------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use  Gtk;
with Gtk.Main;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;
with Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;

with Unbounded_Strings;
use  Unbounded_Strings;

with Version;
use  Version;
with Config;
use  Config;

with Project;
use  Project;
with Systems;
use  Systems;

with Root_Window_Pkg;
use  Root_Window_Pkg;
with Graph_Window_Pkg;
use  Graph_Window_Pkg;
with My_Canvas;
use  My_Canvas;


with Gtk.Widget;
use  Gtk.Widget;
with Gdk.Font;
use  Gdk.Font;
with Gtk.Style;
use  Gtk.Style;

with Gnat.Command_Line;
use  Gnat.Command_Line;

with Gnat.Os_Lib;
use  Gnat.Os_Lib;

with Text_Io;
use  Text_Io;




procedure Cheddar is 



   procedure Usage is 
   begin
      Put_Line (
         "Cheddar is a free real time scheduling simulator.");
      Put_Line(
         "This program provides services to automaticly check temporal constraints of real time tasks.");
      Copyright;
      New_Line;
      Put_Line(
         "Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : cheddar [switch] project-name");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line(
         "            -l select a language : ""fr"" for franais; ""en"" for english ");
      Put("               Default language is ");
      if(Current_Language = English)
            then
         Put("English.");
      else
         Put("Franais.");
      end if;
      New_Line;
   end Usage;


   My_Style    : Gtk_Style;  
   Global_Font : Gdk_Font;  


begin



   -- New project 
   --
   Do_New;


   -- Get arguments 
   --
   loop
      case Getopt ("u l:") is
         when Ascii.Nul =>
            exit;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when 'l' =>
            if (Parameter /= "fr") and
                  (Parameter /= "en")
                  then
               Put_Line("cheddar.adb : invalid language parameter");
               New_Line;
               Usage;
               Os_Exit(0);
            else
               if Parameter = "fr"
                     then
                  Current_Language:=Francais;
               else
                  Current_Language:=English;
               end if;
            end if;

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;


   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;


   Copyright;
   if(File_Name /= Empty_String)
         then
      Current_Directory:=File_Name;
      Is_New:=False;
      Is_Saved:=True;
      Systems.Read_From_Xml_File(Sys, File_Name);
   end if;


   -- Initialize the graphical editor
   --
   Gtk.Main.Set_Locale;
   Gtk.Main.Init;

   Load (Global_Font,
      "-*-*-*-*-*-*-*-*-*-*-*-*-iso8859-1");
   Gtk_New(My_Style);
   Set_Font(My_Style, Global_Font);
   Set_Default_Style (My_Style);

   Gtk_New (Root_Window);
   Show_All(Root_Window);


   -- Graphical editor event loop
   --
   Gtk.Main.Main;



exception
   when Invalid_Switch    =>
      begin
         Put_Line ("cheddar.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Invalid_Parameter =>
      Put_Line ("cheddar.adb : missing parameter for switch " &
         Full_Switch);
   when Xml_Read_Error =>
      Put_Line(
         "cheddar.adb : Can not read project files, Xml Fatal Error");
   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("cheddar.adb : Can not read project files, Name_Error");
   when Ada.Io_Exceptions.Status_Error =>
      Put_Line("cheddar.adb : Can not read project files, Status_Error");
   when Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("cheddar.adb : Can not read project files, Mode_Error");
   when Ada.Io_Exceptions.Use_Error     =>
      Put_Line("cheddar.adb : Can not read project files, Use_Error");
   when Ada.Io_Exceptions.Device_Error  =>
      Put_Line("cheddar.adb : Can not read project files, Device_Error");
   when Ada.Io_Exceptions.End_Error    =>
      Put_Line("cheddar.adb : Can not read project files, End_Error");
   when Ada.Io_Exceptions.Data_Error   =>
      Put_Line("cheddar.adb : Can not read project files, Data_Error");
   when Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("cheddar.adb : Can not read project files, Layout_Error");
   when others =>
      Put_Line("This is an internal Cheddar bug ... sorry");
      Put_Line(
         "WARNING           : your project was saved in the file cheddar_bug.xml");
      Put_Line("Exception name    : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);

      if Graph_Window/=null
            then
         Get_Coordinates(Graph_Window.Buffer, Sys);
      end if;
      Systems.Write_To_Xml_File(Sys, To_Unbounded_String(
            "cheddar_bug.xml"));

      Put_Line("Please, send a bug report to singhoff@univ-brest.fr");
      Put_Line(
         "Do not forget to join the erronous Cheddar project files with your bug report");

end Cheddar;



