------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
-----------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
-- You will find here every constants that can be tune by users.
-- Change definitions contained in this package according to your need
--
-- WARNING : most of the constants defined in this package are used in I/O projects 
--	     operations. Changing one of them can lead to errors during
--	     loading of projects edited with different value of these
--	     constants
------------------------------------------------------------------------------
------------------------------------------------------------------------------






package Config is


   pragma Elaborate_Body;


   -- Maximum processors, tasks, messages, buffers and resources
   -- which can be define in a cheddar simulation
   --
   Max_Processors : constant Natural := 100;  
   Max_Tasks      : constant Natural := 100;  
   Max_Messages   : constant Natural := 50;  
   Max_Buffers    : constant Natural := 30;  
   Max_Resources  : constant Natural := 50;  


   -- Define the largest scheduling interval that the 
   -- simulator can compute
   --
   Max_Scheduling_Period : constant Natural := 5000;  


   -- Definition of priority range. We choose the most current value :
   -- 256 priority levels (from 0 to 255)
   -- Zero is the lowest priority level
   ---
   type Priority_Range is range 0 .. 255; 
   Low_Priority  : constant Priority_Range := Priority_Range'First;  
   High_Priority : constant Priority_Range := Priority_Range'Last;  


   -- Define the language used in the graphical editor of Cheddar
   -- Default language is "english"
   --
   type Languages is 
         (Francais, 
          English); 
   Current_Language : Languages := English;  


   -- Constants above define maximum number of tasks that
   -- can be use a shared resource or a buffer
   --
   Max_Tasks_For_A_Resource : constant Natural := 100;  
   Max_Tasks_For_A_Buffer   : constant Natural := 100;  


   -- Define offset constants : a task can contain at most 3 offsets
   --
   Max_Offsets : constant Natural := 5;  


   -- Define maximum of variables that can be defined 
   -- in parametric scheduler
   --
   Max_Variables : constant Natural := 100;  


   -- Define maximum of user's defined task parameter
   --
   Max_User_Defined_Task_Parameters : constant Natural := 5;  


   -- This constant define the size in "xi" of a discrete 
   -- density probability
   --
   Max_Density_Size : constant Natural := 300;  


   -- Maximum size of the framework request table 
   --
   Max_Request : constant Natural := 400;  


end Config;
