------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------






with Fixed_Strings;
use Fixed_Strings;


use Buffer_Set.Generic_Buffer_Set;
with Buffers;
use Buffers;
use Buffers.Buffer_Roles;


package body Buffer_Set is



   procedure Fill_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       :    out Buffer_Descriptor; 
         A_Buffer   : in     Buffer_Ptr         ) is 

   begin
      Desc.Name := To_Bounded_String(To_String(A_Buffer.Name));
      Desc.Location:= To_Bounded_String(To_String(A_Buffer.Location));
      Desc.Size:= A_Buffer.Size;

      for I in 0..A_Buffer.Roles.Nb_Entries-1 loop

         Desc.Roles.Entries(Io_Roles_Range(I)).Item:=To_Bounded_String(
            To_String(A_Buffer.Roles.Entries(I).Item));
         Desc.Roles.Entries(Io_Roles_Range(I)).Data:=A_Buffer.Roles.
            Entries(I).Data;
      end loop;


      Desc.Roles.Nb_Entries:=Io_Roles_Range(A_Buffer.Roles.Nb_Entries);


   end Fill_Descriptor;


   procedure Extract_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       : in     Buffer_Descriptor; 
         A_Buffer   :    out Buffer_Ptr         ) is 


   begin
      A_Buffer := new Buffer;
      A_Buffer.Name:=To_Unbounded_String(To_String(Desc.Name));
      A_Buffer.Location:=To_Unbounded_String(To_String(Desc.Location));
      A_Buffer.Size:=Desc.Size;

      for I in 0..Desc.Roles.Nb_Entries-1 loop

         A_Buffer.Roles.Entries(Buffer_Roles_Range(I)).Item:=
            To_Unbounded_String(To_String(Desc.Roles.Entries(I).Item));
         A_Buffer.Roles.Entries(Buffer_Roles_Range(I)).Data:=Desc.Roles.
            Entries(I).Data;
      end loop;


      A_Buffer.Roles.Nb_Entries:=Buffer_Roles_Range(Desc.Roles.Nb_Entries);


   end Extract_Descriptor;



   procedure Add_Buffer (
         My_Buffers : in out Buffers_Set;       
         Name       : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ) is 

      Dummy : Buffer_Ptr;  
   begin
      Add_Buffer(My_Buffers, Dummy, Name, Size, Location,
         Roles);
   end Add_Buffer;


   procedure Add_Buffer (
         My_Buffers : in out Buffers_Set;       
         A_Buffer   : in out Buffer_Ptr;        
         Name       : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ) is 


      Desc        : Buffer_Descriptor;  
      My_Iterator : Iterator;  

   begin


      if (Name = "")
            then
         raise Invalid_Parameter;
      end if;

      if (Location = "")
            then
         raise Invalid_Parameter;
      end if;

      if (Size = 0)
            then
         raise Invalid_Parameter;
      end if;

      if(Get_Number_Of_Elements(My_Buffers) > 0)
            then

         Reset_Iterator(My_Iterator);

         loop
            Current_Element(My_Buffers, A_Buffer, My_Iterator);
            if (Name = A_Buffer.Name)
                  then
               raise Buffer_Already_Defined;
            end if;

            exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

            Next_Element(My_Buffers, My_Iterator);
         end loop;
      end if;


      Desc.Name:=To_Bounded_String(To_String(Name));
      Desc.Location:=To_Bounded_String(To_String(Location));
      Desc.Size:=Size;

      for I in 0..Roles.Nb_Entries loop

         Desc.Roles.Entries(Io_Roles_Range(I)).Item:=To_Bounded_String(
            To_String(Roles.Entries(I).Item));
         Desc.Roles.Entries(Io_Roles_Range(I)).Data:=Roles.Entries(I).
            Data;
      end loop;


      Desc.Roles.Nb_Entries:=Io_Roles_Range(Roles.Nb_Entries);



      Extract_Descriptor(My_Buffers, Desc, A_Buffer);


      Add(My_Buffers,A_Buffer);


   end Add_Buffer;

   procedure Update_Buffer (
         My_Buffers : in out Buffers_Set;       
         Name       : in     Unbounded_String;  
         New_Name   : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ) is 
      The_Buffer : Buffer_Ptr;  
   begin
      The_Buffer:=Search_Buffer(My_Buffers,Name);
      The_Buffer.Name:=New_Name;
      The_Buffer.Location:=Location;
      The_Buffer.Size:=Size;
      The_Buffer.Roles:=Roles;

   end Update_Buffer;




   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Natural is 

      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  

   begin

      if ( Param_Name /= Size)
            then
         raise Invalid_Parameter;
      end if;

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if (A_Buffer.Name = Buffer_Name)
               then
            exit;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);

      end loop;

      return A_Buffer.Size;

   end Get;



   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Unbounded_String is 

      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  

   begin

      if ( Param_Name /= Location)
            then
         raise Invalid_Parameter;
      end if;

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if (A_Buffer.Name = Buffer_Name)
               then
            exit;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);

      end loop;

      return A_Buffer.Location;

   end Get;


   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Natural            ) is 
      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  

   begin

      if (Param_Name /= Size)
            then
         raise Invalid_Parameter;
      end if;

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if (A_Buffer.Name = Buffer_Name)
               then
            A_Buffer.Size:=Param_Value;
            exit;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);

      end loop;
   end Set;


   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Unbounded_String   ) is 
      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  

   begin

      if (Param_Name /= Location)
            then
         raise Invalid_Parameter;
      end if;

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if (A_Buffer.Name = Buffer_Name)
               then
            A_Buffer.Location:=Param_Value;
            exit;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);

      end loop;
   end Set;


   function Search_Buffer (
         My_Buffers : in     Buffers_Set;     
         Name       : in     Unbounded_String ) 
     return Buffer_Ptr is 

      My_Iterator : Iterator;  
      A_Buffer    : Buffer_Ptr;  
      Result      : Buffer_Ptr;  

      Found : Boolean := False;  

   begin

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if(A_Buffer.Name = Name) then
            Found:=True;
            Result:=A_Buffer;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);

      end loop;

      if not Found then
         raise Buffer_Not_Found;
      end if;

      return Result;

   end Search_Buffer;



   procedure Delete_Task (
         My_Buffers : in out Buffers_Set;     
         A_Task     : in     Unbounded_String ) is 

      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Buffers_Iterator;  
      Index       : Buffer_Roles_Range := 1;  

   begin

      if(Get_Number_Of_Elements(My_Buffers) > 0)
            then

         Reset_Iterator(My_Iterator);
         loop
            Current_Element(My_Buffers, A_Buffer, My_Iterator);

            -- Looking for a_task in roles
            --
            Index:=1;
            while (Index<=A_Buffer.Roles.Nb_Entries)
                  loop
               if (A_Buffer.Roles.Entries(Index).Item = A_Task)
                     then
                  A_Buffer.Roles.Entries(Index):=A_Buffer.Roles.Entries(
                     A_Buffer.Roles.Nb_Entries);

                  A_Buffer.Roles.Nb_Entries:=
                     A_Buffer.Roles.Nb_Entries-1;
               end if;
            end loop;


            exit when Is_Last_Element(My_Buffers, My_Iterator) = True;
            Next_Element(My_Buffers, My_Iterator);
         end loop;

      end if;

   end Delete_Task;



   procedure Delete_Processor (
         My_Buffers  : in out Buffers_Set;     
         A_Processor : in     Unbounded_String ) is 

      Tmp         : Buffers_Set;  
      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Buffers_Iterator;  

   begin

      if(Get_Number_Of_Elements(My_Buffers) > 0)
            then

         Reset_Iterator(My_Iterator);
         loop
            Current_Element(My_Buffers, A_Buffer, My_Iterator);

            if(A_Buffer.Location=A_Processor)
                  then
               Add(Tmp, A_Buffer);
            end if;

            exit when Is_Last_Element(My_Buffers, My_Iterator) = True;
            Next_Element(My_Buffers, My_Iterator);
         end loop;

         if not Is_Empty(Tmp) then
            Reset_Iterator(My_Iterator);
            loop
               Current_Element(Tmp, A_Buffer, My_Iterator);

               Delete(My_Buffers,A_Buffer);

               exit when Is_Last_Element(Tmp, My_Iterator) = True;
               Next_Element(Tmp, My_Iterator);
            end loop;
         end if;


      end if;
   end Delete_Processor;


   function Get_Number_Of_Buffer_From_Processor (
         My_Buffers     : in     Buffers_Set;     
         Processor_Name : in     Unbounded_String ) 
     return Buffers_Range is 

      Number      : Buffers_Range := 0;  
      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  

   begin

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Buffers,A_Buffer,My_Iterator);

         if (A_Buffer.Location  = Processor_Name)
               then
            Number:=Number+1;
         end if;

         exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

         Next_Element(My_Buffers, My_Iterator);


      end loop;

      return Number;
   end Get_Number_Of_Buffer_From_Processor;




   procedure Export_Xml (
         My_Buffers : in     Buffers_Set; 
         Into       : in     File_Type    ) is 

      My_Iterator : Buffers_Iterator;  
      A_Buffer    : Buffer_Ptr;  


   begin

      if not Is_Empty(My_Buffers) then

         Put_Line(Into, "<buffers>");

         Reset_Iterator(My_Iterator);

         loop
            Current_Element(My_Buffers,A_Buffer,My_Iterator);


            Put(Into,Ascii.Ht & "<buffer");
            if A_Buffer.X/=0
                  then
               Put(Into, " x=""" & A_Buffer.X'Img & """ ");
            end if;
            if A_Buffer.Y/=0
                  then
               Put(Into, "y=""" & A_Buffer.Y'Img & """ ");
            end if;
            Put_Line(Into, ">");


            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<location>" & To_String(
                  A_Buffer.Location) & "</location>");
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<name>" & To_String(
                  A_Buffer.Name) & "</name>");
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<size>" & A_Buffer.Size'
               Img & "</size>");




            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<buffer_used_by>");
            for I in 0..A_Buffer.Roles.Nb_Entries-1 loop
               Put(Into,Ascii.Ht & Ascii.Ht & Ascii.Ht &
                  "<buffer_user");
               if  A_Buffer.Roles.Entries(I).Data.The_Role = Consumer
                     then
                  Put(Into," buffer_role=""consumer"">");
               else
                  Put(Into,">");
               end if;
               Put(Into, " " & To_String(A_Buffer.Roles.Entries(I).Item) &
                  " ");
               Put(Into, " " & A_Buffer.Roles.Entries(I).Data.Size'Img &
                  " ");
               Put_Line(Into, "</buffer_user>");
            end loop;
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "</buffer_used_by>");


            Put_Line(Into,Ascii.Ht & "</buffer>");


            exit when Is_Last_Element(My_Buffers, My_Iterator) = True;

            Next_Element(My_Buffers, My_Iterator);


         end loop;
         Put_Line(Into, "</buffers>");
      end if;


   end Export_Xml;





end Buffer_Set;



