------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use Text_Io;


with Fixed_Strings;
use Fixed_Strings;

with Config;
use Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Buffers;
use Buffers;


with Indexed_Tables;
with Sets;


package Buffer_Set is


   Buffer_Not_Found       : exception;  
   Buffer_Already_Defined : exception;  
   Invalid_Parameter      : exception;  

   package Io_Roles is new Indexed_Tables(Buffer_Role, Fixed_String,
      Config.Max_Tasks, 0,
      Put, Initialize, Fixed_Strings.Put);
   use Io_Roles;

   subtype Io_Roles_Table is Indexed_Table;
   subtype Io_Roles_Range is Indexed_Table_Range;

   type Buffer_Descriptor is 
      record 
         Name     : Fixed_String;  
         Size     : Natural;  
         Location : Fixed_String;  
         Roles    : Io_Roles_Table;  
      end record; 


   type Buffer_Parameters is 
         (Size,     
          Location, 
          Roles); 


   package Generic_Buffer_Set is new Sets (
      Max_Element     => Config.Max_Buffers, 
      File_Descriptor => Buffer_Descriptor,  
      Element         => Buffer_Ptr,         
      Free            => Free,               
      Copy            => Copy,               
      Put             => Put);


   use Generic_Buffer_Set;

   type Buffers_Set is new Generic_Buffer_Set.Set with private; 


   subtype Buffers_Range is Generic_Buffer_Set.Element_Range;
   subtype Buffers_Iterator is   Generic_Buffer_Set.Iterator;


   procedure Export_Xml (
         My_Buffers : in     Buffers_Set; 
         Into       : in     File_Type    ); 


   procedure Fill_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       :    out Buffer_Descriptor; 
         A_Buffer   : in     Buffer_Ptr         ); 


   procedure Extract_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       : in     Buffer_Descriptor; 
         A_Buffer   :    out Buffer_Ptr         ); 


   procedure Add_Buffer (
         My_Buffers : in out Buffers_Set;       
         A_Buffer   : in out Buffer_Ptr;        
         Name       : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ); 


   procedure Add_Buffer (
         My_Buffers : in out Buffers_Set;       
         Name       : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ); 


   procedure Update_Buffer (
         My_Buffers : in out Buffers_Set;       
         Name       : in     Unbounded_String;  
         New_Name   : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Roles      : in     Buffer_Roles_Table ); 


   procedure Delete_Processor (
         My_Buffers  : in out Buffers_Set;     
         A_Processor : in     Unbounded_String ); 


   procedure Delete_Task (
         My_Buffers : in out Buffers_Set;     
         A_Task     : in     Unbounded_String ); 


   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Unbounded_String; 


   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Unbounded_String   ); 


   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Natural; 

   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Natural            ); 


   function Search_Buffer (
         My_Buffers : in     Buffers_Set;     
         Name       : in     Unbounded_String ) 
     return Buffer_Ptr; 


   function Get_Number_Of_Buffer_From_Processor (
         My_Buffers     : in     Buffers_Set;     
         Processor_Name : in     Unbounded_String ) 
     return Buffers_Range; 


private

   type Buffers_Set is new Generic_Buffer_Set.Set with null record; 




end Buffer_Set;
