------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;

with Tasks;
use Tasks;


with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;


with Unbounded_Strings;
use Unbounded_Strings;

with Time_Units;
use Time_Units;


with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Double_Util;
use Double_Util;


with Config;
use Config;
with Translate;
use Translate;


package body Buffers is


   procedure Put (
         B : in     Buffer_Role ) is 
   begin
      Put(B.The_Role'Img & "/" & B.Size'Img);
   end Put;

   procedure Initialize (
         B :    out Buffer_Role ) is 
   begin
      B.The_Role:=No_Role;
      B.Size:=0;
   end Initialize;


   procedure Initialize (
         B :    out Buffer_Size ) is 
   begin
      B.Time:=0;
      B.Size:=0;
   end Initialize;


   procedure Put (
         B : in     Buffer_Size ) is 
   begin
      Put(B.Time'Img & "/" & B.Size'Img);
   end Put;





   function Get_Max_Size (
         Buff : in     Buffer_Size_Table ) 
     return Natural is 

      Max : Natural := 0;  
   begin

      for I in 0..Buff.Nb_Entries-1 loop
         if Buff.Entries(I).Size > Max
               then
            Max:=Buff.Entries(I).Size;
         end if;
      end loop;

      return Max;

   end Get_Max_Size;


   ----------------------------------------------------
   -- We produce in the start of activation
   -- and we consume in the end of the activation
   ----------------------------------------------------
   procedure Get_Buffer_Size (
         Sched    : in     Scheduling_Sequence; 
         My_Buff  : in     Buffer_Ptr;          
         My_Tasks : in     Tasks_Set;           
         Buff     : in out Buffer_Size_Table    ) is 


      type Item is 
         record 
            Ts               : Task_Ptr;  
            Rest_Of_Capacity : Natural;  
            Ru               : Buffer_Role;  
         end record; 

      type Table is array (Tasks_Range) of Item; 

      Tcb             : Table;  
      My_Iterator     : Tasks_Iterator;  
      A_Task          : Task_Ptr;  
      Index2          : Buffer_Roles_Range := 0;  
      Ok              : Boolean            := True;  
      Current_Size    : Natural            := 0;  
      Number_Of_Tasks : Tasks_Range        := 0;  
      A_Buff_Size     : Buffer_Size;  

   begin

      Initialize(Buff);

      -- First, gather information
      --

      Reset_Iterator(My_Iterator);

      loop

         Current_Element(My_Tasks, A_Task, My_Iterator);


         -- Check if the task is producer/consumer of the
         -- buffer
         --
         Index2:=0;
         Ok:=True;
         while ((Index2<=My_Buff.Roles.Nb_Entries) and (Ok) ) loop
            if (A_Task.Name = My_Buff.Roles.Entries(Index2).Item)
                  then

               Ok:=False;

               Tcb(Number_Of_Tasks).Ts:=A_Task;
               Tcb(Number_Of_Tasks).Ru:=My_Buff.Roles.Entries(Index2).
                  Data;
               Tcb(Number_Of_Tasks).Rest_Of_Capacity:=A_Task.Capacity;

               Number_Of_Tasks:=Number_Of_Tasks+1;
            end if;
            Index2:=Index2+1;
         end loop;

         exit when Is_Last_Element(My_Tasks, My_Iterator) = True;
         Next_Element(My_Tasks, My_Iterator);


      end loop;


      -- And then compute buffer size
      --
      for I in 0 .. Sched.Scheduled.Nb_Entries-1 loop

         if (Sched.Scheduled.Entries(I).Item.Etype=Running) then

            -- Find if the task is producer/consumer
            --
            for J in 0..Number_Of_Tasks-1 loop


               if(Tcb(J).Ts.Name=Sched.Scheduled.Entries(I).Item.
                     Running_Task.Name)
                     then

                  -- Produce ?
                  if (Tcb(J).Rest_Of_Capacity=Tcb(J).Ts.Capacity) and
                        (Tcb(J).Ru.The_Role = Producer)
                        then
                     Current_Size:=Current_Size+Tcb(J).Ru.Size;
                  end if;
                  -- Consume ?

                  Tcb(J).Rest_Of_Capacity:=Tcb(J).Rest_Of_Capacity-1;
                  if (Tcb(J).Rest_Of_Capacity=0) and
                        (Tcb(J).Ru.The_Role = Consumer)
                        then
                     if (Current_Size-Tcb(J).Ru.Size <= 0)
                           then
                        Current_Size:=0;
                        -- la capacite du tampon ne peut etre negative!!
                     else
                        Current_Size:=Current_Size-Tcb(J).Ru.Size;
                     end if;


                  end if;

                  if Tcb(J).Rest_Of_Capacity=0
                        then
                     Tcb(J).Rest_Of_Capacity:=Tcb(J).Ts.Capacity;
                  end if;

               end if;

            end loop;

            -- Store buffer size
            --

            A_Buff_Size.Time:=Sched.Scheduled.Entries(I).Data;
            A_Buff_Size.Size:=Current_Size;
            Add(Buff, A_Buff_Size);
         end if;
      end loop;
   end Get_Buffer_Size;




   procedure Initialize (
         A_Buff : in out Buffer ) is 
   begin
      A_Buff.Dependence_Type := Buffer_Dependency;
      A_Buff.Location:=Empty_String;
      A_Buff.Name:=Empty_String;
      A_Buff.Size:=0;
      Initialize(A_Buff.Roles);
   end Initialize;


   procedure Finalize (
         A_Buff : in out Buffer ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Buff : in out Buffer ) is 
   begin
      null;
   end Adjust;


   procedure Dispatched_Put (
         My_Buff : in     Buffer ) is 
   begin
      Put("Buffer name : " & To_String(My_Buff.Name));
      Put(" ; Location : " & To_String(My_Buff.Location));
      Put(" ; Size : " & My_Buff.Size'Img);
      Put(" ; x : " & My_Buff.X'Img);
      Put(" ; y : " & My_Buff.Y'Img);
      New_Line;
      Put(My_Buff.Roles);
      New_Line;
   end Dispatched_Put;




   procedure Put (
         My_Buff : in     Buffer_Ptr ) is 
   begin
      Put(My_Buff.all);
   end Put;



   function Copy (
         My_Buffer : in     Buffer_Ptr ) 
     return Buffer_Ptr is 

      Ptr : Buffer_Ptr;  
   begin
      Ptr:=new Buffer;
      Ptr.all:=My_Buffer.all;

      return Ptr;
   end Copy;

   function Dispatched_Copy (
         My_Buffer : Buffer ) 
     return Dependency_Ptr is 

      Src :
      aliased Buffer := My_Buffer;
      Ptr : Buffer_Ptr;  
   begin

      Ptr:=Src'Unchecked_Access;

      return Dependency_Ptr(Copy(Ptr));
   end Dispatched_Copy;



   procedure Put_Name (
         My_Buff : in     Buffer_Ptr ) is 
   begin
      Put(My_Buff.Name);
   end Put_Name;


   ----------


   procedure Get_Bound (
         My_Buff        : in     Buffer_Ptr;       
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Bound          : in out Natural;          
         Msg            : in out Unbounded_String  ) is 

      N,  
      M : Natural;  

   begin

      -- Controls
      --
      Periodic_Control(My_Tasks, Processor_Name);
      Buffer_Flow_Control(My_Buff, My_Tasks);

      N := 0;
      M := 0;
      Msg:=Empty_String;


      -- Number of producer and consumer
      --
      for J in  0 .. (My_Buff.Roles.Nb_Entries - 1) loop
         if (My_Buff.Roles.Entries(J).Data.The_Role = Producer)
               then
            N := N + 1;
         end if;
         if (My_Buff.Roles.Entries(J).Data.The_Role = Consumer)
               then
            M := M + 1;
         end if;
      end loop;

      if (M <= 1) -- one consumer
            then

         Msg:=To_Unbounded_String(" (") & Lb_See(Current_Language) &
            To_Unbounded_String("[4], page 3, ") & Lb_Theorem(
            Current_Language) &
            To_Unbounded_String("1). ") & Unbounded_Lf;

         if (Is_Cons_Prod_Harmonic(My_Buff, My_Tasks) = True)
               then
            Bound:=2*N;
         else
            Bound:=2*N+1;
         end if;
      else
         raise Task_Model_Error;
      end if;

   end Get_Bound;


   procedure Buffer_Flow_Control (
         My_Buff  : in     Buffer_Ptr; 
         My_Tasks : in     Tasks_Set   ) is 

      Flow_Cons,  
      Flow_Prod : Double;  
      A_Task    : Task_Ptr;  

   begin
      --init
      Flow_Prod := 0.0;
      Flow_Cons := 0.0;

      -- compute data flow
      for I in  0 .. (My_Buff.Roles.Nb_Entries - 1) loop

         -- compute producers data flow
         if (My_Buff.Roles.Entries(I).Data.The_Role = Producer)
               then
            A_Task := Search_Task(My_Tasks,My_Buff.Roles.Entries(I).Item);

            case A_Task.Task_Type is
               when Periodic_Type =>
                  Flow_Prod := Flow_Prod +
                     Double(My_Buff.Roles.Entries(I).Data.Size) /
                     Double(Periodic_Task_Ptr(A_Task).Period);

               when Poisson_Type =>
                  Flow_Prod := Flow_Prod +
                     Double(My_Buff.Roles.Entries(I).Data.Size) /
                     Double(Poisson_Task_Ptr(A_Task).Period);
               when others =>
                  raise Constraint_Error;
            end case;


         end if;

         -- Compute consumers data flow
         --
         if (My_Buff.Roles.Entries(I).Data.The_Role = Consumer)
               then
            A_Task := Search_Task(My_Tasks,My_Buff.Roles.Entries(I).Item);

            case A_Task.Task_Type is
               when Periodic_Type =>
                  Flow_Cons := Flow_Cons +
                     Double(My_Buff.Roles.Entries(I).Data.Size) /
                     Double(Periodic_Task_Ptr(A_Task).Period);
               when Poisson_Type =>
                  Flow_Cons := Flow_Cons +
                     Double(My_Buff.Roles.Entries(I).Data.Size) /
                     Double(Poisson_Task_Ptr(A_Task).Period);
               when others =>
                  raise Constraint_Error;
            end case;
         end if;

      end loop;

      if (not Equal(Flow_Prod,Flow_Cons)) and  (Flow_Prod > Flow_Cons)
            then
         raise Flow_Constraint_Not_Respected;
      end if;
   end Buffer_Flow_Control;



   function Is_Cons_Prod_Harmonic (
         My_Buff  : in     Buffer_Ptr; 
         My_Tasks : in     Tasks_Set   ) 
     return Boolean is 

      A_Task1,  
      A_Task2 : Task_Ptr;  
      Result  : Boolean  := True;  
      Tmp_Mod : Natural;  

   begin

      for I in  0 .. (My_Buff.Roles.Nb_Entries - 1) loop
         A_Task1 := Search_Task(My_Tasks,My_Buff.Roles.Entries(I).Item);
         for J in  I .. (My_Buff.Roles.Nb_Entries - 1) loop
            A_Task2 := Search_Task(My_Tasks,My_Buff.Roles.Entries(J).Item);

            if A_Task1.Task_Type = Periodic_Type then

               if ( Periodic_Task_Ptr(A_Task1).Period >
                     Periodic_Task_Ptr(A_Task2).Period)
                     then
                  Tmp_Mod := Periodic_Task_Ptr(A_Task1).Period mod
                     Periodic_Task_Ptr(A_Task2).Period;
               else
                  Tmp_Mod := Periodic_Task_Ptr(A_Task2).Period mod
                     Periodic_Task_Ptr(A_Task1).Period;
               end if;

            else
               if A_Task1.Task_Type = Poisson_Type then

                  if ( Poisson_Task_Ptr(A_Task1).Period >
                        Poisson_Task_Ptr(A_Task2).Period)
                        then
                     Tmp_Mod := Poisson_Task_Ptr(A_Task1).Period mod
                        Poisson_Task_Ptr(A_Task2).Period;
                  else
                     Tmp_Mod := Poisson_Task_Ptr(A_Task2).Period mod
                        Poisson_Task_Ptr(A_Task1).Period;
                  end if;
               end if;
            end if;

            if (Tmp_Mod /= 0 )
                  then
               Result:=False;
            end if;
         end loop;
      end loop;
      return Result;
   end Is_Cons_Prod_Harmonic;








end Buffers;





