------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Config;
use Config;
with Translate;
use Translate;

with Xml_Tag;
use Xml_Tag;
with Task_Set;
use Task_Set;

with Buffers;
use Buffers;
use Buffers.Buffer_Size_Package;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Unbounded_Strings;
use Unbounded_Strings;

with Call_Scheduling_Framework;
use Call_Scheduling_Framework;

with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;



package body Call_Buffer_Framework is




   procedure Compute_Buffer_Utilization (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ) is 


      A_Buffer        : Buffer_Ptr;  
      My_Iterator     : Buffers_Iterator;  
      Index           : Scheduling_Range  := 0;  
      A_Buffer_Result : Buffer_Size_Table;  


   begin


      -- find the processor on which we want to compute buffer utilization
      --
      for I in 0..Call_Scheduling_Framework.Sched.Nb_Entries-1 loop

         if Call_Scheduling_Framework.Sched.Entries(I).Data.Processor/=null then
            if Call_Scheduling_Framework.Sched.Entries(I).Data.
                  Processor.Name
                  = A_Processor.Name
                  then
               Index:=I;
            end if;
         end if;
      end loop;


      -- Compute buffer utilization on all buffer of the 
      -- selected processor
      --


      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);


         if A_Processor.Name = A_Buffer.Location
               then

            Initialize(A_Buffer_Result);

            Get_Buffer_Size(Sched.Entries(Index).Data.Result,
               A_Buffer, Sys.Tasks, A_Buffer_Result);

            Add(Buff, A_Buffer, A_Buffer_Result);

         end if;

         exit when Is_Last_Element(Sys.Buffers, My_Iterator) =
            True;
         Next_Element(Sys.Buffers, My_Iterator);

      end loop;

   end Compute_Buffer_Utilization;



   procedure Compute_Buffer_Bound (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ) is 



      A_Buffer    : Buffer_Ptr;  
      Bnd         : Natural;  
      Ref         : Unbounded_String;  
      My_Iterator : Iterator;  

   begin

      if Output=Xml_Output then
         Set_Tag;
      else
         Set_Empty;
      end if;
      Reset_Iterator(My_Iterator);

      Result :=  Unbounded_Lf & Start_Line & Unbounded_Lf;

      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         if (A_Buffer.Location = A_Processor.Name) then
            if (Natural(A_Buffer.Roles.Nb_Entries) >= 1)
                  then
               begin

                  Ref:= Empty_String;
                  Get_Bound(A_Buffer, Sys.Tasks,
                     A_Processor.Name, Bnd,  Ref);

                  Result:= Result & Start_Task & To_Unbounded_String(
                     "    ") &
                     A_Buffer.Name &
                     To_Unbounded_String(" => ") &
                     Bnd'Img & End_Task & Start_Ref & Ref & End_Ref  &
                     Unbounded_Lf;
                  Result:=Result & Unbounded_Lf & End_Line & Unbounded_Lf &
                     End_Buf_Bound & Unbounded_Lf;

               exception
                  when Task_Set.Task_Model_Error=>
                     Result:= Result & Lb_Compute_Error_20(
                        Current_Language) &
                        Unbounded_Lf;
                  when Task_Set.Task_Must_Be_Periodic=>
                     Result:=Result & Lb_Compute_Error_17(
                        Current_Language) &
                        Unbounded_Lf;
                  when Buffers.Flow_Constraint_Not_Respected =>
                     Result:=Result & Lb_Compute_Error_16(
                        Current_Language) &
                        Unbounded_Lf;
               end;
            else
               Result:= Result &  To_Unbounded_String("    ") &
                  A_Buffer.
                  Name
                  & To_Unbounded_String(" => NA");
            end if;
         end if;
         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

   end Compute_Buffer_Bound;

end Call_Buffer_Framework;




