------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Systems;
use Systems;

with Call_Framework;
use Call_Framework;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;

with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;

with Multiprocessor_Services;
use Multiprocessor_Services;




package Call_Scheduling_Framework  is


   pragma Elaborate_Body;




   -- Last computed scheduling
   --
   Sched                  : Scheduling_Table_Ptr;  
   Scheduling_Base_Period : Natural;  





   procedure Compute_Simulation_Time_Line (
         Sys                        : in out System;                           
         Result                     : in out Unbounded_String;                 
         Period                     : in     Natural;                          
         Schedule_With_Offsets      : in     Boolean;                          
         Schedule_With_Precedencies : in     Boolean;                          
         Schedule_With_Resources    : in     Boolean;                          
         Seed_Value                 : in     Natural;                          
         Output                     : in     Output_Format    := String_Output ); 


   procedure Compute_Simulation_Basics (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Compute_Simulation_Number_Of_Preemption (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Compute_Simulation_Response_Time (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Compute_Feasibility_Processor_Utilization_Factor (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Compute_Feasibility_Period (
         Sys         : in     System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Compute_Feasibility_Response_Time (
         Sys         : in out System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         Output      : in     Output_Format    := String_Output ); 


   procedure Set_Priorities (
         Sys         : in out System;                           
         Result      : in out Unbounded_String;                 
         A_Processor : in     Processor_Ptr;                    
         A_Scheduler : in     Scheduler_Type;                   
         Output      : in     Output_Format    := String_Output ); 



end Call_Scheduling_Framework;


