------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Finalization;

with Config;
use Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Sets;


package Dependencies is


   type Dependency_Type is 
         (Precedence_Dependency,   
          Buffer_Dependency,       
          Communication_Dependency); 

   type Dependency is abstract new Ada.Finalization.Controlled with 
      record 
         Dependence_Type : Dependency_Type;  
      end record; 
   type Dependency_Ptr is access all Dependency'Class; 

   procedure Free (
         My_Dep : in out Dependency_Ptr ); 
   pragma Import (C, Free, "__gnat_free");



   procedure Initialize (
         A_Dep : in out Dependency ); 
   procedure Finalize (
         A_Dep : in out Dependency ); 
   procedure Adjust (
         A_Dep : in out Dependency ); 



   procedure Put (
         My_Dep : in     Dependency_Ptr ); 

   procedure Put (
         My_Dep : in     Dependency'Class ); 

   procedure Dispatched_Put (
         My_Dep : in     Dependency ) is 
   abstract;



   function Copy (
         My_Dep : in     Dependency_Ptr ) 
     return Dependency_Ptr; 

   function Copy (
         My_Dep : in     Dependency'Class ) 
     return Dependency_Ptr; 

   function Dispatched_Copy (
         My_Dep : in     Dependency ) 
     return Dependency_Ptr is 
   abstract;


   ----------------------------------------------------------------------

   type Precedence is new Dependency with 
      record 
         From_Task : Unbounded_String;  
         To_Task   : Unbounded_String;  
      end record; 
   type Precedence_Ptr is access all Precedence'Class; 


   procedure Initialize (
         A_Pred : in out Precedence ); 
   procedure Finalize (
         A_Pred : in out Precedence ); 
   procedure Adjust (
         A_Pred : in out Precedence ); 

   procedure Dispatched_Put (
         My_Pred : in     Precedence ); 

   function Dispatched_Copy (
         My_Pred : in     Precedence ) 
     return Dependency_Ptr; 

   function Copy (
         My_Precedence : in     Precedence_Ptr ) 
     return Precedence_Ptr; 


   ---------------------------------------
   --  Define generic dependencies set
   ---------------------------------------
   -- File_descriptor is set to "integer"
   -- since this data structure is not design
   -- to I/O
   ---------------------------------------

   package Dep_Set is new Sets (
      Max_Element     => Config.Max_Tasks, 
      File_Descriptor => Integer,          
      Element         => Dependency_Ptr,   
      Put             => Put,              
      Copy            => Copy,             
      Free            => Free);
   use Dep_Set;

   type Depends_Set is new Dep_Set.Set with private; 

   subtype Depends_Range is Dep_Set.Element_Range;
   subtype Depends_Iterator is Dep_Set.Iterator;

private

   type Depends_Set is new Dep_Set.Set with null record; 


end Dependencies;


