------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--
-- This package provides some algorithms to scan dependencies graph and
-- compute scheduling parameters (deadlines, priorities, response times, ...)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Task_Set;
use Task_Set;


with Task_Dependencies;
use Task_Dependencies;

with Scheduler;
use Scheduler;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;



package Dependencies_Services is



   -- Raised when tasks in a DAG have differents
   -- value of period 
   --
   Dependencies_Period_Error : exception;  

   -- Raised when tasks in a DAG are of different type
   --
   Task_Model_Error : exception;  

   -- Raised when a method do not permit a cyclic graph
   --
   Cyclic_Graph_Error : exception;  




   -- Compute task parameters of a DAG organized tasks
   -- according to Chetto/Blazewicz method
   --
   procedure Set_Parameters_According_To_Chetto (
         My_Dependencies    : in     Tasks_Dependencies; 
         My_Tasks           : in out Tasks_Set;          
         Compute_Deadlines  : in     Boolean;            
         Compute_Priorities : in     Boolean             ); 



   -- Compute end to end response time by the Holistic
   -- method
   --
   procedure Compute_End_To_End_Response_Time (
         My_Dependencies  : in     Tasks_Dependencies; 
         My_Processors    : in     Processors_Set;     
         My_Tasks         : in out Tasks_Set;          
         One_Step         : in     Boolean;            
         Update_Tasks_Set : in     Boolean;            
         Msg              : in out Unbounded_String;   
         Rt               : in out Response_Table      ); 



   -- Test is two tables of response time are equals
   --
   function Is_Equal (
         Rtn  : in     Response_Table; 
         Rtn1 : in     Response_Table  ) 
     return Boolean; 



   -- To schedule with precedencies
   -- all task of a dag must have the same priority
   -- If "Processor_Name" is a null string, do this check
   -- on all processor
   --
   procedure Dependencies_Periods_Control (
         My_Tasks       : in     Tasks_Set;          
         My_Deps        : in     Tasks_Dependencies; 
         Processor_Name : in     Unbounded_String    ); 



end Dependencies_Services;





