------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Dependencies;
use Dependencies;


package Messages is


   type Messages_Type is 
         (Periodic_Type,  
          Aperiodic_Type, 
          Generic_Type); 

   --------------------------------------------------

   type Generic_Message is abstract new Dependency with 
      record 
         Name         : Unbounded_String := To_Unbounded_String ("");  
         Message_Type : Messages_Type;  
         Deadline     : Natural          := 0;  
         Size         : Natural          := 0;  
         X            : Natural          := 0;  
         Y            : Natural          := 0;  
      end record; 

   type Message_Ptr is access all Generic_Message'Class; 


   procedure Free (
         My_Message : in out Message_Ptr ); 
   pragma Import (C, Free, "__gnat_free");


   procedure Initialize (
         A_Message : in out Generic_Message ); 
   procedure Finalize (
         A_Message : in out Generic_Message ); 
   procedure Adjust (
         A_Message : in out Generic_Message ); 


   procedure Put (
         My_Message : in     Generic_Message'Class ); 

   procedure Put (
         My_Message : in     Message_Ptr ); 

   procedure Dispatched_Put (
         My_Message : in     Generic_Message ) is 
   abstract;

   function Copy (
         My_Message : in     Message_Ptr ) 
     return Message_Ptr; 


   procedure Put_Name (
         My_Message : in     Message_Ptr ); 


   --------------------------------------------------

   type Periodic_Message is new Generic_Message with 
      record 
         Period : Natural := 0;  
         Jitter : Natural := 0;  
      end record; 

   type Periodic_Message_Ptr is access all Periodic_Message'Class; 

   procedure Initialize (
         A_Message : in out Periodic_Message ); 
   procedure Finalize (
         A_Message : in out Periodic_Message ); 
   procedure Adjust (
         A_Message : in out Periodic_Message ); 

   procedure Dispatched_Put (
         My_Message : in     Periodic_Message ); 

   function Dispatched_Copy (
         My_Message : in     Periodic_Message ) 
     return Dependency_Ptr; 

   --------------------------------------------------


   type Aperiodic_Message is new Generic_Message with null record; 

   type Aperiodic_Message_Ptr is access all Aperiodic_Message'Class; 

   procedure Initialize (
         A_Message : in out Aperiodic_Message ); 
   procedure Finalize (
         A_Message : in out Aperiodic_Message ); 
   procedure Adjust (
         A_Message : in out Aperiodic_Message ); 

   procedure Dispatched_Put (
         My_Message : in     Aperiodic_Message ); 



   function Dispatched_Copy (
         My_Message : in     Aperiodic_Message ) 
     return Dependency_Ptr; 


end Messages;
