------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--      This package provides tools to support multiprocessing.
--      Today, you only find here stuff to scheduling tasks in several processors
--      with dependencies constraints
------------------------------------------------------------------------------
------------------------------------------------------------------------------






with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;


with Processors;
use Processors;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Resources;
use Resources;
use Resources.Task_Lists;

with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Config;
use Config;


with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Indexed_Tables;


with Task_Dependencies;
use Task_Dependencies;



package Multiprocessor_Services is



   type Scheduling_Result is 
      record 
         Processor : Processor_Ptr;  
         Msg       : Unbounded_String;  
         Result    : Scheduling_Sequence;  
      end record; 

   type Scheduling_Result_Ptr is access all Scheduling_Result; 


   procedure Put (
         R : in     Scheduling_Result ); 
   procedure Initialize (
         R :    out Scheduling_Result ); 


   package Scheduling_Result_Per_Processor is new Indexed_Tables(
      Scheduling_Result, Processor_Ptr, Config.Max_Processors, 0, Put,
      Initialize, Processors.Put_Name);
   use Scheduling_Result_Per_Processor;

   subtype Scheduling_Table is Scheduling_Result_Per_Processor.
      Indexed_Table;
   subtype Scheduling_Range is Scheduling_Result_Per_Processor.
      Indexed_Table_Range;

   type Scheduling_Table_Ptr is access all Scheduling_Table; 



   ----------------------------------------

   procedure Build_Multiprocessor_Scheduling (
         My_Processors      : in     Processors_Set;               
         My_Tasks           : in     Tasks_Set;                    
         My_Resources       : in     Resources_Set;                
         My_Dependencies    : in     Tasks_Dependencies;           
         Result             :    out Scheduling_Table_Ptr;         
         Last_Time          :        Natural;                      
         Common_Seed_Value  :        Integer              := 0;    
         With_Offsets       :        Boolean              := True; 
         With_Precedencies  :        Boolean              := True; 
         With_Resources     :        Boolean              := True; 
         Task_Specific_Seed :        Boolean              := True  ); 





end Multiprocessor_Services;


