------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Network;
use Network;


package Network.Predefined is


   type Predefined_Network is new Generic_Network with private; 


   procedure Initialize (
         A_Network : in out Predefined_Network ); 

   procedure Finalize (
         A_Network : in out Predefined_Network ); 

   procedure Adjust (
         A_Network : in out Predefined_Network ); 


   procedure Register_Message (
         A_Network     : in out Predefined_Network; 
         Message_Name  : in     Unbounded_String;   
         Response_Time : in     Double              ); 


   procedure Register_Messages (
         A_Network     : in out Predefined_Network; 
         Number        : in     Response_Range;     
         Response_Time : in     Response_Table      ); 


   procedure Dispatched_Read_From_File (
         My_Network : in out Predefined_Network; 
         File_Name  : in     Unbounded_String    ); 

   procedure Dispatched_Write_To_File (
         My_Network : in     Predefined_Network; 
         File_Name  : in     Unbounded_String    ); 

   procedure Dispatched_Reset (
         My_Network : in out Predefined_Network ); 

   procedure Dispatched_Put (
         My_Network : in     Predefined_Network ); 


   procedure Dispatched_Transmission_Delay (
         My_Network    : in     Predefined_Network; 
         My_Messages   : in     Messages_Set;       
         Response_Time :    out Response_Table      ); 






private


   type Predefined_Network is new Generic_Network with 
      record 
         Number_Of_Messages : Response_Range := 0;  
         Rt                 : Response_Table;  
      end record; 


end Network.Predefined;

