------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Text_Io;
use Text_Io;

package body Network is


   procedure Initialize (
         My_Network : in out Generic_Network ) is 
   begin

      Reset(My_Network);
   end Initialize;


   procedure Finalize (
         My_Network : in out Generic_Network ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         My_Network : in out Generic_Network ) is 
   begin
      null;
   end Adjust;






   procedure Put (
         My_Network : in     Generic_Network'Class ) is 
   begin
      Put_Line("Name = " & To_String(My_Network.Name));
      Put_Line("Architecture = " & My_Network.Architecture'Img);
      Put_Line("Reception_Latency_Bound = " & My_Network.
         Reception_Latency_Bound'Img);
      Put_Line("Emission_Latency_Bound = " & My_Network.
         Emission_Latency_Bound'Img);
      Put_Line("Media_Length = " & My_Network.Media_Length'Img);

      Dispatched_Put(My_Network);
   end Put;


   procedure Reset (
         My_Network : in out Generic_Network'Class ) is 
   begin
      My_Network.Reception_Latency_Bound := 0;
      My_Network.Emission_Latency_Bound  := 0;
      My_Network.Media_Length          := 0;
      My_Network.Transmission_Bandwidth := 0;

      Dispatched_Reset(My_Network);

   end Reset;

   procedure Write_To_File (
         My_Network : in     Generic_Network'Class; 
         File_Name  : in     Unbounded_String       ) is 
   begin
      Dispatched_Write_To_File (My_Network, File_Name);
   end Write_To_File;

   procedure Read_From_File (
         My_Network : in out Generic_Network'Class; 
         File_Name  : in     Unbounded_String       ) is 
   begin
      Dispatched_Read_From_File (My_Network, File_Name);
   end Read_From_File;


   procedure Compute_Transmission_Delay (
         My_Network    : in     Generic_Network'Class; 
         My_Messages   : in     Messages_Set;          
         Response_Time :    out Response_Table         ) is 
   begin
      Dispatched_Transmission_Delay(My_Network, My_Messages, Response_Time);
   end Compute_Transmission_Delay;





   function Get_Name (
         My_Network : in     Generic_Network'Class ) 
     return Unbounded_String is 
   begin
      return My_Network.Name;
   end Get_Name;


   function Get (
         My_Network : in     Generic_Network'Class; 
         Param_Name : in     Network_Parameters     ) 
     return Natural is 
   begin
      if ( (Param_Name /= Media_Length)
            and (Param_Name /= Reception_Latency_Bound)
            and (Param_Name /= Emission_Latency_Bound)
            and (Param_Name /= Transmission_Bandwidth) )
            then
         raise Invalid_Parameter;
      end if;

      if (Param_Name = Media_Length)
            then
         return My_Network.Media_Length;
      else
         if (Param_Name /= Reception_Latency_Bound)
               then
            return My_Network.Reception_Latency_Bound;
         else
            if (Param_Name /= Emission_Latency_Bound)
                  then
               return My_Network.Emission_Latency_Bound;
            else
               return My_Network.Transmission_Bandwidth;
            end if;
         end if;
      end if;
   end Get;


   procedure Set (
         My_Network  : in out Generic_Network'Class; 
         Param_Name  : in     Network_Parameters;    
         Param_Value : in     Natural                ) is 
   begin
      if ( (Param_Name /= Media_Length)
            and (Param_Name /= Reception_Latency_Bound)
            and (Param_Name /= Emission_Latency_Bound)
            and (Param_Name /= Transmission_Bandwidth) )
            then
         raise Invalid_Parameter;
      end if;

      if (Param_Name = Media_Length)
            then
         My_Network.Media_Length:=Param_Value;
      else
         if (Param_Name /= Reception_Latency_Bound)
               then
            My_Network.Reception_Latency_Bound:=Param_Value;
         else
            if (Param_Name /= Emission_Latency_Bound)
                  then
               My_Network.Emission_Latency_Bound:=Param_Value;
            else
               My_Network.Transmission_Bandwidth:=Param_Value;
            end if;
         end if;
      end if;
   end Set;

   function Get (
         My_Network : in     Generic_Network'Class; 
         Param_Name : in     Network_Parameters     ) 
     return Network_Architecture is 
   begin
      if (Param_Name /= Architecture)
            then
         raise Invalid_Parameter;
      end if;

      return My_Network.Architecture;
   end Get;



end Network;
