------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--  This package contains description of user defined parameters 
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Config;
use Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Tables;



package Parameters is



   -- User defined parameters can have several types : boolean, integer, double or string
   -- 	
   type Parameter_Type is 
         (Boolean_Parameter, 
          Integer_Parameter, 
          Double_Parameter,  
          String_Parameter); 

   -- Definition of a user defined parameter
   --
   type Parameter 
         (Ptype : Parameter_Type) is 
      record 

         -- Name of the parameter
         --
         Name : Unbounded_String;  

         -- Type and value of the parameter
         --
case Ptype is when Boolean_Parameter => Boolean_Value : Boolean; 
   when Integer_Parameter =>
   Integer_Value : Integer;  
   when Double_Parameter =>
   Double_Value : Double;  
   when String_Parameter =>
   String_Value : Unbounded_String;  
end case;
end record;

type Parameter_Ptr is access Parameter; 


procedure Initialize (
      My_Parameter :    out Parameter_Ptr ); 

procedure Put (
      My_Parameter : Parameter_Ptr ); 


-- Here, we define a table of parameters : each task can contain a table 
-- of such defined bellow
--
package Parameters_Table_Package is new Tables(Parameter_Ptr,
   Max_User_Defined_Task_Parameters,Put);
use Parameters_Table_Package;

-- Range and type of parameters tables
--
subtype Parameters_Range is Parameters_Table_Package.Table_Range;
subtype Parameters_Table is Parameters_Table_Package.Table;


-- Empty table 
--
No_Parameter :  Parameters_Table;


end Parameters;



