



with Text_Io;
use Text_Io;
with Config;
use Config;
with Scheduler_Lex;
use Scheduler_Lex;
with Scheduler_Dfa;
use Scheduler_Dfa;
with Scheduler_Goto;
use Scheduler_Goto;
with Scheduler_Tokens;
use Scheduler_Tokens;
with Scheduler_Shift_Reduce;
use Scheduler_Shift_Reduce;
with Ada.Numerics.Aux;
use Ada.Numerics.Aux;
with Translate;
use Translate;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
with Ada.Exceptions;
use Ada.Exceptions;
with Simulations;
use Simulations;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Tasks;
use Tasks;
with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;



package body Parser is


   -- The langague is block oriented : the constant below
   -- given the maximum depth level of block
   --
   Max_Block_Level : constant Natural := 10000;  


   -- These variables store statements of inner blocks of code
   --
   Else_Inner_Statement_Ptr   : array (0 .. Max_Block_Level) of Statement_Ptr;
      
   Else_Inner_Statement_Index : Natural                                       := 
     0;  
   Then_Inner_Statement_Ptr   : array (0 .. Max_Block_Level) of Statement_Ptr;
      
   Then_Inner_Statement_Index : Natural                                       := 
     0;  
   Loop_Inner_Statement_Ptr   : array (0 .. Max_Block_Level) of Statement_Ptr;
      
   Loop_Inner_Statement_Index : Natural                                       := 
     0;  


   -- Variables needed to parse a "if" or a "forall" statement
   --
   New_If     : If_Statement_Ptr;  
   Old_Stat   : Statement_Ptr;  
   New_Stat   : Statement_Ptr;  
   New_Forall : Forall_Statement_Ptr;  
   New_While  : While_Statement_Ptr;  


   procedure Yyerror (
         S : String ) is 
   begin
      Raise_Exception(Expressions.Syntax_Error'Identity, "token """ &
         To_String(
            To_Unbounded_String(Yytext) & """" & Lb_Comma & " Line" &
            Positive'Image(Lines) & Lb_Comma & To_Unbounded_String(S) ));
   end Yyerror;



   -- Store the scheduler behavior step
   -- Used to Check is some Statements can be
   -- declared in the Current_Section
   --
   Current_Section : Sections;  


   -- Current Pointers to the parsed code
   -- There is one Pointers per section
   --
   Current_Statement_Pointer : Section_Pointers;  



   -- Procedure used to display content of each
   -- section (Statements and Expressions)
   --
   package Sections_Io is new Text_Io.Enumeration_Io(Sections);
   use Sections_Io;

   procedure Put (
         S : in     Section_Pointers ) is 

   begin
      for I in Sections'range loop
         if S(I)/=null
               then
            Put(" Section : ");
            Put(I);
            New_Line;
            Recursive_Put(S(I));
            New_Line;
            New_Line;
         end if;
      end loop;
   end Put;



   procedure Init_Parser (
         My_Tasks : in     Tasks_Set ) is 


      New_Var           : Variable_Record_Ptr;  
      New_Expr_Array    : Array_Variable_Expr_Ptr;  
      New_Expr_Variable : Variable_Expr_Ptr;  
      My_Iterator       : Iterator;  
      A_Task            : Task_Ptr;  
      Do_Insert         : Boolean;  


   begin

      Else_Inner_Statement_Index:=0;
      Then_Inner_Statement_Index:=0;
      Loop_Inner_Statement_Index:=0;

      Initialize(Variables_Table);


      for I in Sections'range loop
         Root_Statement_Pointer(I):=null;
      end loop;


      ----------------------------------------
      -- Declare predefined Variables ..
      ----------------------------------------


      -- "priority"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("priority");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "deadline"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("deadline");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "start_time"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("start_time");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "capacity"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("capacity");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "rest_of_capacity"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("rest_of_capacity");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "activation_number"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("activation_number");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "used_cpu"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("used_cpu");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);



      -- "period"
      --
      New_Var:= new Variable_Record;
      New_Expr_Array := new
         Array_Variable_Expr;
      New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
      New_Expr_Array.Name:=To_Unbounded_String("period");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
      Add(Variables_Table, New_Var);


      -- "previously_elected"
      --
      New_Var:= new Variable_Record;
      New_Expr_Variable := new Variable_Expr;
      New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
      New_Expr_Variable.Name:=To_Unbounded_String("previously_elected");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Variable);
      Add(Variables_Table, New_Var);


      -- "nb_tasks"
      --
      New_Var:= new Variable_Record;
      New_Expr_Variable := new Variable_Expr;
      New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
      New_Expr_Variable.Name:=To_Unbounded_String("nb_tasks");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Variable);
      Add(Variables_Table, New_Var);


      -- "simulation_time"
      --
      New_Var:= new Variable_Record;
      New_Expr_Variable := new Variable_Expr;
      New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
      New_Expr_Variable.Name:=To_Unbounded_String("simulation_time");
      New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Variable);
      Add(Variables_Table, New_Var);


      -- User defined parameters
      --

      -- Find user defined parameters : scan all task
      -- Variables and add them
      --

      if not Is_Empty(My_Tasks)
            then
         Reset_Iterator(My_Iterator);

         loop
            Current_Element(My_Tasks, A_Task, My_Iterator);

            exit when Is_Last_Element(My_Tasks, My_Iterator) = True;

            for I in 0..A_Task.Param.Nb_Entries-1 loop

               -- Is the Variable already exists ?
               --
               Do_Insert:=True;
               for J in 0..Variables_Table.Nb_Entries-1 loop
                  if Variables_Table.Entries(J).Variable.Name=A_Task.
                        Param.Entries(I).Name
                        then
                     Do_Insert:=False;
                  end if;
               end loop;

               if Do_Insert
                     then
                  New_Var:= new Variable_Record;
                  New_Expr_Array := new
                     Array_Variable_Expr;
                  if A_Task.Param.Entries(I).Ptype=Integer_Parameter
                        then
                     New_Expr_Array.Variable_Type:=
                        Simulation_Array_Integer_Data;
                  end if;
                  if A_Task.Param.Entries(I).Ptype=Double_Parameter
                        then
                     New_Expr_Array.Variable_Type:=
                        Simulation_Array_Double_Data;
                  end if;
                  if A_Task.Param.Entries(I).Ptype=Boolean_Parameter
                        then
                     New_Expr_Array.Variable_Type:=
                        Simulation_Array_Boolean_Data;
                  end if;

                  New_Expr_Array.Name:=A_Task.Param.Entries(I).Name;
                  New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
                  Add(Variables_Table, New_Var);
               end if;

            end loop;

            Next_Element(My_Tasks, My_Iterator);
         end loop;
      end if;


   end Init_Parser;





   procedure Yyparse is 

      -- Rename User Defined Packages to Internal Names.
      package Yy_Goto_Tables         renames
         Scheduler_Goto;
      package Yy_Shift_Reduce_Tables renames
         Scheduler_Shift_Reduce;
      package Yy_Tokens              renames
         Scheduler_Tokens;

      use      Yy_Tokens,  
      Yy_Goto_Tables,  
      Yy_Shift_Reduce_Tables;  

      procedure Yyerrok; 
      procedure Yyclearin; 


      package Yy is

         -- the size of the value and state stacks
         Stack_Size : constant Natural := 300;  

         -- subtype rule         is natural;
         subtype Parse_State  is Natural;
         -- subtype nonterminal  is integer;

         -- encryption constants
         Default           : constant := - 1;  
         First_Shift_Entry : constant := 0;  
         Accept_Code       : constant := - 3001;  
         Error_Code        : constant := - 3000;  

         -- stack data used by the parser
         Tos         : Natural                                      := 0;
            
         Value_Stack : array (0 .. Stack_Size) of Yy_Tokens.Yystype;  
         State_Stack : array (0 .. Stack_Size) of Parse_State;  

         -- current input symbol and action the parser is on
         Action       : Integer;  
         Rule_Id      : Rule;  
         Input_Symbol : Yy_Tokens.Token;  


         -- error recovery flag
         Error_Flag : Natural := 0;  
         -- indicates  3 - (number of valid shifts after an error occurs)
         Look_Ahead : Boolean := True;  
         Index      : Integer;  

         -- Is Debugging option on or off
         Debug : constant Boolean := False;  

      end Yy;


      function Goto_State (
            State : Yy.Parse_State; 
            Sym   : Nonterminal     ) 
        return Yy.Parse_State; 

      function Parse_Action (
            State : Yy.Parse_State; 
            T     : Yy_Tokens.Token ) 
        return Integer; 

      pragma Inline(Goto_State, Parse_Action);


      function Goto_State (
            State : Yy.Parse_State; 
            Sym   : Nonterminal     ) 
        return Yy.Parse_State is 
         Index : Integer;  
      begin
         Index := Goto_Offset(State);
         while  Integer(Goto_Matrix(Index).Nonterm) /= Sym loop
            Index := Index + 1;
         end loop;
         return Integer(Goto_Matrix(Index).Newstate);
      end Goto_State;


      function Parse_Action (
            State : Yy.Parse_State; 
            T     : Yy_Tokens.Token ) 
        return Integer is 
         Index   :          Integer;  
         Tok_Pos :          Integer;  
         Default : constant Integer := - 1;  
      begin
         Tok_Pos := Yy_Tokens.Token'Pos(T);
         Index   := Shift_Reduce_Offset(State);
         while Integer(Shift_Reduce_Matrix(Index).T) /= Tok_Pos and then
               Integer(Shift_Reduce_Matrix(Index).T) /= Default
               loop
            Index := Index + 1;
         end loop;
         return Integer(Shift_Reduce_Matrix(Index).Act);
      end Parse_Action;

      -- error recovery stuff

      procedure Handle_Error is 
         Temp_Action : Integer;  
      begin

         if Yy.Error_Flag = 3 then -- no shift yet, clobber input.
            if Yy.Debug then
               Text_Io.Put_Line("Ayacc.YYParse: Error Recovery Clobbers " &
                  Yy_Tokens.Token'Image(Yy.Input_Symbol));
            end if;
            if Yy.Input_Symbol = Yy_Tokens.End_Of_Input then
               -- don't discard,
               if Yy.Debug then
                  Text_Io.Put_Line(
                     "Ayacc.YYParse: Can't discard END_OF_INPUT, quiting...");
               end if;
               raise Yy_Tokens.Syntax_Error;
            end if;

            Yy.Look_Ahead := True;   -- get next token
            return;                  -- and try again...
         end if;

         if Yy.Error_Flag = 0 then -- brand new error
            Yyerror("Syntax Error");
         end if;

         Yy.Error_Flag := 3;

         -- find state on stack where error is a valid shift --

         if Yy.Debug then
            Text_Io.Put_Line(
               "Ayacc.YYParse: Looking for state with error as valid shift");
         end if;

         loop
            if Yy.Debug then
               Text_Io.Put_Line("Ayacc.YYParse: Examining State " &
                  Yy.Parse_State'Image(Yy.State_Stack(Yy.Tos)));
            end if;
            Temp_Action := Parse_Action(Yy.State_Stack(Yy.Tos), Error);

            if Temp_Action >= Yy.First_Shift_Entry then
               if Yy.Tos = Yy.Stack_Size then
                  Text_Io.Put_Line(" Stack size exceeded on state_stack");
                  raise Yy_Tokens.Syntax_Error;
               end if;
               Yy.Tos := Yy.Tos + 1;
               Yy.State_Stack(Yy.Tos) := Temp_Action;
               exit;
            end if;

            Decrement_Stack_Pointer :
               begin
               Yy.Tos := Yy.Tos - 1;
            exception
               when Constraint_Error =>
                  Yy.Tos := 0;
            end Decrement_Stack_Pointer;

            if Yy.Tos = 0 then
               if Yy.Debug then
                  Text_Io.Put_Line(
                     "Ayacc.YYParse: Error recovery popped entire stack, aborting...");
               end if;
               raise Yy_Tokens.Syntax_Error;
            end if;
         end loop;

         if Yy.Debug then
            Text_Io.Put_Line(
               "Ayacc.YYParse: Shifted error token in state " &
               Yy.Parse_State'Image(Yy.State_Stack(Yy.Tos)));
         end if;

      end Handle_Error;

      -- print debugging information for a shift operation
      procedure Shift_Debug (
            State_Id : Yy.Parse_State; 
            Lexeme   : Yy_Tokens.Token ) is 
      begin
         Text_Io.Put_Line("Ayacc.YYParse: Shift "& Yy.Parse_State'Image(
               State_Id)&" on input symbol "&
            Yy_Tokens.Token'Image(Lexeme) );
      end;

      -- print debugging information for a reduce operation
      procedure Reduce_Debug (
            Rule_Id  : Rule;          
            State_Id : Yy.Parse_State ) is 
      begin
         Text_Io.Put_Line("Ayacc.YYParse: Reduce by rule "&Rule'Image(
               Rule_Id)&" goto state "&
            Yy.Parse_State'Image(State_Id));
      end;

      -- make the parser believe that 3 valid shifts have occured.
      -- used for error recovery.
      procedure Yyerrok is 
      begin
         Yy.Error_Flag := 0;
      end Yyerrok;

      -- called to clear input symbol that caused an error.
      procedure Yyclearin is 
      begin
         -- yy.input_symbol := yylex;
         Yy.Look_Ahead := True;
      end Yyclearin;


   begin
      -- initialize by pushing state 0 and getting the first input symbol
      Yy.State_Stack(Yy.Tos) := 0;


      loop

         Yy.Index := Shift_Reduce_Offset(Yy.State_Stack(Yy.Tos));
         if Integer(Shift_Reduce_Matrix(Yy.Index).T) = Yy.Default then
            Yy.Action := Integer(Shift_Reduce_Matrix(Yy.Index).Act);
         else
            if Yy.Look_Ahead then
               Yy.Look_Ahead   := False;

               Yy.Input_Symbol := Yylex;
            end if;
            Yy.Action :=
               Parse_Action(Yy.State_Stack(Yy.Tos), Yy.Input_Symbol);
         end if;


         if Yy.Action >= Yy.First_Shift_Entry then  -- SHIFT

            if Yy.Debug then
               Shift_Debug(Yy.Action, Yy.Input_Symbol);
            end if;

            -- Enter new state
            if Yy.Tos = Yy.Stack_Size then
               Text_Io.Put_Line(" Stack size exceeded on state_stack");
               raise Yy_Tokens.Syntax_Error;
            end if;
            Yy.Tos := Yy.Tos + 1;
            Yy.State_Stack(Yy.Tos) := Yy.Action;
            Yy.Value_Stack(Yy.Tos) := Yylval;

            if Yy.Error_Flag > 0 then  -- indicate a valid shift
               Yy.Error_Flag := Yy.Error_Flag - 1;
            end if;

            -- Advance lookahead
            Yy.Look_Ahead := True;

         elsif Yy.Action = Yy.Error_Code then       -- ERROR

            Handle_Error;

         elsif Yy.Action = Yy.Accept_Code then
            if Yy.Debug then
               Text_Io.Put_Line("Ayacc.YYParse: Accepting Grammar...");
            end if;
            exit;

         else -- Reduce Action

            -- Convert action into a rule
            Yy.Rule_Id  := -1 * Yy.Action;

            -- Execute User Action
            -- user_action(yy.rule_id);


            case Yy.Rule_Id is

               when  2 =>
                  --#line  63

                  Current_Section:=Start;


               when  5 =>
                  --#line  70

                  Current_Section:=Priority;


               when  8 =>
                  --#line  77

                  Current_Section:=Election;


               when  11 =>
                  --#line  84

                  Current_Section:=Activation;


               when  14 =>
                  --#line  91

                  Current_Section:=Check_Resource;


               when  17 =>
                  --#line  98

                  Current_Section:=Release_Resource;


               when  20 =>
                  --#line  105

                  Current_Section:=Allocate_Resource;


               when  23 =>
                  --#line  113

                  if
                        Yy.Value_Stack(Yy.Tos).Stats/=null then
                     if (Root_Statement_Pointer(Current_Section)=null)
                           then
                        Root_Statement_Pointer(Current_Section):=
                           Yy.Value_Stack(Yy.Tos).Stats;
                        Current_Statement_Pointer(Current_Section):=
                           Root_Statement_Pointer(Current_Section);
                     else
                        Current_Statement_Pointer(Current_Section).
                           Next_Statement:=
                           Yy.Value_Stack(Yy.Tos).Stats;
                        Current_Statement_Pointer(Current_Section):=
                           Yy.Value_Stack(Yy.Tos).Stats;
                     end if;
                  end if;



               when  25 =>
                  --#line  132


                  Yy.Value_Stack(Yy.Tos).Stats.Next_Statement:=
                     Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                  Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;


               when  26 =>
                  --#line  137

                  Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index+
                     1;
                  Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;
                  Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index).
                     Next_Statement:=null;


               when  27 =>
                  --#line  145


                  Yy.Value_Stack(Yy.Tos).Stats.Next_Statement:=
                     Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                  Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;


               when  28 =>
                  --#line  150

                  Then_Inner_Statement_Index:=Then_Inner_Statement_Index+
                     1;
                  Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;
                  Then_Inner_Statement_Ptr(Then_Inner_Statement_Index).
                     Next_Statement:=null;


               when  29 =>
                  --#line  159


                  Yy.Value_Stack(Yy.Tos).Stats.Next_Statement:=
                     Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                  Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;


               when  30 =>
                  --#line  164

                  Else_Inner_Statement_Index:=Else_Inner_Statement_Index+
                     1;
                  Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=
                     Yy.Value_Stack(Yy.Tos).Stats;
                  Else_Inner_Statement_Ptr(Else_Inner_Statement_Index).
                     Next_Statement:=null;


               when  31 =>
                  --#line  174

                  declare
                     New_Put : Put_Statement_Ptr;  
                  begin

                     -- Firstly, we Check if the Variable exists
                     --
                     Check_Variable_Declaration(Variables_Table,
                        Yy.Value_Stack(Yy.Tos-2).String_Value);

                     New_Put:= new Put_Statement;
                     New_Put.Identifier:=
                        Yy.Value_Stack(Yy.Tos-2).String_Value;
                     New_Put.Line_Number:=Scheduler_Lex.Lines;

                     Yyval.Stats:=Statement_Ptr(New_Put);

                  end;


               when  32 =>
                  --#line  193

                  declare
                     New_Affectation : Affectation_Statement_Ptr;  
                     New_Var         : Variable_Record_Ptr;  
                     New_Array_Expr  : Array_Variable_Expr_Ptr;  
                     New_Var_Expr    : Variable_Expr_Ptr;  


                  begin

                     -- Is the Variable already exists ?
                     --
                     for I in 0..Variables_Table.Nb_Entries-1 loop
                        if (Variables_Table.Entries(I).Variable.Name=
                              Yy.Value_Stack(Yy.Tos-4).String_Value)
                              then
                           Raise_Exception(Expressions.Syntax_Error'
                              Identity, To_String(

                                 Yy.Value_Stack(Yy.Tos-4).String_Value &
                                 Lb_Comma & Lb_Undeclared_Identifier(
                                    Current_Language) ));
                        end if;
                     end loop;



                     -- Create the Variable (Array or not)
                     --
                     New_Var:= new Variable_Record;

                     if (
                           Yy.Value_Stack(Yy.Tos-2).Variable_Type=
                           Simulation_Array_Double_Data)
                           or (
                           Yy.Value_Stack(Yy.Tos-2).Variable_Type=
                           Simulation_Array_Integer_Data)
                           or (
                           Yy.Value_Stack(Yy.Tos-2).Variable_Type=
                           Simulation_Array_Boolean_Data)
                           then

                        New_Array_Expr:= new Array_Variable_Expr;
                        New_Array_Expr.Variable_Type:=
                           Yy.Value_Stack(Yy.Tos-2).Variable_Type;
                        New_Array_Expr.Name:=
                           Yy.Value_Stack(Yy.Tos-4).String_Value;
                        New_Var.Variable:=Variable_Expr_Ptr(
                           New_Array_Expr);
                     else
                        New_Var_Expr := new Variable_Expr;
                        New_Var_Expr.Variable_Type:=
                           Yy.Value_Stack(Yy.Tos-2).Variable_Type;
                        New_Var_Expr.Name:=
                           Yy.Value_Stack(Yy.Tos-4).String_Value;
                        New_Var.Variable:=Variable_Expr_Ptr(New_Var_Expr);
                     end if;

                     Add(Variables_Table, New_Var);


                     -- If Affectation is not null, we should register
                     -- a new Affectation
                     --
                     if
                           Yy.Value_Stack(Yy.Tos-1).Expr /= null
                           then
                        New_Affectation:= new Affectation_Statement;
                        New_Affectation.Lvalue:=Expression_Ptr(
                           New_Var_Expr);
                        New_Affectation.Rvalue:=
                           Yy.Value_Stack(Yy.Tos-1).Expr;
                        New_Affectation.Line_Number:=Scheduler_Lex.Lines;

                        Yyval.Stats:=Statement_Ptr(New_Affectation);
                     else
                        Yyval.Stats:=null;

                     end if;
                  end;


               when  33 =>
                  --#line  256

                  declare
                     New_Affectation : Affectation_Statement_Ptr;  

                  begin
                     New_Affectation:= new Affectation_Statement;
                     New_Affectation.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-3).Expr;
                     New_Affectation.Rvalue:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;
                     New_Affectation.Line_Number:=Scheduler_Lex.Lines;


                     Yyval.Stats:=Statement_Ptr(New_Affectation);

                  end;


               when  34 =>
                  --#line  271


                  New_If:= new If_Statement;
                  New_If.Bool_Expression:=
                     Yy.Value_Stack(Yy.Tos-2).Expr;
                  New_If.Line_Number:=Scheduler_Lex.Lines;

                  New_If.Then_Statement:=Then_Inner_Statement_Ptr(
                     Then_Inner_Statement_Index);
                  Then_Inner_Statement_Index:=Then_Inner_Statement_Index-
                     1;


                  Yyval.Stats:=Statement_Ptr(New_If);

                  New_Stat:=Copy(New_If.Then_Statement);
                  New_Stat.Next_Statement:=null;
                  while  New_If.Then_Statement.Next_Statement/=null loop
                     Old_Stat:=Copy(New_If.Then_Statement.Next_Statement);
                     Old_Stat.Next_Statement:=New_Stat;
                     New_Stat:=Old_Stat;
                     New_If.Then_Statement:=New_If.Then_Statement.
                        Next_Statement;
                  end loop;
                  New_If.Then_Statement:=New_Stat;




               when  35 =>
                  --#line  295


                  New_If.Else_Statement:=Else_Inner_Statement_Ptr(
                     Else_Inner_Statement_Index);
                  Else_Inner_Statement_Index:=Else_Inner_Statement_Index-
                     1;

                  Yyval.Stats:=Statement_Ptr(New_If);

                  New_Stat:=Copy(New_If.Else_Statement);
                  New_Stat.Next_Statement:=null;
                  while  New_If.Else_Statement.Next_Statement/=null loop
                     Old_Stat:=Copy(New_If.Else_Statement.Next_Statement);
                     Old_Stat.Next_Statement:=New_Stat;
                     New_Stat:=Old_Stat;
                     New_If.Else_Statement:=New_If.Else_Statement.
                        Next_Statement;
                  end loop;
                  New_If.Else_Statement:=New_Stat;


               when  36 =>
                  --#line  314

                  declare
                     New_Return : Return_Statement_Ptr;  

                  begin
                     New_Return:= new Return_Statement;
                     New_Return.Return_Value:=
                        Yy.Value_Stack(Yy.Tos).Expr;
                     New_Return.Line_Number:=Scheduler_Lex.Lines;


                     Yyval.Stats:=Statement_Ptr(New_Return);

                  end;


               when  37 =>
                  --#line  330

                  New_Forall:= new Forall_Statement;
                  New_Forall.Forall_Index:=Variable_Expr_Ptr(
                     Yy.Value_Stack(Yy.Tos-5).Expr);
                  New_Forall.Line_Number:=Scheduler_Lex.Lines;

                  New_Forall.Included_Statement:=Loop_Inner_Statement_Ptr(
                     Loop_Inner_Statement_Index);
                  Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-
                     1;

                  New_Stat:=Copy(New_Forall.Included_Statement);
                  New_Stat.Next_Statement:=null;
                  while  New_Forall.Included_Statement.Next_Statement/=null loop
                     Old_Stat:=Copy(New_Forall.Included_Statement.
                        Next_Statement);
                     Old_Stat.Next_Statement:=New_Stat;
                     New_Stat:=Old_Stat;
                     New_Forall.Included_Statement:=New_Forall.
                        Included_Statement.Next_Statement;
                  end loop;
                  New_Forall.Included_Statement:=New_Stat;


                  Yyval.Stats:=Statement_Ptr(New_Forall);


               when  38 =>
                  --#line  353

                  New_While:= new While_Statement;
                  New_While.Condition:=
                     Yy.Value_Stack(Yy.Tos-5).Expr;
                  New_While.Line_Number:=Scheduler_Lex.Lines;

                  New_While.Included_Statement:=Loop_Inner_Statement_Ptr(
                     Loop_Inner_Statement_Index);
                  Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-
                     1;

                  New_Stat:=Copy(New_While.Included_Statement);
                  New_Stat.Next_Statement:=null;
                  while  New_While.Included_Statement.Next_Statement/=null loop
                     Old_Stat:=Copy(New_While.Included_Statement.
                        Next_Statement);
                     Old_Stat.Next_Statement:=New_Stat;
                     New_Stat:=Old_Stat;
                     New_While.Included_Statement:=New_While.
                        Included_Statement.Next_Statement;
                  end loop;
                  New_While.Included_Statement:=New_Stat;


                  Yyval.Stats:=Statement_Ptr(New_While);



               when  39 =>
                  --#line  377


                  Yy.Value_Stack(Yy.Tos-2).Expr :=
                     Yy.Value_Stack(Yy.Tos-1).Expr;


               when  40 =>
                  --#line  384

                  null;


               when  41 =>
                  --#line  388

                  declare
                     New_Nop : Nop_Statement_Ptr;  

                  begin
                     New_Nop:= new Nop_Statement;
                     New_Nop.Line_Number:=Scheduler_Lex.Lines;

                     Else_Inner_Statement_Index:=
                        Else_Inner_Statement_Index+1;
                     Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=
                        Statement_Ptr(New_Nop);

                  end;



               when  42 =>
                  --#line  405


                  Yyval.Expr:=
                     Yy.Value_Stack(Yy.Tos-1).Expr;


               when  43 =>
                  --#line  409


                  Yyval.Expr:=null;


               when  44 =>
                  --#line  417


                  Yyval.Variable_Type:=Simulation_Double_Data;


               when  45 =>
                  --#line  421


                  Yyval.Variable_Type:=Simulation_Array_Double_Data;


               when  46 =>
                  --#line  425


                  Yyval.Variable_Type:=Simulation_Integer_Data;


               when  47 =>
                  --#line  429


                  Yyval.Variable_Type:=Simulation_Array_Integer_Data;


               when  48 =>
                  --#line  433


                  Yyval.Variable_Type:=Simulation_Boolean_Data;


               when  49 =>
                  --#line  437


                  Yyval.Variable_Type:=Simulation_Array_Boolean_Data;


               when  50 =>
                  --#line  444


                  Yyval.Expr:=
                     Yy.Value_Stack(Yy.Tos).Expr;


               when  51 =>
                  --#line  448


                  Yyval.Expr:=null;


               when  52 =>
                  --#line  455

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Logic_And;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  53 =>
                  --#line  468

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Modulo;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  54 =>
                  --#line  481

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Inf;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  55 =>
                  --#line  494

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Sup;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  56 =>
                  --#line  507

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Equal_Less;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  57 =>
                  --#line  520

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Equal_Greater;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  58 =>
                  --#line  533

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Not_Equal;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  59 =>
                  --#line  546

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Equal;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  60 =>
                  --#line  559

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Logic_Or;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  61 =>
                  --#line  572

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Logic_Not;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  62 =>
                  --#line  585


                  Yyval.Expr:=
                     Yy.Value_Stack(Yy.Tos-1).Expr;


               when  63 =>
                  --#line  590

                  declare
                     New_Expr : Constant_Expr_Ptr;  
                  begin
                     New_Expr:=new Constant_Expr;
                     New_Expr.Value:= new Simulation_Value'(
                        Simulation_Boolean_Data, True);

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  64 =>
                  --#line  601

                  declare
                     New_Expr : Constant_Expr_Ptr;  
                  begin
                     New_Expr:=new Constant_Expr;
                     New_Expr.Value:= new Simulation_Value'(
                        Simulation_Boolean_Data, False);

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  65 =>
                  --#line  612

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Plus;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  66 =>
                  --#line  625

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Minus;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  67 =>
                  --#line  638

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Mult;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  68 =>
                  --#line  651

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Div;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  69 =>
                  --#line  664

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Lcm;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-5).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos-3).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  70 =>
                  --#line  677

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Max_Operator;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-5).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos-3).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  71 =>
                  --#line  690

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Min_Operator;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-5).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos-3).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  72 =>
                  --#line  703

                  declare
                     New_Expr : Composite_Expr_Ptr;  
                  begin
                     New_Expr:=new Composite_Expr;
                     New_Expr.Ope:=Exp;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-2).Expr;
                     New_Expr.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  73 =>
                  --#line  716

                  declare
                     New_Expr1 : Composite_Expr_Ptr;  
                     New_Expr2 : Constant_Expr_Ptr;  

                  begin
                     New_Expr2:=new Constant_Expr;
                     New_Expr2.Value:=new Simulation_Value'(
                        Simulation_Double_Data,
                        -1.0);
                     New_Expr1:=new Composite_Expr;
                     New_Expr1.Ope:=Mult;
                     New_Expr1.Lvalue:=Expression_Ptr(New_Expr2);
                     New_Expr1.Rvalue:=
                        Yy.Value_Stack(Yy.Tos).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr1);
                  end;


               when  74 =>
                  --#line  734

                  declare
                     New_Expr : Max_To_Index_Expr_Ptr;  
                  begin
                     New_Expr := new Max_To_Index_Expr;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  75 =>
                  --#line  745

                  declare
                     New_Expr : Min_To_Index_Expr_Ptr;  
                  begin
                     New_Expr := new Min_To_Index_Expr;
                     New_Expr.Lvalue:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  76 =>
                  --#line  756

                  declare
                     New_Expr : Constant_Expr_Ptr;  
                  begin
                     New_Expr:=new Constant_Expr;
                     New_Expr.Value:=new Simulation_Value'(
                        Simulation_Double_Data,
                        Yy.Value_Stack(Yy.Tos).Double_Value);

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  77 =>
                  --#line  767

                  declare
                     New_Expr : Constant_Expr_Ptr;  
                  begin
                     New_Expr:=new Constant_Expr;
                     New_Expr.Value:=new Simulation_Value'(
                        Simulation_Integer_Data,
                        Yy.Value_Stack(Yy.Tos).Integer_Value);

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  78 =>
                  --#line  778

                  declare
                     New_Expr : Array_Variable_Expr_Ptr;  
                  begin
                     -- Find the type of the $1 Variable
                     --

                     Yyval.Expr:=Expression_Ptr(Variables_Table.Entries(
                           Find_Variable(Variables_Table,
                              Yy.Value_Stack(Yy.Tos-3).String_Value)).
                        Variable );

                     New_Expr := new
                        Array_Variable_Expr;
                     New_Expr.Variable_Type:=Variable_Expr_Ptr(
                        Yyval.Expr).Variable_Type;
                     New_Expr.Name:=
                        Yy.Value_Stack(Yy.Tos-3).String_Value;
                     New_Expr.Array_Index:=
                        Yy.Value_Stack(Yy.Tos-1).Expr;

                     Yyval.Expr:=Expression_Ptr(New_Expr);
                  end;


               when  79 =>
                  --#line  797

                  -- Find the type of the Variable before
                  -- instanciation
                  --

                  Yyval.Expr:=Expression_Ptr(Variables_Table.Entries(
                        Find_Variable(Variables_Table,
                           Yy.Value_Stack(Yy.Tos).String_Value)).Variable );


               when  80 =>
                  --#line  809


                  Yyval.String_Value := To_Unbounded_String(Scheduler_Dfa.
                     Yytext);


               when  81 =>
                  --#line  815

                  declare
                     Ok : Boolean;  

                  begin
                     To_Double(To_Unbounded_String(Scheduler_Dfa.Yytext),
                        Yyval.Double_Value,Ok);
                     if not Ok
                           then
                        Raise_Exception(Expressions.Syntax_Error'Identity,
                           To_String(
                              To_Unbounded_String(Scheduler_Dfa.Yytext) &
                              Lb_Comma & Lb_Double_Conversion_Error(
                                 Current_Language) ));
                     end if;
                  end;


               when  82 =>
                  --#line  831

                  declare
                     Ok : Boolean;  

                  begin
                     To_Integer(To_Unbounded_String(Scheduler_Dfa.Yytext),
                        Yyval.Integer_Value,Ok);
                     if not Ok
                           then
                        Raise_Exception(Expressions.Syntax_Error'Identity,
                           To_String(
                              To_Unbounded_String(Scheduler_Dfa.Yytext) &
                              Lb_Comma & Lb_Integer_Conversion_Error(
                                 Current_Language) ));
                     end if;
                  end;


               when others =>
                  null;
            end case;


            -- Pop RHS states and goto next state
            Yy.Tos      := Yy.Tos - Rule_Length(Yy.Rule_Id) + 1;
            if Yy.Tos > Yy.Stack_Size then
               Text_Io.Put_Line(" Stack size exceeded on state_stack");
               raise Yy_Tokens.Syntax_Error;
            end if;
            Yy.State_Stack(Yy.Tos) := Goto_State(Yy.State_Stack(Yy.Tos-1) ,
               Get_Lhs_Rule(Yy.Rule_Id));

            Yy.Value_Stack(Yy.Tos) := Yyval;

            if Yy.Debug then
               Reduce_Debug(Yy.Rule_Id,
                  Goto_State(Yy.State_Stack(Yy.Tos - 1),
                     Get_Lhs_Rule(Yy.Rule_Id)));
            end if;

         end if;


      end loop;


   end Yyparse;



end Parser;




