
%with Ada.strings.unbounded;
%use Ada.strings.unbounded;
%with Ada.numerics.aux;
%use Ada.numerics.aux;
%with Statements;
%use Statements;
%with Expressions;
%use Expressions;
%use Expressions.Variables_Type_Package;
%with Simulations;
%use Simulations;


%Token Double_Type_Token Double_Type_Task_Array_Token
%Token Integer_Type_Token Integer_Type_Task_Array_Token
%Token Boolean_Type_Token Boolean_Type_Task_Array_Token

%Token Double_Token Double_Task_Array_Token
%Token Integer_Token Integer_Task_Array_Token
%Token Boolean_Token Boolean_Task_Array_Token

%Token Double_Token Integer_Token Identifier_Token True_Token False_Token

%Token Forall_Token Set_Token While_Token Loop_Token
%Token Return_Token Max_Token Min_Token Lcm_Token Put_Token Mod_Token
%Token Max_To_Index_Token Min_To_Index_Token
%Token Then_Token End_Token Else_Token If_Token
%Token Uniform_Token Random_Token Time_Seed_Token Exponential_Token 
%Token Start_Token Priority_Token Election_Token Activation_Token Check_Resource_Token Release_Resource_Token Allocate_Resource_Token 
%Token '[' ']' '(' ')' '}' '{'
%right     Equal_Token
%left      '+' '-'
%left      '*' '/'
%right     Dummy
%nonassoc  Exp_Token
%Token ':' '|' ';' ',' 
%Token Inf_Token Sup_Token Inf_Equal_Token Sup_Equal_Token Not_Equal_Token Affectation_Token
%Token Or_Token And_Token Not_Token

{

type Key_Type is (Cval, Ival, Noval);

type Yystype is record
  	String_Value 	: unbounded_String;
	Integer_Value 	: Integer;
	Double_Value 	: Double;
  	Expr		: Expression_Ptr;
  	Stats 		: Statement_Ptr;
  	Variable_Type 	: Simulation_Data_Type;
end record;

}

%%

Entry :  Start_Section Priority_Section Election_Section Activation_Section Check_Resource_Section Release_Resource_Section  Allocate_Resource_Section; 



Start_Section : 
	{
		Current_Section:=Start;
	} 
	Start_Token Statements
        | ;

Priority_Section : 
	{
		Current_Section:=Priority;
	} 
	Priority_Token Statements
        | ;

Election_Section  : 
	{
		Current_Section:=Election;
	} 
	Election_Token Statements
        |;

Activation_Section  : 
	{
		Current_Section:=Activation;
	} 
	Activation_Token Statements
        | ;

Check_Resource_Section  : 
	{
		Current_Section:=Check_Resource;
	} 
	Check_Resource_Token Statements
        | ;

Release_Resource_Section  : 
	{
		Current_Section:=Release_Resource;
	} 
	Release_Resource_Token Statements
        | ;

Allocate_Resource_Section : 
	{
		Current_Section:=Allocate_Resource;
	} 
	Allocate_Resource_Token Statements
        | ;

           
Statements : Statements Statement
       {	
        if $2.Stats/=null then
		if (Root_Statement_Pointer(Current_Section)=null)
                then
                	Root_Statement_Pointer(Current_Section):=$2.Stats;
                        Current_Statement_Pointer(Current_Section):=Root_Statement_Pointer(Current_Section);
                else
                        Current_Statement_Pointer(Current_Section).Next_Statement:=$2.Stats;
                        Current_Statement_Pointer(Current_Section):=$2.Stats;
                end if;
        end if;

        }
           |
           ;



Loop_Included_Statements :  Loop_Included_Statements Statement
        {
                $2.Stats.Next_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=$2.Stats;
        }
           | Statement
	{
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index+1;
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=$1.Stats;
                Loop_Inner_Statement_ptr(Loop_Inner_Statement_Index).Next_Statement:=null;
        }; 
          

Then_Included_Statements :  Then_Included_Statements Statement
        {
                $2.Stats.Next_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=$2.Stats;
        }
           | Statement
	{
                Then_Inner_Statement_Index:=Then_Inner_Statement_Index+1;
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=$1.Stats;
                Then_Inner_Statement_ptr(Then_Inner_Statement_Index).Next_Statement:=null;
        }; 
          


Else_Included_Statements :  Else_Included_Statements Statement
        {
                $2.Stats.Next_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=$2.Stats;
        }
           | Statement
	{
                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=$1.Stats;
                Else_Inner_Statement_ptr(Else_Inner_Statement_Index).Next_Statement:=null;
        }; 
          


Statement  : 
	Put_Token '(' Identifier_Data ')' ';'	
	{
	declare
		New_Put : Put_Statement_Ptr;
	begin

		-- Firstly, we Check if the Variable exists
		--
		Check_Variable_Declaration(Variables_Table, $3.String_Value);

		New_Put:= new Put_Statement;
                New_Put.Identifier:=$3.String_Value;
		New_Put.Line_Number:=Scheduler_Lex.Lines;
		$$.Stats:=Statement_Ptr(New_Put);

	end;
	}

	|
	Identifier_Data ':' User_Type Affectation ';'	
	{
	declare
		New_Affectation : Affectation_Statement_Ptr;
		New_Var 	: Variable_Record_Ptr;
		New_Array_Expr 	: Array_Variable_Expr_Ptr;
                New_Var_Expr 	: Variable_Expr_Ptr;

        
	begin

		-- Is the Variable already exists ?
		--
		for I in 0..Variables_Table.Nb_Entries-1 loop
			if (Variables_Table.Entries(i).Variable.Name=$1.String_Value) 
				then
					Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
                         $1.String_Value & Lb_Comma & Lb_Undeclared_Identifier(Current_Language) ));
			end if;
		end loop;



		-- Create the Variable (Array or not)
		--
		New_Var:= new Variable_Record;

                if ($3.Variable_Type=Simulation_Array_Double_Data)
                	or ($3.Variable_Type=Simulation_Array_Integer_Data)
                	or ($3.Variable_Type=Simulation_Array_Boolean_Data) 
			then

                		New_Array_Expr:= new Array_Variable_Expr;
                		New_Array_Expr.Variable_Type:=$3.Variable_Type;
                		New_Array_Expr.Name:=$1.String_Value;
				New_Var.Variable:=variable_expr_Ptr(New_Array_Expr);
			else
				New_Var_Expr := new Variable_Expr;
        		        New_Var_Expr.Variable_Type:=$3.Variable_Type;
				New_Var_Expr.Name:=$1.String_Value;
				New_Var.Variable:=variable_expr_Ptr(New_Var_Expr);
		end if;

		Add(Variables_Table, New_Var);


		-- If Affectation is not null, we should register
		-- a new Affectation
		--
		if $4.Expr /= null 
			then
				New_Affectation:= new Affectation_Statement;
				New_Affectation.Lvalue:=Expression_Ptr(New_Var_Expr);
                		New_Affectation.Rvalue:=$4.Expr;
				New_Affectation.Line_Number:=Scheduler_Lex.Lines;
				$$.Stats:=Statement_Ptr(New_Affectation);
                        else $$.Stats:=null;

		end if;
	end;
	}

	|
	 Expr Affectation_Token Expr ';'
	{
	declare
		New_Affectation : Affectation_Statement_Ptr;
		
	begin
		New_Affectation:= new Affectation_Statement;
                New_Affectation.Lvalue:=$1.Expr;
                New_Affectation.Rvalue:=$3.Expr;
		New_Affectation.Line_Number:=Scheduler_Lex.Lines;

		$$.Stats:=Statement_Ptr(New_Affectation);

	end;
	}
| If_Token Expr Then_Token Then_Included_Statements      
	{

		New_If:= new If_Statement;
                New_If.Bool_Expression:=$2.Expr;
		New_If.Line_Number:=Scheduler_Lex.Lines;

                New_If.Then_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                Then_Inner_Statement_Index:=Then_Inner_Statement_Index-1;

		$$.Stats:=Statement_Ptr(New_If);

                New_Stat:=Copy(New_If.Then_Statement);
                New_Stat.Next_Statement:=null;
                while  New_If.Then_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Then_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Then_Statement:=New_If.Then_Statement.Next_Statement;
                end loop;
                New_If.Then_Statement:=New_Stat;


        }
           Else_Rule End_Token If_Token ';'
        {

                New_If.Else_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Index:=Else_Inner_Statement_Index-1;
		$$.Stats:=Statement_Ptr(New_If);

                New_Stat:=Copy(New_If.Else_Statement);
                New_Stat.Next_Statement:=null;
                while  New_If.Else_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Else_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Else_Statement:=New_If.Else_Statement.Next_Statement;
                end loop;
                New_If.Else_Statement:=New_Stat;
        }


	| Return_Token Return_Expr 
	{
	declare
		New_Return : Return_Statement_Ptr;
		
	begin
		New_Return:= new Return_Statement;
                New_Return.Return_Value:=$2.Expr;
		New_Return.Line_Number:=Scheduler_Lex.Lines;

		$$.Stats:=Statement_Ptr(New_Return);

	end;
	}


        |  Forall_Token Expr Loop_Token Loop_Included_Statements end_Token  Loop_Token ';'
	{
 		New_Forall:= new Forall_Statement;
                New_Forall.Forall_Index:=variable_expr_ptr($2.Expr);
		New_Forall.Line_Number:=Scheduler_Lex.Lines;
                 
                New_Forall.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_Forall.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_Forall.Included_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_Forall.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_Forall.Included_Statement:=New_Forall.Included_Statement.Next_Statement;
                end loop;
                New_Forall.Included_Statement:=New_Stat;

                $$.Stats:=Statement_Ptr(New_Forall);
        }


        |  While_Token Expr Loop_Token Loop_Included_Statements End_Token  Loop_Token ';'
	{
                New_While:= new While_Statement;
                New_While.Condition:=$2.Expr;
                New_While.Line_Number:=Scheduler_Lex.Lines;
                 
                New_While.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_While.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_While.Included_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_While.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_While.Included_Statement:=New_While.Included_Statement.Next_Statement;
                end loop;
                New_While.Included_Statement:=New_Stat;

                $$.Stats:=Statement_Ptr(New_While);

        }


	| set_Token Expr Statements ';'
	{
 		$2.expr := $3.expr;
        };


             
Else_Rule :     Else_Token Else_Included_Statements 
	{
		null;
	}
	|
	{
        declare
                New_Nop : Nop_Statement_Ptr;

        begin
                New_Nop:= new Nop_Statement;
                New_Nop.Line_Number:=Scheduler_Lex.Lines;

                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=Statement_Ptr(New_Nop);

        end;

	};


Return_Expr : Expr ';'
	{
		$$.Expr:=$1.Expr;
	}
	| ';'
	{
		$$.Expr:=null;
	};




User_Type : Double_Type_Token 
	{
		$$.Variable_Type:=Simulation_Double_Data;
	}
	| Double_Type_Task_Array_Token
	{
		$$.Variable_Type:=Simulation_Array_Double_Data;
	}
 	| Integer_Type_Token  
	{
		$$.Variable_Type:=Simulation_Integer_Data;
	}
	| Integer_Type_Task_Array_Token
	{
		$$.Variable_Type:=Simulation_Array_Integer_Data;
	}
 	| Boolean_Type_Token 
	{
		$$.Variable_Type:=Simulation_Boolean_Data;
	}
	| Boolean_Type_Task_Array_Token
	{
		$$.Variable_Type:=Simulation_Array_Boolean_Data;
	}
	;


Affectation : Affectation_Token Expr 
	{
        	$$.Expr:=$2.Expr;
	}
	|
	{
		$$.Expr:=null;
	};


Expr  :
	Expr And_Token Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Logic_And;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Expr Mod_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Modulo;
                New_Expr.Lvalue:=$1.Expr;            
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Inf_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Inf;
                New_Expr.Lvalue:=$1.Expr;            
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Sup_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Sup;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Inf_Equal_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Equal_Less;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Sup_Equal_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Equal_Greater;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Not_Equal_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Not_Equal;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Equal_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Equal;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Or_Token Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Logic_Or;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Not_Token Expr 
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Logic_Not;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$1.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | '(' Expr ')'
	{
        	$$.Expr:=$2.Expr;
	}

        | True_Token 
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=new Constant_Expr;
		New_Expr.Value:= new Simulation_Value'(Simulation_Boolean_Data, true);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | False_Token
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=new Constant_Expr;
		New_Expr.Value:= new Simulation_Value'(Simulation_Boolean_Data, false);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}	

        | Expr '+' Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Plus;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '-' Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Minus;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '*' Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Mult;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '/' Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Div;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Lcm_Token '(' Expr ',' Expr ')'
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Lcm;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Max_Token '(' Expr ',' Expr ')'
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Max_Operator;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Min_Token '(' Expr ',' Expr  ')'
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
                New_Expr.Ope:=Min_Operator;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Exp_Token Expr
	{
	declare
		New_Expr : Composite_Expr_Ptr;	
	begin
		New_Expr:=new Composite_Expr;
		New_Expr.Ope:=Exp;
                New_Expr.Lvalue:=$1.Expr;
                New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| '-' Expr  %prec Dummy
	{
	declare
		New_Expr1 : Composite_Expr_Ptr;	
		New_Expr2 : Constant_Expr_Ptr;	

	begin
                New_Expr2:=new Constant_Expr;
		New_Expr2.value:=new Simulation_Value'(Simulation_Double_Data,
                                -1.0);
		New_Expr1:=new Composite_Expr;
		New_Expr1.Ope:=Mult;
		New_Expr1.Lvalue:=Expression_Ptr(New_Expr2);
                New_Expr1.Rvalue:=$2.Expr;
		$$.Expr:=Expression_Ptr(New_Expr1);
	end;
	}

        | Max_To_Index_Token '(' Expr ')'
	{
	declare 
                New_Expr : Max_To_Index_Expr_Ptr;
	begin
                New_Expr := new Max_To_Index_Expr;
                New_Expr.Lvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Min_To_Index_Token '(' Expr ')'
	{
	declare 
                New_Expr : Min_To_Index_Expr_Ptr;
	begin
                New_Expr := new Min_To_Index_Expr;
                New_Expr.Lvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Double_Data
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Double_Data,$1.Double_Value);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Integer_Data 
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Integer_Data,$1.Integer_Value);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Identifier_Data '[' Expr ']'
	{
	declare
		New_Expr : Array_Variable_Expr_Ptr;	
	begin
		-- Find the type of the $1 Variable
		--
		$$.Expr:=Expression_Ptr(Variables_Table.Entries(
                        Find_Variable(Variables_Table, $1.String_Value)).Variable );
		
		New_Expr := new 
			Array_Variable_Expr;
		New_Expr.Variable_Type:=Variable_Expr_Ptr($$.Expr).Variable_Type;
                New_Expr.Name:=$1.String_Value;
                New_Expr.Array_index:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
	
	| Identifier_Data  
	{
		-- Find the type of the Variable before
		-- instanciation
		--
		$$.Expr:=Expression_Ptr(Variables_Table.Entries(
		Find_Variable(Variables_Table, $1.String_Value)).Variable );
	};




Identifier_Data     : Identifier_Token
	{ 
		$$.String_Value := To_Unbounded_String(Scheduler_Dfa.Yytext); 
	};


Double_Data     : Double_Token
	{ 
	declare 
		Ok : boolean;

	begin
		To_Double(To_Unbounded_String(Scheduler_Dfa.Yytext),$$.Double_Value,Ok);
         	if not Ok
               		then
				Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
			 		To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Double_Conversion_Error(Current_Language) ));
         	end if;
         end;
	} ;


Integer_Data     : Integer_Token
	{ 
	declare 
		Ok : boolean;

	begin
		To_Integer(To_Unbounded_String(Scheduler_Dfa.Yytext),$$.Integer_Value,Ok);
		if not Ok
			then
				Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
					To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Integer_Conversion_Error(Current_Language) ));
         	end if;
	end;
	};



%%
with Statements;
use Statements;
with Expressions; 
use  Expressions;
use Expressions.Variables_Type_Package;
with Task_Set;
use Task_Set;


package Parser is




	-- Function to parse a scheduler behavior
	--
	procedure Yyparse;


	-- Initiliaze parser Data
	-- Do initialize Simulation Data all together
	--
	procedure Init_Parser(My_Tasks : in Tasks_Set);




	-- Root Pointers to the parsed code
	-- There is one Pointers per section
	--
	type Sections is (Start, Priority, Election, Activation, Check_Resource, Allocate_Resource, Release_Resource);


	type Section_Pointers is Array (Sections'range)
		of Statement_Ptr;

	Root_Statement_Pointer : Section_Pointers;



	-- Table to store the list of Variables of 
	-- a parametric scheduler
	--
	Variables_Table : Variables_Table_Type;



	-- Display list of Statements contained
	-- in a section
	--
	procedure Put(S : in Section_Pointers);





end Parser;




with Text_Io;
use Text_Io;
with Config;
use Config;
with Scheduler_Lex;
use Scheduler_Lex;
with Scheduler_Dfa;
use Scheduler_Dfa;
with Scheduler_Goto; 
use Scheduler_Goto;
with Scheduler_Tokens; 
use Scheduler_Tokens;
with Scheduler_Shift_Reduce;
use Scheduler_Shift_Reduce;
with Ada.Numerics.Aux;
use Ada.Numerics.Aux;
with Translate;
use Translate;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
with Ada.Exceptions;
use Ada.Exceptions;
with Simulations;
use Simulations;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Tasks;
use Tasks;
with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;



package body Parser is


        -- The langague is block oriented : the constant below
        -- given the maximum depth level of block
        --
        Max_Block_Level : constant Natural := 10000;


        -- These variables store statements of inner blocks of code
        --
        Else_Inner_Statement_Ptr : Array (0..Max_Block_Level) of Statement_Ptr;
        Else_Inner_Statement_Index : Natural :=0;
        Then_Inner_Statement_Ptr : Array (0..Max_Block_Level) of Statement_Ptr;
        Then_Inner_Statement_Index : Natural :=0;
        Loop_Inner_Statement_Ptr : Array (0..Max_Block_Level) of Statement_Ptr;
        Loop_Inner_Statement_Index : Natural :=0;


        -- Variables needed to parse a "if" or a "forall" statement
        --
        New_If :  If_Statement_Ptr;
        Old_Stat : Statement_Ptr;
        New_Stat : Statement_Ptr;
        New_Forall : Forall_Statement_Ptr;
        New_While : While_Statement_Ptr;


	procedure Yyerror(S: String) is
	begin
                Raise_Exception(Expressions.Syntax_Error'Identity, "token """ & To_String(
                        To_Unbounded_String(Yytext) & """" & Lb_Comma & " Line" & positive'image(Lines) & Lb_Comma & To_Unbounded_String(S) ));
	end Yyerror;



	-- Store the scheduler behavior step
	-- Used to Check is some Statements can be
	-- declared in the Current_Section
	--
	Current_Section : Sections;


	-- Current Pointers to the parsed code
	-- There is one Pointers per section
	--
	Current_Statement_Pointer : Section_Pointers;



	-- Procedure used to display content of each
	-- section (Statements and Expressions)
	--
	package Sections_Io is new Text_Io.Enumeration_Io(Sections);
	use Sections_Io;
	
	procedure Put(S : in Section_Pointers) is 
	
	begin 
		for I in Sections'range loop
			if S(I)/=null 
				then
					Put(" Section : ");
					Put(I); New_Line;	
					Recursive_Put(S(I));
					New_Line;
					New_Line;
			end if;
	    	end loop;
	end Put;



	procedure Init_Parser(My_Tasks : in Tasks_Set) is 


	New_Var    	  : Variable_Record_Ptr;
	New_Expr_Array 	  : Array_Variable_Expr_Ptr;
	New_Expr_Variable : Variable_Expr_Ptr;
	My_Iterator 	  : Iterator;
	A_Task      	  : Task_Ptr;
	Do_Insert 	  : boolean;


	begin

                Else_Inner_Statement_Index:=0;
                Then_Inner_Statement_Index:=0;
                Loop_Inner_Statement_Index:=0;

		Initialize(Variables_Table);


		for I in Sections'range loop
			Root_Statement_Pointer(I):=null;
		end loop;


        	----------------------------------------
		-- Declare predefined Variables ..
	        ----------------------------------------
	

		-- "priority"
		--
		New_Var:= new Variable_record;
       	 	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
		New_Expr_Array.Name:=To_Unbounded_String("priority");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
	
	
		-- "deadline"
		--
		New_Var:= new Variable_record;
       		New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
       	 	New_Expr_Array.Name:=To_Unbounded_String("deadline");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

		-- "start_time"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
        	New_Expr_Array.Name:=To_Unbounded_String("start_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
	

		-- "capacity"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
       	 	New_Expr_Array.Name:=To_Unbounded_String("capacity");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


		-- "rest_of_capacity"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
        	New_Expr_Array.Name:=To_Unbounded_String("rest_of_capacity");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


		-- "activation_number"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
        	New_Expr_Array.Name:=To_Unbounded_String("activation_number");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

		-- "used_cpu"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
		Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
        	New_Expr_Array.Name:=To_Unbounded_String("used_cpu");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		


		-- "period"
		--
		New_Var:= new Variable_record;
        	New_Expr_Array := new 
			Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
        	New_Expr_Array.Name:=To_Unbounded_String("period");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

                -- "previously_elected"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
                New_Expr_Variable.Name:=To_Unbounded_String("previously_elected");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_tasks"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_tasks");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "simulation_time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer_Data;
                New_Expr_Variable.Name:=To_Unbounded_String("simulation_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


		-- User defined parameters
		--

		-- Find user defined parameters : scan all task
		-- Variables and add them
		--

		if not Is_Empty(My_Tasks) 
			then
        			Reset_Iterator(My_Iterator);

         			loop
            				Current_Element(My_Tasks, A_Task, My_Iterator);

            				exit when Is_Last_Element(My_Tasks, My_Iterator) = True;

		 			for I in 0..A_Task.Param.Nb_Entries-1 loop
  
						-- Is the Variable already exists ?
                				--
						Do_Insert:=true;
                				for J in 0..Variables_Table.Nb_Entries-1 loop
                        				if Variables_Table.Entries(J).Variable.Name=a_task.param.Entries(i).Name
                                				then Do_Insert:=false;
                        				end if;
                				end loop;

						if Do_Insert 
							then
								New_Var:= new Variable_record;
						        	New_Expr_Array := new 
									Array_Variable_Expr;
						        	if A_Task.Param.Entries(I).Ptype=Integer_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Integer_Data;
								end if;
						        	if A_Task.Param.Entries(I).Ptype=Double_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Double_Data;
								end if;
						        	if A_Task.Param.Entries(I).Ptype=Boolean_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Boolean_Data;
								end if;
	
						        	New_Expr_Array.Name:=A_Task.Param.Entries(I).Name;
						        	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
								Add(Variables_Table, New_Var);
						end if;
	
					end loop;

           	 			Next_Element(My_Tasks, My_Iterator);
				end loop;
			end if;


        end Init_Parser;





##



end Parser;





