------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
--
--  Here, you will find type definitions of data stored during simulation
--  with parametric scheduler. These are mainly composed of a table 
--  storing of scheduler variable values
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;
with Unbounded_Strings;
use Unbounded_Strings;


package body Simulations is



   function To_Unbounded_String (
         S : Simulation_Value_Ptr ) 
     return Unbounded_String is 

      Msg  : Unbounded_String;  
      Last : Tasks_Range;  

   begin

      if Natural(Tasks_Range'Last) < 6 then
         Last:=Tasks_Range'Last;
      else
         Last:=6;
      end if;

      if (S/=null) then
         case S.Ptype is

            when Simulation_Boolean_Data=>
               return To_Unbounded_String(S.Boolean_Value'Img);

            when Simulation_Integer_Data=>
               return To_Unbounded_String(S.Integer_Value'Img);

            when Simulation_Double_Data=>
               return To_Unbounded_String(S.Double_Value'Img);

            when Simulation_Array_Boolean_Data=>
               for I in Tasks_Range'First..Last loop
                  Msg:=Msg & S.Boolean_Table_Value(I)'Img & ' ';
               end loop;
               return Msg & To_Unbounded_String("...");

            when Simulation_Array_Integer_Data=>
               for I in Tasks_Range'First..Last loop
                  Msg:=Msg & S.Integer_Table_Value(I)'Img & ' ';
               end loop;
               return Msg & To_Unbounded_String("...");

            when Simulation_Array_Double_Data=>
               for I in Tasks_Range'First..Last loop
                  Msg:=Msg & S.Double_Table_Value(I)'Img & ' ';
               end loop;
               return Msg & To_Unbounded_String("...");
         end case;
      else
         return Empty_String;
      end if;

   end To_Unbounded_String;



   procedure Put (
         S : Simulation_Value_Ptr ) is 
   begin
      Put(To_Unbounded_String(S));
   end Put;






end Simulations;



