------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--  Here, you will find type definitions of data stored during simulation
--  with parametric scheduler. These are mainly composed of a table 
--  storing of scheduler variable values
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Task_Set;
use Task_Set;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;



package Simulations is


   ------------------------------------------------------------------
   -- Type of data stored during a scheduling simulation 
   ------------------------------------------------------------------


   type Boolean_Value_Table is array (Tasks_Range) of Boolean; 
   type Integer_Value_Table is array (Tasks_Range) of Integer; 
   type Double_Value_Table is array (Tasks_Range) of Double; 


   type Simulation_Data_Type is 
         (Simulation_Boolean_Data,       
          Simulation_Integer_Data,       
          Simulation_Double_Data,        
          Simulation_Array_Boolean_Data, 
          Simulation_Array_Integer_Data, 
          Simulation_Array_Double_Data); 



   type Simulation_Value 
         (Ptype : Simulation_Data_Type) is 
      record 
case Ptype is when Simulation_Boolean_Data => Boolean_Value : Boolean := True; 
   when Simulation_Integer_Data=>
   Integer_Value : Integer := 0;  
   when Simulation_Double_Data=>
   Double_Value : Double := 0.0;  
   when Simulation_Array_Boolean_Data=>
   Boolean_Table_Value : Boolean_Value_Table := (others => True);  
   when Simulation_Array_Integer_Data=>
   Integer_Table_Value : Integer_Value_Table := (others => 0);  
   when Simulation_Array_Double_Data=>
   Double_Table_Value : Double_Value_Table := (others => 0.0);  
end case;
end record;

type Simulation_Value_Ptr is access Simulation_Value; 


procedure Put (
      S : Simulation_Value_Ptr ); 


function To_Unbounded_String (
      S : Simulation_Value_Ptr ) 
  return Unbounded_String; 





end Simulations;



