------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--  This package provides data structurs to store 
--  statements composing a scheduler behavior
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;


package body Statements is




   procedure Initialize (
         S : in out Generic_Statement ) is 
   begin
      null;
   end Initialize;


   procedure Finalize (
         S : in out Generic_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Generic_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out Put_Statement ) is 
   begin
      S.Statement_Type:=Put_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out Put_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Put_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out Nop_Statement ) is 
   begin
      S.Statement_Type:=Nop_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out Nop_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Nop_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out If_Statement ) is 
   begin
      S.Statement_Type:=If_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out If_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out If_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out Affectation_Statement ) is 
   begin
      S.Statement_Type:=Affectation_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out Affectation_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Affectation_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out Forall_Statement ) is 
   begin
      S.Statement_Type:=Forall_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out Forall_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Forall_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out While_Statement ) is 
   begin
      S.Statement_Type:=While_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out While_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out While_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         S : in out Return_Statement ) is 
   begin
      S.Statement_Type:=Return_Statement_Type;
   end Initialize;


   procedure Finalize (
         S : in out Return_Statement ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         S : in out Return_Statement ) is 
   begin
      null;
   end Adjust;


   procedure Recursive_Put (
         S : in     Statement_Ptr ) is 

   begin

      Put(S.all);

      if (S.Statement_Type = While_Statement_Type) then
         if (While_Statement_Ptr(S).Included_Statement /= null) then
            Put("Included statement : ");
            New_Line;
            Recursive_Put(While_Statement_Ptr(S).Included_Statement);
         end if;
      end if;
      if (S.Statement_Type = Forall_Statement_Type) then
         if (Forall_Statement_Ptr(S).Included_Statement /= null) then
            Put("Included statement : ");
            New_Line;
            Recursive_Put(Forall_Statement_Ptr(S).Included_Statement);
         end if;
      end if;

      if (S.Statement_Type = If_Statement_Type) then
         if (If_Statement_Ptr(S).Then_Statement /= null) then
            Put("Then statement : ");
            New_Line;
            New_Line;
            Recursive_Put(If_Statement_Ptr(S).Then_Statement);
         end if;
         if (If_Statement_Ptr(S).Else_Statement /= null) then
            Put("Else statement : ");
            New_Line;
            New_Line;
            Recursive_Put(If_Statement_Ptr(S).Else_Statement);
         end if;
      end if;

      if (S.Next_Statement /= null) then
         Put("Next statement  : ");
         New_Line;
         New_Line;
         Recursive_Put(S.Next_Statement);
      end if;

   end Recursive_Put;


   procedure Put (
         S : in     Generic_Statement ) is 

   begin
      Put_Line("Type : " & S.Statement_Type'Img);
      Put_Line("Line number : " & S.Line_Number'Img);
      New_Line;
   end Put;


   procedure Put (
         S : in     Nop_Statement ) is 

   begin
      Put_Line("Nop statement :");
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     Put_Statement ) is 

   begin
      Put_Line("Put statement :");
      Put("Identifier : ");
      Put(S.Identifier);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     If_Statement ) is 

   begin
      Put_Line("If statement :");
      Put("Bool expression : ");
      Put(S.Bool_Expression.all);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     Forall_Statement ) is 

   begin
      Put_Line("Forall statement :");
      Put("Forall index : ");
      Put(S.Forall_Index.all);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     While_Statement ) is 

   begin
      Put_Line("While statement :");
      Put("Condition : ");
      Put(S.Condition.all);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     Affectation_Statement ) is 

   begin
      Put_Line("Affectation statement :");
      Put("Variable name/index : ");
      New_Line;
      Put("Lvalue : ");
      Put(S.Lvalue.all);
      New_Line;
      Put("Rvalue : ");
      Put(S.Rvalue.all);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   procedure Put (
         S : in     Return_Statement ) is 

   begin
      Put_Line("Return statement :");
      Put("Return value: ");
      Put(S.Return_Value.all);
      New_Line;
      Put(Generic_Statement(S));
   end Put;


   function Copy (
         S : in     Statement_Ptr ) 
     return Statement_Ptr is 

      New_If_Statement          : If_Statement_Ptr;  
      New_Forall_Statement      : Forall_Statement_Ptr;  
      New_Return_Statement      : Return_Statement_Ptr;  
      New_Affectation_Statement : Affectation_Statement_Ptr;  
      New_Put_Statement         : Put_Statement_Ptr;  
      New_Nop_Statement         : Nop_Statement_Ptr;  
      New_While_Statement       : While_Statement_Ptr;  
      Ptr                       : Statement_Ptr;  

   begin

      case S.Statement_Type is
         when Affectation_Statement_Type   =>
            New_Affectation_Statement       := new Affectation_Statement;
            New_Affectation_Statement.all   := Affectation_Statement_Ptr(
               S).all;
            Ptr                     := Statement_Ptr(
               New_Affectation_Statement);
         when Forall_Statement_Type   =>
            New_Forall_Statement       := new Forall_Statement;
            New_Forall_Statement.all   := Forall_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_Forall_Statement);
         when While_Statement_Type   =>
            New_While_Statement       := new While_Statement;
            New_While_Statement.all   := While_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_While_Statement);
         when If_Statement_Type   =>
            New_If_Statement      := new If_Statement;
            New_If_Statement.all  := If_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_If_Statement);
         when Return_Statement_Type   =>
            New_Return_Statement        := new Return_Statement;
            New_Return_Statement.all    := Return_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_Return_Statement);
         when Put_Statement_Type =>
            New_Put_Statement     := new Put_Statement;
            New_Put_Statement.all := Put_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_Put_Statement);
         when Nop_Statement_Type =>
            New_Nop_Statement     := new Nop_Statement;
            New_Nop_Statement.all := Nop_Statement_Ptr(S).all;
            Ptr                     := Statement_Ptr(New_Nop_Statement);
         when others =>
            raise Constraint_Error;
      end case;

      return Ptr;
   end Copy;




end Statements;


