------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use Text_Io;


with Fixed_Strings;
use Fixed_Strings;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Config;

with Scheduler;
use Scheduler;

with Processors;
use Processors;

with Sets;

with Io_Tools;
use Io_Tools;


package Processor_Set is


   type Processor_Descriptor is 
      record 
         Processor_Name : Fixed_String;  
         Scheduler_Name : Scheduler_Type;  
         Is_Preemptive  : Preemptive_Type;  
         Quantum        : Natural;  
      end record; 


   type Processor_Parameters is 
         (Scheduler); 


   Invalid_Parameter         : exception;  
   Processor_Already_Defined : exception;  
   Scheduler_Syntax_Error    : exception;  


   package Generic_Processor_Set is new Sets (
      Max_Element     => Config.Max_Processors, 
      File_Descriptor => Processor_Descriptor,  
      Element         => Processor_Ptr,         
      Free            => Free,                  
      Copy            => Copy,                  
      Put             => Put);


   use Generic_Processor_Set;

   type Processors_Set is new Generic_Processor_Set.Set with private; 


   subtype Processors_Range is Generic_Processor_Set.Element_Range;
   subtype Processors_Iterator is   Generic_Processor_Set.Iterator;



   --------------------------------------------
   -- Write and Read should be overided
   -- because the scheduler's behavior
   -- should be stored/read in the project
   ---------------------------------------------

   procedure Read_From_File (
         My_Processor : in out Processors_Set'Class; 
         File_Name    : in     Unbounded_String      ); 


   procedure Write_To_File (
         My_Processor : in     Processors_Set'Class; 
         File_Name    : in     Unbounded_String;     
         Write_Mode   : in     Io_Tools.File_Mode    ); 


   ---------------------------------------------


   procedure Export_Xml (
         My_Processors : in     Processors_Set; 
         Into          : in     File_Type       ); 


   procedure Fill_Descriptor (
         My_Processor : in     Processors_Set;       
         Desc         :    out Processor_Descriptor; 
         A_Processor  : in     Processor_Ptr         ); 


   procedure Extract_Descriptor (
         My_Processor : in     Processors_Set;       
         Desc         : in     Processor_Descriptor; 
         A_Processor  :    out Processor_Ptr         ); 



   procedure Add_Processor (
         My_Processors : in out Processors_Set;   
         Name          : in     Unbounded_String; 
         A_Scheduler   : in     Scheduler_Ptr     ); 




   function Get (
         My_Processors  : in     Processors_Set;      
         Processor_Name : in     Unbounded_String;    
         Param_Name     : in     Processor_Parameters ) 
     return Scheduler_Ptr; 

   procedure Set (
         My_Processors  : in out Processors_Set;       
         Processor_Name : in     Unbounded_String;     
         Param_Name     : in     Processor_Parameters; 
         Param_Value    : in     Scheduler_Ptr         ); 





private

   type Processors_Set is new Generic_Processor_Set.Set with null record; 




end Processor_Set;
