------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

package body Queueing_System.Md1 is

   -- return the next customer arrival instant
   --
   function Next_Customer_Arrival (
         A_Queueing_System : in     Md1_Queueing_System; 
         Arrival_Rate      : in     Double;              
         Seed              : in     Generator;           
         Old_Arrival       : in     Double               ) 
     return Double is 

      Tmp_Next_Departure : Double;  

   begin
      Tmp_Next_Departure := Exponential_Inter_Time (A_Queueing_System,
         Arrival_Rate,Seed, Old_Arrival);

      return(Tmp_Next_Departure);
   end Next_Customer_Arrival;

   -- return the next customer departure instant
   --
   function Next_Customer_Departure (
         A_Queueing_System    : in     Md1_Queueing_System; 
         Departure_Rate       : in     Double;              
         Seed                 : in     Generator;           
         Old_Departure        : in     Double;              
         Buffer_Size          : in     Natural;             
         Last_Message_Arrival : in     Double               ) 
     return Double is 

      Tmp_Next_Departure : Double;  
   begin


      Tmp_Next_Departure := Constant_Inter_Time (A_Queueing_System,
         Departure_Rate,
         Old_Departure,Buffer_Size,Last_Message_Arrival);

      return(Tmp_Next_Departure);
   end Next_Customer_Departure;

   -- return the theoric average waiting time of one customer in the queue
   --
   function Qs_Average_Waiting_Time (
         A_Queueing_System : in     Md1_Queueing_System; 
         Arrival_Rate      : in     Double;              
         Departure_Rate    : in     Double               ) 
     return Double is 

      Result : Double;  
      Rau    : Double;  
   begin
      Rau := Arrival_Rate/Departure_Rate;
      Result := (2.0 - Rau) / (2.0*Departure_Rate*(1.0-Rau));

      Put_Line("test");
      return(Result);
   end Qs_Average_Waiting_Time;

   -- return the theoric average number of customer in the queue
   --
   function Qs_Average_Number_Customer (
         A_Queueing_System : in     Md1_Queueing_System; 
         Arrival_Rate      : in     Double;              
         Departure_Rate    : in     Double               ) 
     return Double is 

      Result : Double;  
      Rau    : Double;  
   begin
      Rau := Arrival_Rate/Departure_Rate;
      Result := (2.0*Rau - Rau*Rau)/(2.0*(1.0-Rau));

      return(Result);
   end Qs_Average_Number_Customer;

end Queueing_System.Md1;



