------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

package body Queueing_System.Mm1 is

   -- return the theoric average waiting time of one customer in the queue
   --
   function Qs_Average_Waiting_Time (
         A_Queueing_System : in     Mm1_Queueing_System; 
         Arrival_Rate      : in     Double;              
         Service_Rate      : in     Double               ) 
     return Double is 

      Result : Double;  
   begin
      Result := 1.0 / (Service_Rate - Arrival_Rate) - (1.0 / Service_Rate);
      return(Result);
   end Qs_Average_Waiting_Time;

   -- return the theoric average number of customer in the queue
   --
   function Qs_Average_Number_Customer (
         A_Queueing_System : in     Mm1_Queueing_System; 
         Arrival_Rate      : in     Double;              
         Service_Rate      : in     Double               ) 
     return Double is 
      Utilisation : Double := Arrival_Rate / Service_Rate;  
      Result      : Double;  
   begin
      Result := Utilisation / (1.0 - Utilisation);

      --      Put_Line("average waiting time: " & result'img);
      return(Result);

   end Qs_Average_Number_Customer;

   -- return the probability to be in a certain state
   --
   function Get_Probability_Of_State (
         A_Queueing_System : in     Mm1_Queueing_System; 
         N                 : in     Natural              ) 
     return Double is 
      Utilisation : Double;  
      Result      : Double;  
   begin
      Utilisation := A_Queueing_System.Arrival_Rate.Entries(0).Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;
      Result := Pow(Utilisation,Double(N));
      Result := Result * (1.0 - Utilisation);

      return (Result);
   end;

   -- return the utilistation of queue
   --
   function Get_Utilisation (
         A_Queueing_System : in     Mm1_Queueing_System ) 
     return Double is 
      Result : Double;  
   begin
      Result := A_Queueing_System.Arrival_Rate.Entries(0).Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;

      return (Result);
   end;



end Queueing_System.Mm1;



