------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;


package body Random_Tools is

   function Get_Rand_Parameter (
         Min  : in     Natural;  
         Max  : in     Natural;  
         Seed : in     Generator ) 
     return Natural is 

      Rand_Number : Uniformly_Distributed;  
      Aux         : Double;  
      Res         : Natural;  

   begin
      Rand_Number := Random(Seed); -- between 0 and 1
      Aux := Double(Rand_Number) * (Double(Max) - Double(Min));
      Aux := Aux + Double(Min);
      Res := Natural(Aux);

      if (Res<Min) or (Res>Max)
            then
         raise Constraint_Error;
      end if;

      return Res;
   end Get_Rand_Parameter;

   -- interarrival time or service time of a queuing sytem
   -- Uniform_Parameter should be in ]0,1]
   --
   function Get_Exponential_Time (
         Rate              : in     Double;               
         Uniform_Parameter : in     Uniformly_Distributed 
	) 
     return Natural is 

      Result : Natural;  

   begin
      -- inverse function of exponential function
      -- Result := Natural(- 1.0/Service_Rate * Log(Double(Uniform_Parameter)/Service_Rate));
      Result := Natural(- 1.0/Rate * Log(1.0-Double(Uniform_Parameter)));

      -- small ugly fix
      if (Result = 0) then
         Result :=1;
      end if;

      -- return the result
      return(Result);
   end Get_Exponential_Time;

   -- generate poisson inter arrival time (different from 0)
   function Get_Inter_Arrival_Time (
         Arrival_Rate : in     Double;   
         Seed         : in     Generator ) 
     return Natural is 
      Tmp_Interarrival  : Natural;  
      Uniform_Parameter : Uniformly_Distributed;  
   begin
      -- number between 0 and 1, but not 0
      Uniform_Parameter := Random(Seed);
      while (Uniform_Parameter = Uniformly_Distributed'First) loop
         Uniform_Parameter := Random(Seed);
      end loop;

      -- Put_Line("uniform para = " & Uniform_Parameter'Img);

      -- find the interarrival time different from 0
      Tmp_Interarrival := Get_Exponential_Time(Arrival_Rate,
         Uniform_Parameter);


      -- while (Tmp_Interarrival = 0) loop
      --         -- number between 0 and 1, but not 0
      --         Uniform_Parameter := Random(Seed);
      --         while (Uniform_Parameter = Uniformly_Distributed'first) loop
      --            Uniform_Parameter := Random(Seed);
      --         end loop;
      --         Tmp_interarrival := Get_Exponential_Time(Arrival_Rate,Uniform_Parameter);
      --      end loop;

      return(Tmp_Interarrival);
   end Get_Inter_Arrival_Time;

end Random_Tools;
