------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Resources;
use Resources;

with Time_Units;
use  Time_Units;
use  Time_Units.A_Time_Unit;


use Resource_Set.Generic_Resource_Set;

package body Resource_Set is


   procedure Task_Begin_End (
         Task_Name      : in     Unbounded_String; 
         Resource_Name  : in     Unbounded_String; 
         A_Resource_Set : in     Resources_Set;    
         Starts_Ends    : in out Start_End_Table;  
         Elements       : in out Task_List_Range   ) is 

      My_Resource_Iterator : Resources_Iterator;  
      A_Resource           : Resource_Ptr;  
      Index                : Task_List_Range    := 0;  

      --looking for a resource and a task the end and the begin of this task on this resource
      --Element is the number of element, must start to 0
      --
   begin
      Reset_Iterator(My_Resource_Iterator);
      loop
         Current_Element(A_Resource_Set, A_Resource, My_Resource_Iterator);
         if A_Resource.Name = Resource_Name then

            for I in 0..A_Resource.Task_Tab.Nb_Entries-1 loop
               if (A_Resource.Task_Tab.Entries(Index).Item= Task_Name) then
                  Starts_Ends(Elements+1).Task_Begin:= A_Resource.
                     Task_Tab.Entries(Index).Data.Task_Begin;
                  Starts_Ends(Elements+1).Task_End:= A_Resource.Task_Tab.
                     Entries(Index).Data.Task_End;
                  Elements := Elements + 1;
               end if;
               Index := Index + 1;
            end loop;
            exit;
         end if;
         exit when Is_Last_Element(A_Resource_Set, My_Resource_Iterator) =
            True;
         Next_Element(A_Resource_Set, My_Resource_Iterator);
      end loop;

   end Task_Begin_End;


   procedure Fill_Descriptor (
         My_Resource : in     Resources_Set;       
         Desc        :    out Resource_Descriptor; 
         A_Resource  : in     Resource_Ptr         ) is 


   begin


      Desc.Name := To_Bounded_String(To_String(A_Resource.Name));
      Desc.State := A_Resource.State;
      Desc.Location := To_Bounded_String(To_String(A_Resource.Location));
      Desc.Protocol := To_Bounded_String(A_Resource.Protocol'Img);

      for I in 0..A_Resource.Task_Tab.Nb_Entries-1 loop
         Desc.Affected_Tasks.Entries(Io_Task_List_Range(I)).Item:=
            To_Bounded_String(To_String(A_Resource.Task_Tab.Entries(I).
               Item));
         Desc.Affected_Tasks.Entries(Io_Task_List_Range(I)).Data:=
            A_Resource.Task_Tab.Entries(I).Data;
      end loop;

      Desc.Affected_Tasks.Nb_Entries:=Io_Task_List_Range(A_Resource.
         Task_Tab.Nb_Entries);

   end Fill_Descriptor;


   procedure Extract_Descriptor (
         My_Resource : in     Resources_Set;       
         Desc        : in     Resource_Descriptor; 
         A_Resource  :    out Resource_Ptr         ) is 

      New_Protocol     : Resources_Type;  
      New_Pcp_Resource : Pcp_Resource_Ptr;  
      New_Pip_Resource : Pip_Resource_Ptr;  
      New_Np_Resource  : Np_Resource_Ptr;  

   begin

      if (Desc.Protocol = "PRIORITY_CEILING_PROTOCOL") then

         New_Pcp_Resource := new Pcp_Resource;
         New_Protocol := Priority_Ceiling_Protocol;
         A_Resource:=Resource_Ptr(New_Pcp_Resource);

      elsif (Desc.Protocol = "PRIORITY_INHERITANCE_PROTOCOL") then
         New_Pip_Resource := new Pip_Resource;
         New_Protocol := Priority_Inheritance_Protocol;
         A_Resource:=Resource_Ptr(New_Pip_Resource);

      elsif (Desc.Protocol = "NO_PROTOCOL" ) then
         New_Np_Resource := new Np_Resource;
         New_Protocol := No_Protocol;
         A_Resource:=Resource_Ptr(New_Np_Resource);

      else

         raise Invalid_Parameter;

      end if;

      A_Resource.Name:=To_Unbounded_String(To_String(Desc.Name));
      A_Resource.State:=Desc.State;
      A_Resource.Location:=To_Unbounded_String(To_String(Desc.Location));
      A_Resource.Protocol := New_Protocol;

      A_Resource.Task_Tab.Nb_Entries:=Task_List_Range(Desc.Affected_Tasks.
         Nb_Entries);

      for I in 0..  A_Resource.Task_Tab.Nb_Entries-1 loop
         A_Resource.Task_Tab.Entries(I).Item := To_Unbounded_String(
            To_String(Desc.Affected_Tasks.Entries(Io_Task_List_Range(I)).
               Item));
         A_Resource.Task_Tab.Entries(I).Data:= Desc.Affected_Tasks.
            Entries(Io_Task_List_Range(I)).Data;
      end loop;

   end Extract_Descriptor;

   procedure Update_Resource (
         My_Resources   : in out Resources_Set;    
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Location       : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ) is 

      The_Resource : Resource_Ptr;  

   begin

      The_Resource := Search_Resource(My_Resources,Name);
      Delete(My_Resources,The_Resource);
      Add_Resource(My_Resources,Name,State,Location,Protocol,
         Affected_Tasks);

   end Update_Resource;


   procedure Add_Resource (
         My_Resources   : in out Resources_Set;    
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Cpu            : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ) is 


      A_Resource_Ptr : Resource_Ptr;  

   begin

      Add_Resource(My_Resources,A_Resource_Ptr,Name,State,Cpu,Protocol,
         Affected_Tasks);

   end Add_Resource;


   procedure Add_Resource (
         My_Resources   : in out Resources_Set;    
         A_Resource     : in out Resource_Ptr;     
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Cpu            : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ) is 

      My_Iterator : Iterator;  
      Desc        : Resource_Descriptor;  

   begin

      if (Name = "") then
         raise Invalid_Parameter;
      end if;

      if (Cpu = "") then
         raise Invalid_Parameter;
      end if;

      if (Protocol /= "NO_PROTOCOL")
            and (Protocol /= "PRIORITY_CEILING_PROTOCOL")
            and (Protocol /= "PRIORITY_INHERITANCE_PROTOCOL")
            then
         raise Invalid_Parameter;
      end if;

      for I in Task_List_Range loop
         if Affected_Tasks.Entries(I).Item /= ""
               then
            if (Affected_Tasks.Entries(I).Data.Task_Begin = 0) or
                  (Affected_Tasks.Entries(I).Data.Task_End = 0)
                  then
               raise Invalid_Parameter;
            end if;
         end if;
      end loop;



      if(Get_Number_Of_Elements(My_Resources) > 0) then

         Reset_Iterator(My_Iterator);

         -- Resource already exist ?

         loop
            Current_Element(My_Resources, A_Resource, My_Iterator);

            if (Name = A_Resource.Name) then
               raise Resource_Already_Defined;
            end if;

            exit when Is_Last_Element(My_Resources, My_Iterator) = True;

            Next_Element(My_Resources, My_Iterator);

         end loop;

      end if;

      --Put the new element in the list

      Desc.Name:= To_Bounded_String(To_String(Name));
      Desc.State:=State;
      Desc.Location := To_Bounded_String(To_String(Cpu));
      Desc.Protocol := To_Bounded_String(To_String(Protocol));



      for I in 0..Affected_Tasks.Nb_Entries-1 loop
         Desc.Affected_Tasks.Entries(Io_Task_List_Range(I)).Item:=
            To_Bounded_String(To_String(Affected_Tasks.Entries(I).Item));
         Desc.Affected_Tasks.Entries(Io_Task_List_Range(I)).Data:=
            Affected_Tasks.Entries(I).Data;
      end loop;

      Desc.Affected_Tasks.Nb_Entries:=Io_Task_List_Range(Affected_Tasks.
         Nb_Entries);



      Extract_Descriptor(My_Resources, Desc, A_Resource);

      Add(My_Resources,A_Resource);

   end Add_Resource;


   function Search_Resource (
         My_Resources : in     Resources_Set;   
         Name         : in     Unbounded_String ) 
     return Resource_Ptr is 

      My_Iterator : Iterator;  
      A_Resource  : Resource_Ptr;  
      Result      : Resource_Ptr;  

      Found : Boolean := False;  

   begin

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(My_Resources,A_Resource,My_Iterator);

         if(A_Resource.Name = Name) then
            Found:=True;
            Result:=A_Resource;
         end if;

         exit when Is_Last_Element(My_Resources, My_Iterator) = True;

         Next_Element(My_Resources, My_Iterator);

      end loop;

      if not Found then
         raise Resource_Not_Found;
      end if;

      return Result;

   end Search_Resource;

   procedure Delete_Task (
         My_Resources : in out Resources_Set;   
         A_Task       : in     Unbounded_String ) is 

      A_Resource  : Resource_Ptr;  
      My_Iterator : Resources_Iterator;  
      Index       : Task_List_Range    := 1;  

   begin

      if(Get_Number_Of_Elements(My_Resources) > 0) then

         Reset_Iterator(My_Iterator);
         loop
            Current_Element(My_Resources, A_Resource, My_Iterator);

            -- Looking for a_task in the task_list and delete it
            --
            Index:=0;
            while (Index<A_Resource.Task_Tab.Nb_Entries)
                  loop
               if (A_Resource.Task_Tab.Entries(Index).Item = A_Task)
                     then
                  A_Resource.Task_Tab.Entries(Index):=A_Resource.Task_Tab.
                     Entries(A_Resource.Task_Tab.Nb_Entries);

                  A_Resource.Task_Tab.Nb_Entries:=
                     A_Resource.Task_Tab.Nb_Entries-1;
               end if;
               Index:=Index+1;
            end loop;


            exit when Is_Last_Element(My_Resources, My_Iterator) = True;
            Next_Element(My_Resources, My_Iterator);
         end loop;
      end if;

   end Delete_Task;



   procedure Delete_Processor (
         My_Resources : in out Resources_Set;   
         A_Processor  : in     Unbounded_String ) is 

      Tmp         : Resources_Set;  
      A_Resource  : Resource_Ptr;  
      My_Iterator : Resources_Iterator;  

   begin
      if(Get_Number_Of_Elements(My_Resources) > 0) then

         Reset_Iterator(My_Iterator);
         loop
            Current_Element(My_Resources, A_Resource, My_Iterator);

            if(A_Resource.Location=A_Processor)
                  then
               Add(Tmp, A_Resource);
            end if;

            exit when Is_Last_Element(My_Resources, My_Iterator) = True;
            Next_Element(My_Resources, My_Iterator);
         end loop;

         if not Is_Empty(Tmp) then
            Reset_Iterator(My_Iterator);
            loop
               Current_Element(Tmp, A_Resource, My_Iterator);

               Delete(My_Resources,A_Resource);

               exit when Is_Last_Element(Tmp, My_Iterator) = True;
               Next_Element(Tmp, My_Iterator);
            end loop;
         end if;

      end if;
   end Delete_Processor;




   procedure Export_Xml (
         My_Resources : in     Resources_Set; 
         Into         : in     File_Type      ) is 

      My_Iterator : Resources_Iterator;  
      A_Resource  : Resource_Ptr;  


   begin

      if not Is_Empty(My_Resources) then

         Put_Line(Into, "<resources>");

         Reset_Iterator(My_Iterator);

         loop
            Current_Element(My_Resources,A_Resource,My_Iterator);

            Put_Line(Into,Ascii.Ht & "<resource>");

            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<location>" & To_String(
                  A_Resource.Location) & "</location>");
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<name>" & To_String(
                  A_Resource.Name) & "</name>");
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<state>" & A_Resource.
               State'Img & "</state>");
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<protocol>" & A_Resource.
               Protocol'Img & "</protocol>");


            Put_Line(Into,Ascii.Ht & Ascii.Ht & "<resource_used_by>");
            for I in 0..A_Resource.Task_Tab.Nb_Entries-1 loop
               Put(Into,Ascii.Ht & Ascii.Ht & Ascii.Ht & "<resource_user>");
               Put(Into, " " & To_String(A_Resource.Task_Tab.Entries(I).
                     Item) & " ");
               Put(Into, A_Resource.Task_Tab.Entries(I).Data.Task_Begin'
                  Img & " " );
               Put(Into, A_Resource.Task_Tab.Entries(I).Data.Task_End'Img &
                  " " );
               Put_Line(Into, "</resource_user>");
            end loop;
            Put_Line(Into,Ascii.Ht & Ascii.Ht & "</resource_used_by>");


            Put_Line(Into,Ascii.Ht & "</resource>");


            exit when Is_Last_Element(My_Resources, My_Iterator) = True;

            Next_Element(My_Resources, My_Iterator);

         end loop;
         Put_Line(Into, "</resources>");
      end if;


   end Export_Xml;





end Resource_Set;








