------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

with Fixed_Strings;
use Fixed_Strings;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Config;
use Config;

with Resources;
use Resources;
use Resources.Task_Lists;

with Indexed_Tables;
with Sets;

package Resource_Set is

   Resource_Already_Defined : exception;  
   Invalid_Parameter        : exception;  
   Resource_Not_Found       : exception;  


   package Io_Task_Lists is new Indexed_Tables(
      Affected_Task_List, Fixed_String,
      Config.Max_Tasks_For_A_Resource, 0,
      Put, Initialize, Fixed_Strings.Put);
   use Io_Task_Lists;

   subtype Io_Task_List is Io_Task_Lists.Indexed_Table;
   subtype Io_Task_List_Range is Io_Task_Lists.Indexed_Table_Range;

   type Resource_Descriptor is 
      record 
         Name           : Fixed_String;  
         State          : Integer;  
         Protocol       : Fixed_String;  
         Location       : Fixed_String;  
         Affected_Tasks : Io_Task_List;  
      end record; 

   package Generic_Resource_Set is new Sets (
      Max_Element     => Config.Max_Resources, 
      File_Descriptor => Resource_Descriptor,  
      Element         => Resource_Ptr,         
      Free            => Free,                 
      Copy            => Copy,                 
      Put             => Put);

   use Generic_Resource_Set;

   type Resources_Set is new Generic_Resource_Set.Set with private; 


   subtype Resources_Range is Generic_Resource_Set.Element_Range;
   subtype Resources_Iterator is Generic_Resource_Set.Iterator;




   procedure Task_Begin_End (
         Task_Name      : in     Unbounded_String; 
         Resource_Name  : in     Unbounded_String; 
         A_Resource_Set : in     Resources_Set;    
         Starts_Ends    : in out Start_End_Table;  
         Elements       : in out Task_List_Range   ); 


   procedure Export_Xml (
         My_Resources : in     Resources_Set; 
         Into         : in     File_Type      ); 


   procedure Fill_Descriptor (
         My_Resource : in     Resources_Set;       
         Desc        :    out Resource_Descriptor; 
         A_Resource  : in     Resource_Ptr         ); 

   procedure Extract_Descriptor (
         My_Resource : in     Resources_Set;       
         Desc        : in     Resource_Descriptor; 
         A_Resource  :    out Resource_Ptr         ); 

   procedure Update_Resource (
         My_Resources   : in out Resources_Set;    
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Location       : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ); 


   procedure Add_Resource (
         My_Resources   : in out Resources_Set;    
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Cpu            : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ); 

   procedure Add_Resource (
         My_Resources   : in out Resources_Set;    
         A_Resource     : in out Resource_Ptr;     
         Name           : in     Unbounded_String; 
         State          : in     Integer;          
         Cpu            : in     Unbounded_String; 
         Protocol       : in     Unbounded_String; 
         Affected_Tasks : in     Task_List         ); 

   procedure Delete_Task (
         My_Resources : in out Resources_Set;   
         A_Task       : in     Unbounded_String ); 

   procedure Delete_Processor (
         My_Resources : in out Resources_Set;   
         A_Processor  : in     Unbounded_String ); 



   function Search_Resource (
         My_Resources : in     Resources_Set;   
         Name         : in     Unbounded_String ) 
     return Resource_Ptr; 


private

   type Resources_Set is new Generic_Resource_Set.Set with null record; 

end Resource_Set;




