------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Resources;
use Resources;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

package body Resources.Pcp is


   procedure Initialize (
         A_Blocking_Time : in out Pcp_Resource ) is 
   begin
      Reset(A_Blocking_Time);
   end Initialize;

   function Pi_Rk (
         Resource_Name  : in     Unbounded_String; 
         A_Resource_Set : in     Resources_Set;    
         A_Task_Set     : in     Tasks_Set         ) 
     return Priority_Range is 


      My_Resource_Iterator : Resources_Iterator;  
      A_Resource           : Resource_Ptr;  
      Max_Priority         : Priority_Range     := Low_Priority;  
      Index                : Task_List_Range    := 0;  
      Task_Name            : Unbounded_String;  
      A_Task               : Task_Ptr;  

   begin

      Reset_Iterator(My_Resource_Iterator);

      loop

         Current_Element(A_Resource_Set, A_Resource, My_Resource_Iterator);

         if A_Resource.Name = Resource_Name then

            for I in Task_List_Range loop

               exit when A_Resource.Task_Tab.Entries(Index).Data.Task_End =
                  0;

               Task_Name := A_Resource.Task_Tab.Entries(Index).Item;

               A_Task := Search_Task(A_Task_Set,Task_Name);

               if A_Task.Priority > Max_Priority then

                  Max_Priority := A_Task.Priority;

               end if;

               Index := Index + 1;


            end loop;

         end if;

         exit when Is_Last_Element(A_Resource_Set, My_Resource_Iterator) =
            True;

         Next_Element(A_Resource_Set, My_Resource_Iterator);

      end loop;

      return Max_Priority;

   end Pi_Rk;


   function Blocking_Time (
         Task_Name      : in     Unbounded_String; 
         A_Task_Set     : in     Tasks_Set;        
         A_Resource_Set : in     Resources_Set     ) 
     return Natural is 
   begin

      return Pcp_Critical_Tasks(Task_Name,A_Task_Set,A_Resource_Set);

   exception
      when Others_Resources_Protocol_Found =>
         raise Others_Protocol_Found;


   end Blocking_Time;


   function Pcp_Critical_Tasks (
         Task_Name      : in     Unbounded_String; 
         A_Task_Set     : in     Tasks_Set;        
         A_Resource_Set : in     Resources_Set     ) 
     return Natural is 

      The_Task              : Task_Ptr;  
      A_Task                : Task_Ptr;  
      A_Resource            : Resource_Ptr;  
      My_Resources_Iterator : Resources_Iterator;  
      Index                 : Task_List_Range    := 0;  
      The_Begin             : Natural            := 0;  
      The_End               : Natural            := 0;  
      Intersect_Time        : Natural            := 0;  
      Start_End             : Start_End_Table;  
      Index_Time            : Task_List_Range    := 0;  
      Elements              : Task_List_Range    := 0;  
      Max_Critical          : Natural            := 0;  
      Max_Priority          : Priority_Range;  
      Save_Elements         : Task_List_Range    := 0;  

   begin

      --Looking for the Task
      --
      The_Task := Search_Task(A_Task_Set,Task_Name);

      -- List the resources
      --
      Reset_Iterator(My_Resources_Iterator);

      loop

         Current_Element(A_Resource_Set, A_Resource, My_Resources_Iterator);

         if (A_Resource.Protocol/=Priority_Ceiling_Protocol) then

            raise Others_Resources_Protocol_Found;

         else
            --list the affected task tab
            --
            Save_Elements := Elements;

            Task_Begin_End(Task_Name,A_Resource.Name,A_Resource_Set,
               Start_End,Elements);

            loop

               exit when Elements-Save_Elements = 0;
               --This task is not in this resource

               The_Begin := Start_End(Index_Time).Task_Begin;
               The_End := Start_End(Index_Time).Task_End;

               Index := 0;

               for I in Task_List_Range loop

                  exit when A_Resource.Task_Tab.Entries(Index).Data.
                     Task_End = 0 ;

                  --If he start before the Begining and if he finish after the beginning
                  --
                  if ( A_Resource.Task_Tab.Entries(Index).Data.Task_Begin <=
                        The_Begin )
                        and ( A_Resource.Task_Tab.Entries(Index).Data.
                        Task_End > The_Begin)
                        and ( A_Resource.Task_Tab.Entries(Index).Item/=
                        Task_Name)  then

                     A_Task := Search_Task(A_Task_Set,A_Resource.Task_Tab.
                        Entries(Index).Item);
                     Max_Priority := Pi_Rk(A_Resource.Name,A_Resource_Set,
                        A_Task_Set);

                     --Pj<Pi and Pi(Rk) >= Pi
                     --
                     if(A_Task.Priority < The_Task.Priority )
                           and (A_Task.Name /= Task_Name)
                           and (The_Task.Priority >= Max_Priority)then

                        Intersect_Time := A_Resource.Task_Tab.Entries(
                           Index).Data.Task_End - The_Begin;

                        if Intersect_Time > Max_Critical then
                           Max_Critical := Intersect_Time;
                        end if;

                     end if;

                  end if;

                  --next element
                  --
                  Index := Index + 1;


               end loop;


               exit when Index_Time = Elements;

               Index_Time := Index_Time + 1;

            end loop;
         end if;
         exit when Is_Last_Element(A_Resource_Set, My_Resources_Iterator) =
            True;

         Next_Element(A_Resource_Set, My_Resources_Iterator);

      end loop;

      return Max_Critical;

   end Pcp_Critical_Tasks;


end Resources.Pcp;




