------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;

with Tasks;
use Tasks;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Text_Io;
use Text_Io;

package body Resources.Pip is

   procedure Initialize (
         A_Blocking_Time : in out Pip_Resource ) is 
   begin
      Reset(A_Blocking_Time);
   end Initialize;


   procedure Make_Table (
         Object_Name : in     Unbounded_String; 
         Max         : in     Natural;          
         N           : in out Natural;          
         Table       : in out Temp_Table        ) is 

      --N Must Be Initialize to 0 this is the number of task stored;
      --
      Index : Natural := 0;  
      Find  : Natural := 0;  
   begin

      pragma Debug ( Put_Line("Make Table") );

      loop

         if (Table(Index).Object_Name = Object_Name) then
            if (Table(Index).Max_Critical < Max ) then
               pragma Debug ( Put_Line("Element Exist") );
               Table(Index).Max_Critical := Max;
               Find := 1;
               exit;
            end if;
         end if;
         Index := Index+1;
         exit when Index = Max_Tasks_For_A_Resource + 1;

      end loop;

      if (Find = 0) then
         --New task
         --
         pragma Debug ( Put_Line("New Element") );
         Table(N+1).Object_Name := Object_Name;
         Table(N+1).Max_Critical := Max;
         N := N + 1;
      end if;

   end Make_Table;


   function Blocking_Time (
         Task_Name      : in     Unbounded_String; 
         A_Task_Set     : in     Tasks_Set;        
         A_Resource_Set : in     Resources_Set     ) 
     return Natural is 

      Tasks_Critical     : Natural := 0;  
      Resources_Critical : Natural := 0;  
      Tasks_N            : Natural := 0;  
      Resources_N        : Natural := 0;  
      Tasks_Mult         : Natural := 0;  
      Resources_Mult     : Natural := 0;  

   begin

      Pip_Critical_Tasks(Task_Name,A_Task_Set,A_Resource_Set,
         Tasks_Critical,Tasks_N);
      Pip_Critical_Resources(Task_Name,A_Task_Set,A_Resource_Set,
         Resources_Critical,Resources_N);
      Tasks_Mult := Tasks_N * Tasks_Critical;
      Resources_Mult := Resources_N * Resources_Critical;

      if (Tasks_Mult < Resources_Mult ) then
         return Tasks_Mult;
      else
         return Resources_Mult;
      end if;

   exception
      when Others_Resources_Protocol_Found =>
         raise Others_Resources_Protocol_Found;

   end Blocking_Time;

   procedure Pip_Critical_Tasks (
         Task_Name            : in     Unbounded_String; 
         A_Task_Set           : in     Tasks_Set;        
         A_Resource_Set       : in     Resources_Set;    
         Critical_Section_Sum : in out Natural;          
         N                    : in out Natural           ) is 

      The_Task              : Task_Ptr;  
      A_Task                : Task_Ptr;  
      A_Resource            : Resource_Ptr;  
      My_Resources_Iterator : Resources_Iterator;  
      Index                 : Task_List_Range    := 0;  
      The_Begin             : Natural            := 0;  
      The_End               : Natural            := 0;  
      Intersect_Time        : Natural            := 0;  
      Task_Table            : Temp_Table;  
      Sum_Index             : Natural            := 1;  
      Elements              : Task_List_Range    := 0;  
      Start_End             : Start_End_Table;  
      Index_Time            : Task_List_Range    := 0;  
      Save_Elements         : Task_List_Range    := 0;  

   begin

      --Looking for the Task

      The_Task := Search_Task(A_Task_Set,Task_Name);

      -- List the resources

      Reset_Iterator(My_Resources_Iterator);

      loop

         Current_Element(A_Resource_Set, A_Resource, My_Resources_Iterator);

         if (A_Resource.Protocol/=Priority_Inheritance_Protocol) then

            raise Others_Protocol_Found;

         else
            --list the affected task tab
            Save_Elements := Elements;

            Task_Begin_End(Task_Name,A_Resource.Name,A_Resource_Set,
               Start_End,Elements);

            loop

               pragma Debug (
                  Put_Line("Begin End Elements :"));
               pragma Debug (
                  Put_Line(To_String(To_Unbounded_String(Task_List_Range'
                           Image(Elements)))));
               pragma Debug (
                  Put_Line("Begin End Index :"));
               pragma Debug (
                  Put_Line(To_String(To_Unbounded_String(Task_List_Range'
                           Image(Index)))));

               exit when Elements-Save_Elements = 0;

               The_Begin := Start_End(Index_Time).Task_Begin;
               The_End := Start_End(Index_Time).Task_End;

               Index := 1;

               loop

                  exit when Index>A_Resource.Task_Tab.Nb_Entries;

                  --If he start before the Begining and if he finish after the beginning
                  --
                  if ( A_Resource.Task_Tab.Entries(Index).Data.Task_Begin <=
                        The_Begin )
                        and ( A_Resource.Task_Tab.Entries(Index).Data.
                        Task_End > The_Begin)
                        and ( A_Resource.Task_Tab.Entries(Index).Item/=
                        Task_Name)  then

                     A_Task := Search_Task(A_Task_Set,A_Resource.Task_Tab.
                        Entries(Index).Item);

                     if(A_Task.Priority > The_Task.Priority ) and (A_Task.
                           Name /= Task_Name) then

                        Intersect_Time := A_Resource.Task_Tab.Entries(
                           Index).Data.Task_End - The_Begin;
                        pragma Debug (
                           Put_Line("**Appel Make Table**")) ;
                        Make_Table(A_Resource.Task_Tab.Entries(Index).
                           Item,Intersect_Time,N,Task_Table);

                     end if;

                  end if;

                  --next element

                  Index := Index + 1;

                  exit when Index = Task_List_Range(
                     Max_Tasks_For_A_Resource) + 1 ;

               end loop;

               exit when Index_Time = Elements;

               Index_Time := Index_Time + 1;

            end loop;

         end if;

         exit when Is_Last_Element(A_Resource_Set, My_Resources_Iterator) =
            True;

         Next_Element(A_Resource_Set, My_Resources_Iterator);

      end loop;

      pragma Debug (
         Put_Line("Elements task :") );
      pragma Debug (
         Put_Line(To_String(To_Unbounded_String(Natural'Image(N)))) );

      loop

         exit when N = 0;
         exit when Task_Table(Sum_Index).Max_Critical = 0;

         pragma Debug (
            Put_Line("Critical task:") );
         pragma Debug (
            Put_Line(To_String(To_Unbounded_String(Natural'Image(
                        Critical_Section_Sum)))) );
         pragma Debug (
            Put_Line("Index task:") );
         pragma Debug (
            Put_Line(To_String(To_Unbounded_String(Natural'Image(
                        Sum_Index)))) );

         Critical_Section_Sum :=  Critical_Section_Sum + Task_Table(
            Sum_Index).Max_Critical;

         Sum_Index := Sum_Index + 1;

         exit when Sum_Index = N + 1;

      end loop;

   end Pip_Critical_Tasks;



   procedure Pip_Critical_Resources (
         Task_Name            : in     Unbounded_String; 
         A_Task_Set           : in     Tasks_Set;        
         A_Resource_Set       : in     Resources_Set;    
         Critical_Section_Sum : in out Natural;          
         N                    : in out Natural           ) is 

      The_Task              : Task_Ptr;  
      A_Task                : Task_Ptr;  
      A_Resource            : Resource_Ptr;  
      My_Resources_Iterator : Resources_Iterator;  
      Index                 : Task_List_Range    := 1;  
      The_Begin             : Natural            := 0;  
      The_End               : Natural            := 0;  
      Intersect_Time        : Natural            := 0;  
      Task_Table            : Temp_Table;  
      Sum_Index             : Natural            := 1;  
      Elements              : Task_List_Range    := 0;  
      Start_End             : Start_End_Table;  
      Index_Time            : Task_List_Range    := 1;  
      Save_Elements         : Task_List_Range    := 0;  

   begin

      --Looking for the Task

      The_Task := Search_Task(A_Task_Set,Task_Name);

      -- List the resources

      Reset_Iterator(My_Resources_Iterator);

      loop

         Current_Element(A_Resource_Set, A_Resource, My_Resources_Iterator);

         if (A_Resource.Protocol/=Priority_Inheritance_Protocol) then
            raise Others_Resources_Protocol_Found;
         else
            --list the affected task tab
            Save_Elements := Elements;

            Task_Begin_End(Task_Name,A_Resource.Name,A_Resource_Set,
               Start_End,Elements);

            loop

               exit when Elements-Save_Elements = 0;
               --This task is not in this resource

               The_Begin := Start_End(Index_Time).Task_Begin;
               The_End := Start_End(Index_Time).Task_End;
               Index := 1;
               for I in Task_List_Range loop

                  exit when A_Resource.Task_Tab.Entries(Index).Data.
                     Task_End = 0 ;

                  --If he start before the Begining and if he finish after the beginning

                  if ( A_Resource.Task_Tab.Entries(Index).Data.Task_Begin <=
                        The_Begin )
                        and ( A_Resource.Task_Tab.Entries(Index).Data.
                        Task_End > The_Begin)
                        and ( A_Resource.Task_Tab.Entries(Index).Item/=
                        Task_Name)  then

                     A_Task := Search_Task(A_Task_Set,A_Resource.Task_Tab.
                        Entries(Index).Item);

                     if(A_Task.Priority > The_Task.Priority ) and (A_Task.
                           Name /= Task_Name) then

                        Intersect_Time := A_Resource.Task_Tab.Entries(
                           Index).Data.Task_End - The_Begin;

                        pragma Debug (
                           Put_Line("**Appel Make Table Resource**") );

                        Make_Table(A_Resource.Name,Intersect_Time,N,
                           Task_Table);

                     end if;

                  end if;

                  --next element

                  Index := Index + 1;


               end loop;

               exit when Index_Time = Elements;

               Index_Time := Index_Time + 1;

            end loop;

         end if;

         exit when Is_Last_Element(A_Resource_Set, My_Resources_Iterator) =
            True;

         Next_Element(A_Resource_Set, My_Resources_Iterator);

      end loop;


      pragma Debug (
         Put_Line("Elements Ress:") );
      pragma Debug (
         Put_Line(To_String(To_Unbounded_String(Natural'Image(N)))) );


      loop

         exit when N = 0;
         exit when Task_Table(Sum_Index).Max_Critical = 0;

         pragma Debug (
            Put_Line("Critical res :") );
         pragma Debug (
            Put_Line(To_String(To_Unbounded_String(Natural'Image(
                        Critical_Section_Sum)))) );
         pragma Debug (
            Put_Line("Index res:") );
         pragma Debug (
            Put_Line(To_String(To_Unbounded_String(Natural'Image(
                        Sum_Index)))) );


         Critical_Section_Sum :=  Critical_Section_Sum + Task_Table(
            Sum_Index).Max_Critical;

         Sum_Index := Sum_Index + 1;

         exit when Sum_Index = N + 1;

      end loop;

   end Pip_Critical_Resources;



   function Pip_Number_Of_Resources_For_A_Task (
         A_Resource_Set : in     Resources_Set;   
         Task_Name      : in     Unbounded_String ) 
     return Natural is 

      A_Resource          : Resource_Ptr;  
      My_Iterator         : Resources_Iterator;  
      Number_Of_Resources : Natural            := 0;  
      Index               : Task_List_Range    := 1;  

   begin

      Reset_Iterator(My_Iterator);

      loop

         Current_Element(A_Resource_Set, A_Resource, My_Iterator);

         for I in Task_List_Range loop

            if (A_Resource.Task_Tab.Entries(Index).Item= Task_Name) then

               Number_Of_Resources := Number_Of_Resources+1;

            end if;


            Index := Index + 1;

         end loop;


         exit when Is_Last_Element(A_Resource_Set, My_Iterator) = True;

         Next_Element(A_Resource_Set, My_Iterator);

      end loop;

      return Number_Of_Resources;

   end Pip_Number_Of_Resources_For_A_Task;

end Resources.Pip;




