------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Task_Set;
use Task_Set;


package Resources.Pip is


   Others_Protocol_Found           : exception;  
   Others_Resources_Protocol_Found : exception;  

   type Pip_Resource is new Generic_Blocking_Time with private; 

   type Temp_Table_Record is 
      record 
         Object_Name  : Unbounded_String;  
         Max_Critical : Natural;  
      end record; 

   type Temp_Table is array (Natural range 1 .. Max_Tasks_For_A_Resource) of Temp_Table_Record; 


   procedure Initialize (
         A_Blocking_Time : in out Pip_Resource ); 


   procedure Make_Table (
         Object_Name : in     Unbounded_String; 
         Max         : in     Natural;          
         N           : in out Natural;          
         Table       : in out Temp_Table        ); 


   procedure Pip_Critical_Tasks (
         Task_Name            : in     Unbounded_String; 
         A_Task_Set           : in     Tasks_Set;        
         A_Resource_Set       : in     Resources_Set;    
         Critical_Section_Sum : in out Natural;          
         N                    : in out Natural           ); 

   function Pip_Number_Of_Resources_For_A_Task (
         A_Resource_Set : in     Resources_Set;   
         Task_Name      : in     Unbounded_String ) 
     return Natural; 

   procedure Pip_Critical_Resources (
         Task_Name            : in     Unbounded_String; 
         A_Task_Set           : in     Tasks_Set;        
         A_Resource_Set       : in     Resources_Set;    
         Critical_Section_Sum : in out Natural;          
         N                    : in out Natural           ); 

   function Blocking_Time (
         Task_Name      : in     Unbounded_String; 
         A_Task_Set     : in     Tasks_Set;        
         A_Resource_Set : in     Resources_Set     ) 
     return Natural; 

private
   type Pip_Resource is new Generic_Blocking_Time with 
      record 
null; --Blocking_Time: PIP_Blocking_Time;
end record;


end Resources.Pip;
