------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;

with Ada.Finalization;
with Convert_Strings;
with Convert_Unbounded_Strings;
with Indexed_Tables;


package Resources is

   type Resources_Type is 
         (No_Protocol,                  
          Priority_Ceiling_Protocol,    
          Priority_Inheritance_Protocol); 


   procedure To_Protocol is 
   new Convert_Strings(Resources_Type,No_Protocol);
   procedure To_Protocol is 
   new Convert_Unbounded_Strings(Resources_Type,No_Protocol);


   type Generic_Blocking_Time is abstract tagged private; 
   type Blocking_Time_Ptr is access all Generic_Blocking_Time'Class; 

   procedure Set_Blocking_Time_Type (
         My_Blocking_Time : in out Generic_Blocking_Time'Class; 
         Blocking_Type    : in     Resources_Type               ); 

   procedure Reset (
         A_Blocking_Time : in out Generic_Blocking_Time'Class ); 


   function Get_Blocking_Time_Type (
         My_Blocking_Time : in     Generic_Blocking_Time'Class ) 
     return Unbounded_String; 


   type Affected_Task_List is 
      record 
         Task_Begin : Natural := 0;  
         Task_End   : Natural := 0;  
      end record; 


   procedure Put (
         A : in     Affected_Task_List ); 
   procedure Initialize (
         A :    out Affected_Task_List ); 

   package Task_Lists is new Indexed_Tables(
      Affected_Task_List, Unbounded_String,
      Config.Max_Tasks_For_A_Resource, 0,
      Put, Initialize, Put);
   use Task_Lists;

   subtype Task_List is Indexed_Table;
   subtype Task_List_Range is Indexed_Table_Range;



   type Start_End_Table is array (Task_List_Range) of Affected_Task_List; 



   type Generic_Resource is new Ada.Finalization.Controlled with 
      record 
         Name     : Unbounded_String := To_Unbounded_String ("");  
         State    : Integer          := 0;  
         Protocol : Resources_Type;  
         Task_Tab : Task_List;  
         Location : Unbounded_String := To_Unbounded_String ("");  
      end record; 

   type Resource_Ptr is access all Generic_Resource'Class; 

   procedure Initialize (
         A_Resource : in out Generic_Resource ); 
   procedure Finalize (
         A_Resource : in out Generic_Resource ); 
   procedure Adjust (
         A_Resource : in out Generic_Resource ); 



   procedure Free (
         A_Resource : in out Resource_Ptr ); 
   --pragma Import (C, free, "__gnat_free");

   procedure Put (
         My_Resource : in     Resource_Ptr ); 


   function Copy (
         My_Resource : in     Resource_Ptr ) 
     return Resource_Ptr; 



   type Pcp_Resource is new Generic_Resource with private; 
   type Pcp_Resource_Ptr is access all Pcp_Resource'Class; 

   procedure Initialize (
         A_Resource : in out Pcp_Resource ); 
   procedure Finalize (
         A_Resource : in out Pcp_Resource ); 
   procedure Adjust (
         A_Resource : in out Pcp_Resource ); 

   type Np_Resource is new Generic_Resource with private; 
   type Np_Resource_Ptr is access all Np_Resource'Class; 

   procedure Initialize (
         A_Resource : in out Np_Resource ); 
   procedure Finalize (
         A_Resource : in out Np_Resource ); 
   procedure Adjust (
         A_Resource : in out Np_Resource ); 

   type Pip_Resource is new Generic_Resource with private; 
   type Pip_Resource_Ptr is access all Pip_Resource'Class; 

   procedure Initialize (
         A_Resource : in out Pip_Resource ); 
   procedure Finalize (
         A_Resource : in out Pip_Resource ); 
   procedure Adjust (
         A_Resource : in out Pip_Resource ); 


private
   type Generic_Blocking_Time is abstract new Ada.Finalization.Controlled with 
      record 
         Name : Resources_Type;  
      end record; 

   type Pcp_Resource is new Generic_Resource with null record; 
   type Pip_Resource is new Generic_Resource with null record; 
   type Np_Resource is new Generic_Resource with null record; 

end Resources;

