------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Text_Io;
use Text_Io;

with Tasks;
use Tasks;

with Scheduler;
use Scheduler;


with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;







package body Scheduler.Dynamic_Priority.Edf is


   procedure Initialize (
         A_Scheduler : in out Edf_Scheduler ) is 
   begin
      Reset(A_Scheduler);
      A_Scheduler.Name:= Earliest_Deadline_First;
   end Initialize;


   procedure Finalize (
         A_Scheduler : in out Edf_Scheduler ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Scheduler : in out Edf_Scheduler ) is 
   begin
      null;
   end Adjust;



   function Copy (
         A_Scheduler : in     Edf_Scheduler ) 
     return Scheduler_Ptr is 

      Ptr : Edf_Scheduler_Ptr;  

   begin

      Ptr:= new Edf_Scheduler;

      Ptr.Name:=A_Scheduler.Name;
      Ptr.Is_Preemptive:=A_Scheduler.Is_Preemptive;
      Ptr.Quantum:=A_Scheduler.Quantum;
      Ptr.Previously_Elected:=A_Scheduler.Previously_Elected;

      return Scheduler_Ptr(Ptr);

   end Copy;


   procedure Do_Election (
         My_Scheduler      : in out Edf_Scheduler;          
         Si                : in out Scheduling_Information; 
         Processor_Name    : in     Unbounded_String;       
         At_Time           :        Natural;                
         My_Dependencies   : in     Tasks_Dependencies;     
         With_Offsets      :        Boolean;                
         With_Precedencies :        Boolean;                
         With_Resources    :        Boolean;                
         Elected           : in out Tasks_Range;            
         No_Task           : in out Boolean                 ) is 

      Smallest_Deadline : Natural     := Natural'Last;  
      I                 : Tasks_Range := 0;  
   begin

      loop
         if (Si.Tcbs(I).Tsk.Location = Processor_Name) then
            if (Si.Tcbs(I).Wake_Up_Time <= At_Time) and
                  (Dynamic_Priority_Tcb_Ptr(Si.Tcbs(I)).Dynamic_Deadline <
                  Smallest_Deadline) and
                  (Si.Tcbs(I).Rest_Of_Capacity /= 0)
                  then
               if (With_Offsets=False) or
                     Check_Offset(Si.Tcbs(I), At_Time)
                     then
                  if (With_Precedencies=False) or
                        Check_Precedencies(Si, My_Dependencies, At_Time,
                        Si.Tcbs(I))
                        then

                     Smallest_Deadline:=Dynamic_Priority_Tcb_Ptr(Si.Tcbs(
                           I)).
                        Dynamic_Deadline;
                     Elected:=I;
                  end if;
               end if;
            end if;
         end if;

         I:=I+1;
         exit when Si.Tcbs(I)=null;
      end loop;


      if Smallest_Deadline= Natural'Last
            then
         No_Task:=True;
      else
         No_Task:=False;

         -- last time unit : compute dynamic deadline for periodic task
         --
         if (Si.Tcbs(Elected).Rest_Of_Capacity = 1)
               and (Si.Tcbs(Elected).Tsk.Task_Type = Periodic_Type)
               then
            Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).Dynamic_Deadline:=
               Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).
               Dynamic_Deadline
               +Periodic_Task_Ptr(Si.Tcbs(Elected).Tsk).Period;
         end if;

      end if;

   end Do_Election;





end Scheduler.Dynamic_Priority.Edf ;


