------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
-- 
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package Scheduler.Dynamic_Priority is


   type Dynamic_Priority_Scheduler is abstract new Generic_Scheduler with private; 
   type Dynamic_Priority_Scheduler_Ptr is access all Dynamic_Priority_Scheduler'Class; 


   type Dynamic_Priority_Tcb is new Tcb with 
      record 
         Dynamic_Deadline : Natural;  
      end record; 
   type Dynamic_Priority_Tcb_Ptr is access all Dynamic_Priority_Tcb'Class; 




   function Build_Tcb (
         My_Scheduler : in     Dynamic_Priority_Scheduler; 
         A_Task       :        Task_Ptr                    ) 
     return Tcb_Ptr; 


   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Dynamic_Priority_Scheduler; 
         My_Tasks       : in     Tasks_Set;                  
         Processor_Name : in     Unbounded_String            ); 


   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Dynamic_Priority_Scheduler; 
         Si             : in out Scheduling_Information;     
         Processor_Name : in     Unbounded_String;           
         My_Tasks       : in out Tasks_Set;                  
         My_Resources   : in out Resources_Set               ); 


   procedure Bound_On_Processor_Utilization (
         My_Scheduler   : in     Dynamic_Priority_Scheduler; 
         My_Tasks       : in     Tasks_Set;                  
         Processor_Name : in     Unbounded_String;           
         Result         : in out Double;                     
         Msg            : in out Unbounded_String            ); 


   procedure Compute_Response_Time (
         My_Scheduler   : in     Dynamic_Priority_Scheduler; 
         My_Tasks       : in out Tasks_Set;                  
         Processor_Name : in     Unbounded_String;           
         Msg            : in out Unbounded_String;           
         Response_Time  :    out Response_Table              ); 






private

   type Dynamic_Priority_Scheduler is abstract new Generic_Scheduler with null record; 

end Scheduler.Dynamic_Priority ;




