------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Fifos;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;


with Scheduler;
with Scheduler.Fixed_Priority;



package Scheduler.Fixed_Priority.Hpf is


   type Hpf_Scheduler is new Fixed_Priority_Scheduler with private; 
   type Hpf_Scheduler_Ptr is access all Hpf_Scheduler'Class; 


   type Hpf_Tcb is new Fixed_Priority_Tcb with 
      record 
         Task_Quantum : Natural := 0;  
      end record; 
   type Hpf_Tcb_Ptr is access all Hpf_Tcb'Class; 


   procedure Initialize (
         A_Scheduler : in out Hpf_Scheduler ); 

   procedure Finalize (
         A_Scheduler : in out Hpf_Scheduler ); 

   procedure Adjust (
         A_Scheduler : in out Hpf_Scheduler ); 



   function Copy (
         A_Scheduler : in     Hpf_Scheduler ) 
     return Scheduler_Ptr; 




   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Hpf_Scheduler;   
         My_Tasks       : in     Tasks_Set;       
         Processor_Name : in     Unbounded_String ); 


   procedure Compute_Response_Time (
         My_Scheduler   : in     Hpf_Scheduler;    
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ); 


   function Build_Tcb (
         My_Scheduler : in     Hpf_Scheduler; 
         A_Task       :        Task_Ptr       ) 
     return Tcb_Ptr; 


   procedure Do_Election (
         My_Scheduler      : in out Hpf_Scheduler;          
         Si                : in out Scheduling_Information; 
         Processor_Name    : in     Unbounded_String;       
         At_Time           :        Natural;                
         My_Dependencies   : in     Tasks_Dependencies;     
         With_Offsets      :        Boolean;                
         With_Precedencies :        Boolean;                
         With_Resources    :        Boolean;                
         Elected           : in out Tasks_Range;            
         No_Task           : in out Boolean                 ); 



   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Hpf_Scheduler;          
         Si             : in out Scheduling_Information; 
         Processor_Name : in     Unbounded_String;       
         My_Tasks       : in out Tasks_Set;              
         My_Resources   : in out Resources_Set           ); 


   procedure Dispatched_Change_Current_Priority (
         My_Scheduler : in out Hpf_Scheduler;          
         A_Tcb        : in     Fixed_Priority_Tcb_Ptr; 
         New_Priority :        Priority_Range          ); 



private


   procedure Put_Tcb (
         E : Hpf_Tcb_Ptr ); 

   package Tcb_Fifos is new Fifos(Config.Max_Tasks,
      Hpf_Tcb_Ptr, Put_Tcb);
   use Tcb_Fifos;

   type Fifo_Table is array (Priority_Range) of Fifo; 

   type Hpf_Scheduler is new Fixed_Priority_Scheduler with 
      record 
         Priority_Fifos : Fifo_Table;  
      end record; 




end Scheduler.Fixed_Priority.Hpf ;



