------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use  Text_Io;


with Scheduler;
use Scheduler;
use Scheduler.Double_Tasks_Parameters;


with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package body Scheduler.Fixed_Priority.Rm is


   procedure Initialize (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      Reset(A_Scheduler);
      A_Scheduler.Name:= Rate_Monotonic;
   end Initialize;

   procedure Finalize (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      null;
   end Adjust;



   function Copy (
         A_Scheduler : in     Rm_Scheduler ) 
     return Scheduler_Ptr is 

      Ptr : Rm_Scheduler_Ptr;  

   begin

      Ptr:= new Rm_Scheduler;

      Ptr.Name:=A_Scheduler.Name;
      Ptr.Is_Preemptive:=A_Scheduler.Is_Preemptive;
      Ptr.Quantum:=A_Scheduler.Quantum;
      Ptr.Previously_Elected:=A_Scheduler.Previously_Elected;
      Ptr.Current_Ceiling_Priority:=A_Scheduler.Current_Ceiling_Priority;
      Ptr.Used_Resource:=A_Scheduler.Used_Resource;

      return Scheduler_Ptr(Ptr);

   end Copy;



   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Rm_Scheduler;    
         My_Tasks       : in     Tasks_Set;       
         Processor_Name : in     Unbounded_String ) is 

   begin

      Periodic_Control(My_Tasks, Processor_Name);
      Have_Deadlines_Equal_Than_Periods_Control
         (My_Tasks, Processor_Name);

   end Check_Before_Scheduling;







   procedure Compute_Response_Time (
         My_Scheduler   : in     Rm_Scheduler;     
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ) is 

   begin


      -- check if tasks are periodics
      Periodic_Control(My_Tasks, Processor_Name);

      -- check start time
      Start_Time_Control(My_Tasks, Processor_Name);

      -- check offset
      Offset_Control(My_Tasks, Processor_Name);


      -- check if deadlines are equal  than period
      Have_Deadlines_Equal_Than_Periods_Control
         (My_Tasks, Processor_Name);


      -- Check processor utilization first
      if (Processor_Utilization_Over_Period(My_Tasks,
               Processor_Name) > 1.0)
            then
         raise Processor_Utilization_Exceeded;
      end if;



      -- Compute response time
      --

      Scheduler.Fixed_Priority.Compute_Response_Time(
         Fixed_Priority_Scheduler(My_Scheduler),
         My_Tasks, Processor_Name, Msg, Response_Time);
   end Compute_Response_Time;









end Scheduler.Fixed_Priority.Rm ;


