------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------






with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Unbounded_Strings;
use Unbounded_Strings;


with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Config;
use Config;

with Scheduler;



package Scheduler.Fixed_Priority is


   type Fixed_Priority_Scheduler is abstract new Generic_Scheduler with private; 


   type Fixed_Priority_Tcb is new Tcb with 
      record 
         Current_Priority : Priority_Range;  
         Is_Ready         : Boolean;  
      end record; 
   type Fixed_Priority_Tcb_Ptr is access all Fixed_Priority_Tcb'Class; 

   ----------------------------------------

   type Fixed_Priority_Resource is new Shared_Resource with 
      record 
         Priority_Ceiling : Priority_Range;  
      end record; 

   type Fixed_Priority_Resource_Ptr is access all Fixed_Priority_Resource'Class; 



   ----------------------------------------

   function Build_Resource (
         My_Scheduler : in     Fixed_Priority_Scheduler; 
         A_Resource   :        Resource_Ptr              ) 
     return Shared_Resource_Ptr; 


   function Build_Tcb (
         My_Scheduler : in     Fixed_Priority_Scheduler; 
         A_Task       :        Task_Ptr                  ) 
     return Tcb_Ptr; 


   procedure Do_Election (
         My_Scheduler      : in out Fixed_Priority_Scheduler; 
         Si                : in out Scheduling_Information;   
         Processor_Name    : in     Unbounded_String;         
         At_Time           :        Natural;                  
         My_Dependencies   : in     Tasks_Dependencies;       
         With_Offsets      :        Boolean;                  
         With_Precedencies :        Boolean;                  
         With_Resources    :        Boolean;                  
         Elected           : in out Tasks_Range;              
         No_Task           : in out Boolean                   ); 


   procedure Compute_Response_Time (
         My_Scheduler   : in     Fixed_Priority_Scheduler; 
         My_Tasks       : in out Tasks_Set;                
         Processor_Name : in     Unbounded_String;         
         Msg            : in out Unbounded_String;         
         Response_Time  :    out Response_Table            ); 


   procedure Bound_On_Processor_Utilization (
         My_Scheduler   : in     Fixed_Priority_Scheduler; 
         My_Tasks       : in     Tasks_Set;                
         Processor_Name : in     Unbounded_String;         
         Result         : in out Double;                   
         Msg            : in out Unbounded_String          ); 


   procedure Set_Priority_According_To_Dm (
         My_Scheduler   : in     Fixed_Priority_Scheduler;                
         My_Tasks       : in out Tasks_Set;                               
         Processor_Name : in     Unbounded_String         := Empty_String ); 


   procedure Set_Priority_According_To_Rm (
         My_Scheduler   : in     Fixed_Priority_Scheduler;                
         My_Tasks       : in out Tasks_Set;                               
         Processor_Name : in     Unbounded_String         := Empty_String ); 


   procedure Check_Resource (
         My_Scheduler : in out Fixed_Priority_Scheduler; 
         Si           : in out Scheduling_Information;   
         A_Tcb        : in     Tcb_Ptr;                  
         Is_Ready     :    out Boolean                   ); 


   procedure Allocate_Resource (
         My_Scheduler      : in out Fixed_Priority_Scheduler; 
         Si                : in out Scheduling_Information;   
         Result            : in out Scheduling_Sequence;      
         Current_Time_Unit : in out Time_Unit_Range;          
         Current_Time      : in     Natural;                  
         A_Tcb             :        Tcb_Ptr                   ); 


   procedure Release_Resource (
         My_Scheduler      : in out Fixed_Priority_Scheduler; 
         Si                : in out Scheduling_Information;   
         Result            : in out Scheduling_Sequence;      
         Current_Time_Unit : in out Time_Unit_Range;          
         Current_Time      : in     Natural;                  
         A_Tcb             :        Tcb_Ptr                   ); 




   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Fixed_Priority_Scheduler; 
         Si             : in out Scheduling_Information;   
         Processor_Name : in     Unbounded_String;         
         My_Tasks       : in out Tasks_Set;                
         My_Resources   : in out Resources_Set             ); 


   procedure Compute_Ceiling_Of_Resources (
         My_Scheduler   : in out Fixed_Priority_Scheduler; 
         Si             : in out Scheduling_Information;   
         Processor_Name : in     Unbounded_String;         
         My_Tasks       : in out Tasks_Set;                
         My_Resources   : in out Resources_Set             ); 


   procedure Dispatched_Change_Current_Priority (
         My_Scheduler : in out Fixed_Priority_Scheduler; 
         A_Tcb        : in     Fixed_Priority_Tcb_Ptr;   
         New_Priority :        Priority_Range            ); 


   procedure Change_Current_Priority (
         My_Scheduler : in out Fixed_Priority_Scheduler'Class; 
         A_Tcb        : in     Fixed_Priority_Tcb_Ptr;         
         New_Priority :        Priority_Range                  ); 



private

   type Fixed_Priority_Scheduler is abstract new Generic_Scheduler with 
      record 
         Current_Ceiling_Priority : Priority_Range;  
         Used_Resource            : Boolean        := False;  
      end record; 


end Scheduler.Fixed_Priority ;


