------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;

with Ada.Exceptions;
use Ada.Exceptions;


with  Scheduler_Io;
use  Scheduler_Io;
with Text_Io;
use Text_Io;

with Statements;
use Statements;
with Simulations;
use Simulations;

with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;

with Task_Set;
use Task_Set;


package body Scheduler.Parametrics is




   procedure Initialize (
         A_Scheduler : in out Parametric_Scheduler ) is 
   begin
      Reset(A_Scheduler);
      A_Scheduler.Name:= Parametric;
      A_Scheduler.String_Behavior:= new Unbounded_String;
      A_Scheduler.Behavior_File_Name:= new Unbounded_String;

   end Initialize;


   procedure Finalize (
         A_Scheduler : in out Parametric_Scheduler ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Scheduler : in out Parametric_Scheduler ) is 
   begin
      null;
   end Adjust;




   function Copy (
         A_Scheduler : in     Parametric_Scheduler ) 
     return Scheduler_Ptr is 

      Ptr : Parametric_Scheduler_Ptr;  

   begin

      Ptr:= new Parametric_Scheduler;

      Ptr.Name:=A_Scheduler.Name;
      Ptr.Is_Preemptive:=A_Scheduler.Is_Preemptive;
      Ptr.Quantum:=A_Scheduler.Quantum;
      Ptr.Previously_Elected:=A_Scheduler.Previously_Elected;

      Ptr.String_Behavior:= new Unbounded_String;
      Ptr.String_Behavior.all:=A_Scheduler.String_Behavior.all;
      Ptr.Root_Statement_Pointer:=A_Scheduler.Root_Statement_Pointer;
      Ptr.Variables_Table:=A_Scheduler.Variables_Table;

      return Scheduler_Ptr(Ptr);

   end Copy;

   ---------------------------------------------------------------------
   --- Functions and procedure in this part of the package run
   --  parametric scheduler statements
   ----------------------------------------------------------------------


   procedure Dispatch (
         My_Scheduler    : in out Parametric_Scheduler; 
         Current         : in     Statement_Ptr;        
         Current_Section : in     Sections              ); 



   procedure If_Dispatch (
         My_Scheduler    : in out Parametric_Scheduler; 
         Current         : in     If_Statement_Ptr;     
         Current_Section : in     Sections;             
         Next            : in out Statement_Ptr         ) is 

      Simu_Ptr : Simulation_Value_Ptr := Value_Of (Current.Bool_Expression.
      all, My_Scheduler.Variables_Table);

   begin


      if Simu_Ptr.Ptype/=Simulation_Boolean_Data
            then
         Raise_Exception (Type_Error'Identity, "If statement, line " &
            Next.Line_Number'Img
            & ", expression type error");
      end if;

      if Simu_Ptr.Boolean_Value
            then
         Dispatch(My_Scheduler, Current.Then_Statement, Current_Section);
      else
         Dispatch(My_Scheduler, Current.Else_Statement, Current_Section);
      end if;

      Next:=Current.Next_Statement;

   end If_Dispatch;




   procedure Affectation_Dispatch (
         My_Scheduler : in out Parametric_Scheduler;      
         Current      : in     Affectation_Statement_Ptr; 
         Next         : in out Statement_Ptr              ) is 

      Var     : Variables_Range;  
      Indexed : Boolean              := False;  
      Val,  
      Index   : Simulation_Value_Ptr;  

   begin

      if Current.Lvalue.Expression_Type=Array_Variable_Type
            then
         if Array_Variable_Expr_Ptr(Current.Lvalue).Array_Index/=null
               then
            Indexed:=True;
         end if;
      end if;

      if not Indexed then
         Var:=Find_Variable(My_Scheduler.Variables_Table,
            Variable_Expr_Ptr(Current.Lvalue).Name);

         My_Scheduler.Variables_Table.Entries(Var).Simulation:=
            Value_Of(Current.Rvalue.all, My_Scheduler.Variables_Table);


      else

         Index:=Value_Of(Array_Variable_Expr_Ptr(Current.Lvalue).
            Array_Index.all,
            My_Scheduler.Variables_Table);


         if Index.Ptype/=Simulation_Integer_Data then
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line " &
               Current.Line_Number'Img & ", index expression type error");
         end if;


         Var:=Find_Variable(My_Scheduler.Variables_Table,
            Variable_Expr_Ptr(Current.Lvalue).Name);

         Val:=Value_Of(Current.Rvalue.all, My_Scheduler.Variables_Table);

         if (Val.Ptype/=Simulation_Integer_Data)
               and (Val.Ptype/=Simulation_Boolean_Data)
               and (Val.Ptype/=Simulation_Double_Data)
               then
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line "
               & Current.Line_Number'Img &
               ", rvalue expression type error");
         end if;

         if Val.Ptype=Simulation_Integer_Data  then
            My_Scheduler.Variables_Table.Entries(Var).Simulation.
               Integer_Table_Value(Tasks_Range(Index.Integer_Value)):=Val.
               Integer_Value;
         else
            if Val.Ptype=Simulation_Boolean_Data  then
               My_Scheduler.Variables_Table.Entries(Var).Simulation.
                  Boolean_Table_Value(Tasks_Range(Index.Integer_Value)):=
                  Val.Boolean_Value;
            else
               My_Scheduler.Variables_Table.Entries(Var).Simulation.
                  Double_Table_Value(Tasks_Range(Index.Integer_Value)):=
                  Val.Double_Value;

            end if;
         end if;
      end if;

      Next:=Current.Next_Statement;


   end Affectation_Dispatch;




   procedure Put_Dispatch (
         My_Scheduler : in out Parametric_Scheduler; 
         Current      : in     Put_Statement_Ptr;    
         Next         : in out Statement_Ptr         ) is 

      Var : Variables_Range;  

   begin


      Put("Line " & Current.Line_Number'Img & ", ");
      Put(To_String(Put_Statement_Ptr(Current).Identifier) & "=");

      Var:=Find_Variable(My_Scheduler.Variables_Table,
         Put_Statement_Ptr(Current).Identifier);

      Put(My_Scheduler.Variables_Table.Entries(Var).Simulation);
      New_Line;


      Next:=Current.Next_Statement;


   end Put_Dispatch;




   procedure While_Dispatch (
         My_Scheduler    : in out Parametric_Scheduler; 
         Current         : in     While_Statement_Ptr;  
         Current_Section : in     Sections;             
         Next            : in out Statement_Ptr         ) is 


      Current_Included : Statement_Ptr        := Statement_Ptr (Current.Included_Statement);
         
      Simu_Ptr         : Simulation_Value_Ptr;  

   begin


      -- Let's run included statements :
      -- do a loop iteration
      --
      loop

         Simu_Ptr := Value_Of (Current.Condition.all, My_Scheduler.
            Variables_Table);
         if Simu_Ptr.Ptype/=Simulation_Boolean_Data
               then
            Raise_Exception (Type_Error'Identity, "while statement, line " &
               Current.Line_Number'Img & ", expression type error");
         end if;

         exit when Simu_Ptr.Boolean_Value = False;

         Dispatch(My_Scheduler, Current_Included, Current_Section);

      end loop;


      Next:=Statement_Ptr(Current.Next_Statement);

   end While_Dispatch;


   procedure Forall_Dispatch (
         My_Scheduler    : in out Parametric_Scheduler; 
         Current         : in     Forall_Statement_Ptr; 
         Current_Section : in     Sections;             
         Next            : in out Statement_Ptr         ) is 

      Current_Included : Statement_Ptr   := Statement_Ptr (Current.Included_Statement);
         
      Var1,  
      Var2             : Variables_Range;  

   begin


      -- Test is the expression included in the forall
      -- statement is a variable ... raise a exception
      -- in the other cases
      --
      if Current.Forall_Index.Expression_Type=Variable_Type
            then
         if Current.Forall_Index.Variable_Type/=Simulation_Integer_Data
               then
            Raise_Exception (Type_Error'Identity,
               "forall statement, line " & Current.Line_Number'Img &
               ", forall expression have to be an integer variable");
         end if;
      else
         Raise_Exception (Type_Error'Identity, "forall statement, line " &
            Current.Line_Number'Img &
            ", forall expression have to be an integer variable");
      end if;


      -- Let's run included statements :
      -- do a loop iteration
      --
      Var1:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("nb_tasks"));


      for I in 0..My_Scheduler.Variables_Table.Entries(Var1).Simulation.
            Integer_Value-1 loop


         Var2:=Find_Variable(My_Scheduler.Variables_Table,
            Variable_Expr_Ptr(Current.Forall_Index).Name);
         My_Scheduler.Variables_Table.Entries(Var2).Simulation.
            Integer_Value:=I;

         Dispatch(My_Scheduler, Current_Included, Current_Section);


      end loop;


      Next:=Statement_Ptr(Current.Next_Statement);


   end Forall_Dispatch;



   procedure Dispatch (
         My_Scheduler    : in out Parametric_Scheduler; 
         Current         : in     Statement_Ptr;        
         Current_Section : in     Sections              ) is 

      Next       : Statement_Ptr;  
      My_Current : Statement_Ptr := Current;  

   begin


      if My_Current/=null then
         loop



            -- Run the associated statement
            --
            case My_Current.Statement_Type is

               when Put_Statement_Type =>
                  Put_Dispatch(My_Scheduler, Put_Statement_Ptr(My_Current),
                     Next);

               when 	Nop_Statement_Type =>
                  Next:=My_Current.Next_Statement;

               when       If_Statement_Type =>
                  If_Dispatch(My_Scheduler, If_Statement_Ptr(My_Current),
                     Current_Section, Next);

               when   Affectation_Statement_Type =>
                  Affectation_Dispatch(My_Scheduler,
                     Affectation_Statement_Ptr(My_Current), Next);

               when       Forall_Statement_Type =>
                  Forall_Dispatch(My_Scheduler, Forall_Statement_Ptr(
                        My_Current), Current_Section, Next);

               when   Return_Statement_Type =>
                  -- Check if a return statement is called
                  --
                  if (Return_Statement_Ptr(My_Current).Return_Value /= null)
                        and (Current_Section /= Election)  then
                     Raise_Exception (Type_Error'Identity,
                        Current_Section'Img & " section, line " & Current.
                        Line_Number' Img &
                        ", invalid return value from return statement");
                  end if;
                  exit;

               when     While_Statement_Type =>
                  While_Dispatch(My_Scheduler, While_Statement_Ptr(
                        My_Current), Current_Section, Next);

            end case;



            -- Last statement
            --
            exit when Next=null;


            My_Current:=Next;

         end loop;
      end if;


   end Dispatch;



   ---------------------------------------------------------------------
   --  Standard procedures and functions of scheduler class
   ----------------------------------------------------------------------



   procedure Bound_On_Processor_Utilization (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in     Tasks_Set;            
         Processor_Name : in     Unbounded_String;     
         Result         : in out Double;               
         Msg            : in out Unbounded_String      ) is 

   begin
      raise Invalid_Scheduler;
   end Bound_On_Processor_Utilization;


   procedure Compute_Response_Time (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in out Tasks_Set;            
         Processor_Name : in     Unbounded_String;     
         Msg            : in out Unbounded_String;     
         Response_Time  :    out Response_Table        ) is 

   begin
      raise Invalid_Scheduler;
   end Compute_Response_Time;


   function Get_String_Behavior (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String is 

   begin
      return My_Scheduler.String_Behavior.all;
   end Get_String_Behavior;


   function Get_String_Behavior (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String_Ptr is 

   begin
      return My_Scheduler.String_Behavior;
   end Get_String_Behavior;


   procedure Put (
         My_Scheduler : in     Parametric_Scheduler ) is 
   begin
      Put(Generic_Scheduler(My_Scheduler));
      Put_Line("Behavior file  : " & My_Scheduler.String_Behavior.all);
      Put(My_Scheduler.Root_Statement_Pointer);
      Put(My_Scheduler.Variables_Table);
   end Put;


   procedure Set_Behavior_File_Name (
         My_Scheduler : in out Parametric_Scheduler; 
         To_Set       : in     Unbounded_String      ) is 
   begin
      My_Scheduler.Behavior_File_Name:= new Unbounded_String;
      My_Scheduler.Behavior_File_Name.all:=To_Set;
   end Set_Behavior_File_Name;


   function Get_Behavior_File_Name (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String is 
   begin
      return My_Scheduler.Behavior_File_Name.all;
   end Get_Behavior_File_Name;


   function Get_Behavior_File_Name (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String_Ptr is 
   begin
      return My_Scheduler.Behavior_File_Name;
   end Get_Behavior_File_Name;


   procedure Set_String_Behavior (
         My_Scheduler : in out Parametric_Scheduler; 
         To_Set       : in     Unbounded_String      ) is 
   begin
      My_Scheduler.String_Behavior:= new Unbounded_String;
      My_Scheduler.String_Behavior.all:=To_Set;
   end Set_String_Behavior;


   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in     Tasks_Set;            
         Processor_Name : in     Unbounded_String      ) is 

   begin
      null;
   end Check_Before_Scheduling;


   procedure Do_Election (
         My_Scheduler      : in out Parametric_Scheduler;   
         Si                : in out Scheduling_Information; 
         Processor_Name    : in     Unbounded_String;       
         At_Time           : in     Natural;                
         My_Dependencies   : in     Tasks_Dependencies;     
         With_Offsets      : in     Boolean;                
         With_Precedencies : in     Boolean;                
         With_Resources    : in     Boolean;                
         Elected           : in out Tasks_Range;            
         No_Task           : in out Boolean                 ) is 


      Return_Val             : Expression_Ptr;  
      Current                : Statement_Ptr;  
      Election_Value         : Simulation_Value_Ptr;  
      Current_Selected_Value : Simulation_Value_Ptr;  
      I                      : Tasks_Range          := 0;  
      Var                    : Variables_Range;  


   begin


      -- By default, no task are elected : it's a way
      -- to safer schedule a parametric scheduler without 
      -- a correctly designed code (election part of it)
      --
      No_Task:=True;


      -- Before priority section is started, we
      -- retrieve some task run time information
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("activation_number"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Table_Value(My_Scheduler.Previously_Elected):=
         Integer(Si.Tcbs(My_Scheduler.Previously_Elected).Activation);


      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("rest_of_capacity"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Table_Value(My_Scheduler.Previously_Elected):=
         Integer(Si.Tcbs(My_Scheduler.Previously_Elected).
         Rest_Of_Capacity);


      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("used_cpu"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Table_Value(My_Scheduler.Previously_Elected):=
         Integer(Si.Tcbs(My_Scheduler.Previously_Elected).Used_Cpu);


      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("previously_elected"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(My_Scheduler.Previously_Elected);


      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("simulation_time"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(At_Time);



      -- Now, do "priority" section
      --
      Current := My_Scheduler.Root_Statement_Pointer(Parser.Priority);
      Dispatch(My_Scheduler, Current, Parser.Priority);


      -- Do "election" section
      --

      -- A election is done in 3 steps :
      --        1) we check and retrieve return information
      --        2) tasks queues should be updated
      --        3) we find the running task
      --
      Current := My_Scheduler.Root_Statement_Pointer(Parser.Election);

      -- Only one type of statement is allowed on "election" section :
      -- a return statement
      --
      if Current/=null then
         if Current.all not in Return_Statement'Class
               then
            Raise_Exception (Statement_Error'Identity,
               "Election section, line " & Current.Line_Number'Img &
               ", return statement expected");
         end if;


         -- Do return statement
         --

         -- The return statement should be :
         --      - either a min/max_to_index expression composed of
         --        a array variable only. Boolean/double/string array variable are forbidden
         --      - or a integer constant
         --      - or a integer variable
         -- 
         Return_Val:=Return_Statement_Ptr(Current).Return_Value;


         -- The return statement is a constant or a variable
         --   
         if (Return_Val.Expression_Type=Constant_Type) or
               (Return_Val.Expression_Type=Variable_Type)

               then

            if (Return_Val.Expression_Type=Constant_Type)
                  then
               if Constant_Expr_Ptr(Return_Val).Value.Ptype/=
                     Simulation_Integer_Data
                     then
                  Raise_Exception (Statement_Error'Identity,
                     "Election section, line " & Current.Line_Number'Img &
                     ", invalid expression type referenced in a return statement");
               end if;
            else
               if (Constant_Expr_Ptr(Return_Val).Value.Integer_Value<
                     Integer(Tasks_Range'First)) or
                     (Constant_Expr_Ptr(Return_Val).Value.Integer_Value>
                     Integer(Tasks_Range'
                        Last)) then
                  Raise_Exception (Statement_Error'Identity,
                     "Election section, line " & Current.Line_Number'Img &
                     ", invalid range type referenced in a return statement");
               end if;

               I:=Tasks_Range(Constant_Expr_Ptr(Return_Val).Value.
                  Integer_Value);
            end if;

            if   (Return_Val.Expression_Type=Variable_Type)
                  then
               Current_Selected_Value:=Value_Of(Variable_Expr_Ptr(
                     Return_Val).all,My_Scheduler.
                  Variables_Table);
               if Current_Selected_Value.Ptype/=Simulation_Integer_Data
                     then
                  Raise_Exception (Statement_Error'Identity,
                     "Election section, line " & Current.Line_Number'Img &
                     ", invalid expression type referenced in a return statement");
               end if;

               if (Current_Selected_Value.Integer_Value<Integer(
                        Tasks_Range'First)) or
                     (Current_Selected_Value.Integer_Value>Integer(
                        Tasks_Range'
                        Last)) then
                  Raise_Exception (Statement_Error'Identity,
                     "Election section, line " & Current.Line_Number'Img &
                     ", invalid range type referenced in a return statement");
               end if;
               I:=Tasks_Range(Current_Selected_Value.Integer_Value);
            end if;


            if (Si.Tcbs(I).Tsk.Location = Processor_Name) then
               if (Si.Tcbs(I).Wake_Up_Time <= At_Time) and
                     (Si.Tcbs(I).Rest_Of_Capacity /= 0)
                     then
                  if (With_Offsets=False) or
                        Check_Offset(Si.Tcbs(I), At_Time)
                        then
                     if (With_Precedencies=False) or
                           Check_Precedencies(Si, My_Dependencies,
                           At_Time,
                           Si.Tcbs(I))
                           then
                        Elected:=I;
                        No_Task:=False;

                     end if;
                  end if;
               end if;
            end if;

            return;

         end if;

         -- Now, let see the case when the return expression is a max/min_to_index
         --
         if (Return_Val.Expression_Type/=Min_To_Index_Type)
               and (Return_Val.Expression_Type/=Max_To_Index_Type)
               then
            Raise_Exception (Statement_Error'Identity,
               "Election section, line " & Current.Line_Number'Img &
               ", invalid expression type referenced in a return statement");
         end if;


         if (Return_Val.Expression_Type=Min_To_Index_Type) then
            if (Min_To_Index_Expr_Ptr(Return_Val).Lvalue.
                  Expression_Type/=
                  Array_Variable_Type)
                  then
               Raise_Exception (Statement_Error'Identity,
                  "Election section, line " & Current.Line_Number'Img &
                  ", invalid min_to_index expression in return statement");
            end if;

            if (Array_Variable_Expr_Ptr(Min_To_Index_Expr_Ptr(
                        Return_Val).
                     Lvalue).
                  Variable_Type
                  /=Simulation_Array_Integer_Data)
                  and (Array_Variable_Expr_Ptr(Min_To_Index_Expr_Ptr(
                        Return_Val).
                     Lvalue).
                  Variable_Type
                  /=Simulation_Array_Double_Data)

                  then
               Raise_Exception (Statement_Error'Identity,
                  "Election section, line " & Current.Line_Number'Img &
                  ", min_to_index expression in return statement should be an integer array variable");
            end if;
         end if;

         if (Return_Val.Expression_Type=Max_To_Index_Type) then
            if (Max_To_Index_Expr_Ptr(Return_Val).Lvalue.Expression_Type/=
                  Array_Variable_Type)
                  then
               Raise_Exception (Statement_Error'Identity,
                  "Election section, line " & Current.Line_Number'Img &
                  ", invalid max_to_index expression in return statement");
            end if;


            if (Array_Variable_Expr_Ptr(Max_To_Index_Expr_Ptr(Return_Val).
                     Lvalue).
                  Variable_Type
                  /=Simulation_Array_Integer_Data)
                  then
               Raise_Exception (Statement_Error'Identity,
                  "Election section, line " & Current.Line_Number'Img &
                  ", max_to_index expression in return statement should be an integer array variable");
            end if;
         end if;


         -----------------------------------------------------
         -- All the code is now done : we do the last
         -- operation : election !!!
         -----------------------------------------------------

         if Return_Val.Expression_Type=Min_To_Index_Type
               then
            Election_Value:=Value_Of(Array_Variable_Expr_Ptr(
                  Min_To_Index_Expr_Ptr(
                     Return_Val).Lvalue).all,My_Scheduler.
               Variables_Table);
         else
            Election_Value:=Value_Of(Array_Variable_Expr_Ptr(
                  Max_To_Index_Expr_Ptr(
                     Return_Val).Lvalue).all,My_Scheduler.
               Variables_Table);
         end if;


         if (Election_Value.Ptype=Simulation_Array_Double_Data) then
            Current_Selected_Value:= new Simulation_Value (
               Simulation_Double_Data);
            if Return_Val.Expression_Type=Min_To_Index_Type then
               Current_Selected_Value.Double_Value:=Double'Last;
            else
               Current_Selected_Value.Double_Value:=Double'First;
            end if;
         else
            Current_Selected_Value:= new Simulation_Value (
               Simulation_Integer_Data);
            if Return_Val.Expression_Type=Min_To_Index_Type then
               Current_Selected_Value.Integer_Value:=Integer'Last;
            else
               Current_Selected_Value.Integer_Value:=Integer'First;
            end if;
         end if;


         -- Now, loop on the array variable to find the appropriate task to be
         -- elected
         --

         loop
            if (Si.Tcbs(I).Tsk.Location = Processor_Name) then
               if (Si.Tcbs(I).Wake_Up_Time <= At_Time) and
                     (Si.Tcbs(I).Rest_Of_Capacity /= 0)
                     then
                  if (With_Offsets=False) or
                        Check_Offset(Si.Tcbs(I), At_Time)
                        then
                     if (With_Precedencies=False) or
                           Check_Precedencies(Si, My_Dependencies,
                           At_Time,
                           Si.Tcbs(I))
                           then

                        if (Election_Value.Ptype=
                              Simulation_Array_Integer_Data) then

                           if ((Return_Val.Expression_Type=
                                    Min_To_Index_Type) and
                                 (Election_Value.Integer_Table_Value(I) <
                                    Current_Selected_Value.
                                    Integer_Value))
                                 or
                                 ((Return_Val.Expression_Type=
                                    Max_To_Index_Type) and
                                 (Election_Value.Integer_Table_Value(I) >
                                    Current_Selected_Value.
                                    Integer_Value))

                                 then
                              Elected:=I;
                              Current_Selected_Value.Integer_Value:=
                                 Election_Value.
                                 Integer_Table_Value(I);
                           end if;
                        else
                           if ((Return_Val.Expression_Type=
                                    Min_To_Index_Type) and
                                 (Election_Value.Double_Table_Value(I) <
                                    Current_Selected_Value.
                                    Double_Value))
                                 or
                                 ((Return_Val.Expression_Type=
                                    Max_To_Index_Type) and
                                 (Election_Value.Double_Table_Value(I) >
                                    Current_Selected_Value.
                                    Double_Value))

                                 then
                              Elected:=I;
                              Current_Selected_Value.Double_Value:=
                                 Election_Value.
                                 Double_Table_Value(I);
                           end if;


                        end if;
                     end if;
                  end if;
               end if;
            end if;


            I:=I+1;
            exit when Si.Tcbs(I)=null;
         end loop;



         if (Election_Value.Ptype=
               Simulation_Array_Integer_Data) then

            if ( (Return_Val.Expression_Type=Min_To_Index_Type) and (
                     Current_Selected_Value.Integer_Value/=Integer'
                     Last)  )
                  or ( (Return_Val.Expression_Type=Max_To_Index_Type) and (
                     Current_Selected_Value.Integer_Value/=Integer'
                     First)  )
                  then
               No_Task:=False;
            end if;
         else
            if ( (Return_Val.Expression_Type=Min_To_Index_Type) and (
                     Current_Selected_Value.Double_Value/=Double'Last)  )
                  or ( (Return_Val.Expression_Type=Max_To_Index_Type) and (
                     Current_Selected_Value.Double_Value/=Double'First)  )
                  then
               No_Task:=False;
            end if;
         end if;


         -- End of return statement
         --
      end if;


   end Do_Election;



   -- Do "start" section statements
   --
   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Parametric_Scheduler;   
         Si             : in out Scheduling_Information; 
         Processor_Name : in     Unbounded_String;       
         My_Tasks       : in out Tasks_Set;              
         My_Resources   : in out Resources_Set           ) is 


      Current : Statement_Ptr;  

      Var : Variables_Range;  



   begin

      pragma Debug ( Put_Line("parametric : do start section") );


      -- We should check the scheduler behavior syntax ...
      --
      Open_Input(My_Scheduler.String_Behavior.all);
      Parser.Init_Parser(My_Tasks);
      Parser.Yyparse;


      -- Store syntax tree and compiling information
      --
      My_Scheduler.Root_Statement_Pointer:=
         Parser.Root_Statement_Pointer;

      My_Scheduler.Variables_Table:=
         Parser.Variables_Table;

      Current := My_Scheduler.Root_Statement_Pointer (Start);



      -- Initialize simulation data
      --
      for I in 0..My_Scheduler.Variables_Table.Nb_Entries-1 loop

         if My_Scheduler.Variables_Table.Entries(I).Variable.all in
               Variable_Expr'Class then

            My_Scheduler.Variables_Table.Entries(I).Simulation:=new
               Simulation_Value(
               Variable_Expr_Ptr(My_Scheduler.Variables_Table.
                  Entries(
                     I).
                  Variable).Variable_Type);

         else
            My_Scheduler.Variables_Table.Entries(I).Simulation:=new
               Simulation_Value(
               Simulation_Integer_Data);

         end if;
      end loop;


      -- Set task parameter : "priority"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("priority"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Tsk.Priority);
      end loop;


      -- Set task parameter : "deadline"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("deadline"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Tsk.Deadline);
      end loop;



      -- Set task parameter : "start_time"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("start_time"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Tsk.Start_Time);
      end loop;


      -- Set task parameter : "capacity"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("capacity"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Tsk.Capacity);
      end loop;


      -- Set task parameter : "period"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("period"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;

         case Si.Tcbs(I).Tsk.Task_Type is
            when Periodic_Type =>
               My_Scheduler.Variables_Table.Entries(Var).Simulation.
                  Integer_Table_Value(I):=
                  Integer(Periodic_Task_Ptr(Si.Tcbs(I).Tsk).Period);
            when Poisson_Type =>
               My_Scheduler.Variables_Table.Entries(Var).Simulation.
                  Integer_Table_Value(I):=
                  Integer(Poisson_Task_Ptr(Si.Tcbs(I).Tsk).Period);
            when others =>
               My_Scheduler.Variables_Table.Entries(Var).Simulation.
                  Integer_Table_Value(I):=0;
         end case;
      end loop;


      -- Set task parameter : "rest_of_capacity"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("rest_of_capacity"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Rest_Of_Capacity);
      end loop;


      -- Set task parameter : "activation_number"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("activation_number"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Activation);
      end loop;


      -- Set task parameter : "used_cpu"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("used_cpu"));
      for I in Tasks_Range loop
         exit when Si.Tcbs(I)=null;
         My_Scheduler.Variables_Table.Entries(Var).Simulation.
            Integer_Table_Value(I):=
            Integer(Si.Tcbs(I).Used_Cpu);
      end loop;


      -- Set variable : "previously_elected"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("previously_elected"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(My_Scheduler.Previously_Elected);


      -- Set variable : "nb_tasks"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("nb_tasks"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(Get_Number_Of_Task_From_Processor(
            My_Tasks, Processor_Name));


      -- Set variable : "simulation_time"
      --
      Var:=Find_Variable(My_Scheduler.Variables_Table,
         To_Unbounded_String("simulation_time"));
      My_Scheduler.Variables_Table.Entries(Var).Simulation.
         Integer_Value:=0;


      -- Set user defined task's parameter :
      --

      for K in Tasks_Range loop
         exit when Si.Tcbs(K)=null;


         for I in 0..Si.Tcbs(K).Tsk.Param.Nb_Entries-1 loop

            -- Find the user defined variable
            -- and set its value
            --
            for J in 0..Variables_Table.Nb_Entries-1 loop
               if Variables_Table.Entries(J).Variable.Name=
                     Si.Tcbs(K).Tsk.Param.Entries(I).Name
                     then

                  -- Now, set the value
                  --
                  if Si.Tcbs(K).Tsk.Param.Entries(I).Ptype=
                        Boolean_Parameter then
                     My_Scheduler.Variables_Table.Entries(J).
                        Simulation.
                        Boolean_Table_Value(K):=Boolean(Si.Tcbs(K).Tsk.
                        Param.Entries(I).Boolean_Value);
                  end if;
                  if Si.Tcbs(K).Tsk.Param.Entries(I).Ptype=
                        Integer_Parameter then
                     My_Scheduler.Variables_Table.Entries(J).
                        Simulation.
                        Integer_Table_Value(K):=Integer(Si.Tcbs(K).Tsk.
                        Param.Entries(I).Integer_Value);
                  end if;
                  if Si.Tcbs(K).Tsk.Param.Entries(I).Ptype=
                        Double_Parameter then
                     My_Scheduler.Variables_Table.Entries(J).
                        Simulation.
                        Double_Table_Value(K):=Double(Si.Tcbs(K).Tsk.
                        Param.Entries(I).Double_Value);
                  end if;
               end if;
            end loop;
         end loop;
      end loop;



      -- Now : run "start" statements
      --
      Dispatch(My_Scheduler, Current, Start);


   end Specific_Scheduler_Initialization;



end Scheduler.Parametrics ;


