------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;

with Expressions;
use Expressions;
use Expressions.Variables_Type_Package;

with Parser;
use Parser;



package Scheduler.Parametrics is




   type Parametric_Scheduler is new Generic_Scheduler with private; 
   type Parametric_Scheduler_Ptr is access all Parametric_Scheduler'Class; 






   procedure Initialize (
         A_Scheduler : in out Parametric_Scheduler ); 


   procedure Finalize (
         A_Scheduler : in out Parametric_Scheduler ); 


   procedure Adjust (
         A_Scheduler : in out Parametric_Scheduler ); 


   function Copy (
         A_Scheduler : in     Parametric_Scheduler ) 
     return Scheduler_Ptr; 


   procedure Set_Behavior_File_Name (
         My_Scheduler : in out Parametric_Scheduler; 
         To_Set       : in     Unbounded_String      ); 


   function Get_Behavior_File_Name (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String; 


   function Get_Behavior_File_Name (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String_Ptr; 


   procedure Set_String_Behavior (
         My_Scheduler : in out Parametric_Scheduler; 
         To_Set       : in     Unbounded_String      ); 


   function Get_String_Behavior (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String; 


   function Get_String_Behavior (
         My_Scheduler : in     Parametric_Scheduler ) 
     return Unbounded_String_Ptr; 


   procedure Put (
         My_Scheduler : in     Parametric_Scheduler ); 


   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in     Tasks_Set;            
         Processor_Name : in     Unbounded_String      ); 


   procedure Do_Election (
         My_Scheduler      : in out Parametric_Scheduler;   
         Si                : in out Scheduling_Information; 
         Processor_Name    : in     Unbounded_String;       
         At_Time           :        Natural;                
         My_Dependencies   : in     Tasks_Dependencies;     
         With_Offsets      : in     Boolean;                
         With_Precedencies : in     Boolean;                
         With_Resources    : in     Boolean;                
         Elected           : in out Tasks_Range;            
         No_Task           : in out Boolean                 ); 


   procedure Specific_Scheduler_Initialization (
         My_Scheduler   : in out Parametric_Scheduler;   
         Si             : in out Scheduling_Information; 
         Processor_Name : in     Unbounded_String;       
         My_Tasks       : in out Tasks_Set;              
         My_Resources   : in out Resources_Set           ); 


   procedure Bound_On_Processor_Utilization (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in     Tasks_Set;            
         Processor_Name : in     Unbounded_String;     
         Result         : in out Double;               
         Msg            : in out Unbounded_String      ); 


   procedure Compute_Response_Time (
         My_Scheduler   : in     Parametric_Scheduler; 
         My_Tasks       : in out Tasks_Set;            
         Processor_Name : in     Unbounded_String;     
         Msg            : in out Unbounded_String;     
         Response_Time  :    out Response_Table        ); 






private

   type Parametric_Scheduler is new Generic_Scheduler with 
      record 

         -- Scheduler behavior in string format
         --
         String_Behavior : Unbounded_String_Ptr;  

         -- Scheduler behavior in string format
         --
         Behavior_File_Name : Unbounded_String_Ptr;  

         -- Statements for each scheduling
         -- section (pointers given by the parser)
         --
         Root_Statement_Pointer : Section_Pointers;  

         -- Variables table for the parsed scheduler
         -- (given by the parser)
         --
         Variables_Table : Variables_Table_Type;  
      end record; 



end Scheduler.Parametrics ;
