------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

with Io_Tools;
use Io_Tools;

with Network;
use Network;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;

with Xml_Parsers;
use Xml_Parsers;
with Input_Sources.File;
use Input_Sources.File;
with Sax.Readers;
use Sax.Readers;



package body Systems is


   procedure Read_From_Xml_File (
         A_System  : in out System; 
         File_Name : in     String  ) is 
   begin
      Read_From_Xml_File(A_System, To_Unbounded_String(File_Name));
   end Read_From_Xml_File;


   procedure Write_To_Xml_File (
         A_System  : in     System; 
         File_Name : in     String  ) is 
   begin
      Write_To_Xml_File(A_System, To_Unbounded_String(File_Name));
   end Write_To_Xml_File;


   procedure Read_From_Xml_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ) is 

      Read       :          File_Input;  
      My_Reader  :          Xml_Parser;  
      Name_Start :          Natural;  
      S          : constant String     := To_String (File_Name);  


   begin

      --  Base file name should be used as the public Id
      --
      Name_Start := S'Last;
      while Name_Start >= S'First  and then S (Name_Start) /= '/' loop
         Name_Start := Name_Start - 1;
      end loop;
      Set_Public_Id (Read, S (Name_Start + 1 .. S'Last));
      Set_System_Id (Read, S);
      Open (S, Read);

      --  xmlns:* attributes will be reported in Start_Element
      --
      Set_Feature (My_Reader, Namespace_Prefixes_Feature, False);
      Set_Feature (My_Reader, Validation_Feature, True);

      Parse (My_Reader, Read);
      A_System:=Xml_Parsers.Parsed_System;
      Close (Read);


   exception
      when Xml_Fatal_Error =>
         Close(Read);
         raise Xml_Read_Error;


   end Read_From_Xml_File;



   procedure Read_From_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ) is 

   begin

      Processor_Set.Read_From_File(A_System.Processors, File_Name &
         ".proc" );


      begin
         Read_From_File(A_System.Tasks, File_Name & ".tsk" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            Put_Line("Warning : " & To_String(File_Name) & ".tsk"  &
               " : Ada.Io_Exceptions.Name_Error");
         when Ada.Io_Exceptions.Status_Error =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Status_Error");
         when Ada.Io_Exceptions.Mode_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Mode_Error");
         when Ada.Io_Exceptions.Use_Error     =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Use_Error");
         when Ada.Io_Exceptions.Device_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Device_Error");
         when Ada.Io_Exceptions.End_Error    =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.End_Error");
         when Ada.Io_Exceptions.Data_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Data_Error");
         when Ada.Io_Exceptions.Layout_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".tsk" &
               " : Ada.Io_Exceptions.Layout_Error");

            Processor_Set.Reset(A_System.Processors);

      end;

      begin
         Read_From_File(A_System.Messages, File_Name & ".msg" );

      exception
         when Ada.Io_Exceptions.Name_Error =>
            Put_Line("Warning : " & To_String(File_Name) & ".msg"  &
               " : Ada.Io_Exceptions.Name_Error");
         when Ada.Io_Exceptions.Status_Error =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Status_Error");
         when Ada.Io_Exceptions.Mode_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Mode_Error");
         when Ada.Io_Exceptions.Use_Error     =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Use_Error");
         when Ada.Io_Exceptions.Device_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Device_Error");
         when Ada.Io_Exceptions.End_Error    =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.End_Error");
         when Ada.Io_Exceptions.Data_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Data_Error");
         when Ada.Io_Exceptions.Layout_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".msg" &
               " : Ada.Io_Exceptions.Layout_Error");

      end;

      begin
         Read_From_File(A_System.Buffers, File_Name & ".buff" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            Put_Line("Warning : " & To_String(File_Name) & ".buff"  &
               " : Ada.Io_Exceptions.Name_Error");
         when Ada.Io_Exceptions.Status_Error =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Status_Error");
         when Ada.Io_Exceptions.Mode_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Mode_Error");
         when Ada.Io_Exceptions.Use_Error     =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Use_Error");
         when Ada.Io_Exceptions.Device_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Device_Error");
         when Ada.Io_Exceptions.End_Error    =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.End_Error");
         when Ada.Io_Exceptions.Data_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Data_Error");
         when Ada.Io_Exceptions.Layout_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".buff" &
               " : Ada.Io_Exceptions.Layout_Error");
      end;

      begin
         Read_From_File(A_System.Resources, File_Name & ".res" );
      exception
         when Ada.Io_Exceptions.Name_Error =>
            Put_Line("Warning : " & To_String(File_Name) & ".res"  &
               " : Ada.Io_Exceptions.Name_Error");
         when Ada.Io_Exceptions.Status_Error =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Status_Error");
         when Ada.Io_Exceptions.Mode_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Mode_Error");
         when Ada.Io_Exceptions.Use_Error     =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Use_Error");
         when Ada.Io_Exceptions.Device_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Device_Error");
         when Ada.Io_Exceptions.End_Error    =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.End_Error");
         when Ada.Io_Exceptions.Data_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Data_Error");
         when Ada.Io_Exceptions.Layout_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".res" &
               " : Ada.Io_Exceptions.Layout_Error");
      end;

      begin
         Read_From_File(A_System.Dependencies,
            A_System.Tasks,
            A_System.Messages,
            A_System.Buffers,
            File_Name & ".dep");
      exception
         when Ada.Io_Exceptions.Name_Error =>
            Put_Line("Warning : " & To_String(File_Name) & ".dep"  &
               " : Ada.Io_Exceptions.Name_Error");
         when Ada.Io_Exceptions.Status_Error =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Status_Error");
         when Ada.Io_Exceptions.Mode_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Mode_Error");
         when Ada.Io_Exceptions.Use_Error     =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Use_Error");
         when Ada.Io_Exceptions.Device_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Device_Error");
         when Ada.Io_Exceptions.End_Error    =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.End_Error");
         when Ada.Io_Exceptions.Data_Error   =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Data_Error");
         when Ada.Io_Exceptions.Layout_Error  =>
            Put_Line("Warning : " & To_String(File_Name)  & ".dep" &
               " : Ada.Io_Exceptions.Layout_Error");
      end;
   end Read_From_File;


   procedure Write_To_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ) is 

   begin


      Processor_Set.Write_To_File(A_System.Processors, File_Name &
         ".proc",
         Save_In_New_File );

      Write_To_File(A_System.Tasks, File_Name & ".tsk",
         Save_In_New_File );

      Write_To_File(A_System.Messages, File_Name & ".msg",
         Save_In_New_File );

      Write_To_File(A_System.Buffers, File_Name & ".buff",
         Save_In_New_File );

      Write_To_File(A_System.Resources, File_Name & ".res",
         Save_In_New_File );

      Write_To_File(A_System.Dependencies, File_Name & ".dep",
         Save_In_New_File );

   end Write_To_File;


   procedure Write_To_Xml_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ) is 


      Into : File_Type;  

   begin

      -- Open file and Write DTD/XML Headr
      --
      Create(Into,
         Mode => Out_File,
         Name => To_String(File_Name));

      Put_Line(Into, "<?xml version=""1.0"" standalone=""yes""?>  ");
      Put_Line(Into,
         "<?xml-stylesheet type=""text/xsl"" href=""cheddar_project.xsl""?>");
      Put_Line(Into, "<!DOCTYPE cheddar [  ");
      Put_Line(Into, "<!ELEMENT name (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT scheduler (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT parameter (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT offset (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT period (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT capacity (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT deadline (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT blocking_time (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT policy (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT priority (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT location (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT jitter (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT seed (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT parametric_file_name (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT state (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT protocol (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT size (#PCDATA) > ");
      Put_Line(Into, "<!ELEMENT buffer_user (#PCDATA) >  ");
      Put_Line(Into, "<!ELEMENT resource_user (#PCDATA) >  ");
      Put_Line(Into,
         "<!ELEMENT cheddar (processors, (tasks)?, ((buffers)?|(resources)?|(messages)?),(dependencies)?  )  >   ");
      Put_Line(Into, "<!ELEMENT processors (processor)+ >  ");
      Put_Line(Into, "<!ELEMENT processor (name|scheduler) >  ");
      Put_Line(Into, "<!ELEMENT tasks (task)+>  ");
      Put_Line(Into,
         "<!ELEMENT task (name|location|capacity|start_time|(period)?|(deadline)?|(parameters)?|(offsets)?|(jitter)?|(policy)?|(priority)?|(blocking_time)?|(seed)?|(parametric_file_name)?) >  ");
      Put_Line(Into, "<!ELEMENT offsets (offset)+ >  ");
      Put_Line(Into, "<!ELEMENT parameters (parameter)+ >  ");
      Put_Line(Into, "<!ELEMENT messages (message)+ >  ");
      Put_Line(Into,
         "<!ELEMENT message (name|size|(period)?|(deadline)?|(jitter)?) >  ");
      Put_Line(Into, "<!ELEMENT buffers (buffer)+ >  ");
      Put_Line(Into,
         "<!ELEMENT buffer (location|name|size|(buffer_used_by)?) >  ");
      Put_Line(Into, "<!ELEMENT buffer_used_by (buffer_user)+ >  ");
      Put_Line(Into, "<!ELEMENT resources (resource)+ >  ");
      Put_Line(Into,
         "<!ELEMENT resource (location|name|protocol|(state)?|(resource_used_by)?) >  ");
      Put_Line(Into, "<!ELEMENT resource_used_by (resource_user)+ >  ");
      Put_Line(Into, "<!ELEMENT dependencies (dependency)+ >  ");
      Put_Line(Into, "<!ELEMENT dependency (#PCDATA)  >  ");
      Put_Line(Into, "<!ATTLIST scheduler ");
      Put_Line(Into, "        quantum CDATA  ""0""  ");
      Put_Line(Into, "        is_preemptive CDATA  ""PREEMPTIVE""  ");
      Put_Line(Into, "        parametric_file_name CDATA """" >  ");
      Put_Line(Into, "<!ATTLIST task  ");
      Put_Line(Into, "        task_type CDATA  ""APERIODIC_TYPE""  ");
      Put_Line(Into, "        x CDATA ""0""  ");
      Put_Line(Into, "        y CDATA ""0"" > ");
      Put_Line(Into, "<!ATTLIST buffer  ");
      Put_Line(Into, "        x CDATA ""0"" ");
      Put_Line(Into, "        y CDATA ""0"" > ");
      Put_Line(Into, "<!ATTLIST message  ");
      Put_Line(Into, "        x CDATA ""0"" ");
      Put_Line(Into, "        y CDATA ""0"" > ");
      Put_Line(Into, "<!ATTLIST buffer_user ");
      Put_Line(Into,
         "        buffer_role CDATA  ""producer"" >  ");
      Put_Line(Into, "<!ATTLIST dependency ");
      Put_Line(Into, "        from_type CDATA ""task"" ");
      Put_Line(Into, "        to_type CDATA ""task"" > ");
      Put_Line(Into, "<!ATTLIST parameter " );
      Put_Line(Into,
         "        parameter_type CDATA  ""integer"" >  ");
      Put_Line(Into, "] >  ");
      New_Line(Into,4);


      Put_Line(Into, "<cheddar>");


      -- Call each export part of the system
      --
      Export_Xml(A_System.Processors, Into);
      Export_Xml(A_System.Tasks, Into);
      Export_Xml(A_System.Buffers, Into);
      Export_Xml(A_System.Messages, Into);
      Export_Xml(A_System.Resources, Into);
      Export_Xml(A_System.Dependencies, Into);


      Put_Line(Into, "</cheddar>");
      New_Line(Into);
      New_Line(Into);

      Close(Into);


   end Write_To_Xml_File;




   procedure Put (
         A_System : in     System ) is 
   begin
      -- Put(A_System.Network.all);


      Put_Line("***** LIST OF PROCESSORS *****");
      Put(A_System.Processors);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF TASKS *****");
      Put(A_System.Tasks);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF RESSOURCES *****");
      Put(A_System.Resources);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF BUFFERS *****");
      Put(A_System.Buffers);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF MESSAGES *****");
      Put(A_System.Messages);
      New_Line;
      New_Line;
      New_Line;
      Put_Line("***** LIST OF DEPENCENCIES *****");
      Put(A_System.Dependencies);

   end Put;



   procedure Initialize (
         A_System : in out System ) is 
   begin
      A_System.Network:= null;
      Reset(A_System.Tasks);
      Reset(A_System.Messages);
      Reset(A_System.Resources);
      Reset(A_System.Dependencies);
      Reset(A_System.Buffers);
      Reset(A_System.Processors);
   end Initialize;


   procedure Read_From_Xml_File (
         A_System  : in out System_Ptr; 
         File_Name : in     Char_Array  ) is 
   begin
      Read_From_Xml_File(A_System.all, To_Unbounded_String(To_Ada(
               File_Name)));
   end Read_From_Xml_File;


   procedure Write_To_Xml_File (
         A_System  : in     System_Ptr; 
         File_Name : in     Char_Array  ) is 
   begin
      Write_To_Xml_File(A_System.all, To_Unbounded_String(To_Ada(
               File_Name)));
   end Write_To_Xml_File;


   procedure Initialize (
         A_System : in out System_Ptr ) is 
   begin
      Initialize(A_System.all);
   end Initialize;


   procedure Allocate (
         A_System : in out System_Ptr ) is 
   begin
      A_System:= new System;
   end Allocate;


   procedure Put (
         A_System : in     System_Ptr ) is 
   begin
      Put(A_System.all);
   end Put;




end Systems;


