------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Network;
use Network;

with Interfaces.C;
use Interfaces.C;



package Systems is


   Xml_Read_Error : exception;  


   type System is tagged 
      record 
         Network      : Network_Ptr;  
         Tasks        : Tasks_Set;  
         Resources    : Resources_Set;  
         Messages     : Messages_Set;  
         Dependencies : Tasks_Dependencies;  
         Processors   : Processors_Set;  
         Buffers      : Buffers_Set;  
      end record; 

   type System_Ptr is access System; 




   procedure Initialize (
         A_System : in out System ); 


   procedure Initialize (
         A_System : in out System_Ptr ); 
   pragma Export(C, Initialize,
      "initialize_system");


   procedure Allocate (
         A_System : in out System_Ptr ); 
   pragma Export(C, Allocate,
      "allocate_system");


   procedure Put (
         A_System : in     System ); 


   procedure Put (
         A_System : in     System_Ptr ); 
   pragma Export(C, Put,
      "put_system");


   procedure Read_From_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ); 


   procedure Write_To_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ); 


   procedure Read_From_Xml_File (
         A_System  : in out System;          
         File_Name : in     Unbounded_String ); 


   procedure Write_To_Xml_File (
         A_System  : in     System;          
         File_Name : in     Unbounded_String ); 


   pragma Warnings (Off);

   procedure Read_From_Xml_File (
         A_System  : in out System; 
         File_Name : in     String  ); 


   procedure Write_To_Xml_File (
         A_System  : in     System; 
         File_Name : in     String  ); 


   procedure Read_From_Xml_File (
         A_System  : in out System_Ptr; 
         File_Name : in     Char_Array  ); 
   pragma Export(C, Read_From_Xml_File,
      "read_from_xml_file");


   procedure Write_To_Xml_File (
         A_System  : in     System_Ptr; 
         File_Name : in     Char_Array  ); 
   pragma Export(C, Write_To_Xml_File,
      "write_to_xml_file");

   pragma Warnings (On);



end Systems;








