------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Config;
use Config;

with Tables;



package Task_Offsets is


   -- An offset is defined by : 
   --		- a value (delay between task activation and
   --		  first runable instant)
   -- 		- a activation which describe the activation
   --		  number when the delay should be applicated.
   --		  Activation number begins at 1.
   --  
   type Offset_Type is 
      record 
         Value      : Integer;  
         Activation : Integer;  
      end record; 



   procedure Initialize (
         My_Offset :    out Offset_Type ); 

   procedure Put (
         My_Offset : Offset_Type ); 


   -- A table of offset. User can define several offset for each task
   --
   package Offsets_Table_Package is new Tables(Offset_Type,Max_Offsets,
      Put);
   use Offsets_Table_Package;

   subtype Offsets_Range is Offsets_Table_Package.Table_Range;
   subtype Offsets_Table is Offsets_Table_Package.Table;


end Task_Offsets;




