------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;



package body Tasks is


   --------------------------------------------------

   procedure Initialize (
         A_Task : in out Generic_Task ) is 
   begin
      Initialize(A_Task.Offset);
      Initialize(A_Task.Param);
   end Initialize;

   procedure Finalize (
         A_Task : in out Generic_Task ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Task : in out Generic_Task ) is 
   begin
      null;
   end Adjust;


   procedure Initialize (
         A_Task : in out Aperiodic_Task ) is 
   begin
      A_Task.Task_Type:=Aperiodic_Type;
   end Initialize;


   procedure Finalize (
         A_Task : in out Aperiodic_Task ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Task : in out Aperiodic_Task ) is 
   begin
      null;
   end Adjust;

   procedure Initialize (
         A_Task : in out Periodic_Task ) is 
   begin
      A_Task.Task_Type:=Periodic_Type;
   end Initialize;


   procedure Finalize (
         A_Task : in out Periodic_Task ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Task : in out Periodic_Task ) is 
   begin
      null;
   end Adjust;

   procedure Initialize (
         A_Task : in out Poisson_Task ) is 
   begin
      A_Task.Task_Type := Poisson_Type;
   end Initialize;


   procedure Finalize (
         A_Task : in out Poisson_Task ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Task : in out Poisson_Task ) is 
   begin
      null;
   end Adjust;

   procedure Initialize (
         A_Task : in out Parametric_Task ) is 
   begin
      A_Task.Task_Type:=Parametric_Type;
   end Initialize;


   procedure Finalize (
         A_Task : in out Parametric_Task ) is 
   begin
      null;
   end Finalize;


   procedure Adjust (
         A_Task : in out Parametric_Task ) is 
   begin
      null;
   end Adjust;


   --------------------------------------------------

   procedure Put (
         My_Task : in     Task_Ptr ) is 
   begin
      Put(My_Task.all);
   end Put;


   procedure Put (
         My_Task : in     Generic_Task ) is 
   begin
      Put ("Name : " & To_String(My_Task.Name));
      Put (" ; Processor : " & To_String(My_Task.Location));
      Put (" ; Type : " & My_Task.Task_Type'Img);
      Put (" ; Capacity : " &
         My_Task.Capacity'Img & " ; Start time : " & My_Task.Start_Time'
         Img & " ; ");
      Put ("Deadline : " & My_Task.Deadline'Img & " ; " );
      Put ("Priority : " & My_Task.Priority'Img & " ; " );
      Put ("Blocking time : " & My_Task.Blocking_Time'Img & " ; " );
      Put ("Scheduling policy : " & My_Task.Policy'Img & " ; " );
      Put(" ; x : " & My_Task.X'Img);
      Put(" ; y : " & My_Task.Y'Img);

      Put(My_Task.Offset);
      Put(My_Task.Param);

   end Put;



   procedure Put_Name (
         A_Task : in     Task_Ptr ) is 
   begin
      Put(To_String(A_Task.Name));
   end Put_Name;


   procedure Put (
         My_Task : in     Periodic_Task ) is 
   begin
      Put(Generic_Task(My_Task));
      Put ("Period : " & My_Task.Period'Img & " ; " );
      Put ("Jitter : " & My_Task.Jitter'Img);
   end Put;

   procedure Put (
         My_Task : in     Poisson_Task ) is 
   begin
      Put(Generic_Task(My_Task));
      Put ("Period : " & My_Task.Period'Img & " ; " );
      Put ("Jitter : " & My_Task.Jitter'Img & " ; " );
      Put ("Seed : " & My_Task.Seed'Img);
   end Put;

   procedure Put (
         My_Task : in     Parametric_Task ) is 
   begin
      Put(Generic_Task(My_Task));
      Put ("File name : " & To_String(My_Task.File_Name) & " ; " );
   end Put;

   function Copy (
         My_Task : in     Task_Ptr ) 
     return Task_Ptr is 

      New_Periodic_Task   : Periodic_Task_Ptr;  
      New_Aperiodic_Task  : Aperiodic_Task_Ptr;  
      New_Poisson_Task    : Poisson_Task_Ptr;  
      New_Parametric_Task : Parametric_Task_Ptr;  
      Ptr                 : Task_Ptr;  

   begin

      case My_Task.Task_Type is
         when Periodic_Type   =>
            New_Periodic_Task       := new Periodic_Task;
            New_Periodic_Task.all   := Periodic_Task_Ptr(My_Task).all;
            Ptr                     := Task_Ptr(New_Periodic_Task);
         when Aperiodic_Type  =>
            New_Aperiodic_Task      := new Aperiodic_Task;
            New_Aperiodic_Task.all  := Aperiodic_Task_Ptr(My_Task).all;
            Ptr                     := Task_Ptr(New_Aperiodic_Task);
         when Poisson_Type    =>
            New_Poisson_Task        := new Poisson_Task;
            New_Poisson_Task.all    := Poisson_Task_Ptr(My_Task).all;
            Ptr                     := Task_Ptr(New_Poisson_Task);
         when Parametric_Type =>
            New_Parametric_Task     := new Parametric_Task;
            New_Parametric_Task.all := Parametric_Task_Ptr(My_Task).all;
            Ptr                     := Task_Ptr(New_Parametric_Task);
         when others =>
            Ptr := null;
      end case;

      return Ptr;
   end Copy;

   --------------------------------------------------
   -- Laurent : modelise tasks occurences

   procedure Put (
         My_Task_Occurence : in     Task_Occurence ) is 
   begin
      -- Put (My_Task_Occurence.Parent_Task);
      Put ("Arrival_Date : " & My_Task_Occurence.Arrival_Date'Img & " ; " );
      Put ("Absolute_Deadline : " & My_Task_Occurence.Absolute_Deadline'
         Img);
      Put ("Residual_capacity : " & My_Task_Occurence.Residual_Capacity'
         Img);
      Put ("Response_Time : " & My_Task_Occurence.Response_Time'Img);
   end Put;

   procedure Initialize (
         My_Task_Occurence : in out Task_Occurence;        
         A_Task            : in     Periodic_Task_Ptr;     
         Arrival_Date      : in     Natural           := 0 ) is 
   begin
      My_Task_Occurence.Parent_Task       := Task_Ptr(A_Task) ;
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity ;
      My_Task_Occurence.Arrival_Date      := Arrival_Date;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.
         Arrival_Date + A_Task.Deadline ;

   end Initialize ;

   procedure Initialize (
         My_Task_Occurence : in out Task_Occurence;    
         A_Task            : in     Aperiodic_Task_Ptr ) is 
   begin
      My_Task_Occurence.Parent_Task       := Task_Ptr(A_Task) ;
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity ;
      My_Task_Occurence.Arrival_Date      := A_Task.Start_Time ;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.
         Arrival_Date + A_Task.Deadline ;
   end Initialize ;


   procedure Initialize (
         My_Task_Occurence : in out Task_Occurence;       
         A_Task            : in     Poisson_Task_Ptr;     
         Arrival_Date      : in     Natural          := 0 ) is 
   begin
      My_Task_Occurence.Parent_Task       := Task_Ptr(A_Task) ;
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity ;
      My_Task_Occurence.Arrival_Date      := Arrival_Date;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.
         Arrival_Date + A_Task.Deadline ;

   end Initialize ;


   procedure Initialize (
         My_Task_Occurence : in out Task_Occurence;     
         A_Task            : in     Parametric_Task_Ptr ) is 
   begin
      My_Task_Occurence.Parent_Task       := Task_Ptr(A_Task) ;
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity ;
      My_Task_Occurence.Arrival_Date      := A_Task.Start_Time ;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.
         Arrival_Date + A_Task.Deadline ;
   end Initialize ;

end Tasks;
