------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Config;

with Natural_Util;
use Natural_Util;

with Tasks;
use Tasks;
with Resources;
use Resources;

with Indexed_Tables;

package Time_Units is


   type Time_Unit_Event_Type is 
         (Running,           
          Activation,        
          Allocate_Resource, 
          Release_Resource); 

   type Time_Unit_Event 
         (Etype : Time_Unit_Event_Type) is 
      record 
case Etype is when Running => Running_Task : Task_Ptr; 
   when Activation =>
   Activated_Task : Task_Ptr;  
   when Allocate_Resource =>
   Allocated_Resource : Resource_Ptr;  
   when Release_Resource =>
   Released_Resource : Resource_Ptr;  
end case;
end record;

type Time_Unit_Event_Ptr is access Time_Unit_Event; 


procedure Put (
      E : in     Time_Unit_Event_Ptr ); 
procedure Put_Line (
      E : in     Time_Unit_Event_Ptr ); 
procedure Initialize (
      E : in out Time_Unit_Event_Ptr ); 



package A_Time_Unit is new Indexed_Tables(Natural, Time_Unit_Event_Ptr,
   Config.Max_Scheduling_Period, 0,
   Natural_Util.Put, Natural_Util.Initialize, Put);
use A_Time_Unit;

subtype Time_Unit_Table is Indexed_Table;
subtype Time_Unit_Range is Indexed_Table_Range;

end Time_Units;



