------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Text_Io;
use Ada.Text_Io;
with Sax.Readers;
use Sax.Readers;
with Sax.Exceptions;
use Sax.Exceptions;
with Sax.Locators;
use Sax.Locators;
with Sax.Attributes;
use Sax.Attributes;
with Unicode.Ces;
use Unicode.Ces;
with Unicode;
use Unicode;

with Config;
use Config;
with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;
with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Strings;
use Strings;
with Unbounded_Strings;
use Unbounded_Strings;
with Io_Tools;
use Io_Tools;

with Dependencies;
use Dependencies;
with Resources;
use Resources;
use Resources.Task_Lists;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffers;
use Buffers;
use Buffers.Buffer_Roles;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Scheduler;
use Scheduler;
with Scheduler.Dynamic_Priority.Edf;
use Scheduler.Dynamic_Priority.Edf;
with Scheduler.Dynamic_Priority.Llf;
use Scheduler.Dynamic_Priority.Llf;
with Scheduler.Fixed_Priority.Dm;
use Scheduler.Fixed_Priority.Dm;
with Scheduler.Fixed_Priority.Rm;
use Scheduler.Fixed_Priority.Rm;
with Scheduler.Fixed_Priority.Hpf;
use Scheduler.Fixed_Priority.Hpf;
with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Task_Dependencies;
use Task_Dependencies;


package body Xml_Parsers is


   -- Information retrieved from the xml parser
   --
   Name                 : Unbounded_String;  
   X,  
   Y                    : Natural;  
   Deadline             : Natural;  
   Start_Time           : Natural;  
   Blocking_Time        : Natural;  
   State                : Integer;  
   Period               : Natural;  
   Capacity             : Natural;  
   Location             : Unbounded_String;  
   Jitter               : Natural;  
   Policy               : Policies;  
   Task_Type            : Tasks_Type;  
   Parametric_File_Name : Unbounded_String;  
   Seed                 : Integer;  
   Quantum              : Natural;  
   Priority             : Integer;  
   Scheduler            : Scheduler_Ptr;  
   Size                 : Natural;  
   Protocol             : Unbounded_String;  
   Is_Preemptive        : Preemptive_Type;  
   Behavior_File_Name   : Unbounded_String;  
   Behavior             : Unbounded_String;  

   Roles  : Buffer_Roles_Table;  
   A_Role : Buffer_Role;  

   Offset   : Offsets_Table;  
   A_Offset : Offset_Type;  

   A_Task_List_Entry : Affected_Task_List;  
   A_Task_List       : Task_List;  

   Param   : Parameters_Table;  
   A_Param : Parameter_Ptr;  

   From_Type : Unbounded_String;  
   To_Type   : Unbounded_String;  

   Ok : Boolean;  



   -- Status information (store parameters of each tag)
   --
   Current_Parameter : Integer;  
   Max_Parameters    : Natural                                         := 
     10000;  
   Parameter_List    : array (1 .. Max_Parameters) of Unbounded_String; 
     
















   -- Initialize all data
   --
   procedure Initialize is 
   begin

      Current_Parameter:=1;

      X:=0;
      Y:=0;
      State:=0;
      Task_Type:=Aperiodic_Type;
      Parametric_File_Name:=To_Unbounded_String("");
      Seed := 0;
      Priority:=1;
      Name:=To_Unbounded_String("");
      Deadline:=0;
      Period:=0;
      Protocol:=To_Unbounded_String("NO_PROTOCOL");
      Location:=To_Unbounded_String("");
      Jitter:=0;
      Initialize(Offset);
      Initialize(Param);
      Quantum:=0;
      Scheduler:=null;
      Is_Preemptive:=Preemptive;
      Behavior_File_Name:=To_Unbounded_String("");
      Behavior:=To_Unbounded_String("");
      Size:=0;
      Initialize(Roles);
      A_Role.The_Role:=Consumer;
      Initialize(A_Task_List_Entry);
   end Initialize;


   --------------------
   -- Start_Document --
   --------------------

   procedure Start_Document (
         Handler : in out Xml_Parser ) is 
   begin

      Initialize;
      Initialize(Parsed_System);


   end Start_Document;


   -------------------
   -- Start_Element --
   -------------------

   procedure Start_Element (
         Handler       : in out Xml_Parser;                            
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := ""; 
         Local_Name    :        Unicode.Ces.Byte_Sequence       := ""; 
         Qname         :        Unicode.Ces.Byte_Sequence       := ""; 
         Atts          :        Sax.Attributes.Attributes'Class        ) is 
   begin





      Current_Parameter:=1;



      -- Store attributes
      --
      if Get_Length (Atts)>0 then

         if Qname="task"
               then

            for J in 0 .. Get_Length (Atts) - 1 loop


               if Get_Qname (Atts, J)="task_type"
                     then
                  To_Task_Type(Get_Value (Atts, J),Task_Type,Ok);
                  if not Ok then
                     Put_Line("Warning : Error on data type. From " &
                        To_String (Handler.Locator.all));
                  end if;
               end if;

               if Get_Qname (Atts, J)="x"
                     then
                  To_Natural(Get_Value (Atts, J),X,Ok);
                  if not Ok then
                     Put_Line("Warning : Error on data type. From " &
                        To_String (Handler.Locator.all));
                  end if;
               end if;


               if Get_Qname (Atts, J)="y"
                     then
                  To_Natural(Get_Value (Atts, J),Y,Ok);
                  if not Ok then
                     Put_Line("Warning : Error on data type. From " &
                        To_String (Handler.Locator.all));
                  end if;
               end if;



            end loop;

         else

            if Qname="scheduler"
                  then

               for J in 0 .. Get_Length (Atts) - 1 loop


                  if Get_Qname (Atts, J)="quantum"
                        then
                     To_Natural(Get_Value (Atts, J),Quantum,Ok);
                     if not Ok then
                        Put_Line("Warning : Error on data type. From " &
                           To_String (Handler.Locator.all));
                     end if;
                  end if;


                  if Get_Qname (Atts, J)="is_preemptive"
                        then
                     To_Preemptive(Get_Value (Atts, J),Is_Preemptive,Ok);
                     if not Ok then
                        Put_Line("Warning : Error on data type. From " &
                           To_String (Handler.Locator.all));
                     end if;
                  end if;


                  if Get_Qname (Atts, J)="parametric_file_name"
                        then
                     Behavior_File_Name:=To_Unbounded_String(Get_Value (
                           Atts,
                           J));
                  end if;


               end loop;

            else

               if Qname="buffer"
                     then

                  for J in 0 .. Get_Length (Atts) - 1 loop


                     if Get_Qname (Atts, J)="x"
                           then
                        To_Natural(Get_Value (Atts, J),X,Ok);
                        if not Ok then
                           Put_Line(
                              "Warning : Error on data type. From " &
                              To_String (Handler.Locator.all));
                        end if;
                     end if;


                     if Get_Qname (Atts, J)="y"
                           then
                        To_Natural(Get_Value (Atts, J),Y,Ok);
                        if not Ok then
                           Put_Line(
                              "Warning : Error on data type. From " &
                              To_String (Handler.Locator.all));
                        end if;
                     end if;


                  end loop;

               else
                  if Qname="message"
                        then

                     for J in 0 .. Get_Length (Atts) - 1 loop


                        if Get_Qname (Atts, J)="x"
                              then
                           To_Natural(Get_Value (Atts, J),X,Ok);
                           if not Ok then
                              Put_Line(
                                 "Warning : Error on data type. From " &
                                 To_String (Handler.Locator.all));
                           end if;
                        end if;


                        if Get_Qname (Atts, J)="y"
                              then
                           To_Natural(Get_Value (Atts, J),Y,Ok);
                           if not Ok then
                              Put_Line(
                                 "Warning : Error on data type. From " &
                                 To_String (Handler.Locator.all));
                           end if;
                        end if;


                     end loop;

                  else
                     if Qname="buffer_user"
                           then

                        for J in 0 .. Get_Length (Atts) - 1 loop

                           if Get_Qname (Atts, J)="buffer_role"
                                 then
                              if (Get_Value (Atts, J)/="consumer")
                                    and (Get_Value (Atts, J)/=
                                    "producer")
                                    then
                                 Put_Line(
                                    "Warning : Error on data type. From " &
                                    To_String (
                                       Handler.Locator.all));
                              end if;
                              if Get_Value (Atts, J)="consumer"
                                    then
                                 A_Role.The_Role:=Consumer;
                              else
                                 A_Role.The_Role:=Producer;
                              end if;
                           end if;

                        end loop;

                     else
                        if Qname="parameter"
                              then

                           for J in 0 .. Get_Length (Atts) - 1 loop

                              if Get_Qname (Atts, J)="parameter_type"
                                    then

                                 if (Get_Value (Atts, J)/="integer")
                                       and (Get_Value (Atts, J)/=
                                       "double")
                                       and (Get_Value (Atts, J)/=
                                       "string")
                                       and (Get_Value (Atts, J)/=
                                       "boolean")
                                       then
                                    Put_Line(
                                       "Warning : Error on data type. From " &
                                       To_String (
                                          Handler.Locator.all));
                                 end if;

                                 if Get_Value (Atts, J)="integer"
                                       then
                                    A_Param:= new Parameter(
                                       Integer_Parameter);
                                 end if;
                                 if Get_Value (Atts, J)="boolean"
                                       then
                                    A_Param:= new Parameter(
                                       Boolean_Parameter);
                                 end if;
                                 if Get_Value (Atts, J)="string"
                                       then
                                    A_Param:= new Parameter(
                                       String_Parameter);
                                 end if;
                                 if Get_Value (Atts, J)="double"
                                       then
                                    A_Param:= new Parameter(
                                       Double_Parameter);
                                 end if;
                              end if;

                           end loop;

                        else
                           if Qname="dependency"
                                 then

                              for J in 0 .. Get_Length (Atts) - 1 loop


                                 if Get_Qname (Atts, J)="from_type"
                                       then
                                    if (Get_Value (Atts, J)/="buffer")
                                          and (Get_Value (Atts, J)/=
                                          "message")
                                          and (Get_Value (Atts, J)/=
                                          "task")
                                          then
                                       Put_Line(
                                          "Warning : Error on data type. From " &
                                          To_String (
                                             Handler.Locator.all));
                                    end if;
                                    From_Type:=To_Unbounded_String(
                                       Get_Value (Atts, J));
                                 end if;


                                 if Get_Qname (Atts, J)="to_type"
                                       then
                                    if (Get_Value (Atts, J)/="buffer")
                                          and (Get_Value (Atts, J)/=
                                          "message")
                                          and (Get_Value (Atts, J)/=
                                          "task")
                                          then
                                       Put_Line(
                                          "Warning : Error on data type. From " &
                                          To_String (
                                             Handler.Locator.all));
                                    end if;
                                    To_Type:=To_Unbounded_String(
                                       Get_Value (Atts, J));
                                 end if;

                              end loop;

                           end if;
                        end if;
                     end if;
                  end if;
               end if;
            end if;
         end if;
      end if;



   end Start_Element;

   -----------------
   -- End_Element --
   -----------------

   procedure End_Element (
         Handler       : in out Xml_Parser;                      
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := ""; 
         Local_Name    :        Unicode.Ces.Byte_Sequence := ""; 
         Qname         :        Unicode.Ces.Byte_Sequence := ""  ) is 


      Buff_Iterator : Buffers_Iterator;  
      Mess_Iterator : Messages_Iterator;  
      A_Buff        : Buffer_Ptr;  
      A_Mess        : Message_Ptr;  
      A_Task        : Task_Ptr;  

   begin


      -----------------------------------
      -- Element with one parameter
      -----------------------------------


      if Qname="name"
            then
         Name:=Parameter_List(1);
      end if;

      if Qname="location"
            then
         Location:=Parameter_List(1);
      end if;

      if Qname="scheduler"
            then

         if (Parameter_List(1) = To_Unbounded_String(
                  "EARLIEST_DEADLINE_FIRST"))
               then
            Scheduler:= new Edf_Scheduler;
         else
            if (Parameter_List(1) = To_Unbounded_String(
                     "LEAST_LAXITY_FIRST"))
                  then
               Scheduler:= new Llf_Scheduler;
            else
               if (Parameter_List(1) = To_Unbounded_String(
                        "RATE_MONOTONIC"))
                     then
                  Scheduler:= new Rm_Scheduler;
               else
                  if (Parameter_List(1) = To_Unbounded_String(
                           "DEADLINE_MONOTONIC") )
                        then
                     Scheduler:= new Dm_Scheduler;
                  else
                     if (Parameter_List(1) = To_Unbounded_String(
                              "HIGHIEST_PRIORITY_FIRST"))
                           then
                        Scheduler:= new Hpf_Scheduler;
                     else
                        if (Parameter_List(1) = To_Unbounded_String(
                                 "PARAMETRIC"))
                              then
                           Scheduler:= new Parametric_Scheduler;

                           Set_Behavior_File_Name(Parametric_Scheduler(
                                 Scheduler.all),
                              Behavior_File_Name);
                           begin
                              Behavior:=Read_Sequential_File(
                                 Behavior_File_Name);
                              Set_String_Behavior (
                                 Parametric_Scheduler(Scheduler.all),
                                 Behavior);

                           exception
                              when others =>
                                 Put_Line(
                                    "Warning : can not read parametric file :  "
                                    & To_String(Behavior_File_Name) &
                                    "; from " &
                                    To_String (
                                       Handler.Locator.all));
                           end ;

                        end if;
                     end if;
                  end if;
               end if;
            end if;
         end if;

         Set_Quantum(Scheduler.all, Quantum);
         Set_Preemptive(Scheduler.all, Is_Preemptive);
      end if;

      if Qname="parametric_file_name"
            then
         Parametric_File_Name:=Parameter_List(1);
      end if;

      if Qname="period"
            then
         To_Natural(Parameter_List(1),Period,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="size"
            then
         To_Natural(Parameter_List(1),Size,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="deadline"
            then
         To_Natural(Parameter_List(1),Deadline,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="state"
            then
         To_Integer(Parameter_List(1),State,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="seed"
            then
         To_Integer(Parameter_List(1),Seed,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="protocol"
            then
         Protocol:=Parameter_List(1);
      end if;

      if Qname="policy"
            then
         To_Policy(Parameter_List(1),Policy,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="jitter"
            then
         To_Natural(Parameter_List(1),Jitter,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="capacity"
            then
         To_Natural(Parameter_List(1),Capacity,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="priority"
            then
         To_Integer(Parameter_List(1),Priority,Ok);
         if not Ok then
            Priority:=1;
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
         if Priority<Integer(Priority_Range'First)  or Priority>Integer(
               Priority_Range'Last) then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="start_time"
            then
         To_Natural(Parameter_List(1),Start_Time,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      if Qname="blocking_time"
            then
         To_Natural(Parameter_List(1),Blocking_Time,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
      end if;

      -----------------------------------
      -- Element with several parameters
      -----------------------------------



      if Qname="resource_user"
            then
         To_Natural(Parameter_List(2),A_Task_List_Entry.Task_Begin,Ok);
         if not Ok
               then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
         To_Natural(Parameter_List(3),A_Task_List_Entry.Task_End,Ok);
         if not Ok
               then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
         Add(A_Task_List,Parameter_List(1),A_Task_List_Entry);
      end if;


      if Qname="buffer_user"
            then
         To_Natural(Parameter_List(2),A_Role.Size,Ok);
         if not Ok then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;

         Add(Roles,Parameter_List(1),A_Role);
      end if;

      if Qname="offset"
            then
         To_Natural(Parameter_List(1),A_Offset.Activation,Ok);
         if not Ok
               then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
         To_Natural(Parameter_List(2),A_Offset.Value,Ok);
         if not Ok
               then
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.all));
         end if;
         Add(Offset, A_Offset);
      end if;



      if Qname="parameter"
            then
         A_Param.Name:=Parameter_List(2);
         if A_Param.Ptype=String_Parameter
               then
            A_Param.String_Value:=Parameter_List(1);
         end if;
         if A_Param.Ptype=Boolean_Parameter
               then
            To_Boolean(Parameter_List(1),A_Param.Boolean_Value,Ok);
            if not Ok
                  then
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.all));
            end if;
         end if;
         if A_Param.Ptype=Integer_Parameter
               then
            To_Integer(Parameter_List(1),A_Param.Integer_Value,Ok);
            if not Ok
                  then
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.all));
            end if;
         end if;
         if A_Param.Ptype=Double_Parameter
               then
            To_Double(Parameter_List(1),A_Param.Double_Value,Ok);
            if not Ok
                  then
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.all));
            end if;
         end if;
         Add(Param, A_Param);
      end if;


      ----------------------------------------
      -- End of Element is the end of an objet
      ----------------------------------------

      if Qname="processor"
            then
         Add_Processor(Parsed_System.Processors, Name, Scheduler);
         Initialize;
      end if;

      if Qname="buffer"
            then
         Add_Buffer(Parsed_System.Buffers, A_Buff, Name, Size, Location,
            Roles);
         A_Buff.X:=X;
         A_Buff.Y:=Y;
         Initialize;
      end if;

      if Qname="message"
            then
         Add_Message(Parsed_System.Messages, A_Mess, Name, Size, Period,
            Deadline,
            Jitter);
         A_Mess.X:=X;
         A_Mess.Y:=Y;
         Initialize;
      end if;

      if Qname="resource"
            then
         Add_Resource(Parsed_System.Resources, Name, State, Location,
            Protocol, A_Task_List);
         Initialize;
      end if;


      -- Dont forget "parametric_file_name"
      --
      if Qname="task"
            then
         Add_Task(Parsed_System.Tasks, A_Task, Name, Location, Task_Type,
            Start_Time, Capacity, Period, Deadline, Jitter,
            Blocking_Time, Priority_Range(Priority), Policy,
            Offset, Param, Parametric_File_Name, Seed);
         A_Task.X:=X;
         A_Task.Y:=Y;
         Initialize;
      end if;



      if Qname="dependency"
            then
         if (From_Type=To_Unbounded_String("task"))
               and  (To_Type=To_Unbounded_String("task"))
               then
            Add_Tasks_Dependencies(Parsed_System.Dependencies,
               Parsed_System.Tasks, Parameter_List(1), Parameter_List(2));
         else
            if (From_Type=To_Unbounded_String("buffer"))
                  or  (To_Type=To_Unbounded_String("buffer"))
                  then

               Reset_Iterator(Buff_Iterator);

               loop
                  Current_Element(Parsed_System.Buffers, A_Buff ,
                     Buff_Iterator);

                  if A_Buff.Name = Parameter_List(1)
                        then
                     Add_Tasks_Dependencies(Parsed_System.Dependencies,
                        Parsed_System.Tasks, Parameter_List(2),
                        Dependency_Ptr(
                           A_Buff), To_Task);
                     exit;
                  end if;

                  if A_Buff.Name = Parameter_List(2)
                        then
                     Add_Tasks_Dependencies(Parsed_System.Dependencies,
                        Parsed_System.Tasks, Parameter_List(1),
                        Dependency_Ptr(
                           A_Buff), To_Dependency);
                     exit;
                  end if;

                  if Is_Last_Element(Parsed_System.Buffers, Buff_Iterator) =
                        True
                        then
                     raise Buffer_Set.Buffer_Not_Found;
                  end if;

                  Next_Element(Parsed_System.Buffers, Buff_Iterator);
               end loop;

            else
               if (From_Type=To_Unbounded_String("message"))
                     or  (To_Type=To_Unbounded_String("message"))
                     then

                  Reset_Iterator(Mess_Iterator);

                  loop
                     Current_Element(Parsed_System.Messages, A_Mess ,
                        Mess_Iterator);

                     if A_Mess.Name = Parameter_List(1)
                           then
                        Add_Tasks_Dependencies(Parsed_System.Dependencies,
                           Parsed_System.Tasks, Parameter_List(2),
                           Dependency_Ptr(
                              A_Mess), To_Task);
                        exit;
                     end if;


                     if A_Mess.Name = Parameter_List(2)
                           then
                        Add_Tasks_Dependencies(Parsed_System.Dependencies,
                           Parsed_System.Tasks, Parameter_List(1),
                           Dependency_Ptr(
                              A_Mess), To_Dependency);
                        exit;
                     end if;

                     if Is_Last_Element(Parsed_System.Messages,
                           Mess_Iterator) = True
                           then
                        raise Message_Set.Message_Not_Found;
                     end if;

                     Next_Element(Parsed_System.Messages, Mess_Iterator);
                  end loop;


               end if;
            end if;
         end if;
         Initialize;
      end if;

   end End_Element;

   ----------------
   -- Characters --
   ----------------

   procedure Characters (
         Handler : in out Xml_Parser;               
         Ch      :        Unicode.Ces.Byte_Sequence ) is 



   begin

      Parameter_List(Current_Parameter):=To_Unbounded_String(Ch);
      Current_Parameter:=Current_Parameter+1;

   end Characters;




   --------------------------
   -- Set_Document_Locator --
   --------------------------

   procedure Set_Document_Locator (
         Handler : in out Xml_Parser;                
         Loc     : access Sax.Locators.Locator'Class ) is 
   begin
      Handler.Locator := Locator_Access (Loc);
   end Set_Document_Locator;



   -------------
   -- Warning --
   -------------

   procedure Warning (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ) is 
   begin
      Put_Line ("Xml warning ("
         & Get_Message (Except) & ", at "
         & To_String (Get_Locator (Except)) & ')');
   end Warning;

   -----------
   -- Error --
   -----------

   procedure Error (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ) is 
   begin
      Put_Line ("Xml parser error ("
         & Get_Message (Except) & ", at "
         & To_String (Get_Locator (Except)) & ')');
   end Error;

   -----------------
   -- Fatal_Error --
   -----------------

   procedure Fatal_Error (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ) is 
   begin
      Put_Line ("Xml parser fatal error (" & Get_Message (Except) & ')');
      Fatal_Error (Reader (Handler), Except);
   end Fatal_Error;




end Xml_Parsers;
