------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Sax.Exceptions;
with Sax.Locators;
with Sax.Readers;
with Sax.Attributes;
with Unicode.Ces;


with Systems;
use Systems;


package Xml_Parsers is


   -- At the end of the parser execution, the project 
   -- is store in the objet below
   --
   Parsed_System : System;  


   -- This tagged type is the parser ...
   --
   type Xml_Parser is new Sax.Readers.Reader with private; 


   -- Override methods of the Reader to custom the treatement the 
   -- parser have to do
   --
   procedure Start_Document (
         Handler : in out Xml_Parser ); 
   procedure Start_Element (
         Handler       : in out Xml_Parser;                            
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := ""; 
         Local_Name    :        Unicode.Ces.Byte_Sequence       := ""; 
         Qname         :        Unicode.Ces.Byte_Sequence       := ""; 
         Atts          :        Sax.Attributes.Attributes'Class        ); 
   procedure End_Element (
         Handler       : in out Xml_Parser;                      
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := ""; 
         Local_Name    :        Unicode.Ces.Byte_Sequence := ""; 
         Qname         :        Unicode.Ces.Byte_Sequence := ""  ); 
   procedure Characters (
         Handler : in out Xml_Parser;               
         Ch      :        Unicode.Ces.Byte_Sequence ); 
   procedure Set_Document_Locator (
         Handler : in out Xml_Parser;                
         Loc     : access Sax.Locators.Locator'Class ); 


   procedure Error (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ); 
   procedure Fatal_Error (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ); 
   procedure Warning (
         Handler : in out Xml_Parser;                              
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class ); 



private

   type Xml_Parser is new Sax.Readers.Reader with 
      record 
         Locator : Sax.Locators.Locator_Access;  
      end record; 

end Xml_Parsers;
