------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Dependencies;
use Dependencies;
use Dependencies.Dep_Set;

with Task_Dependencies;
use Task_Dependencies;


with Text_Io;
use Text_Io;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;


procedure Acyclic is 


   Tasks1 : Tasks_Set;  
   Dep1   : Tasks_Dependencies;  

   Offset : Offsets_Table;  
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");  



begin

   Initialize(Offset);
   Reset(Dep1);

   Add_Task(Tasks1, To_Unbounded_String("T1"), Cpu , Aperiodic_Type, 0, 1,
      0, 5, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T2"), Cpu , Aperiodic_Type, 0, 2,
      0, 10, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T3"), Cpu , Aperiodic_Type, 0, 1,
      0, 8, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T4"), Cpu , Aperiodic_Type, 0, 2,
      0, 14, 0, 0, 1,
      Sched_Fifo, Offset);


   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"));


   Put(Dep1);
   New_Line;
   if Is_Acyclic(Dep1) then
      Put_Line("Dep1 is acyclic");
   else
      Put_Line("ERROR");
   end if;



   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T2"), To_Unbounded_String("T3"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T2"), To_Unbounded_String("T4"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T4"), To_Unbounded_String("T2"));


   Put(Dep1);
   New_Line;
   if not Is_Acyclic(Dep1) then
      Put_Line("Dep1 is cyclic");
   else
      Put_Line("ERROR");
   end if;


   Reset(Dep1);

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T2"), To_Unbounded_String("T3"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T3"), To_Unbounded_String("T2"));


   Put(Dep1);
   New_Line;
   if not Is_Acyclic(Dep1) then
      Put_Line("Dep1 is cyclic");
   else
      Put_Line("ERROR");
   end if;



   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T3"), To_Unbounded_String("T4"));



   Put(Dep1);
   New_Line;
   if not Is_Acyclic(Dep1) then
      Put_Line("Dep1 is cyclic");
   else
      Put_Line("ERROR");
   end if;


   Reset(Dep1);

   Put(Dep1);
   New_Line;
   if Is_Acyclic(Dep1) then
      Put_Line("Dep1 is acyclic");
   else
      Put_Line("ERROR");
   end if;



   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T2"), To_Unbounded_String("T4"));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T3"), To_Unbounded_String("T4"));


   Put(Dep1);
   New_Line;
   if Is_Acyclic(Dep1) then
      Put_Line("Dep1 is acyclic");
   else
      Put_Line("ERROR");
   end if;



   Reset(Dep1);

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T1"));

   Put(Dep1);
   New_Line;
   if not Is_Acyclic(Dep1) then
      Put_Line("Dep1 is cyclic");
   else
      Put_Line("ERROR");
   end if;


   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"));

   Put(Dep1);
   New_Line;
   if not Is_Acyclic(Dep1) then
      Put_Line("Dep1 is cyclic");
   else
      Put_Line("ERROR");
   end if;



end  Acyclic ;

