------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_Io;
use Text_Io;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Systems;
use Systems;

with Call_Framework;
use Call_Framework;
use Call_Framework.Framework_Response_Package;
use Call_Framework.Framework_Request_Package;

with Call_Buffer_Framework;
use Call_Buffer_Framework;
use Call_Buffer_Framework.Buffer_Result_Package;

with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;



procedure Comprehensive_Call is 

   Sys : System;  

   Response_List : Framework_Response_Table;  
   Request_List  : Framework_Request_Table;  
   A_Request     : Framework_Request;  
   A_Param       : Parameter_Ptr;  


begin


   -- Test on periodic tasks
   --
   Read_From_Xml_File(Sys,To_Unbounded_String("jitter.xml"));

   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=Scheduling_Feasibility_Basics;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=Scheduling_Simulation_Time_Line;
   A_Param := new Parameter(Integer_Parameter);
   A_Param.Name:=To_Unbounded_String("period");
   A_Param.Integer_Value:=100;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_offsets");
   A_Param.Boolean_Value:=True;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_precedencies");
   A_Param.Boolean_Value:=True;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_resources");
   A_Param.Boolean_Value:=True;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Integer_Parameter);
   A_Param.Name:=To_Unbounded_String("seed_value");
   A_Param.Integer_Value:=0;
   Add(A_Request.Param, A_Param);
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Request_List);
   Initialize(Response_List);
   Initialize(A_Request);
   A_Request.Statement:=Scheduling_Simulation_Basics;
   Add(Request_List, A_Request);
   Initialize(Response_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Scheduling_Set_Priorities_According_To_Deadline_Monotonic;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Request_List);
   Initialize(Response_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Scheduling_Set_Priorities_According_To_Rate_Monotonic;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Request_List);
   Initialize(Response_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Dependency_Compute_End_To_End_Response_Time_All_Steps;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Request_List);
   Initialize(Response_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Dependency_Set_End_To_End_Response_Time_All_Steps;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);



   -- Test on resources
   --
   Initialize(Sys);
   Read_From_Xml_File(Sys,To_Unbounded_String("test_resource.xml"));

   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Resource_Set_Blocking_Time;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);


   -- Test on buffers 
   --
   Initialize(Sys);
   Read_From_Xml_File(Sys,To_Unbounded_String("test_buffer.xml"));

   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Buffer_Bound;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);



   -- Test on random services 
   --
   Initialize(Sys);
   Read_From_Xml_File(Sys,To_Unbounded_String("test_random.xml"));

   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=Scheduling_Simulation_Time_Line;
   A_Param := new Parameter(Integer_Parameter);
   A_Param.Name:=To_Unbounded_String("period");
   A_Param.Integer_Value:=30;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_offsets");
   A_Param.Boolean_Value:=False;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_precedencies");
   A_Param.Boolean_Value:=False;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Boolean_Parameter);
   A_Param.Name:=To_Unbounded_String("schedule_with_resources");
   A_Param.Boolean_Value:=False;
   Add(A_Request.Param, A_Param);
   A_Param := new Parameter(Integer_Parameter);
   A_Param.Name:=To_Unbounded_String("seed_value");
   A_Param.Integer_Value:=0;
   Add(A_Request.Param, A_Param);
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);
   Initialize(Response_List);
   Initialize(Request_List);
   Initialize(A_Request);
   A_Request.Statement:=
      Random_Response_Time_Density;
   Add(Request_List, A_Request);
   Sequential_Framework_Request (
      Sys, Request_List, Response_List);
   Put(Response_List);






end Comprehensive_Call;
