------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Scheduler;
use Scheduler;
with Scheduler.Dynamic_Priority.Edf;
use Scheduler.Dynamic_Priority.Edf;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Text_Io;
use Text_Io;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Tasks;
use Tasks;

procedure Edf is 


   My_Offset    : Offsets_Table;  
   My_Scheduler : Edf_Scheduler;  
   A_Task       : Task_Ptr;  
   Tasks1,  
   Tasks2,  
   Tasks3       : Tasks_Set;  

   Period : Natural             := 0;  
   Used   : Double              := 0.0;  
   Sched1,  
   Sched2,  
   Sched3 : Scheduling_Sequence;  

   Cpu : Unbounded_String := To_Unbounded_String ("cpu");  

begin

   Initialize(My_Offset);

   Add_Task(Tasks1, A_Task, To_Unbounded_String("T1"), Cpu , Periodic_Type,
      0,
      5, 12, 0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks1, A_Task, To_Unbounded_String("T2"), Cpu , Periodic_Type,
      0,
      2, 6,  0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks1, A_Task, To_Unbounded_String("T3"), Cpu , Periodic_Type,
      0,
      5, 24, 0, 0, 0, 1, Sched_Fifo, My_Offset);

   Put(Tasks1);

   Used:=Processor_Utilization_Over_Period(Tasks1, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period:=Scheduling_Period(Tasks1, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence(My_Scheduler, Tasks1, Cpu, Sched1);
   Put(Sched1);



   New_Line;
   New_Line;

   Add_Task(Tasks2, A_Task, To_Unbounded_String("T1"), Cpu , Periodic_Type,
      0,
      4, 8,  0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks2, A_Task, To_Unbounded_String("T2"), Cpu , Periodic_Type,
      0,
      5, 10, 0, 0, 0, 1, Sched_Fifo, My_Offset);

   Put(Tasks2);

   Used:=Processor_Utilization_Over_Period(Tasks2, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period:=Scheduling_Period(Tasks2, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence(My_Scheduler, Tasks2, Cpu, Sched2);
   Put(Sched2);



   New_Line;
   New_Line;

   Add_Task(Tasks3, To_Unbounded_String("T1"), Cpu , Periodic_Type, 1,
      1, 8, 0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks3, To_Unbounded_String("T2"), Cpu , Periodic_Type, 3,
      1, 10, 0, 0, 0, 1, Sched_Fifo, My_Offset);

   Put(Tasks3);

   Used:=Processor_Utilization_Over_Period(Tasks3, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period:=Scheduling_Period(Tasks3, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence(My_Scheduler, Tasks3, Cpu, Sched3);
   Put(Sched3);




end Edf;
