------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Dependencies;
use Dependencies;
use Dependencies.Dep_Set;


with Task_Dependencies;
use Task_Dependencies;

with Messages;
use Messages;
with Message_Set;
use Message_Set;

with Text_Io;
use Text_Io;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;


procedure Get_Depend is 


   Tasks1,  
   Tasks2,  
   Tasks3 : Tasks_Set;  
   Dep1   : Tasks_Dependencies;  
   Msg1,  
   Msg2   : Message_Ptr;  
   Msg    : Messages_Set;  
   Res    : Depends_Set;  
   Offset : Offsets_Table;  
   Cpu    : Unbounded_String   := To_Unbounded_String ("cpu");  


begin

   Initialize(Offset);
   Reset(Dep1);

   Add_Task(Tasks1, To_Unbounded_String("T1"), Cpu , Periodic_Type, 0, 5,
      12, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T2"), Cpu , Periodic_Type, 0, 2,
      6,  0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T3"), Cpu , Periodic_Type, 0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("T4"), Cpu , Periodic_Type, 0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);

   Add_Message(Msg, Msg1, To_Unbounded_String("m1"), 1000, 20, 0, 0);
   Add_Message(Msg, Msg2, To_Unbounded_String("m2"), 1000, 10, 0, 0);

   Put(Tasks1);
   Put(Msg);


   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("T3"), To_Unbounded_String("T4"), Dependency_Ptr(
         Msg2));

   Put(Dep1);

   New_Line;

   if (Has_Predecessor(Dep1, To_Unbounded_String("T3")))
         then
      Put_Line("T3 has predecessors");
   end if;

   if (not Has_Predecessor(Dep1, To_Unbounded_String("T1")))
         then
      Put_Line("T1 has no predecessor");
   end if;

   if (Has_Successor(Dep1, To_Unbounded_String("T1")))
         then
      Put_Line("T1 has successors");
   end if;

   if (not Has_Successor(Dep1, To_Unbounded_String("T4")))
         then
      Put_Line("T4 has no successor");
   end if;

   New_Line;


   Tasks2:=Get_Successors_List(Dep1, To_Unbounded_String("T1"));
   Put_Line("T1 successors : ");
   Put(Tasks2);

   New_Line;

   Tasks3:=Get_Predecessors_List(Dep1, To_Unbounded_String("T4"));
   Put_Line("T4 predecessors : ");
   Put(Tasks3);


   Res:=Get_Dependencies(Dep1, To_Unbounded_String("T1"),
      To_Unbounded_String(
         "T3"));
   Put(Res);


end  Get_Depend;

