------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--  This program read a ".sc" file and run the corresponding scheduler on
--  a very simple task set
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Ada.Exceptions;
use Ada.Exceptions;

with Ada.Command_Line;
use Ada.Command_Line;


with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Scheduler;
use Scheduler;
with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Text_Io;
use Text_Io;

with Simple_Mmi;
use Simple_Mmi;
with Scheduling_Sequences;
use Scheduling_Sequences;
with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;


with Tasks;
use Tasks;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Io_Tools;
use Io_Tools;


procedure Parametric is 


   A_Scheduler      : Parametric_Scheduler;  
   Scheduler_String : Unbounded_String;  

   Dep1      : Tasks_Dependencies;  
   My_Offset : Offsets_Table;  
   A_Task    : Task_Ptr;  
   Tasks1    : Tasks_Set;  
   Sched1    : Scheduling_Sequence;  
   Cpu       : Unbounded_String    := To_Unbounded_String ("cpu");  


begin

   if Argument_Count/=1
         then
      Raise_Exception(Constraint_Error'Identity,
         "usage : parametric sc-file");
   end if;

   Initialize(My_Offset);

   Scheduler_String:=Read_Sequential_File(Argument(1));

   Set_String_Behavior (
      A_Scheduler, Scheduler_String );
   Set_Behavior_File_Name (
      A_Scheduler, Scheduler_String );


   Add_Task(Tasks1, A_Task, To_Unbounded_String("T1"), Cpu , Periodic_Type,
      0,
      5, 12, 0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks1, A_Task, To_Unbounded_String("T2"), Cpu , Periodic_Type,
      0,
      2, 6,  0, 0, 0, 1, Sched_Fifo, My_Offset);
   Add_Task(Tasks1, A_Task, To_Unbounded_String("T3"), Cpu , Periodic_Type,
      0,
      5, 24, 0, 0, 0, 1, Sched_Fifo, My_Offset);

   Put(Tasks1);

   Build_Scheduling_Sequence(A_Scheduler, Tasks1, Dep1, Cpu, Sched1);

   Draw_Scheduling(Tasks1, 24, Sched1);





end Parametric;



