------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Task_Set;
use Task_Set;
with Scheduler;
use Scheduler;
with Scheduler.Fixed_Priority.Rm;
use Scheduler.Fixed_Priority.Rm;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Text_Io;
use Text_Io;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Buffers;
use Buffers;
use Buffers.Buffer_Roles;
use Buffers.Buffer_Size_Package;

with Tasks;
use Tasks;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;



procedure Tb1 is 


   My_Buff      : Buffer_Ptr;  
   My_Scheduler : Rm_Scheduler;  
   Tasks1       : Tasks_Set;  
   Offset       : Offsets_Table;  
   Sched1       : Scheduling_Sequence;  
   My_Roles     : Buffer_Roles_Table;  
   My_Size      : Buffer_Size_Table;  
   Cpu          : Unbounded_String    := To_Unbounded_String ("cpu");  
   T1,  
   T2           : Task_Ptr;  
   Bound        : Natural;  


begin

   Initialize(Offset);
   Initialize(My_Roles);
   Initialize(My_Size);
   My_Buff:= new Buffer;

   Add_Task(Tasks1, T1, To_Unbounded_String("T1"), Cpu , Periodic_Type, 0,
      3, 10, 10, 0,
      0, 1, Sched_Fifo,
      Offset);
   Add_Task(Tasks1, T2, To_Unbounded_String("T2"), Cpu , Periodic_Type, 0,
      3,  10, 10, 0,
      0, 1, Sched_Fifo,
      Offset);

   Put(Tasks1);

   Build_Scheduling_Sequence(My_Scheduler, Tasks1, Cpu, Sched1,20);
   Put(Sched1);


   My_Buff.Roles.Entries(0).Item:=T1.Name;
   My_Buff.Roles.Entries(0).Data.Size:=1;
   My_Buff.Roles.Entries(0).Data.The_Role:=Producer;
   My_Buff.Roles.Entries(1).Item:=T2.Name;
   My_Buff.Roles.Entries(1).Data.The_Role:=Consumer;
   My_Buff.Roles.Entries(1).Data.Size:=1;
   My_Buff.Roles.Nb_Entries:=2;




   Get_Buffer_Size( Sched1, My_Buff, Tasks1, My_Size);
   Bound:=Get_Max_Size(My_Size);

   Put_Line ("max buffer size = " & Bound'Img);
   New_Line;
   for I in 0 .. Sched1.Scheduled.Nb_Entries-1 loop
      Put_Line("Time = " & My_Size.Entries(Buffer_Size_Range(I)).Time'Img &
         "/Size = " & My_Size.Entries(Buffer_Size_Range(I)).
         Size'Img);
   end loop;

end Tb1;

