------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;
with Unbounded_Strings;
use Unbounded_Strings;

with Scheduler;
use Scheduler;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Text_Io;
use Text_Io;

with Buffers;
use Buffers;
use Buffers.Buffer_Roles;
use Buffers.Buffer_Size_Package;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Systems;
use Systems;

with Processors;
use Processors;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Task_Dependencies;
use Task_Dependencies;

with Gnat.Command_Line;
use Gnat.Command_Line;

with  Gnat.Os_Lib;
use Gnat.Os_Lib;




-- use of the command :
-- test_delay_buffer
--
procedure Test_Delay_Buffer is 

   Sched           : Scheduling_Sequence;  
   My_Size         : Buffer_Size_Table;  
   Bound           : Natural;  
   Sys             : System;  
   A_Processor     : Processor_Ptr;  
   A_Buffer        : Buffer_Ptr;  
   My_Iterator     : Processors_Iterator;  
   My_Iterator_Buf : Buffers_Iterator;  
   File_Name       : Unbounded_String;  


   procedure Copyright is 
   begin
      Put_Line(
         "Test_Delay_Buffer version 1.0, Copyright (C) 2002, J. Legrand ");
      Put_Line(
         "Test_Delay_Buffer is developped by the EA 2215 Team, University of Brest");
      Put_Line("Test_Delay_Buffer comes with ABSOLUTELY NO WARRANTY; ");
      Put_Line(
         "This is free software, and you are welcome to redistribute it");
      Put_Line(
         "under certain conditions; See the GNU General Public License for details.");
   end Copyright;

   procedure Usage is 
   begin
      Put_Line (
         "test_delay_buffer allows you to display in a textual manner the maximum buffer size of a Cheddar project file.");

      Copyright;
      New_Line;
      Put_Line(
         "See Cheddar home page :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : test_delay_buffer project-name");
      New_Line;
      New_Line;
   end Usage;

begin
   Put_Line("debut du programme");

   -- Get arguments
   loop
      case Getopt ("u") is
         when Ascii.Nul =>
            exit;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;

   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;

   Put_Line("NOM DU FICHIER : " & To_String(File_Name));

   -- read the project created with cheddar if a name is specified
   if(File_Name = Empty_String)
         then
      Usage;
      Os_Exit(0);

   else
      Initialize(Sys);

      Read_From_Xml_File(Sys, File_Name);

      Put(Sys);

   end if;

   -- Put(Sys.Tasks);

   -- build scheduling sequence and check buffer size
   -- for each processor of the readed cheddar project
   Reset_Iterator(My_Iterator);
   loop
      Initialize(My_Size);
      Current_Element(Sys.Processors, A_Processor, My_Iterator);

      -- build scheduling sequence
      Build_Scheduling_Sequence(A_Processor.Scheduler.all,
         Sys.Tasks, A_Processor.Name,
         Sched,512); -- changer le 512 !!!


      Put_Line("SCHEDULING SEQUENCE : ");
      Put(Sched);




      Put_Line("BUFFER SIZE NUMBER SEQUENCE : ");
      Reset_Iterator(My_Iterator_Buf);
      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator_Buf);

         -- name of the buffer
         Put_Line("");
         Put_Line("Buffer : " & A_Buffer.Name);

         -- compute the maximum occupation of for the buffer
         Get_Buffer_Size( Sched, A_Buffer, Sys.Tasks, My_Size);
         Bound:=Get_Max_Size(My_Size);

         -- write it
         Put_Line ("max buffer size = " & Bound'Img);
         New_Line;
         for I in 0 .. Sched.Scheduled.Nb_Entries-1 loop
            Put_Line("Time = " & My_Size.Entries(Buffer_Size_Range(I)).
               Time'Img & "/Size = " &
               My_Size.Entries(Buffer_Size_Range(I)).
               Size'Img);
         end loop;

         -- Next buffer
         exit when Is_Last_Element(Sys.Buffers, My_Iterator_Buf) = True;
         Next_Element(Sys.Buffers, My_Iterator_Buf);
      end loop;

      -- next processor
      exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
      Next_Element(Sys.Processors, My_Iterator);
   end loop;
end Test_Delay_Buffer;

