------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Queueing_System;
use Queueing_System;

with Queueing_System.Md1;
use Queueing_System.Md1;

with Text_Io;
use Text_Io;


with Ada.Numerics.Float_Random;
use Ada.Numerics.Float_Random;



procedure Test_Md1 is 

   My_Md1                : Md1_Queueing_System;  
   Arrival_Rate          : Arrival_Rate_Table;  --Double; 
   Departure_Rate        : Double;  
   Rau                   : Double;  
   Max_Interarrival_Time : Natural;  
   Min_Interarrival_Time : Natural;  
   Nb_Inter_Time         : Natural;  
   Max_Buffer_Size       : Natural;  
   Avg_Buffer_Size       : Double;  
   Var_Buffer_Size       : Double;  
   Buffer_Max_Wait_Time  : Double;  
   Buffer_Avg_Wait_Time  : Double;  
   Simu_Number           : Natural;  
   Project_Number        : Natural;  
   Seed_Value            : Natural;  
   Tmp_Utilization       : Double;  
   Tmp_Service_Time      : Natural;  
   Dis_Buffer_Size       : Boolean;  
   Seed_Project          : Generator;  


begin

   -- init
   Nb_Inter_Time := 1000000;
   Simu_Number := 10;
   Max_Interarrival_Time := 100000;
   -- big value to avoid 0 generation for interaarival time
   Min_Interarrival_Time := 10000;
   Project_Number := 100;
   Seed_Value := 0;
   Arrival_Rate.Nb_Entries := 1;
   Departure_Rate := 0.0;
   Dis_Buffer_Size := False;
   Reset(Seed_Project,0);

   -- generate from utilization 0,0..01 to 0,9..9 different flows
   for I in 1 .. Simu_Number-1 loop
      Tmp_Utilization := Double(I) / Double(Simu_Number);

      -- generate Project_Number project for one utilization
      for J in 1 .. Project_Number loop
         -- project generation
         Random_Qs_Project (Tmp_Utilization,
            Arrival_Rate,
            Departure_Rate,
            Seed_Project,
            10000);
         Put_Line(" ");
         Put_Line("# PROJECT ");
         Rau := 0.0; -- Tmp_Utilization; -- Arrival_Rate/Departure_Rate;
         for L in 1 .. Arrival_Rate.Nb_Entries loop
            Put_Line("# la"& L'Img & " = 1/" & Natural(1.0/Arrival_Rate.
                  Entries(L).Data)'Img);
            Rau := Rau + Arrival_Rate.Entries(L).Data;
         end loop;
         Rau := Rau / Departure_Rate;
         Put_Line("# mu = 1/" & Natural(1.0/Departure_Rate)'Img);
         Put_Line("# rau = " & Rau'Img );
         Put_Line("#");

         -- MD1 simulation
         Tmp_Service_Time := Natural(1.0/Departure_Rate);

         Simulate(My_Md1,
            Arrival_Rate,
            --                  Arrival_Rate.Nb_entries,
            Tmp_Service_Time,
            Nb_Inter_Time,
            Max_Buffer_Size,
            Avg_Buffer_Size,
            Var_Buffer_Size,
            Buffer_Max_Wait_Time,
            Buffer_Avg_Wait_Time,
            Dis_Buffer_Size,
            Seed_Value);

         -- gnu plot output
         Put_Line("# * Simulation results : ");
         Put_Line("MW" & Rau'Img & Buffer_Max_Wait_Time'Img);
         Put_Line("MO" & Rau'Img & Max_Buffer_Size'Img);
         Put_Line("AW" & Rau'Img & Buffer_Avg_Wait_Time'Img);
         Put_Line("AO" & Rau'Img & Avg_Buffer_Size'Img);

         Theoric_Results(My_Md1,
            Arrival_Rate,
            Tmp_Service_Time,
            Rau);

      end loop;
   end loop;

end Test_Md1;
