------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Io_Tools;
use Io_Tools;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Dependencies;
use Dependencies;
with Messages;
use Messages;
with Message_Set;
use Message_Set;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Systems;
use Systems;


procedure Write_Depend is 


   Sys    : System;  
   Msg1,  
   Msg2   : Message_Ptr;  
   Offset : Offsets_Table;  
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");  

begin


   Initialize(Offset);
   Initialize(Sys);

   Add_Task(Sys.Tasks, To_Unbounded_String("T1"), Cpu, Periodic_Type, 0, 5,
      12, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, To_Unbounded_String("T2"), Cpu, Periodic_Type, 0, 2,
      6,  0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, To_Unbounded_String("T3"), Cpu, Periodic_Type, 0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, To_Unbounded_String("T4"), Cpu, Periodic_Type, 0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);

   Add_Message(Sys.Messages, Msg1, To_Unbounded_String("m1"), 1000, 20, 0,
      0);
   Add_Message(Sys.Messages, Msg2, To_Unbounded_String("m2"), 1000, 10, 0,
      0);

   Put(Sys);


   Add_Tasks_Dependencies(Sys.Dependencies, Sys.Tasks,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Sys.Dependencies, Sys.Tasks,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"), Dependency_Ptr(
         Msg1));

   Add_Tasks_Dependencies(Sys.Dependencies, Sys.Tasks,
      To_Unbounded_String("T3"), To_Unbounded_String("T4"), Dependency_Ptr(
         Msg2));

   Put(Sys);

   Write_To_Xml_File(Sys, To_Unbounded_String("depend.cheddar"));

   Delete_Tasks_Dependencies(Sys.Dependencies,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"));

   Put(Sys);


end  Write_Depend;

