------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Tasks;
use Tasks;

with Buffers;
use Buffers;
use Buffers.Buffer_Roles;


with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;
with Scheduler.Dynamic_Priority.Edf;
use Scheduler.Dynamic_Priority.Edf;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Messages;
use Messages;
with Message_Set;
use Message_Set;

with Dependencies;
use Dependencies;
with Task_Dependencies;
use Task_Dependencies;

with Network;
use Network;
with Network.Uni_Processor;
use Network.Uni_Processor;

with Systems;
use Systems;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;


procedure Write_Sys is 


   Sys : System;  
   Edf : Scheduler_Ptr;  
   Cpu : Unbounded_String := To_Unbounded_String ("mon_processeur.univ-brest.fr");
      

   T1       : Task_Ptr;  
   T2       : Task_Ptr;  
   Msg1     : Message_Ptr;  
   Offset   : Offsets_Table;  
   My_Roles : Buffer_Roles_Table;  


begin

   Initialize(My_Roles);
   Initialize(Offset);

   Initialize(Sys);

   Sys.Network:= new Monoprocessor_Network;


   Edf:= new Edf_Scheduler;

   Add_Processor(Sys.Processors,Cpu,Edf);


   Add_Task(Sys.Tasks, T1, To_Unbounded_String("T1"), Cpu,
      Periodic_Type, 0, 1, 12, 0, 0, 0, 1, Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, T2, To_Unbounded_String("T2"), Cpu,
      Periodic_Type, 1, 3, 6, 0, 0, 0, 1, Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, To_Unbounded_String("T3"), Cpu,
      Periodic_Type, 0, 1, 3, 0, 0, 0, 1, Sched_Fifo, Offset);


   My_Roles.Entries(0).Item:=T1.Name;
   My_Roles.Entries(0).Data.Size:=1;
   My_Roles.Entries(0).Data.The_Role:=Producer;
   My_Roles.Entries(1).Item:=T2.Name;
   My_Roles.Entries(1).Data.The_Role:=Consumer;
   My_Roles.Entries(1).Data.Size:=1;

   My_Roles.Nb_Entries:=2;


   Add_Buffer(Sys.Buffers, To_Unbounded_String("B1"), 1, Cpu,
      My_Roles);


   Add_Message(Sys.Messages, Msg1, To_Unbounded_String("m1"), 1000, 20, 0,
      0);

   Add_Tasks_Dependencies(Sys.Dependencies, Sys.Tasks,
      To_Unbounded_String("T1"), To_Unbounded_String("T2"), Dependency_Ptr(
         Msg1));
   Add_Tasks_Dependencies(Sys.Dependencies, Sys.Tasks,
      To_Unbounded_String("T1"), To_Unbounded_String("T3"), Dependency_Ptr(
         Msg1));


   Put(Sys);


   Write_To_Xml_File(Sys, To_Unbounded_String("sys.cheddar"));


end  Write_Sys;

