------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Main;

with Text_Io;
use Text_Io;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Scrolling_Draw_Buffer_Pkg;
use Scrolling_Draw_Buffer_Pkg;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Scheduler;
use Scheduler;
with Scheduler.Fixed_Priority.Rm;
use Scheduler.Fixed_Priority.Rm;

with Scheduling_Sequences;
use Scheduling_Sequences;


with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;


with Buffers;
use Buffers;
use Buffers.Buffer_Roles;
use Buffers.Buffer_Size_Package;

with Tasks;
use Tasks;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;



procedure Xtb1 is 


   Bu           : Buffer_Ptr;  
   My_Scheduler : Rm_Scheduler;  
   Tasks1       : Tasks_Set;  
   O1,  
   O2,  
   O3,  
   O4           : Offsets_Table;  
   Sched1       : Scheduling_Sequence;  
   My_Size      : Buffer_Size_Table;  
   Cpu          : Unbounded_String    := To_Unbounded_String ("cpu");  
   T1,  
   T2,  
   T3,  
   T4           : Task_Ptr;  
   Bound        : Natural;  


begin

   Initialize(O1);
   Initialize(O2);
   Initialize(O3);
   Initialize(O4);
   Initialize(My_Size);
   Bu:= new Buffer;


   Add_Task(Tasks1, T1, To_Unbounded_String("T1"), Cpu , Periodic_Type, 0,
      1, 20, 20, 0,
      0, 1, Sched_Fifo, O1);
   Add_Task(Tasks1, T2, To_Unbounded_String("T2"), Cpu , Periodic_Type, 0,
      1,  34, 34, 0,
      0, 1, Sched_Fifo, O2);
   Add_Task(Tasks1, T3, To_Unbounded_String("T3"), Cpu , Periodic_Type, 0,
      1,  120, 120,
      0, 0, 1, Sched_Fifo, O3);
   Add_Task(Tasks1, T4, To_Unbounded_String("T4"), Cpu , Periodic_Type, 0,
      1,  130, 130,
      0, 0, 1, Sched_Fifo, O4);

   Put(Tasks1);

   Build_Scheduling_Sequence(My_Scheduler, Tasks1, Cpu, Sched1,130);
   Put(Sched1);


   Bu.Name:=To_Unbounded_String("My own buffer");
   Bu.Size:=20;
   Bu.Roles.Entries(0).Item:=T1.Name;
   Bu.Roles.Entries(0).Data.The_Role:=Consumer;
   Bu.Roles.Entries(0).Data.Size:=1;
   Bu.Roles.Entries(1).Item:=T2.Name;
   Bu.Roles.Entries(1).Data.The_Role:=Producer;
   Bu.Roles.Entries(1).Data.Size:=1;
   Bu.Roles.Entries(2).Item:=T3.Name;
   Bu.Roles.Entries(2).Data.The_Role:=Producer;
   Bu.Roles.Entries(2).Data.Size:=1;
   Bu.Roles.Entries(3).Item:=T4.Name;
   Bu.Roles.Entries(3).Data.The_Role:=Producer;
   Bu.Roles.Entries(3).Data.Size:=1;
   Bu.Roles.Nb_Entries:=4;

   Get_Buffer_Size( Sched1, Bu, Tasks1, My_Size);
   Bound:=Get_Max_Size(My_Size);

   Put_Line ("Max buffer size = " & Bound'Img);
   New_Line;
   for I in 0 .. Sched1.Scheduled.Nb_Entries-1 loop
      Put_Line("Time = " & My_Size.Entries(Buffer_Size_Range(I)).Time'Img &
         "/Size = " & My_Size.Entries(Buffer_Size_Range(I)).
         Size'Img);
   end loop;

   Gtk.Main.Set_Locale;
   Gtk.Main.Init;


   Gtk_New(Scrolling_Draw_Buffer,To_String(Bu.Name));
   Show_All(Scrolling_Draw_Buffer);
   Clear(Scrolling_Draw_Buffer);
   Draw(Scrolling_Draw_Buffer, Bu, My_Size, 130, Sched1.Scheduled.
      Nb_Entries);



   -- MMI event loop
   Gtk.Main.Main;



end Xtb1;

