------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Mmi_Tools;
use Mmi_Tools;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Project;
use Project;



with Task_Dependencies;
use Task_Dependencies;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;


with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Buffer_Set;
use Buffer_Set;
with Buffers;
use Buffers;
use Buffers.Buffer_Roles;


with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Translate;
use Translate;
with Config;
use Config;
with Graph_Item_Pkg;
use Graph_Item_Pkg;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with My_Canvas;
use My_Canvas;

package body Buffers_Parameters_Pkg.Callbacks is

   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      A_Processor  : Processor_Ptr;  
      Cpu_Index    : Gint;  
      The_Name     : Unbounded_String;  
      A_Buffer     : Buffer_Ptr;  
      The_Size     : Integer;  
      The_Location : Unbounded_String;  
      Ok           : Boolean;  
      Current      : Unbounded_String;  

   begin

      The_Name:=To_Unbounded_String(Get_Text(Buffers_Parameters.Name));

      if The_Name="" then
         Show_Message_Box(Lb_Buffer_Name(Current_Language) &  Lb_Mandatory(
               Current_Language));
         return;
      end if;

      if not Is_A_Valid_Identifier(The_Name)
            then
         Show_Message_Box(Lb_Processor_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;


      Current:=To_Unbounded_String(Get_Text(Buffers_Parameters.
            Size));
      if(Current="")
            then
         The_Size:=0;
      else
         To_Integer(Current,The_Size,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      if (The_Size<=0)
            then
         Show_Message_Box(Lb_Size(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Buffers_Parameters.Clist_Location_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      The_Location:=A_Processor.Name;

      Add_Buffer(Sys.Buffers,A_Buffer, The_Name,Natural(The_Size),
         The_Location,My_Roles);


      Build_Dependencies(Sys.Dependencies, Sys.Tasks, A_Buffer);



      if  Buffers_Parameters.Parent = To_Unbounded_String("graph_window") then
         Set_Name(Items_List(Last_Item-1),The_Name);

         Refresh_Canvas(Graph_Window.Buffer);
         Import_Dependencies(Graph_Window);

      else
         Rebuild_Canvas(Graph_Window);

      end if;


      Is_New:=False;
      Is_Saved:=False;




      Hide(Buffers_Parameters);


   exception
      when Buffer_Already_Defined =>
         begin
            Show_Message_Box(Lb_Already_Defined(Current_Language));
            return;
         end;
   end On_Ok_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin
      if Buffers_Parameters.Parent = To_Unbounded_String("graph_window") then
         My_Canvas.Remove(Graph_Window.Buffer,Items_List(Last_Item-1));
      end if;
      Hide(Buffers_Parameters);
   end On_Cancel_Pressed;

   procedure On_Add_Role_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Cpu_Index    : Gint;  
      A_Processor  : Processor_Ptr;  
      The_Location : Unbounded_String;  
      Ok           : Boolean;  
      Current      : Unbounded_String;  
      The_Task     : Unbounded_String;  
      A_Task       : Task_Ptr;  
      Role_Size    : Integer;  
      Role_Type    : Unbounded_String;  
      Line,  
      Task_Index   : Gint;  
      Index        : Gint;  
      Role_Share   : Chars_Ptr_Array (1 .. 3);  

   begin

      Index:=Gint(Selected_Button (Buffers_Parameters.Clist_Role_Menu));


      Current:=To_Unbounded_String(Get_Text(Buffers_Parameters.
            Affect_Size));
      if(Current="") then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Mandatory(
               Current_Language) );
         return;
      else
         To_Integer(Current,Role_Size,Ok);
         if not Ok then
            Show_Message_Box(Lb_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Task_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Tasks)) -
         Gint(Selected_Button (Buffers_Parameters.Clist_Task_Menu)));
      Task_Index:=Task_Index-1;

      Get_Element_Number(Sys.Tasks,A_Task,Tasks_Range(Task_Index));
      The_Task:=A_Task.Name;


      if (Index=1) then
         My_Roles.Entries(Buffer_Roles_Range(Buffers_Parameters.
               Role_Number)).Data.The_Role :=Producer;
         Role_Type:=Lb_Role_Producer(Current_Language);
      elsif (Index=0) then
         My_Roles.Entries(Buffer_Roles_Range(Buffers_Parameters.
               Role_Number)).Data.The_Role :=Consumer;
         Role_Type:=Lb_Role_Consumer(Current_Language);
      end if;


      -- Get the processor name !
      --
      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Buffers_Parameters.Clist_Location_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      The_Location:=A_Processor.Name;


      if A_Task.Location/=The_Location
            then
         Show_Message_Box(Lb_Task_Not_Permitted(Current_Language));
         return;
      end if;


      My_Roles.Entries(Buffer_Roles_Range(Buffers_Parameters.Role_Number)).
         Data.Size := Role_Size;
      My_Roles.Entries(Buffer_Roles_Range(Buffers_Parameters.Role_Number)).
         Item := A_Task.Name;


      -- Show Task on Clist
      --
      Role_Share:= To_String(The_Task) +
         To_String(To_Unbounded_String(Natural'Image(Role_Size)))
         +   To_String(Role_Type) ;

      Line := Append(Buffers_Parameters.Affected_Role_List,Role_Share);


      Set_Text(Buffers_Parameters.Affect_Size,"");
      Buffers_Parameters.Role_Number:=Buffers_Parameters.Role_Number + 1;
      My_Roles.Nb_Entries:= My_Roles.Nb_Entries +1;



   exception
      when Task_Set.Generic_Task_Set.Empty_Set =>
         begin
            Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
            return;
         end;
      when Task_Set.Task_Not_Found =>
         begin
            Show_Message_Box(Lb_Task_Not_Found(Current_Language));
            return;
         end;


   end On_Add_Role_Pressed;



end Buffers_Parameters_Pkg.Callbacks;








