------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk;
use Gtk;
with Gtk.Enums;
use Gtk.Enums;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Buffers_Parameters_Pkg.Callbacks;
use Buffers_Parameters_Pkg.Callbacks;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with  Unbounded_Strings;
use  Unbounded_Strings;
use  Unbounded_Strings.Strings_Table_Package;
with Gtk.Widget;
use Gtk.Widget;
with Mmi_Tools;
use Mmi_Tools;
with Translate;
use Translate;
with Gtkada.Types;
use Gtkada.Types;
with Gtk.Scrolled_Window;
use Gtk.Scrolled_Window;
with Gtk.Clist;
use Gtk.Clist;
with Gtk.Frame;
with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
with Buffers;
use Buffers;
use Buffers.Buffer_Roles;
with Project;
use Project;
use Processor_Set.Generic_Processor_Set;
with Translate;
use Translate;
with Config;
use Config;
package body Buffers_Parameters_Pkg is

   procedure Gtk_New (
         Buffers_Parameters :    out Buffers_Parameters_Access; 
         From               : in     Unbounded_String           ) is 
   begin
      Buffers_Parameters := new Buffers_Parameters_Record;
      Buffers_Parameters_Pkg.Initialize (Buffers_Parameters);
      Buffers_Parameters.Parent := From;

   end Gtk_New;

   procedure Initialize (
         Buffers_Parameters : access Buffers_Parameters_Record'Class ) is 

      A_Processor : Processor_Ptr;  
      A_Task      : Task_Ptr;  
      My_Iterator : Iterator;  
      Items       : Strings_Table;  
      Items2      : Strings_Table;  

      My_Iterator2 : Task_Set.Generic_Task_Set.Iterator;  
   begin

      Initialize(My_Roles);
      Initialize(Items);

      Reset_Iterator(My_Iterator);

      --Create CPU list
      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         Add(Items,A_Processor.Name);

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;


      Initialize(Items2);

      Reset_Iterator(My_Iterator);

      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator2);
         Add(Items2, A_Task.Name);

         exit when  Is_Last_Element(Sys.Tasks, My_Iterator2) = True;

         Next_Element(Sys.Tasks, My_Iterator2);

      end loop;

      --Boxes
      Gtk.Window.Initialize(Buffers_Parameters , Window_Toplevel);
      Set_Title (Buffers_Parameters, To_String(Lb_Add_Buffer(
               Current_Language)));
      Set_Policy (Buffers_Parameters, True, True, True);
      Set_Position (Buffers_Parameters, Win_Pos_Center_Always);
      Set_Modal (Buffers_Parameters, True);
      Grab_Focus(Buffers_Parameters);
      Gtk.Box.Gtk_New_Vbox (Buffers_Parameters.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Buffers_Parameters.Hbox_Add_Cancel,False,10);
      Gtk.Box.Gtk_New_Hbox (Buffers_Parameters.Hbox_Add_Affect,False,10);
      Gtk.Box.Gtk_New_Hbox (Buffers_Parameters.Hbox_Table_Name_Location,
         False,10);

      Add(Buffers_Parameters,Buffers_Parameters.Vbox_Main);
      Add(Buffers_Parameters.Vbox_Main,Buffers_Parameters.
         Hbox_Table_Name_Location);
      Add(Buffers_Parameters.Vbox_Main,Buffers_Parameters.Hbox_Add_Affect);

      --Frame Name_Location

      Gtk.Frame.Gtk_New(Buffers_Parameters.Frame_Name_Location,To_String(
            Lb_Buffer_Information(Current_Language)));
      Add(Buffers_Parameters.Hbox_Table_Name_Location,
         Buffers_Parameters.Frame_Name_Location);

      --Frame Affect

      Gtk.Frame.Gtk_New(Buffers_Parameters.Frame_Affect, To_String(
            Lb_Role(Current_Language)));
      Add(Buffers_Parameters.Hbox_Add_Affect,Buffers_Parameters.
         Frame_Affect);

      -- Table Name_Location

      Gtk.Table.Gtk_New (Buffers_Parameters.Table_Name_Location,4 , 2,
         False); --2 columns
      Add(Buffers_Parameters.Frame_Name_Location,Buffers_Parameters.
         Table_Name_Location);


      Gtk.Option_Menu.Gtk_New(Buffers_Parameters.Location);

      Buffers_Parameters.Clist_Location_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Buffers_Parameters.Location,Buffers_Parameters.
         Clist_Location_Menu, Items, -1, null);


      Gtk.Gentry.Gtk_New(Buffers_Parameters.Name,50);
      -- name limited to 50 characters
      Gtk.Gentry.Gtk_New(Buffers_Parameters.Size,50);
      -- name limited to 50 characters
      Gtk.Label.Gtk_New(Buffers_Parameters.Name_Label, To_String(
            Lb_Buffer_Name(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Buffers_Parameters.Size_Label,To_String(Lb_Size(
               Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Buffers_Parameters.Location_Label,To_String(
            Lb_Processor(Current_Language) & Lb_Colon));


      -- Attach Table

      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Name, 1, 2, 0, 1);
      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Location_Label, 0, 1, 1, 2);
      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Location, 1, 2, 1, 2);
      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Size_Label, 0, 1, 2, 3);
      Attach_Defaults(Buffers_Parameters.Table_Name_Location,
         Buffers_Parameters.Size, 1, 2, 2, 3);

      --Table Add_Affect

      Gtk.Table.Gtk_New(Buffers_Parameters.Table_Add_Affect,2,4,False);
      --2 columns and 2 lines
      Add(Buffers_Parameters.Frame_Affect,Buffers_Parameters.
         Table_Add_Affect);

      --Box Add_Affect




      Gtk.Option_Menu.Gtk_New(Buffers_Parameters.Affect_Task);

      Buffers_Parameters.Clist_Task_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Buffers_Parameters.Affect_Task,
         Buffers_Parameters.Clist_Task_Menu, Items2, -1, null);

      Gtk.Gentry.Gtk_New(Buffers_Parameters.Affect_Size,10);


      Initialize(Items);
      Add(Items, Lb_Role_Producer(Current_Language));
      Add(Items, Lb_Role_Consumer(Current_Language));
      Buffers_Parameters.Clist_Role_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Buffers_Parameters.Affect_Roletype,
         Buffers_Parameters.Clist_Role_Menu, Items, -1, null);


      Gtk.Button.Gtk_New(Buffers_Parameters.Button_Affect, To_String(
            Lb_Add(Current_Language)));
      Gtk.Label.Gtk_New(Buffers_Parameters.Task_Label,To_String(Lb_Task(
               Current_Language)));
      Gtk.Label.Gtk_New(Buffers_Parameters.Rolesize_Label,To_String(
            Lb_Size(Current_Language)));
      Gtk.Label.Gtk_New(Buffers_Parameters.Roletype_Label,To_String(
            Lb_Role_Type(Current_Language)));

      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Task_Label, 0, 1, 0, 1);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Rolesize_Label, 1, 2, 0, 1);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Roletype_Label, 2, 3, 0, 1);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Affect_Task, 0, 1, 1, 2);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Affect_Size, 1, 2, 1,2);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Affect_Roletype, 2, 3, 1, 2);
      Attach_Defaults(Buffers_Parameters.Table_Add_Affect,
         Buffers_Parameters.Button_Affect, 3, 4, 1, 2);

      --The List of affected Role

      Gtk.Scrolled_Window.Gtk_New(Buffers_Parameters.
         Scroll_Affected_Role_List, null, null);
      Add(Buffers_Parameters.Vbox_Main,Buffers_Parameters.
         Scroll_Affected_Role_List);
      Buffers_Parameters.Affect_List_Titles:= To_String(Lb_Task(
            Current_Language))
         + To_String(Lb_Size(Current_Language)) + To_String(Lb_Role_Type(
            Current_Language));
      Gtk.Clist.Gtk_New(Buffers_Parameters.Affected_Role_List,3,
         Buffers_Parameters.Affect_List_Titles); -- 3 Columns
      Set_Column_Width(Buffers_Parameters.Affected_Role_List, 0, 150);
      Set_Usize(Buffers_Parameters.Affected_Role_List,100,200);
      Add(Buffers_Parameters.Scroll_Affected_Role_List,Buffers_Parameters.
         Affected_Role_List);

      --Box Add_Cancel

      Gtk.Button.Gtk_New(Buffers_Parameters.Button_Add, To_String(Lb_Add(
               Current_Language)));
      Gtk.Button.Gtk_New(Buffers_Parameters.Button_Cancel, To_String(
            Lb_Cancel(Current_Language)));

      Button_Callback.Connect (Buffers_Parameters.Button_Cancel, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));
      Button_Callback.Connect (Buffers_Parameters.Button_Add, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Button_Callback.Connect (Buffers_Parameters.Button_Affect, "clicked",
         Button_Callback.To_Marshaller (On_Add_Role_Pressed'access));

      Add(Buffers_Parameters.Hbox_Add_Cancel,Buffers_Parameters.
         Button_Add);
      Add(Buffers_Parameters.Hbox_Add_Cancel,Buffers_Parameters.
         Button_Cancel);
      Add(Buffers_Parameters.Vbox_Main,Buffers_Parameters.Hbox_Add_Cancel);


      Show_All(Buffers_Parameters);
   exception
      when Processor_Set.Generic_Processor_Set.Empty_Set =>
         begin
            Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
            return;
         end;
      when Task_Set.Generic_Task_Set.Empty_Set =>
         begin
            Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
            return;
         end;


   end Initialize;

end Buffers_Parameters_Pkg;



