------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gdk.Drawable;
use Gdk.Drawable;
with Gtk.Arguments;
use Gtk.Arguments;
with Gtk.Drawing_Area;
use Gtk.Drawing_Area;
with Gtk.Handlers;
use Gtk.Handlers;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Tasks;
use Tasks;
with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Unbounded_Strings;
use Unbounded_Strings;

with Text_Io;
use Text_Io;


with Call_Scheduling_Framework;
use Call_Scheduling_Framework;


package body Draw_Scheduling_Pkg is


   package Draw_Cb is new Gtk.Handlers.Return_Callback
      (
      Widget_Type => Gtk_Double_Buffer_Record, 
      Return_Type => Boolean);

   package Allocation_Cb is new Handlers.Callback (
      Gtk_Double_Buffer_Record);
   package Allocation_Marshaller is new
      Allocation_Cb.Marshallers.Generic_Marshaller
      (Gtk_Allocation_Access, To_Allocation);



   procedure Color_Allocation (
         Draw_Scheduling : access Draw_Scheduling_Record ) is 
   begin

      if not Draw_Scheduling.Have_Colors then

         Gdk.Gc.Gdk_New (Draw_Scheduling.White_Gc, Get_Window (
               Draw_Scheduling.Buffer));
         Gdk.Gc.Set_Foreground
            (Draw_Scheduling.White_Gc, Gdk.Color.White (Gtk.Widget.
               Get_Default_Colormap));

         Gdk.Gc.Gdk_New (Draw_Scheduling.Black_Gc, Get_Window (
               Draw_Scheduling.Buffer));
         Gdk.Gc.Set_Foreground
            (Draw_Scheduling.Black_Gc, Gdk.Color.Black (Gtk.Widget.
               Get_Default_Colormap));

         Draw_Scheduling.Red_Color := Gdk.Color.Parse ("Red");
         Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Draw_Scheduling.
            Red_Color);


         Draw_Scheduling.Have_Colors:=True;

      end if;
   end Color_Allocation;


   function Configure (
         Buffer : access Gtk_Double_Buffer_Record'Class; 
         Event  :        Gdk.Event.Gdk_Event             ) 
     return Boolean is 


   begin
      if Draw_Scheduling.Initialized
            then
         Redraw_Task(Draw_Scheduling);
      else
         Color_Allocation(Draw_Scheduling);

         -- Check if window size has changed !
         --

         Draw_Scheduling.Current_Width:=Gint(Get_Allocation_Width(
               Draw_Scheduling.Buffer));
         Draw_Scheduling.Current_Height:=Gint(Get_Allocation_Height(
               Draw_Scheduling.Buffer));

         Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer),
            Draw_Scheduling.White_Gc, Filled => True,
            X     => 0,   Y      => 0,
            Width => Draw_Scheduling.Current_Width, Height =>
            Draw_Scheduling.Current_Height);

         Double_Buffer.Draw (Draw_Scheduling.Buffer);

      end if;

      return True;
   end Configure;


   procedure Size_Allocate (
         Buffer     : access Gtk_Double_Buffer_Record'Class; 
         Allocation :        Gtk_Allocation_Access           ) is 
      --  This function is called once gtk has decided what size and position
      --  the widget will actually have, or everytime the widget is resized.
      --  This would be a good time for instance for resizing the component
      --  sub-widgets.
      --  Note that we have to move and resize the widget ourselves, and that
      --  for such a simple case, we could simply rely on the ancestor's
      --  size_allocation function.
   begin
      if Realized_Is_Set (Buffer) then
         Draw_Scheduling.Current_Width:=Gint(Get_Allocation_Width(
               Draw_Scheduling.Buffer));
         Draw_Scheduling.Current_Height:=Gint(Get_Allocation_Height(
               Draw_Scheduling.Buffer));
         if Draw_Scheduling.Initialized
               then
            Redraw_Task(Draw_Scheduling);
         else
            Color_Allocation(Draw_Scheduling);

            Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer),
               Draw_Scheduling.White_Gc, Filled => True,
               X     => 0,   Y      => 0,
               Width => Draw_Scheduling.Current_Width, Height =>
               Draw_Scheduling.Current_Height);

            Double_Buffer.Draw (Draw_Scheduling.Buffer);

         end if;
      end if;
      Gtk.Handlers.Emit_Stop_By_Name (Buffer, "size_allocate");
   end Size_Allocate;



   procedure Gtk_New (
         Draw_Scheduling :    out Draw_Scheduling_Access ) is 

   begin
      Draw_Scheduling:=new Draw_Scheduling_Record;
      Draw_Scheduling_Pkg.Initialize (Draw_Scheduling);
   end Gtk_New;


   procedure Initialize (
         Draw_Scheduling : access Draw_Scheduling_Record'Class ) is 

   begin


      Draw_Scheduling.Current_Width:=Draw_Scheduling.First_Width;
      Draw_Scheduling.Current_Height:=Draw_Scheduling.First_Height;

      Gtk_New(Draw_Scheduling.Buffer);

      Gtk.Drawing_Area.Set_Usize (Gtk_Drawing_Area(Draw_Scheduling.Buffer),
         Draw_Scheduling.First_Width, Draw_Scheduling.First_Height);
      Show_All (Draw_Scheduling.Buffer);

      Load (Draw_Scheduling.Font,
         "-adobe-helvetica-medium-o-normal--14-140-75-75-p-78-iso8859-1");


      Draw_Cb.Connect (Draw_Scheduling.Buffer, "configure_event",
         Draw_Cb.To_Marshaller (Configure'access));
      Allocation_Cb.Connect (Draw_Scheduling.Buffer, "size_allocate",
         Allocation_Marshaller.To_Marshaller (Size_Allocate'access));



   end Initialize;


   procedure Clear_Task (
         Draw_Scheduling : access Draw_Scheduling_Record ) is 
   begin


      Color_Allocation(Draw_Scheduling);

      Draw_Scheduling.Initialized:=False;


      Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer), Draw_Scheduling.
         White_Gc, Filled => True,
         X     => 0,   Y      => 0,
         Width => Draw_Scheduling.Current_Width, Height => Draw_Scheduling.
         Current_Height);

      Double_Buffer.Draw (Draw_Scheduling.Buffer);

   end Clear_Task;




   procedure Draw_Task (
         Draw_Scheduling : access Draw_Scheduling_Record; 
         Task1           :        Tasks_Set;              
         Period1         :        Natural;                
         Sched1          :        Scheduling_Table_Ptr    ) is 

      I              : Gint           := 0;  
      Ok             : Boolean;  
      My_Iterator    : Tasks_Iterator;  
      A_Task         : Task_Ptr;  
      Task_Name_Size : Natural        := 0;  

   begin



      Draw_Scheduling.Tasks:=Task1 ;
      Draw_Scheduling.Period:=Period1 ;
      Draw_Scheduling.Sched:=Sched1;


      -- Compute Left_margin
      --
      Reset_Iterator(My_Iterator);

      loop

         Current_Element(Draw_Scheduling.Tasks, A_Task, My_Iterator);

         Task_Name_Size:=Natural'Max(Task_Name_Size, Length(A_Task.Name));

         exit when Is_Last_Element(Draw_Scheduling.Tasks, My_Iterator) =
            True;
         Next_Element(Draw_Scheduling.Tasks, My_Iterator);

      end loop;

      Task_Name_Size:=Task_Name_Size+1;
      Draw_Scheduling.Left_Margin:=Gint(Task_Name_Size*15);


      -- Compute Widget size
      --
      Ok:=False;
      I:=Gint(Period1)*Draw_Scheduling.Timeunit_Margin+Draw_Scheduling.
         Left_Margin+Draw_Scheduling.Right_Margin;
      if I > Draw_Scheduling.Current_Width
            then
         Draw_Scheduling.Current_Width:=I;
         Ok:=True;
      end if;

      I:=Draw_Scheduling.High_Margin+Draw_Scheduling.Interligne*Gint(
         Get_Number_Of_Elements(Task1));
      if I > Draw_Scheduling.Current_Height
            then
         Draw_Scheduling.Current_Height:=I;
         Ok:=True;
      end if;

      if Ok then
         Gtk.Drawing_Area.Set_Usize (Gtk_Drawing_Area(Draw_Scheduling.
               Buffer),
            Draw_Scheduling.Current_Width, Draw_Scheduling.Current_Height);
      end if;



      Draw_Scheduling.Initialized:=True;

      Redraw_Task(Draw_Scheduling);


   end Draw_Task;


   procedure Redraw_Task (
         Draw_Scheduling : access Draw_Scheduling_Record'Class ) is 

      My_Iterator : Tasks_Iterator;  
      A_Task      : Task_Ptr;  
      I           : Gint                                    := 0;  
      Height,  
      Width       : Gint;  
      Msg,  
      Name        : Unbounded_String;  
      Task_Name   : array (Tasks_Range) of Unbounded_String;  
      Task_Id     : Gint;  
      Time_U      : Gint;  
      Ok          : Boolean;  

   begin

      ---------------------------------
      -- Clear work space
      ---------------------------------


      Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer), Draw_Scheduling.
         White_Gc, Filled => True,
         X     => 0,   Y      => 0,
         Width => Draw_Scheduling.Current_Width, Height => Draw_Scheduling.
         Current_Height);

      ---------------------------------
      -- Draw a line per task
      ---------------------------------


      I:=0;
      Reset_Iterator(My_Iterator);

      loop

         Current_Element(Draw_Scheduling.Tasks, A_Task, My_Iterator);

         case A_Task.Task_Type is
            when Periodic_Type =>
               Msg:=To_Unbounded_String("Pe=")
                  & Periodic_Task_Ptr(A_Task).Period'Img
                  & To_Unbounded_String("; C=")
                  & A_Task.Capacity'Img
                  & To_Unbounded_String("; D=")
                  & Periodic_Task_Ptr(A_Task).Deadline'Img
                  & To_Unbounded_String("; S=")
                  & A_Task.Start_Time'Img
                  & To_Unbounded_String("; Pr=")
                  & A_Task.Priority'Img
                  & To_Unbounded_String("; Cpu=")
                  & A_Task.Location;
            when Aperiodic_Type =>
               Msg:=To_Unbounded_String("C=")
                  & A_Task.Capacity'Img
                  & To_Unbounded_String("; D=")
                  & Aperiodic_Task_Ptr(A_Task).Deadline'Img
                  & To_Unbounded_String("; S=")
                  & A_Task.Start_Time'Img
                  & To_Unbounded_String("; Pr=")
                  & A_Task.Priority'Img
                  & To_Unbounded_String("; Cpu=")
                  & A_Task.Location;
            when Poisson_Type =>
               Msg:=To_Unbounded_String("Pe=")
                  & Poisson_Task_Ptr(A_Task).Period'Img
                  & To_Unbounded_String("; C=")
                  & A_Task.Capacity'Img
                  & To_Unbounded_String("; D=")
                  & Poisson_Task_Ptr(A_Task).Deadline'Img
                  & To_Unbounded_String("; S=")
                  & A_Task.Start_Time'Img
                  & To_Unbounded_String("; Pr=")
                  & A_Task.Priority'Img
                  & To_Unbounded_String("; Cpu=")
                  & A_Task.Location;
            when Parametric_Type =>
               raise Constraint_Error;
         end case;

         Width := String_Width (Draw_Scheduling.Font, To_String(Msg));
         Height := String_Height (Draw_Scheduling.Font, To_String(Msg));
         Draw_Text (Get_Pixmap(Draw_Scheduling.Buffer), Draw_Scheduling.
            Font,
            Draw_Scheduling.Black_Gc,
            Draw_Scheduling.Left_Margin,
            Draw_Scheduling.High_Margin+I*Draw_Scheduling.Interligne+20,
            To_String(Msg));

         Width := String_Width (Draw_Scheduling.Font, To_String(A_Task.
               Name));
         Height := String_Height (Draw_Scheduling.Font, To_String(A_Task.
               Name));
         Draw_Text (Get_Pixmap(Draw_Scheduling.Buffer), Draw_Scheduling.
            Font,
            Draw_Scheduling.Black_Gc,
            10, Draw_Scheduling.High_Margin+I*Draw_Scheduling.Interligne,
            To_String(A_Task.Name));

         Draw_Line (Get_Pixmap(Draw_Scheduling.Buffer),
            Draw_Scheduling.Black_Gc,
            Draw_Scheduling.Left_Margin,  Draw_Scheduling.High_Margin+I*
            Draw_Scheduling.Interligne,
            Draw_Scheduling.Left_Margin+Gint(Draw_Scheduling.Period)*
            Draw_Scheduling.Timeunit_Margin, Draw_Scheduling.High_Margin+
            I*Draw_Scheduling.Interligne);

         -- For each line, draw unit time lines
         for J in 0 .. Gint(Draw_Scheduling.Period)   loop
            Draw_Line (Get_Pixmap(Draw_Scheduling.Buffer),
               Draw_Scheduling.Black_Gc,
               Draw_Scheduling.Left_Margin+J*Draw_Scheduling.
               Timeunit_Margin,  Draw_Scheduling.High_Margin+I*
               Draw_Scheduling.Interligne-3,
               Draw_Scheduling.Left_Margin+J*Draw_Scheduling.
               Timeunit_Margin, Draw_Scheduling.High_Margin+I*
               Draw_Scheduling.Interligne+3);
         end loop;


         -- Register each task in the name table
         Task_Name(Tasks_Range(I)):=A_Task.Name;

         I:=I+1;

         exit when Is_Last_Element(Draw_Scheduling.Tasks, My_Iterator) =
            True;
         Next_Element(Draw_Scheduling.Tasks, My_Iterator);

      end loop;


      ------------------------------------------
      -- Draw information from scheduling table
      ------------------------------------------
      for U in 0..Sched.Nb_Entries-1 loop -- processors
         if  Draw_Scheduling.Sched.Entries(U).Data.Msg=Empty_String then

            for K in 0..Draw_Scheduling.Sched.Entries(U).
                  Data.Result.Scheduled.Nb_Entries-1 loop -- current time

               case Draw_Scheduling.Sched.Entries(U).Data.Result.
                        Scheduled.Entries(K).Item.Etype is
                  when Running =>
                     ----------------------------------------
                     -- Draw scheduling
                     ----------------------------------------

                     Name := Draw_Scheduling.Sched.Entries(U).Data.Result.
                        Scheduled.Entries(K).Item.Running_Task.Name;

                     Ok:=False;
                     for L in Tasks_Range loop
                        if (Task_Name(L) = Name) then
                           Ok:=True;
                           Task_Id:=Gint(L);
                           Time_U:=
                              Gint(Draw_Scheduling.Sched.Entries(U).
                              Data.Result.Scheduled.Entries(K).Data);
                        end if;
                     end loop;

                     if not Ok then
                        raise Task_Not_Found;
                     end if;

                     Gdk.Gc.Set_Foreground
                        (Draw_Scheduling.Black_Gc, Gdk.Color.Black
                        (Gtk.Widget.Get_Default_Colormap));

                     Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer),
                        Draw_Scheduling.Black_Gc, True,
                        Draw_Scheduling.Timeunit_Margin*
                        Time_U+Draw_Scheduling.Left_Margin,
                        Task_Id*Draw_Scheduling.Interligne+
                        Draw_Scheduling.High_Margin-3,
                        Draw_Scheduling.Timeunit_Margin, 7);

                  when Activation =>
                     -----------------------------------------------------
                     -- For each task, draw the activation in red color
                     -----------------------------------------------------

                     Name := Draw_Scheduling.Sched.Entries(U).Data.Result.
                        Scheduled.Entries(K).Item.Activated_Task.Name;

                     Ok:=False;
                     for L in Tasks_Range loop
                        if (Task_Name(L) = Name) then
                           Ok:=True;
                           Task_Id:=Gint(L);
                           Time_U:=
                              Gint(Draw_Scheduling.Sched.Entries(U).
                              Data.Result.Scheduled.Entries(K).Data);
                        end if;
                     end loop;

                     if not Ok then
                        raise Task_Not_Found;
                     end if;

                     Gdk.Gc.Set_Foreground (Draw_Scheduling.Black_Gc,
                        Draw_Scheduling.Red_Color );

                     Draw_Rectangle (Get_Pixmap(Draw_Scheduling.Buffer),
                        Draw_Scheduling.Black_Gc, True,
                        Draw_Scheduling.Left_Margin+
                        Draw_Scheduling.Timeunit_Margin*Time_U,
                        Draw_Scheduling.High_Margin+Task_Id*
                        Draw_Scheduling.Interligne-8, 3, 16 );

                  when Allocate_Resource =>
                     null;
                  when Release_Resource =>
                     null;


               end case;


            end loop;
         end if;
      end loop;

      Gdk.Gc.Set_Foreground
         (Draw_Scheduling.Black_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));

      ---------------------
      -- Flush data in the window
      ---------------------
      Draw (Draw_Scheduling.Buffer);

   end Redraw_Task;




end Draw_Scheduling_Pkg;





