------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gdk;
use Gdk;
with Gdk.Event;
use Gdk.Event;
with Gdk.Font;
use Gdk.Font;
with Gdk.Gc;
use Gdk.Gc;
with Gdk.Color;
use Gdk.Color;
with Gtk;
use Gtk;
with Double_Buffer;
use Double_Buffer;
with Gtk.Widget;
use Gtk.Widget;

with Task_Set;
use Task_Set;

with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;


package Draw_Scheduling_Pkg is


   type Draw_Scheduling_Record is tagged 
      record 
         Font   : Gdk_Font;  
         Buffer : Gtk_Double_Buffer;  

         Tasks  : Tasks_Set;  
         Period : Natural;  
         Sched  : Scheduling_Table_Ptr;  

         White_Gc  : Gdk.Gc.Gdk_Gc;  
         Black_Gc  : Gdk.Gc.Gdk_Gc;  
         Red_Color : Gdk_Color;  

         Left_Margin     : Gint := 0;  
         Right_Margin    : Gint := 20;  
         High_Margin     : Gint := 20;  
         Interligne      : Gint := 50;  
         Timeunit_Margin : Gint := 20;  

         First_Width    : Gint := 800;  
         First_Height   : Gint := 300;  
         Current_Width  : Gint;  
         Current_Height : Gint;  

         Have_Colors : Boolean := False;  
         Initialized : Boolean := False;  
      end record; 

   type Draw_Scheduling_Access is access all Draw_Scheduling_Record'Class; 

   procedure Gtk_New (
         Draw_Scheduling :    out Draw_Scheduling_Access ); 
   procedure Initialize (
         Draw_Scheduling : access Draw_Scheduling_Record'Class ); 

   procedure Redraw_Task (
         Draw_Scheduling : access Draw_Scheduling_Record'Class ); 
   procedure Draw_Task (
         Draw_Scheduling : access Draw_Scheduling_Record; 
         Task1           :        Tasks_Set;              
         Period1         :        Natural;                
         Sched1          :        Scheduling_Table_Ptr    ); 
   procedure Clear_Task (
         Draw_Scheduling : access Draw_Scheduling_Record ); 

   procedure Color_Allocation (
         Draw_Scheduling : access Draw_Scheduling_Record ); 
   function Configure (
         Buffer : access Gtk_Double_Buffer_Record'Class; 
         Event  :        Gdk.Event.Gdk_Event             ) 
     return Boolean; 
   procedure Size_Allocate (
         Buffer     : access Gtk_Double_Buffer_Record'Class; 
         Allocation :        Gtk_Allocation_Access           ); 


   Draw_Scheduling : Draw_Scheduling_Access;  


   pragma Inline (Redraw_Task);
   pragma Inline (Color_Allocation);
   pragma Inline (Clear_Task);



end Draw_Scheduling_Pkg;
