------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Mmi_Tools;
use Mmi_Tools;
with Project;
use Project;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
with Message_Set;
use Message_Set;
use  Message_Set.Generic_Message_Set;
with Translate;
use Translate;
with Config;
use Config;
with Graph_Item_Pkg;
use Graph_Item_Pkg;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with My_Canvas;
use My_Canvas;

package body Messages_Parameters_Pkg.Callbacks is




   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 


      Name     : Unbounded_String;  
      Jitter   : Integer;  
      Deadline : Integer;  
      Period   : Integer;  
      Size     : Integer;  
      Ok       : Boolean;  
      Current  : Unbounded_String;  

   begin

      Name:=To_Unbounded_String(Get_Text(Messages_Parameters.Entry_Name));
      if Name=""
            then
         Show_Message_Box(Lb_Message_Name(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return;
      end if;

      if not Is_A_Valid_Identifier(Name)
            then
         Show_Message_Box(Lb_Processor_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;


      Current:=To_Unbounded_String(Get_Text(Messages_Parameters.
            Entry_Jitter));
      if(Current="")
            then
         Jitter:=0;
      else
         To_Integer(Current,Jitter,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      Current:=To_Unbounded_String(Get_Text(Messages_Parameters.
            Entry_Deadline));
      if(Current="")
            then
         Deadline:=0;
      else
         To_Integer(Current,Deadline,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      Current:=To_Unbounded_String(Get_Text(Messages_Parameters.
            Entry_Period));
      if(Current="")
            then
         Period:=0;
      else
         To_Integer(Current,Period,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;


      Current:=To_Unbounded_String(Get_Text(Messages_Parameters.
            Entry_Size));
      if(Current="")
            then
         Size:=0;
      else
         To_Integer(Current,Size,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;


      if (Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Size<=0)
            then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Period/=0 and Jitter>Deadline)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         return;
      end if;


      if (Period/=0 and Jitter=0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Mandatory(
               Current_Language));
         return;
      end if;


      Add_Message(Sys.Messages,Name,Size,Period,Deadline,Jitter);

      if Messages_Parameters.Parent = To_Unbounded_String("graph_window") then
         Set_Name(Items_List(Last_Item-1),Name);
         Refresh_Canvas(Graph_Window.Buffer);
      end if;

      Is_New:=False;
      Is_Saved:=False;
      Hide(Messages_Parameters);

   exception
      when Message_Already_Defined =>
         begin
            Show_Message_Box(Lb_Message(Current_Language) &
               Lb_Already_Defined(Current_Language));
            return;
         end;

   end On_Ok_Pressed;




   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      if Messages_Parameters.Parent = To_Unbounded_String("graph_window") then
         My_Canvas.Remove(Graph_Window.Buffer,Items_List(Last_Item-1));
      else
         null;
      end if;
      Hide(Messages_Parameters);
   end On_Cancel_Pressed;

end Messages_Parameters_Pkg.Callbacks;
