------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Enums;
use Gtk.Enums;
with Gtk.Widget;
use Gtk.Widget;
with Translate;
use Translate;
with Config;
use Config;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Messages_Parameters_Pkg.Callbacks;
use Messages_Parameters_Pkg.Callbacks;

package body Messages_Parameters_Pkg is

   procedure Gtk_New (
         Messages_Parameters :    out Messages_Parameters_Access; 
         From                : in     Unbounded_String            ) is 
   begin
      Messages_Parameters := new Messages_Parameters_Record;
      Messages_Parameters_Pkg.Initialize (Messages_Parameters);
      Messages_Parameters.Parent := From;

   end Gtk_New;

   procedure Initialize (
         Messages_Parameters : access Messages_Parameters_Record'Class ) is 

   begin
      Gtk.Window.Initialize (Messages_Parameters, Window_Toplevel);
      Set_Title (Messages_Parameters,  To_String(Lb_Add_Message(
               Current_Language)));
      Set_Policy (Messages_Parameters, True, True, True);
      Set_Position (Messages_Parameters, Win_Pos_Center_Always);
      Set_Modal (Messages_Parameters, True);
      Set_Extension_Events (Messages_Parameters, Extension_Events_All);
      Set_Flags (Messages_Parameters, Can_Default);
      Grab_Focus (Messages_Parameters);

      Gtk_New (Messages_Parameters.Box, 7, 2, True);
      Set_Border_Width (Messages_Parameters.Box, 10);
      Set_Row_Spacings (Messages_Parameters.Box, 0);
      Set_Col_Spacings (Messages_Parameters.Box, 0);
      Add (Messages_Parameters, Messages_Parameters.Box);

      Gtk_New (Messages_Parameters.Name, To_String(Lb_Message_Name(
               Current_Language) & Lb_Colon));
      Set_Alignment (Messages_Parameters.Name, 0.5, 0.5);
      Set_Padding (Messages_Parameters.Name, 0, 0);
      Set_Justify (Messages_Parameters.Name, Justify_Left);
      Set_Line_Wrap (Messages_Parameters.Name, False);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.Name,
         0, 1, 0, 1);

      Gtk_New (Messages_Parameters.Jitter, To_String(Lb_Jitter(
               Current_Language) & Lb_Colon));
      Set_Alignment (Messages_Parameters.Jitter, 0.5, 0.5);
      Set_Padding (Messages_Parameters.Jitter, 0, 0);
      Set_Justify (Messages_Parameters.Jitter, Justify_Left);
      Set_Line_Wrap (Messages_Parameters.Jitter, False);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Jitter, 0, 1, 1, 2);

      Gtk_New (Messages_Parameters.Deadline, To_String(Lb_Deadline(
               Current_Language) & Lb_Colon));
      Set_Alignment (Messages_Parameters.Deadline, 0.5, 0.5);
      Set_Padding (Messages_Parameters.Deadline, 0, 0);
      Set_Justify (Messages_Parameters.Deadline, Justify_Left);
      Set_Line_Wrap (Messages_Parameters.Deadline, False);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Deadline, 0, 1, 2, 3);

      Gtk_New (Messages_Parameters.Period, To_String(Lb_Period(
               Current_Language) & Lb_Colon));
      Set_Alignment (Messages_Parameters.Period, 0.5, 0.5);
      Set_Padding (Messages_Parameters.Period, 0, 0);
      Set_Justify (Messages_Parameters.Period, Justify_Left);
      Set_Line_Wrap (Messages_Parameters.Period, False);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Period, 0, 1, 3, 4);

      Gtk_New (Messages_Parameters.Size, To_String(Lb_Size(
               Current_Language) & Lb_Colon));
      Set_Alignment (Messages_Parameters.Size, 0.5, 0.5);
      Set_Padding (Messages_Parameters.Size, 0, 0);
      Set_Justify (Messages_Parameters.Size, Justify_Left);
      Set_Line_Wrap (Messages_Parameters.Size, False);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.Size,
         0, 1, 4, 5);


      Gtk_New (Messages_Parameters.Entry_Name);
      Set_Editable (Messages_Parameters.Entry_Name, True);
      Set_Max_Length (Messages_Parameters.Entry_Name, 0);
      Set_Text (Messages_Parameters.Entry_Name, "");
      Set_Visibility (Messages_Parameters.Entry_Name, True);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Entry_Name, 1, 2, 0, 1);

      Gtk_New (Messages_Parameters.Entry_Jitter);
      Set_Editable (Messages_Parameters.Entry_Jitter, True);
      Set_Max_Length (Messages_Parameters.Entry_Jitter, 0);
      Set_Text (Messages_Parameters.Entry_Jitter, "");
      Set_Visibility (Messages_Parameters.Entry_Jitter, True);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Entry_Jitter, 1, 2, 1, 2);

      Gtk_New (Messages_Parameters.Entry_Deadline);
      Set_Editable (Messages_Parameters.Entry_Deadline, True);
      Set_Max_Length (Messages_Parameters.Entry_Deadline, 0);
      Set_Text (Messages_Parameters.Entry_Deadline, "");
      Set_Visibility (Messages_Parameters.Entry_Deadline, True);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Entry_Deadline, 1, 2, 2, 3);

      Gtk_New (Messages_Parameters.Entry_Period);
      Set_Editable (Messages_Parameters.Entry_Period, True);
      Set_Max_Length (Messages_Parameters.Entry_Period, 0);
      Set_Text (Messages_Parameters.Entry_Period, "");
      Set_Visibility (Messages_Parameters.Entry_Period, True);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Entry_Period, 1, 2, 3, 4);

      Gtk_New (Messages_Parameters.Entry_Size);
      Set_Editable (Messages_Parameters.Entry_Size, True);
      Set_Max_Length (Messages_Parameters.Entry_Size, 0);
      Set_Text (Messages_Parameters.Entry_Size, "");
      Set_Visibility (Messages_Parameters.Entry_Size, True);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Entry_Size, 1, 2, 4, 5);

      Gtk_New_Hseparator (Messages_Parameters.Hseparator);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.
         Hseparator, 0, 2, 5, 6);

      Gtk_New_Hbox (Messages_Parameters.Hbox, True, 0);
      Set_Border_Width (Messages_Parameters.Hbox, 6);
      Attach_Defaults (Messages_Parameters.Box, Messages_Parameters.Hbox,
         0, 2, 6, 7);

      Gtk_New (Messages_Parameters.Ok_Button, To_String(Lb_Add(
               Current_Language)));
      Pack_Start (Messages_Parameters.Hbox, Messages_Parameters.Ok_Button,
         True, True, 0);
      Button_Callback.Connect
         (Messages_Parameters.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Gtk_New_Vseparator (Messages_Parameters.Vseparator);
      Set_Sensitive (Messages_Parameters.Vseparator, False);
      Pack_Start (Messages_Parameters.Hbox, Messages_Parameters.
         Vseparator,  True, True, 0);

      Gtk_New (Messages_Parameters.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Messages_Parameters.Hbox, Messages_Parameters.
         Cancel_Button, True, True, 0);
      Button_Callback.Connect
         (Messages_Parameters.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));

      Show_All(Messages_Parameters);

   end Initialize;

end Messages_Parameters_Pkg;
