------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Open_File_Selection_Pkg.Callbacks;
use Open_File_Selection_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;
with Project;
use Project;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


package body Open_File_Selection_Pkg is

   procedure Gtk_New (
         Open_File_Selection :    out Open_File_Selection_Access ) is 
   begin
      Open_File_Selection := new Open_File_Selection_Record;
      Open_File_Selection_Pkg.Initialize (Open_File_Selection);
   end Gtk_New;

   procedure Initialize (
         Open_File_Selection : access Open_File_Selection_Record'Class ) is 
   begin

      Gtk.File_Selection.Initialize (Open_File_Selection, To_String(
            Lb_Open_File(Current_Language)));
      Set_Show_File_Op_Buttons (Open_File_Selection, True);
      Set_Border_Width (Open_File_Selection, 10);
      Set_Title (Open_File_Selection, To_String(Lb_Open_File(
               Current_Language)));
      Set_Policy (Open_File_Selection, False, True, False);
      Set_Position (Open_File_Selection, Win_Pos_Center_Always);
      Set_Modal (Open_File_Selection, True);

      Open_File_Selection.Ok_Button := Get_Ok_Button (Open_File_Selection);
      Set_Flags (Open_File_Selection.Ok_Button, Can_Default);
      Button_Callback.Connect
         (Open_File_Selection.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Open_Filesel_Ok_Button_Clicked'access));

      Open_File_Selection.Cancel_Button := Get_Cancel_Button (
         Open_File_Selection);
      Set_Flags (Open_File_Selection.Cancel_Button, Can_Default);
      Button_Callback.Connect
         (Open_File_Selection.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (
            On_Open_Filesel_Cancel_Button_Clicked'access));


      if Current_Directory=To_Unbounded_String("")
            then
         Current_Directory:=To_Unbounded_String(Get_Filename(
               Open_File_Selection));
      else
         Set_Filename(Open_File_Selection,To_String(Current_Directory));
      end if;


   end Initialize;

end Open_File_Selection_Pkg;
