------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk.Check_Button;
use Gtk.Check_Button;

with Gtk.Gentry;
use Gtk.Gentry;

with Mmi_Tools;
use Mmi_Tools;

with Translate;
use Translate;
with Config;
use Config;

with Project;
use Project;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Strings;
use Strings;

with Ada.Numerics.Float_Random;
use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;
use Ada.Numerics.Aux;


package body Options_Pkg.Callbacks is




   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Ok : Boolean;  

   begin

      if Get_Active(Options.With_Offsets)
            then
         Schedule_With_Offsets:=True;
      else
         Schedule_With_Offsets:=False;
      end if;

      if Get_Active(Options.With_Precedencies)
            then
         Schedule_With_Precedencies:=True;
      else
         Schedule_With_Precedencies:=False;
      end if;

      if Get_Active(Options.With_Resources)
            then
         Schedule_With_Resources:=True;
      else
         Schedule_With_Resources:=False;
      end if;

      if Get_Active(Options.Seed_Task)
            then
         Task_Specific_Seed:=True;
      else
         Task_Specific_Seed:=False;
      end if;

      if Get_Active(Options.Seed_Predictable)
            then
         Task_Specific_Seed:=False;
      else
         Task_Specific_Seed:=True;
      end if;

      To_Integer(Get_Text(Options.Seed_Entry),Project.Common_Seed_Value,
         Ok);
      if not Ok then
         if Get_Text(Options.Seed_Entry) /= "" then
            Show_Message_Box(Lb_Seed(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      Hide(Options);

   end On_Ok_Pressed;




   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide(Options);
   end On_Cancel_Pressed;

   procedure On_Seed_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Seed : Generator;  
      Nmb  : Integer;  
   begin
      -- Give random value for the schedulers seed
      --
      Reset(Seed);
      -- range from integer'first .. integer'last
      Nmb := Integer(Double(Random(Seed))*Double(Integer'Last-Integer'
            First)+
         Double(Integer'First));
      Gtk.Gentry.Set_Text(Options.Seed_Entry,Nmb'Img);

   end On_Seed_Pressed;


end Options_Pkg.Callbacks;



