------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Options_Pkg.Callbacks;
use Options_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Project;
use Project;


package body Options_Pkg is

   procedure Gtk_New (
         Options :    out Options_Access ) is 

   begin
      Options := new Options_Record;
      Options_Pkg.Initialize (Options);
   end Gtk_New;

   procedure Initialize (
         Options : access Options_Record'Class ) is 

   begin

      Gtk.Window.Initialize (Options, Window_Toplevel);
      Set_Title (Options, To_String(Lb_Scheduling_Options(
               Current_Language)));
      Set_Policy (Options, True, True, True);
      Set_Position (Options, Win_Pos_Center_Always);
      Set_Modal (Options, True);
      Set_Extension_Events (Options, Extension_Events_All);
      Set_Flags (Options, Can_Default);
      Grab_Focus (Options);

      Gtk_New_Vbox (Options.Options_Vbox);
      Add (Options, Options.Options_Vbox);

      Gtk_New (Options.Value_Table,8,2,True);
      Set_Border_Width (Options.Value_Table, 14);
      Set_Row_Spacings (Options.Value_Table, 8);
      Set_Col_Spacings (Options.Value_Table, 0);
      Pack_Start (Options.Options_Vbox, Options.Value_Table);

      Gtk.Box.Gtk_New_Hbox(Options.Hbox_Seed, False, 10);
      Attach_Defaults(Options.Value_Table, Options.Hbox_Seed, 1, 2, 7, 8);


      Gtk_New (Options.Label, To_String(Lb_Schedule_According_To(
               Current_Language)));
      Set_Alignment (Options.Label, 0.5, 0.5);
      Set_Padding (Options.Label, 0, 0);
      Set_Justify (Options.Label, Justify_Left);
      Set_Line_Wrap (Options.Label, False);
      Attach (Options.Value_Table, Options.Label,
         0, 1, 0, 1);

      Gtk_New (Options.With_Offsets, To_String(Lb_Offsets(
               Current_Language)));
      if Schedule_With_Offsets = True
            then
         Set_Active(Options.With_Offsets, True);
      else
         Set_Active(Options.With_Offsets, False);
      end if;
      Attach (Options.Value_Table, Options.With_Offsets,
         1, 2, 1, 2);

      Gtk_New (Options.With_Precedencies, To_String(Lb_Precedencies(
               Current_Language)));
      if Schedule_With_Precedencies = True
            then
         Set_Active(Options.With_Precedencies, True);
      else
         Set_Active(Options.With_Precedencies, False);
      end if;
      Attach (Options.Value_Table, Options.With_Precedencies,
         1, 2, 2, 3);

      Gtk_New (Options.With_Resources, To_String(Lb_Resources(
               Current_Language)));
      if Schedule_With_Resources = True
            then
         Set_Active(Options.With_Resources, True);
      else
         Set_Active(Options.With_Resources, False);
      end if;
      Attach (Options.Value_Table, Options.With_Resources,
         1, 2, 3, 4);

      Gtk_New (Options.Label_Seed,
         To_String(Lb_Seed_Options(Current_Language)));
      Set_Alignment (Options.Label_Seed, 0.5, 0.5);
      Set_Padding (Options.Label_Seed, 0, 0);
      Set_Justify (Options.Label_Seed, Justify_Left);
      Set_Line_Wrap (Options.Label_Seed, False);
      Attach (Options.Value_Table, Options.Label_Seed,
         0, 1, 4, 5);

      Gtk_New (Options.Seed_Task, Options.Seed_Group,
         To_String(Lb_Task_Specific(Current_Language)));
      Options.Seed_Group := Group(Options.Seed_Task);
      if Task_Specific_Seed = True
            then
         Set_Active(Options.Seed_Task, True);
      else
         Set_Active(Options.Seed_Task, False);
      end if;
      Attach (Options.Value_Table, Options.Seed_Task,
         1, 2, 5, 6);

      Gtk_New (Options.Seed_Predictable, Options.Seed_Group,
         To_String(Lb_All_Predictable(Current_Language)));
      Options.Seed_Group := Group(Options.Seed_Predictable);
      if Task_Specific_Seed = True
            then
         Set_Active(Options.Seed_Predictable, False);
      else
         Set_Active(Options.Seed_Predictable, True);
      end if;
      Attach (Options.Value_Table, Options.Seed_Predictable,
         1, 2, 6, 7);

      Gtk.Gentry.Gtk_New(Options.Seed_Entry, 50);
      Add(Options.Hbox_Seed, Options.Seed_Entry);
      Set_Text(Options.Seed_Entry, Project.Common_Seed_Value'Img);

      Gtk.Button.Gtk_New(Options.Seed_Button,
         To_String(Lb_Seed(Current_Language)));
      Add(Options.Hbox_Seed, Options.Seed_Button);

      --       Gtk_New (Options.Seed_unpredictable, Options.Seed_Group,
      --                To_String(Lb_All_unpredictable(Current_Language)));
      --       Options.Seed_Group := Group(Options.Seed_unPredictable);
      --       if Seed_All_unpredictable = True
      --             then
      --          Set_Active(Options.Seed_unpredictable, True);
      --       else
      --          Set_Active(Options.Seed_unpredictable, False);
      --       end if;
      --       Attach (Options.Value_table, Options.Seed_unpredictable,
      --               1, 2, 7, 8);

      Gtk_New_Hseparator (Options.Hseparator);
      Pack_Start(Options.Options_Vbox,
         Options.Hseparator);

      Gtk_New (Options.Button_Hbox);
      Set_Layout(Options.Button_Hbox, Buttonbox_Spread);
      Set_Spacing(Options.Button_Hbox, 10);
      Set_Child_Size(Options.Button_Hbox, 20, 20);
      Set_Child_Ipadding(Options.Button_Hbox, 7, 0);
      Pack_Start (Options.Options_Vbox,
         Options.Button_Hbox,
         False, True, 5);

      Gtk_New (Options.Ok_Button, "Ok");
      Add (Options.Button_Hbox, Options.Ok_Button);
      Button_Callback.Connect
         (Options.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Gtk_New (Options.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Add (Options.Button_Hbox, Options.Cancel_Button);
      Button_Callback.Connect
         (Options.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));
      Button_Callback.Connect
         (Options.Seed_Button, "clicked",
         Button_Callback.To_Marshaller (On_Seed_Pressed'access));


   end Initialize;

end Options_Pkg;
