------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Glib;
use Glib;

with Gtk;
use Gtk;
with Gtk.Adjustment;
use Gtk.Adjustment;
with Gtk.Box;
use Gtk.Box;
with Gtk.Button;
use Gtk.Button;
with Gtk.Check_Button;
use Gtk.Check_Button;
with Gtk.Color_Selection;
use Gtk.Color_Selection;
with Gtk.Color_Selection_Dialog;
use Gtk.Color_Selection_Dialog;
with Gtk.Dialog;
use Gtk.Dialog;
with Gtk.Enums;
use Gtk.Enums;
with Gtk.Hbutton_Box;
use Gtk.Hbutton_Box;
with Gtk.Label;
use Gtk.Label;
with Gtk.Notebook;
use Gtk.Notebook;
with Gtk.Separator;
use Gtk.Separator;
with Gtk.Spin_Button;
use Gtk.Spin_Button;
with Gtk.Table;
use Gtk.Table;
with Gtk.Widget;
use Gtk.Widget;

with Gtkada.Intl;
use Gtkada.Intl;



package body Prefs_Dialog_Pkg is

   procedure Gtk_New (
         Prefs_Dialog :    out Prefs_Dialog_Access ) is 
   begin
      Prefs_Dialog := new Prefs_Dialog_Record;
      Prefs_Dialog_Pkg.Initialize (Prefs_Dialog);
   end Gtk_New;

   function Int_To_Hex_Str (
         Value : Integer ) 
     return String is 
   begin
      if (Value < 16) then
         return Hexa_List (Value);
      else
         return (Int_To_Hex_Str (Value / 16) & Hexa_List (Value mod 16));
      end if;
   end Int_To_Hex_Str;


   procedure Color_Sel_Dialog_Ok (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
      Label : Gtk_Label;  
      Color : Color_Array;  
      R,  
      G,  
      B     : Integer;  
   begin
      Destroy (Prefs_Dialog.Color_Sel_Dialog);
      Get_Color (Get_Colorsel (Prefs_Dialog.Color_Sel_Dialog),
         Color);

      R := Integer (Color (Red)   * Gdouble (255));
      G := Integer (Color (Green) * Gdouble (255));
      B := Integer (Color (Blue)  * Gdouble (255));

      case Prefs_Dialog.Col_Sel_Type is
         when Item     =>
            Label := Prefs_Dialog.Item_Color;
         when Font1 	  =>
            Label := Prefs_Dialog.Font1_Color;
         when Tasks    =>
            Label := Prefs_Dialog.Tasks_Color;
         when Timeline =>
            Label := Prefs_Dialog.Timeline_Color;
         when Font2    =>
            Label := Prefs_Dialog.Font2_Color;
      end case;

      Set_Text (Label, "RGB:" &
         Int_To_Hex_Str (R) & "/" &
         Int_To_Hex_Str (G) & "/" &
         Int_To_Hex_Str (B));
   end Color_Sel_Dialog_Ok;

   procedure Color_Sel_Dialog_Cancel (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Destroy (Prefs_Dialog.Color_Sel_Dialog);
   end Color_Sel_Dialog_Cancel;

   function Color_Sel_Dialog_Delete_Event (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) 
     return Gint is 
   begin
      Destroy (Prefs_Dialog.Color_Sel_Dialog);

      return 1; -- The destroy event mustn't be emitted
   end Color_Sel_Dialog_Delete_Event;

   procedure Show_Item_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Prefs_Dialog.Col_Sel_Type := Item;
      Show (Prefs_Dialog.Color_Sel_Dialog);
   end Show_Item_Color_Selection_Dialog;

   procedure Show_Font1_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Prefs_Dialog.Col_Sel_Type := Font1;
      Show (Prefs_Dialog.Color_Sel_Dialog);
   end Show_Font1_Color_Selection_Dialog;

   procedure Show_Tasks_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Prefs_Dialog.Col_Sel_Type := Tasks;
      Show (Prefs_Dialog.Color_Sel_Dialog);
   end Show_Tasks_Color_Selection_Dialog;

   procedure Show_Timeline_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Prefs_Dialog.Col_Sel_Type := Timeline;
      Show (Prefs_Dialog.Color_Sel_Dialog);
   end Show_Timeline_Color_Selection_Dialog;

   procedure Show_Font2_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Prefs_Dialog.Col_Sel_Type := Font2;
      Show (Prefs_Dialog.Color_Sel_Dialog);
   end Show_Font2_Color_Selection_Dialog;

   procedure Create_And_Attach (
         Spinbutton :    out Gtk_Spin_Button;        
         Lower      :        Gfloat;                 
         Current    :        Gfloat;                 
         Upper      :        Gfloat;                 
         Table      : access Gtk_Table_Record'Class; 
         Pos_X      :        Guint;                  
         Pos_Y      :        Guint                   ) is 
      Spinbutton_Adj : Gtk_Adjustment;  
      -- We need an Gtk_Adjustment for each 
      -- spinbutton otherwise all the spinbuttons would have the same value !
   begin
      Gtk_New (Spinbutton_Adj, Current, Lower, Upper, 1.0, 10.0, 10.0);
      Gtk.Spin_Button.Gtk_New (Spinbutton, Spinbutton_Adj, 1.0, 0);

      Set_Numeric (Spinbutton, True);
      Set_Snap_To_Ticks (Spinbutton, False);
      Set_Update_Policy (Spinbutton, Update_If_Valid);
      Set_Wrap (Spinbutton, False);
      Attach (Table, Spinbutton,
         Pos_X, Pos_X + 1, Pos_Y, Pos_Y + 1,
         Expand or Shrink or Fill,
         0, 0, 0);
   end;

   function Process_Prefs_Dialog_Delete_Event (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) 
     return Gint is 
   begin
      Destroy  (Prefs_Dialog);

      return 1;
   end Process_Prefs_Dialog_Delete_Event;

   procedure Ok_Clicked_Handler (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Destroy (Prefs_Dialog);

      -- Force the graphs to be redrawn the next time
      Prefs_Dialog.Graphs_Window_Prefs_Changed := True;
      Prefs_Dialog.Sched_Window_Prefs_Changed := True;
   end Ok_Clicked_Handler;

   procedure Cancel_Clicked_Handler (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
   begin
      Destroy (Prefs_Dialog);
   end Cancel_Clicked_Handler;

   procedure Initialize (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) is 
      Vbox,  
      Inner_Vbox     : Gtk_Vbox;  
      Hsep           : Gtk_Hseparator;  
      Action_Area    : Gtk_Hbox;  
      Hbuttonbox     : Gtk_Hbutton_Box;  
      Button         : Gtk_Button;  
      Notebook,  
      Inner_Notebook : Gtk_Notebook;  
      Table          : Gtk_Table;  
      Label          : Gtk_Label;  

   begin
      -- A dialog is a modal window
      Gtk.Dialog.Initialize (Prefs_Dialog);
      Set_Title (Prefs_Dialog, -("Preferences"));
      Set_Usize (Prefs_Dialog, Default_Window_Width, Default_Window_Height);

      Prefs_Dialog_Gint_Cb.Connect (Prefs_Dialog, "delete_event",
         Prefs_Dialog_Gint_Cb.To_Marshaller (
            Process_Prefs_Dialog_Delete_Event'access));

      -- with a vbox
      Vbox := Get_Vbox (Prefs_Dialog);
      Set_Homogeneous (Vbox, False);
      Set_Spacing (Vbox, 0);

      -- and an action area
      Action_Area := Get_Action_Area (Prefs_Dialog);
      Set_Border_Width (Action_Area, 10);
      Set_Homogeneous (Action_Area, True);
      Set_Spacing (Action_Area, 5);

      -- which contains buttons
      Gtk_New (Hbuttonbox);
      Set_Spacing (Hbuttonbox, 30);
      Set_Layout (Hbuttonbox, Buttonbox_End);
      Set_Child_Ipadding (Hbuttonbox, 7, 0);
      Pack_Start (Action_Area, Hbuttonbox, True, True, 0);

      Gtk_New (Button, -("Ok"));
      Set_Flags (Button, Can_Default);
      Prefs_Dialog_Cb.Object_Connect (Button, "clicked", Prefs_Dialog_Cb.
         To_Marshaller (Ok_Clicked_Handler'access), Prefs_Dialog);
      Add (Hbuttonbox, Button);

      Gtk_New (Button, -("Cancel"));
      Set_Flags (Button, Can_Default);
      Prefs_Dialog_Cb.Object_Connect (Button, "clicked", Prefs_Dialog_Cb.
         To_Marshaller (Cancel_Clicked_Handler'access), Prefs_Dialog);
      Add (Hbuttonbox, Button);

      -- the vbox contains a notebook
      Gtk_New (Notebook);
      Set_Scrollable (Notebook, True);
      Pack_Start (Vbox, Notebook, True, True, 0);

      -------------------------------------------------------------------------
      -- which first page contains an inner vbox
      Gtk_New_Vbox (Inner_Vbox);
      Add (Notebook, Inner_Vbox);

      -- which contains the common properties of all the graphs
      Gtk_New (Table, 3, 2, False);

      -- Items color
      Gtk_New (Label, -("Items color : "));
      Attach (Table, Label,
         0, 1, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Prefs_Dialog.Item_Color, "Forest green");
      -- Musn't be translated !
      Attach (Table, Prefs_Dialog.Item_Color,
         1, 2, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Button, -("Change"));
      Prefs_Dialog_Cb.Object_Connect
         (Button, "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Show_Item_Color_Selection_Dialog'access),
         Prefs_Dialog);
      Attach (Table, Button,
         2, 3, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      -- Font color
      Gtk_New (Label, -("Font color : "));
      Attach (Table, Label,
         0, 1, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Prefs_Dialog.Font1_Color, "Red");
      -- Musn't be translated !
      Attach (Table, Prefs_Dialog.Font1_Color,
         1, 2, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Button, -("Change"));
      Prefs_Dialog_Cb.Object_Connect
         (Button, "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Show_Font1_Color_Selection_Dialog'access),
         Prefs_Dialog);
      Attach (Table, Button,
         2, 3, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Pack_Start (Inner_Vbox, Table, False, False, 0);

      Gtk_New_Hseparator (Hsep);
      Pack_Start (Inner_Vbox, Hsep, False, False, 5);

      -- and an inner notebook
      Gtk_New (Inner_Notebook);
      Set_Tab_Pos (Inner_Notebook, Pos_Right);
      Pack_Start (Inner_Vbox, Inner_Notebook, True, True, 0);

      -- the table needed for the inner notebook first page
      Gtk_New (Table, 2, 2, False);
      Add (Inner_Notebook, Table);

      Gtk_New (Label, -("Space allocated for borders : "));
      Attach (Table, Label,
         0, 1, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Create_And_Attach (Prefs_Dialog.G1_Sb_Borders_Space,
         0.0, 50.0, 100.0, Table, 1, 0);

      Gtk_New (Label, -("Space between items : "));
      Attach (Table, Label,
         0, 1, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Create_And_Attach (Prefs_Dialog.G1_Sb_Space_Between_Items,
         50.0, 100.0, 200.0, Table, 1, 1);

      -- the inner notebook first page
      Gtk_New (Label, -("Graph 1"));
      Set_Tab (Inner_Notebook, 0, Label);

      -- the table needed for the inner notebook second page
      Gtk_New (Table, 2, 2, False);
      Add (Inner_Notebook, Table);

      Gtk_New (Label, -("Space allocated for borders : "));
      Attach (Table, Label,
         0, 1, 0, 1,
         Fill or Expand or Shrink,
         0, 0, 0);

      Create_And_Attach (Prefs_Dialog.G2_Sb_Borders_Space,
         0.0, 50.0, 100.0, Table, 1, 0);

      Gtk_New (Prefs_Dialog.G2_Cb_Force_Circle, -(
            "Constraints to use a circle"));
      Set_Active (Prefs_Dialog.G2_Cb_Force_Circle, False);
      Attach (Table, Prefs_Dialog.G2_Cb_Force_Circle,
         0, 2, 1, 2,
         Fill or Expand or Shrink,
         0, 0, 0);

      -- the inner notebook first page
      Gtk_New (Label, -("Graph 2"));
      Set_Tab (Inner_Notebook, 1, Label);

      -- the table needed for the inner notebook third page
      Gtk_New (Table, 2, 2, False);
      Add (Inner_Notebook, Table);

      Gtk_New (Label, -("Space allocated for borders : "));
      Attach (Table, Label,
         0, 1, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Create_And_Attach (Prefs_Dialog.G3_Sb_Borders_Space,
         0.0, 50.0, 100.0, Table, 1, 0);

      Gtk_New (Label, -("Space between items :"));
      Attach (Table, Label,
         0, 1, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Create_And_Attach (Prefs_Dialog.G3_Sb_Space_Between_Items,
         50.0, 100.0, 200.0, Table, 1, 1);

      -- the inner notebook third page (feww, the last page has been done :)
      Gtk_New (Label, -("Graph 3"));
      Set_Tab (Inner_Notebook, 2, Label);

      -- so we've done with the first main notebook's page
      Gtk_New (Label, -("Graphs"));
      Set_Tab (Notebook, 0, Label);

      -------------------------------------------------------------------------
      -- the second page of the main notebook, which contains an inner vbox
      Gtk_New_Vbox (Inner_Vbox);
      Add (Notebook, Inner_Vbox);

      -- which contains the color properties
      Gtk_New (Table, 3, 3, False);

      -- Tasks color
      Gtk_New (Label, -("Tasks color : "));
      Attach (Table, Label,
         0, 1, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Prefs_Dialog.Tasks_Color, "Green");
      -- Musn't be translated !
      Attach (Table, Prefs_Dialog.Tasks_Color,
         1, 2, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Button, -("Change"));
      Prefs_Dialog_Cb.Object_Connect
         (Button, "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Show_Tasks_Color_Selection_Dialog'access),
         Prefs_Dialog);
      Attach (Table, Button,
         2, 3, 0, 1,
         Expand or Shrink or Fill,
         0, 0, 0);

      -- Timeline color
      Gtk_New (Label, -("Timeline color : "));
      Attach (Table, Label,
         0, 1, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Prefs_Dialog.Timeline_Color, "Black");
      -- Musn't be translated !
      Attach (Table, Prefs_Dialog.Timeline_Color,
         1, 2, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Button, -("Change"));
      Prefs_Dialog_Cb.Object_Connect
         (Button, "clicked",
         Prefs_Dialog_Cb.To_Marshaller (
            Show_Timeline_Color_Selection_Dialog'access),
         Prefs_Dialog);
      Attach (Table, Button,
         2, 3, 1, 2,
         Expand or Shrink or Fill,
         0, 0, 0);

      -- Font color
      Gtk_New (Label, -("Font color : "));
      Attach (Table, Label,
         0, 1, 2, 3,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Prefs_Dialog.Font2_Color, "Black");
      -- Musn't be translated !
      Attach (Table, Prefs_Dialog.Font2_Color,
         1, 2, 2, 3,
         Expand or Shrink or Fill,
         0, 0, 0);

      Gtk_New (Button, -("Change"));
      Prefs_Dialog_Cb.Object_Connect
         (Button, "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Show_Font2_Color_Selection_Dialog'access),
         Prefs_Dialog);
      Attach (Table, Button,
         2, 3, 2, 3,
         Expand or Shrink or Fill,
         0, 0, 0);

      -- Pack this table at the beginning of the inner vbox
      Pack_Start (Inner_Vbox, Table, False, False, 0);

      -- And an horizontal separator
      Gtk_New_Hseparator (Hsep);
      Pack_Start (Inner_Vbox, Hsep, False, False, 5);

      -- Before we construct another table (pfuiii ...)
      Gtk_New (Table, 2, 4, False);

      Gtk_New (Label, -("Space between tasks : "));
      Attach (Table, Label, 0, 1, 0, 1, Expand or Shrink or Fill, 0, 0, 0);
      Create_And_Attach (Prefs_Dialog.Sb_Space_Between_Tasks,
         40.0, 40.0, 100.0, Table, 1, 0);


      Gtk_New (Label, -("Scale value : "));
      Attach (Table, Label, 0, 1, 1, 2, Expand or Shrink or Fill, 0, 0, 0);
      Create_And_Attach (Prefs_Dialog.Sb_Scale,
         0.0, 100.0, 1000.0, Table, 1, 1);

      Gtk_New (Label, -("Arrows size : "));
      Attach (Table, Label, 0, 1, 2, 3, Expand or Shrink or Fill, 0, 0, 0);
      Create_And_Attach (Prefs_Dialog.Sb_Arrow_Size,
         4.0, 4.0, 8.0, Table, 1, 2);

      Gtk_New (Label, -("Delimiters size : "));
      Attach (Table, Label, 0, 1, 3, 4, Expand or Shrink or Fill, 0, 0, 0);
      Create_And_Attach (Prefs_Dialog.Sb_Delimiter_Size,
         7.0, 7.0, 15.0, Table, 1, 3);

      -- Finally add the inner_vbox in the main notebook
      Pack_Start (Inner_Vbox, Table, True, True, 5);

      Gtk_New (Label, -("Scheduling"));
      Set_Tab (Notebook, 1, Label);

      -------------------------------------------------------------------------
      -- its last page don't contains a lot of things ;)
      Gtk_New (Label, -("Nothing for now on :)"));
      Add (Notebook, Label);

      Gtk_New (Label, -("Network"));
      Set_Tab (Notebook, 2, Label);

      -- Init the changed states
      Prefs_Dialog.Graphs_Window_Prefs_Changed := False;
      Prefs_Dialog.Sched_Window_Prefs_Changed := False;

      -- Create the color selection dialog
      Gtk_New (Prefs_Dialog.Color_Sel_Dialog, -("Select a color"));
      Destroy (Get_Help_Button (Prefs_Dialog.Color_Sel_Dialog));

      Prefs_Dialog_Gint_Cb.Object_Connect
         (Prefs_Dialog.Color_Sel_Dialog, "delete_event",
         Prefs_Dialog_Gint_Cb.To_Marshaller (
            Color_Sel_Dialog_Delete_Event'access),
         Prefs_Dialog);

      Prefs_Dialog_Cb.Object_Connect
         (Get_Ok_Button (Prefs_Dialog.Color_Sel_Dialog),
         "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Color_Sel_Dialog_Ok'access),
         Prefs_Dialog);

      Prefs_Dialog_Cb.Object_Connect
         (Get_Cancel_Button (Prefs_Dialog.Color_Sel_Dialog),
         "clicked",
         Prefs_Dialog_Cb.To_Marshaller (Color_Sel_Dialog_Cancel'access),
         Prefs_Dialog);

      Show_All (Vbox);
      Show_All (Action_Area);
   end Initialize;

end Prefs_Dialog_Pkg;
