------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Gtk.Check_Button;
use Gtk.Check_Button;
with Gtk.Color_Selection_Dialog;
use Gtk.Color_Selection_Dialog;
with Gtk.Dialog;
use Gtk.Dialog;
with Gtk.Label;
use Gtk.Label;
with Gtk.Table;
use Gtk.Table;
with Gtk.Spin_Button;
use Gtk.Spin_Button;
with Gtk.Handlers;


package Prefs_Dialog_Pkg is
   -- Color type specification
   type Color_Type is 
         (Item,     
          Font1,    -- Graphs view
          Tasks,    
          Timeline, 
          Font2); -- Scheduling view

   -- Prefs_Dialog specification

   -- Note : We Can't tag private the record, since the compiler would know
   -- it's a Prefs_Dialog_Record but wouldn't directly know it's a
   -- Gtk_Dialog_Record too, so Gtk_Widget_Show can't be used for instance.
   type Prefs_Dialog_Record is new Gtk_Dialog_Record with 
      record 
         -- First view properties
         G1_Sb_Space_Between_Items : Gtk_Spin_Button;  
         G1_Sb_Borders_Space       : Gtk_Spin_Button;  
         G2_Sb_Borders_Space       : Gtk_Spin_Button;  
         G2_Cb_Force_Circle        : Gtk_Check_Button;  
         G3_Sb_Space_Between_Items : Gtk_Spin_Button;  
         G3_Sb_Borders_Space       : Gtk_Spin_Button;  

         -- Second view properties
         Sb_Space_Between_Tasks : Gtk_Spin_Button;  
         Sb_Scale               : Gtk_Spin_Button;  
         Sb_Arrow_Size          : Gtk_Spin_Button;  
         Sb_Delimiter_Size      : Gtk_Spin_Button;  

         Col_Sel_Type : Color_Type;  

         Item_Color     : Gtk_Label;  
         Font1_Color    : Gtk_Label;  
         Tasks_Color    : Gtk_Label;  
         Timeline_Color : Gtk_Label;  
         Font2_Color    : Gtk_Label;  

         Color_Sel_Dialog : Gtk_Color_Selection_Dialog;  

         Graphs_Window_Prefs_Changed : Boolean;  
         Sched_Window_Prefs_Changed  : Boolean;  
      end record; 

   type Prefs_Dialog_Access is access all Prefs_Dialog_Record'Class; 

   -- Methods
   procedure Gtk_New (
         Prefs_Dialog :    out Prefs_Dialog_Access ); 

   Prefs_Dialog : Prefs_Dialog_Access;  

private
   -- Default window size
   Default_Window_Width  : constant Gint := 300;  
   Default_Window_Height : constant Gint := 400;  

   -- Hexadecimal code list
   Hexa_List : constant
   array (0..15) of String (1..1) :=
      ("0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
      "A", "B", "C", "D", "E", "F");

   -- Callbacks packages
   package Prefs_Dialog_Cb is new Gtk.Handlers.Callback (
      Prefs_Dialog_Record);
   package Prefs_Dialog_Gint_Cb is new Gtk.Handlers.Return_Callback (
      Prefs_Dialog_Record, Gint);

   -- Methods
   procedure Initialize (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   function Process_Prefs_Dialog_Delete_Event (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) 
     return Gint; 
   procedure Ok_Clicked_Handler (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   procedure Cancel_Clicked_Handler (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 

   function Int_To_Hex_Str (
         Value : Integer ) 
     return String; 

   function Color_Sel_Dialog_Delete_Event (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ) 
     return Gint; 
   procedure Color_Sel_Dialog_Cancel (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   procedure Color_Sel_Dialog_Ok (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 

   procedure Show_Item_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   procedure Show_Font1_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 

   procedure Show_Tasks_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   procedure Show_Timeline_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 
   procedure Show_Font2_Color_Selection_Dialog (
         Prefs_Dialog : access Prefs_Dialog_Record'Class ); 

   procedure Create_And_Attach (
         Spinbutton :    out Gtk_Spin_Button;        
         Lower      :        Gfloat;                 
         Current    :        Gfloat;                 
         Upper      :        Gfloat;                 
         Table      : access Gtk_Table_Record'Class; 
         Pos_X      :        Guint;                  
         Pos_Y      :        Guint                   ); 


end Prefs_Dialog_Pkg;
