------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk.Check_Button;
use Gtk.Check_Button;
with Scrolling_Editor_Pkg;
use Scrolling_Editor_Pkg;
with Read_Scheduler_From_File_Pkg;
use Read_Scheduler_From_File_Pkg;

with Mmi_Tools;
use Mmi_Tools;

with Project;
use Project;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Scheduler;
use Scheduler;
with Scheduler.Dynamic_Priority.Edf;
use Scheduler.Dynamic_Priority.Edf;
with Scheduler.Fixed_Priority.Rm;
use Scheduler.Fixed_Priority.Rm;
with Scheduler.Dynamic_Priority.Llf;
use Scheduler.Dynamic_Priority.Llf;
with Scheduler.Fixed_Priority.Dm;
use Scheduler.Fixed_Priority.Dm;
with Scheduler.Fixed_Priority.Hpf;
use Scheduler.Fixed_Priority.Hpf;
with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Config;
use Config;

with Translate;
use Translate;

with Unbounded_Strings;
use Unbounded_Strings;

with  Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Expressions;
use Expressions;

with Text_Io;
use Text_Io;


package body Processors_Parameters_Pkg.Callbacks is


   procedure On_Edit_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Sched : Gint;  

   begin

      Sched:= Gint'Val (5 -
         Gint(Selected_Button (Processors_Parameters.Clist_Scheduler_Menu)));

      if (Sched/=5)
            then
         Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
         return;
      end if;


      if Scrolling_Editor/=null then
         if Scrolling_Editor.Is_Displayed=True
               then
            Show_Message_Box(
               Lb_Can_Not_Edit_To_Parametric_Object(Current_Language));
            return;
         end if;
      end if;


      Gtk_New (Scrolling_Editor,
         Lb_Edit(Current_Language) &
         Unbounded_Sp &
         Lb_Scheduler_Behavior(Current_Language),
         Processors_Parameters.String_Behavior);
      Set_Usize(Scrolling_Editor,800,450);
      Show_All (Scrolling_Editor);
   end On_Edit_Pressed;





   procedure On_Read_From_File_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Sched : Gint;  

   begin

      Sched:= Gint'Val (5 - Gint(Selected_Button (Processors_Parameters.
               Clist_Scheduler_Menu)));

      if (Sched/=5)
            then
         Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
         return;
      end if;

      Gtk_New(Read_Scheduler_From_File,
         Processors_Parameters.String_Behavior, Processors_Parameters.
         Behavior_File_Name);
      Show_All (Read_Scheduler_From_File);
   end On_Read_From_File_Pressed;


   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Name           : Unbounded_String;  
      Sched          : Gint;  
      A_Scheduler    : Scheduler_Ptr;  
      Quantum        : Integer;  
      Quantum_String : Unbounded_String;  
      Ok             : Boolean;  


   begin


      Name:=To_Unbounded_String(Get_Text(Processors_Parameters.Entry_Name));



      if Name=""
            then
         Show_Message_Box(
            Lb_Processor_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         return;
      end if;

      if not Is_A_Valid_Identifier(Name)
            then
         Show_Message_Box(Lb_Processor_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;


      Sched:= Gint'Val (5 -
         Gint(Selected_Button (Processors_Parameters.Clist_Scheduler_Menu)));


      if (Sched=0)
            then
         A_Scheduler := new Edf_Scheduler;
      end if;
      if (Sched=1)
            then
         A_Scheduler := new Llf_Scheduler;
      end if;
      if (Sched=2)
            then
         A_Scheduler := new Rm_Scheduler;
      end if;
      if (Sched=3)
            then
         A_Scheduler := new Dm_Scheduler;
      end if;
      if (Sched=4)
            then
         A_Scheduler := new Hpf_Scheduler;
      end if;
      if (Sched=5)
            then
         A_Scheduler := new Parametric_Scheduler;

         begin
            Set_String_Behavior ( Parametric_Scheduler(A_Scheduler.all),
               Processors_Parameters.String_Behavior.all ) ;
            Set_Behavior_File_Name ( Parametric_Scheduler(A_Scheduler.all),
               Processors_Parameters.Behavior_File_Name.all ) ;


         exception
            when Expressions.Syntax_Error=>
               Show_Message_Box(Lb_Scheduler_Behavior(Current_Language) &
                  Lb_Colon & Exception_Message & To_Unbounded_String("  ") );
               return;
         end;
      end if;

      Quantum_String:=To_Unbounded_String(Get_Text(Processors_Parameters.
            Entry_Quantum));
      if(Quantum_String="")
            then
         Quantum:=0;
      else
         To_Integer(Quantum_String,Quantum,Ok);
         if not Ok
               then
            Show_Message_Box("Quantum" &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      if (Quantum/=0) and
            (Sched/=4)
            then
         Show_Message_Box(Lb_Quantum_Control(Current_Language));
         return;
      end if;


      if (Quantum<0)
            then
         Show_Message_Box("Quantum" &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;


      Set_Quantum(A_Scheduler.all, Natural(Quantum));


      if Get_Active(Processors_Parameters.Entry_Option)
            then
         Set_Preemptive(A_Scheduler.all, Preemptive);
      else
         Set_Preemptive(A_Scheduler.all, Not_Preemptive);
      end if;

      Add_Processor(Processors_Parameters.Sys.Processors,Name,A_Scheduler);


   exception
      when Processor_Already_Defined =>
         begin
            Show_Message_Box(Lb_Processor_Name(Current_Language)
               & Lb_Already_Defined(Current_Language));
            return;
         end;


   end On_Add_Pressed;




   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Sys:=Processors_Parameters.Sys;

      Is_New:=False;
      Is_Saved:=False;

      Hide(Processors_Parameters);

   end On_Ok_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide(Processors_Parameters);
   end On_Cancel_Pressed;

end Processors_Parameters_Pkg.Callbacks;
