------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;
use Glib;
with Gtk;
use Gtk;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;
with Gdk.Types;
use Gdk.Types;

with Callbacks_Cheddar;
use Callbacks_Cheddar;

with Project;
use Project;

with Processors_Parameters_Pkg.Callbacks;
use Processors_Parameters_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Mmi_Tools;
use Mmi_Tools;


package body Processors_Parameters_Pkg is

   procedure Gtk_New (
         Processors_Parameters :    out Processors_Parameters_Access ) is 

   begin
      Processors_Parameters := new Processors_Parameters_Record;
      Processors_Parameters_Pkg.Initialize (Processors_Parameters);
   end Gtk_New;

   procedure Initialize (
         Processors_Parameters : access Processors_Parameters_Record'Class ) is 


      Items : Strings_Table;  


   begin


      Processors_Parameters.String_Behavior:=new Unbounded_String;
      Processors_Parameters.Behavior_File_Name:=new Unbounded_String;

      Processors_Parameters.Sys:=Sys;

      Initialize(Items);

      Add(Items,To_Unbounded_String("Earliest Deadline First"));
      Add(Items,To_Unbounded_String("Least Laxity First"));
      Add(Items,To_Unbounded_String("Rate Monotonic"));
      Add(Items,To_Unbounded_String("Deadline Monotonic"));
      Add(Items,To_Unbounded_String("Highiest Priority First/POSIX 4"));
      Add(Items,To_Unbounded_String("Parametric Scheduler"));


      Gtk.Window.Initialize (Processors_Parameters, Window_Toplevel);
      Set_Title (Processors_Parameters, To_String(Lb_Add_Processor(
               Current_Language)));
      Set_Policy (Processors_Parameters, True, True, True);
      Set_Position (Processors_Parameters, Win_Pos_Center_Always);
      Set_Modal (Processors_Parameters, True);
      Set_Extension_Events (Processors_Parameters, Extension_Events_All);
      Set_Flags (Processors_Parameters, Can_Default);
      Grab_Focus (Processors_Parameters);


      Gtk_New (Processors_Parameters.Vbox, 4, 2, True);
      Set_Border_Width (Processors_Parameters.Vbox, 14);
      Set_Row_Spacings (Processors_Parameters.Vbox, 8);
      Set_Col_Spacings (Processors_Parameters.Vbox, 0);
      Add (Processors_Parameters, Processors_Parameters.Vbox);

      Gtk_New (Processors_Parameters.Name,  To_String(Lb_Processor_Name(
               Current_Language) & Lb_Colon));
      Set_Alignment (Processors_Parameters.Name, 0.5, 0.5);
      Set_Padding (Processors_Parameters.Name, 0, 0);
      Set_Justify (Processors_Parameters.Name, Justify_Left);
      Set_Line_Wrap (Processors_Parameters.Name, False);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.Name, 0, 1,
         0, 1,
         0, 0,
         0, 0);

      Gtk_New (Processors_Parameters.Scheduler, To_String(Lb_Scheduler(
               Current_Language) & Lb_Colon));
      Set_Alignment (Processors_Parameters.Scheduler, 0.5, 0.5);
      Set_Padding (Processors_Parameters.Scheduler, 0, 0);
      Set_Justify (Processors_Parameters.Scheduler, Justify_Left);
      Set_Line_Wrap (Processors_Parameters.Scheduler, False);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.Scheduler,
         0, 1, 1, 2,
         0, 0,
         0, 0);

      Gtk_New (Processors_Parameters.Quantum, "Quantum : ");
      Set_Alignment (Processors_Parameters.Quantum, 0.5, 0.5);
      Set_Padding (Processors_Parameters.Quantum, 0, 0);
      Set_Justify (Processors_Parameters.Quantum, Justify_Left);
      Set_Line_Wrap (Processors_Parameters.Quantum, False) ;
      Attach (Processors_Parameters.Vbox, Processors_Parameters.Quantum,
         0, 1, 2, 3,
         0, 0,
         0, 0);

      Gtk_New (Processors_Parameters.Option, To_String(Lb_Option(
               Current_Language) & Lb_Colon));
      Set_Alignment (Processors_Parameters.Option, 0.5, 0.5);
      Set_Padding (Processors_Parameters.Option, 0, 0);
      Set_Justify (Processors_Parameters.Option, Justify_Left);
      Set_Line_Wrap (Processors_Parameters.Option, False);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.Option, 0,
         1, 3, 4,
         0, 0,
         0, 0);


      --
      --

      Gtk_New (Processors_Parameters.Entry_Name);
      Set_Editable (Processors_Parameters.Entry_Name, True);
      Set_Max_Length (Processors_Parameters.Entry_Name, 0);
      Set_Text (Processors_Parameters.Entry_Name, "");
      Set_Visibility (Processors_Parameters.Entry_Name, True);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Entry_Name, 1, 2, 0, 1,
         Expand or Fill, 0,
         0, 0);


      Processors_Parameters.Clist_Scheduler_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Processors_Parameters.Scheduler_Menu,
         Processors_Parameters.Clist_Scheduler_Menu, Items, -1, null);

      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Scheduler_Menu, 1, 2, 1, 2,
         Expand or Fill, 0,
         0, 0);



      Gtk_New (Processors_Parameters.Entry_Quantum);
      Set_Editable (Processors_Parameters.Entry_Quantum, True);
      Set_Max_Length (Processors_Parameters.Entry_Quantum, 0);
      Set_Text (Processors_Parameters.Entry_Quantum, "0");
      Set_Visibility (Processors_Parameters.Entry_Quantum, True);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Entry_Quantum, 1, 2, 2, 3,
         Expand or Fill, 0,
         0, 0);


      Gtk_New (Processors_Parameters.Entry_Option, To_String(
            Lb_Preemptive(Current_Language)));
      Set_Active(Processors_Parameters.Entry_Option, True);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Entry_Option, 1, 2, 3, 4,
         Expand or Fill, 0,
         0, 0);


      --
      --

      Gtk_New_Hbox (Processors_Parameters.Hbox, True, 0);
      Set_Border_Width (Processors_Parameters.Hbox, 6);

      Attach (Processors_Parameters.Vbox, Processors_Parameters.Hbox, 0, 2,
         4, 5,
         Fill, Expand or Fill,
         0, 0);


      Gtk_New (Processors_Parameters.Behavior, To_String(
            Lb_Scheduler_Behavior(
               Current_Language) & Lb_Colon));
      Set_Alignment (Processors_Parameters.Behavior, 0.5, 0.5);
      Set_Padding (Processors_Parameters.Behavior, 0, 0);
      Set_Justify (Processors_Parameters.Behavior, Justify_Left);
      Set_Line_Wrap (Processors_Parameters.Behavior, False);
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Behavior , True, True, 0);


      Gtk_New (Processors_Parameters.Edit_Behavior, To_String(Lb_Edit(
               Current_Language)));
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Edit_Behavior, True, True, 0);
      Button_Callback.Connect
         (Processors_Parameters.Edit_Behavior, "clicked",
         Button_Callback.To_Marshaller (On_Edit_Pressed'access));


      Gtk_New (Processors_Parameters.Read_From_File_Behavior, To_String(
            Lb_Read_From_File(Current_Language)));
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Read_From_File_Behavior, True, True, 0);
      Button_Callback.Connect
         (Processors_Parameters.Read_From_File_Behavior, "clicked",
         Button_Callback.To_Marshaller (On_Read_From_File_Pressed'access));



      --
      --


      Gtk_New_Hseparator (Processors_Parameters.Hseparator5);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Hseparator5, 1, 2, 5, 6,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New_Hseparator (Processors_Parameters.Hseparator6);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.
         Hseparator6, 0, 1, 5, 6,
         Fill, Fill,
         0, 0);

      Gtk_New_Hbox (Processors_Parameters.Hbox, True, 0);
      Set_Border_Width (Processors_Parameters.Hbox, 6);
      Attach (Processors_Parameters.Vbox, Processors_Parameters.Hbox, 0, 2,
         6, 7,
         Fill, Expand or Fill,
         0, 0);



      Gtk_New (Processors_Parameters.Ok_Button, To_String(Lb_Close(
               Current_Language)));
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Ok_Button, True, True, 0);
      Button_Callback.Connect
         (Processors_Parameters.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));


      Gtk_New_Vseparator (Processors_Parameters.Vseparator1);
      Set_Sensitive (Processors_Parameters.Vseparator1, False);
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Vseparator1,  True, True, 0);


      Gtk_New (Processors_Parameters.Add_Button, To_String(Lb_Add(
               Current_Language)));
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Add_Button, True, True, 0);
      Button_Callback.Connect
         (Processors_Parameters.Add_Button, "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'access));



      Gtk_New_Vseparator (Processors_Parameters.Vseparator1);
      Set_Sensitive (Processors_Parameters.Vseparator1, False);
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Vseparator1,  True, True, 0);


      Gtk_New (Processors_Parameters.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Processors_Parameters.Hbox, Processors_Parameters.
         Cancel_Button, True, True, 0);
      Button_Callback.Connect
         (Processors_Parameters.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));


   end Initialize;

end Processors_Parameters_Pkg;
