------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Main;
use Gtk.Main;
with Gtkada.Dialogs;
use Gtkada.Dialogs;


with Mmi_Tools;
use Mmi_Tools;

with Ada.Io_Exceptions;
use Ada.Io_Exceptions;


with Open_File_Selection_Pkg;
use Open_File_Selection_Pkg;
with Save_File_Selection_Pkg;
use Save_File_Selection_Pkg;
with Root_Window_Pkg;
use Root_Window_Pkg;
with Root_Window_Pkg.Callbacks;
use Root_Window_Pkg.Callbacks;
with Call_Scheduling_Framework;
use Call_Scheduling_Framework;
with My_Canvas;
use My_Canvas;
with Graph_Window_Pkg;
use Graph_Window_Pkg;

with Config;
use Config;
with Translate;
use Translate;


with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;




package body Project is


   procedure Do_New is 
   begin

      if Root_Window /= null then

         if (not Is_Saved)
               then
            if (not Is_New)
                  then
               if Message_Dialog (To_String(
                        Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                        Lb_Continue(Current_Language)),
                     Confirmation,
                     Button_Yes or Button_Cancel,
                     Button_Cancel, -- default
                     "", -- help message
                     "") /= Button_Yes then
                  return;
               end if;
            end if;
         end if;
         Clear_Work_Space;

      end if;

      Initialize(Sys);
      File_Name:=To_Unbounded_String("");
      Is_Saved:=False;
      Is_New:=True;


      Schedule_With_Offsets:=True;
      Schedule_With_Resources:=False;
      Schedule_With_Precedencies:=False;


      Initialize(Call_Scheduling_Framework.Sched.all);

   end Do_New;


   procedure Do_Open is 
   begin

      if (not Is_Saved)
            then
         if (not Is_New)
               then
            if Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes or Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") /= Button_Yes then
               return;
            end if;
         end if;
      end if;

      Clear_Work_Space;

      Gtk_New (Open_File_Selection);
      Show_All (Open_File_Selection);


   exception
      when Xml_Read_Error =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Xml Fatal Error"));
      when Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      when Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      when Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      when Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      when Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      when Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", End_Error"));
      when Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      when Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      when others =>
         Show_Message_Box(Lb_Can_Not_Open_Project(Current_Language));

   end  Do_Open;

   procedure Do_Save is 
   begin


      if Graph_Window/=null
            then
         Get_Coordinates(Graph_Window.Buffer, Sys);
      end if;

      if (File_Name /= To_Unbounded_String(""))
            then
         Systems.Write_To_Xml_File(Sys, File_Name);

         Is_Saved:=True;
      else
         Gtk_New (Save_File_Selection);
         Show_All (Save_File_Selection);
      end if;

   exception
      when Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      when Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      when Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      when Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      when Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      when Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", End_Error"));
      when Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      when Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      when others =>
         Show_Message_Box(Lb_Can_Not_Save_Project(Current_Language));

   end  Do_Save;



   procedure Do_Save_As is 
   begin
      Gtk_New (Save_File_Selection);
      Show_All (Save_File_Selection);
   end  Do_Save_As;



   procedure Do_Exit is 
   begin
      if (not Is_Saved)
            then
         if (not Is_New)
               then
            if Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes or Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") = Button_Yes then
               Gtk.Main.Gtk_Exit (0);
            else
               return;
            end if;
         end if;
      end if;
      Gtk.Main.Gtk_Exit (0);
   end  Do_Exit;


end Project;

