------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Widget;
use Gtk.Widget;

with Mmi_Tools;
use Mmi_Tools;

with Config;
use Config;

with Translate;
use Translate;

with Io_Tools;
use Io_Tools;

with Ada.Io_Exceptions;
use Ada.Io_Exceptions;



package body Read_Scheduler_From_File_Pkg.Callbacks is


   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin
      Hide (Read_Scheduler_From_File);

      Read_Scheduler_From_File.Read_File_Name.all := To_Unbounded_String(
         Get_Filename (
            Read_Scheduler_From_File));

      Read_Scheduler_From_File.Read_Behavior.all:=Read_Sequential_File(
         Read_Scheduler_From_File.Read_File_Name.all);


   exception
      when Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      when Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      when Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      when Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      when Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      when Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", End_Error"));
      when Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      when Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      when others =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language));

   end On_Ok_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide (Get_Toplevel (Gtk_Widget (Object)));
   end On_Cancel_Pressed;



end Read_Scheduler_From_File_Pkg.Callbacks;
