------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Read_Scheduler_From_File_Pkg.Callbacks;
use Read_Scheduler_From_File_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;
with Project;
use Project;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


package body Read_Scheduler_From_File_Pkg is

   procedure Gtk_New (
         Read_Scheduler_From_File :    out Read_Scheduler_From_File_Access; 
         Read_Behavior            : in     Unbounded_String_Ptr;            
         Read_File_Name           : in     Unbounded_String_Ptr             ) is 
   begin
      Read_Scheduler_From_File := new Read_Scheduler_From_File_Record;
      Read_Scheduler_From_File_Pkg.Initialize (Read_Scheduler_From_File,
         Read_Behavior, Read_File_Name);
   end Gtk_New;

   procedure Initialize (
         Read_Scheduler_From_File : access Read_Scheduler_From_File_Record'Class; 
         Read_Behavior            : in     Unbounded_String_Ptr;                  
         Read_File_Name           : in     Unbounded_String_Ptr                   ) is 
   begin

      Read_Scheduler_From_File.Read_Behavior:=Read_Behavior;
      Read_Scheduler_From_File.Read_File_Name:=Read_File_Name;

      Gtk.File_Selection.Initialize (Read_Scheduler_From_File, To_String(
            Lb_Read_From_File(Current_Language)));
      Set_Show_File_Op_Buttons (Read_Scheduler_From_File, True);
      Set_Border_Width (Read_Scheduler_From_File, 10);
      Set_Title (Read_Scheduler_From_File, To_String(Lb_Read_From_File(
               Current_Language)));
      Set_Policy (Read_Scheduler_From_File, False, True, False);
      Set_Position (Read_Scheduler_From_File, Win_Pos_Center_Always);
      Set_Modal (Read_Scheduler_From_File, True);

      Read_Scheduler_From_File.Ok_Button := Get_Ok_Button (
         Read_Scheduler_From_File);
      Set_Flags (Read_Scheduler_From_File.Ok_Button, Can_Default);
      Button_Callback.Connect
         (Read_Scheduler_From_File.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Read_Scheduler_From_File.Cancel_Button := Get_Cancel_Button (
         Read_Scheduler_From_File);
      Set_Flags (Read_Scheduler_From_File.Cancel_Button, Can_Default);
      Button_Callback.Connect
         (Read_Scheduler_From_File.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (
            On_Cancel_Pressed'access));


      if Current_Directory=To_Unbounded_String("")
            then
         Current_Directory:=To_Unbounded_String(Get_Filename(
               Read_Scheduler_From_File));
      else
         Set_Filename(Read_Scheduler_From_File,To_String(
               Current_Directory));
      end if;


   end Initialize;

end Read_Scheduler_From_File_Pkg;
