------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use Text_Io;

with Glib;
use Glib;

with Unbounded_Strings;
use Unbounded_Strings;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Mmi_Tools;
use Mmi_Tools;

with Translate;
use Translate;

with Resources;
use Resources;
use Resources.Task_Lists;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Config;
use Config;

with Project;
use Project;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;


package body Resources_Parameters_Pkg.Callbacks  is

   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin

      Destroy(Resource_Parameter);

   end  On_Cancel_Pressed;

   procedure On_Affect_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Affect_Task_Share  : Affected_Task_List;  
      Task_Name          : Unbounded_String;  
      Share_Begin_String : Unbounded_String;  
      Share_End_String   : Unbounded_String;  
      Share_Begin        : Integer                  := 0;  
      Share_End          : Integer                  := 0;  
      Ok                 : Boolean;  
      Line               : Gint;  
      Task_Share         : Chars_Ptr_Array (1 .. 3);  

      A_Task : Task_Ptr;  
      Ite2   : Tasks_Iterator;  

   begin
      Task_Name := To_Unbounded_String(Get_Text(Get_Entry(
               Resource_Parameter.Resource_Affect_Task)));
      Share_Begin_String := To_Unbounded_String(Get_Text(
            Resource_Parameter.Resource_Affect_Begin));
      Share_End_String := To_Unbounded_String(Get_Text(Resource_Parameter.
            Resource_Affect_End));


      To_Integer(Share_Begin_String, Share_Begin, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;

      To_Integer(Share_End_String, Share_End, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_End(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;


      if (Share_Begin<=0)
            then
         Show_Message_Box( Lb_Task_Begin(Current_Language)  &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if (Share_End<=0)
            then
         Show_Message_Box( Lb_Task_End(Current_Language)  &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if Share_Begin > Share_End
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) & Lb_Must_Be(
               Current_Language)
            & To_Unbounded_String(" <= ")
            & Lb_Task_End(Current_Language) );
         return;
      end if;

      Reset_Iterator(Ite2);
      loop
         Current_Element(Sys.Tasks, A_Task, Ite2);

         if A_Task.Name=Task_Name
               then
            if (Natural(Share_End)>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_End(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );
               return;
            end if;
            if (Natural(Share_Begin)>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_Begin(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );

               return;
            end if;
            exit;
         end if;

         exit when Is_Last_Element(Sys.Tasks, Ite2) = True;

         Next_Element(Sys.Tasks, Ite2);
      end loop;

      Task_Share:= To_String(Task_Name)+To_String(Share_Begin_String)+
         To_String(Share_End_String);
      Affect_Task_Share.Task_Begin := Natural(Share_Begin);
      Affect_Task_Share.Task_End := Natural(Share_End);

      -- Add Task on list
      Resource_Parameter.Task_List_Share.Entries(Resource_Parameter.
         Task_List_Share.Nb_Entries).Item:=Task_Name;
      Resource_Parameter.Task_List_Share.Entries(Resource_Parameter.
         Task_List_Share.Nb_Entries).Data := Affect_Task_Share;
      Resource_Parameter.Task_List_Share.Nb_Entries:=
         Resource_Parameter.Task_List_Share.Nb_Entries+1;

      -- Show Task on Clist
      Line := Append(Resource_Parameter.Affected_Task_List, Task_Share);
      Gtk.Gentry.Set_Text(Resource_Parameter.Resource_Affect_Begin,"");
      Gtk.Gentry.Set_Text(Resource_Parameter.Resource_Affect_End,"");

   end On_Affect_Pressed;



   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Resource_Name         : Unbounded_String;  
      Resource_State_String : Unbounded_String;  
      Resource_State        : Integer;  
      Resource_Protocol     : Unbounded_String;  
      Processor             : Unbounded_String;  
      A_Processor           : Processor_Ptr;  
      Cpu_Index             : Gint;  
      Ok                    : Boolean;  

   begin

      Resource_Name := To_Unbounded_String(Get_Text(Resource_Parameter.
            Name));
      if Resource_Name=""
            then
         Show_Message_Box(   Lb_Resource_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         return;
      end if;

      if not Is_A_Valid_Identifier(Resource_Name)
            then
         Show_Message_Box(Lb_Resource_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;


      Resource_State_String := To_Unbounded_String(Get_Text(
            Resource_Parameter.State));
      if Resource_State_String=""
            then
         Resource_State:=0;
      else
         To_Integer(Resource_State_String, Resource_State, Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Resource_State(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;


      Resource_Protocol := To_Unbounded_String(Gtk.Gentry.Get_Text(
            Get_Entry(Resource_Parameter.Protocol)));

      if Resource_Protocol="PCP" then
         Resource_Protocol:=To_Unbounded_String(Resources_Type'Image(
               Priority_Ceiling_Protocol));
      else
         if Resource_Protocol="PIP" then
            Resource_Protocol:=To_Unbounded_String(Resources_Type'Image(
                  Priority_Inheritance_Protocol));
         else
            Resource_Protocol:=To_Unbounded_String(Resources_Type'Image(
                  No_Protocol));
         end if;
      end if;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Resource_Parameter.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;



      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Processor:=A_Processor.Name;

      Add_Resource(Sys.Resources, Resource_Name, Resource_State, Processor,
         Resource_Protocol, Resource_Parameter.Task_List_Share);

      Is_New:=False;
      Is_Saved:=False;
      Hide(Resource_Parameter);

   end On_Ok_Pressed;


end Resources_Parameters_Pkg.Callbacks;

