------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Translate;
use Translate;
with Config;
use Config;
with Gtk.Window;
use Gtk.Window;
with Gtk.Clist;
use Gtk.Clist;
with Gtk.Combo;
use Gtk.Combo;
with Resources_Parameters_Pkg.Callbacks;
use Resources_Parameters_Pkg.Callbacks;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Glib;
use Glib;
with Gtk;
use Gtk;
with  Unbounded_Strings;
use  Unbounded_Strings;
use  Unbounded_Strings.Strings_Table_Package;
with Gtk.Enums;
use Gtk.Enums;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Processors;
use Processors;
with Project;
use Project;
with Mmi_Tools;
use Mmi_Tools;
with Gtk.Window;
use Gtk.Window;
with Gtk.Table;
use Gtk.Table;
with Gtk.Label;
use Gtk.Label;
with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Box;
use Gtk.Box;
with Gtk.Button;
use Gtk.Button;
with Gtk.Option_Menu;
use Gtk.Option_Menu;
with Gtk.Box;
use Gtk.Box;
with Gtk.Widget;
use Gtk.Widget;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Resources;
use Resources;
use Resources.Task_Lists;


package body Resources_Parameters_Pkg is

   procedure Gtk_New (
         Resource_Parameter :    out Resource_Parameter_Access ) is 

   begin

      Resource_Parameter := new Resource_Parameter_Record;
      Resources_Parameters_Pkg.Initialize (Resource_Parameter);

   end Gtk_New;


   procedure Initialize (
         Resource_Parameter : access Resource_Parameter_Record'Class ) is 

      A_Processor  : Processor_Ptr;  
      My_Iterator  : Processors_Iterator;  
      Items        : Strings_Table;  
      A_Task       : Task_Ptr;  
      My_Iterator2 : Tasks_Iterator;  
      The_Name     : Unbounded_String;  

   begin


      Initialize(Resource_Parameter.Task_List_Share);

      --Boxes
      Gtk.Window.Initialize(Resource_Parameter , Window_Toplevel);
      Set_Policy (Resource_Parameter, True, True, True);
      Set_Title (Resource_Parameter, To_String(Lb_Add_Resource(
               Current_Language)));
      Set_Position (Resource_Parameter, Win_Pos_Center_Always);
      Set_Modal (Resource_Parameter, True);
      Grab_Focus(Resource_Parameter);
      Gtk.Box.Gtk_New_Vbox (Resource_Parameter.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Resource_Parameter.Hbox_Add_Cancel,False,10);
      Gtk.Box.Gtk_New_Hbox (Resource_Parameter.Hbox_Add_Affect,False,10);
      Gtk.Box.Gtk_New_Hbox (Resource_Parameter.Hbox_Table_Name_State,
         False,10);

      Add(Resource_Parameter,Resource_Parameter.Vbox_Main);
      Add(Resource_Parameter.Vbox_Main,Resource_Parameter.
         Hbox_Table_Name_State);
      Add(Resource_Parameter.Vbox_Main,Resource_Parameter.Hbox_Add_Affect);

      --Frame Name_State

      Gtk.Frame.Gtk_New(Resource_Parameter.Frame_Name_State,To_String(
            Lb_Resource_Information(Current_Language)));
      Add(Resource_Parameter.Hbox_Table_Name_State,Resource_Parameter.
         Frame_Name_State);

      --Frame Affect

      Gtk.Frame.Gtk_New(Resource_Parameter.Frame_Affect, To_String(
            Lb_Resource_Affect(Current_Language)));
      Add(Resource_Parameter.Hbox_Add_Affect,Resource_Parameter.
         Frame_Affect);

      -- Table Name_State

      Gtk.Table.Gtk_New (Resource_Parameter.Table_Name_State,4 , 2, False);
      --2 columns and 2 lines
      Add(Resource_Parameter.Frame_Name_State,Resource_Parameter.
         Table_Name_State);

      -- Widgets in Table Name_State

      Gtk.Gentry.Gtk_New(Resource_Parameter.Name,50);
      --Resource name limited to 50 characters
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_Name_Label, To_String(
            Lb_Resource_Name(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_Combo_Label,To_String(
            Lb_Use_Protocol(Current_Language)));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_State_Label,To_String(
            Lb_Resource_State(Current_Language)));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_Cpu_Label, To_String(
            Lb_Processor(Current_Language) & Lb_Colon));
      Gtk.Gentry.Gtk_New(Resource_Parameter.State,50);
      Gtk.Combo.Gtk_New(Resource_Parameter.Protocol);

      Initialize(Items);
      Reset_Iterator(My_Iterator);

      --Create CPU list

      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         Add(Items,A_Processor.Name);

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;


      Gtk.Option_Menu.Gtk_New(Resource_Parameter.Cpu);

      Resource_Parameter.Clist_Processor_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Resource_Parameter.Cpu,Resource_Parameter.
         Clist_Processor_Menu, Items, -1, null);

      Set_Editable(Get_Entry(Resource_Parameter.Protocol),False);

      --Combo Items

      String_List.Append (Resource_Parameter.Protocol_Items, "No Protocol");
      String_List.Append (Resource_Parameter.Protocol_Items, "PCP");
      String_List.Append (Resource_Parameter.Protocol_Items, "PIP");
      Set_Popdown_Strings (Resource_Parameter.Protocol,Resource_Parameter.
         Protocol_Items);

      -- Attach Table

      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Resource_Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Name, 1, 2, 0, 1);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Resource_State_Label, 0, 1, 1, 2);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.State, 1, 2, 1, 2);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Protocol, 1, 2, 2, 3);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Resource_Combo_Label, 0, 1, 2, 3);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Cpu, 1, 2, 3, 4);
      Attach_Defaults(Resource_Parameter.Table_Name_State,
         Resource_Parameter.Resource_Cpu_Label, 0, 1, 3, 4);

      --Table Add_Affect

      Gtk.Table.Gtk_New(Resource_Parameter.Table_Add_Affect,2,4,False);
      --4 columns and 2 lines
      Add(Resource_Parameter.Frame_Affect,Resource_Parameter.
         Table_Add_Affect);

      --Box Add_Affect
      Reset_Iterator(My_Iterator2);

      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator2);
         The_Name  := A_Task.Name;
         String_List.Append(Resource_Parameter.Items, To_String(The_Name));

         exit when Is_Last_Element(Sys.Tasks, My_Iterator2) = True;
         Next_Element(Sys.Tasks, My_Iterator2);

      end loop;


      Gtk_New (Resource_Parameter.Resource_Affect_Task);
      Gtk.Gentry.Gtk_New(Resource_Parameter.Resource_Affect_Begin,10);
      Gtk.Gentry.Gtk_New(Resource_Parameter.Resource_Affect_End,10);
      Gtk.Button.Gtk_New(Resource_Parameter.Button_Affect, To_String(
            Lb_Add(Current_Language)));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_Task_Label,To_String(
            Lb_Task_Name(Current_Language)));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_Begin_Label,To_String(
            Lb_Task_Begin(Current_Language)));
      Gtk.Label.Gtk_New(Resource_Parameter.Resource_End_Label, To_String(
            Lb_Task_End(Current_Language)));

      Set_Popdown_Strings (Resource_Parameter.Resource_Affect_Task,
         Resource_Parameter.Items);
      Set_Editable(Get_Entry(Resource_Parameter.Resource_Affect_Task),
         False);

      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_Task_Label, 0, 1, 0, 1);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_Begin_Label, 1, 2, 0, 1);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_End_Label, 2, 3, 0, 1);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_Affect_Task, 0, 1, 1, 2);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_Affect_Begin, 1, 2, 1,2);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Resource_Affect_End, 2, 3, 1, 2);
      Attach_Defaults(Resource_Parameter.Table_Add_Affect,
         Resource_Parameter.Button_Affect, 3, 4, 1, 2);

      --The List of affected task

      Gtk.Scrolled_Window.Gtk_New(Resource_Parameter.
         Scroll_Affected_Task_List, null, null);
      Add(Resource_Parameter.Vbox_Main,Resource_Parameter.
         Scroll_Affected_Task_List);
      Resource_Parameter.Affect_List_Titles:= To_String(Lb_Task(
            Current_Language))
         +To_String(Lb_Task_Begin(Current_Language))
         +To_String(Lb_Task_End(Current_Language));
      Gtk.Clist.Gtk_New(Resource_Parameter.Affected_Task_List,3,
         Resource_Parameter.Affect_List_Titles); -- 3 Columns
      Set_Column_Width(Resource_Parameter.Affected_Task_List, 0, 150);
      Set_Usize(Resource_Parameter.Affected_Task_List,100,200);
      Add(Resource_Parameter.Scroll_Affected_Task_List,Resource_Parameter.
         Affected_Task_List);

      --Box Add_Cancel

      Gtk.Button.Gtk_New(Resource_Parameter.Button_Add,To_String(Lb_Add(
               Current_Language)));
      Gtk.Button.Gtk_New(Resource_Parameter.Button_Cancel, To_String(
            Lb_Cancel(Current_Language)));

      Button_Callback.Connect (Resource_Parameter.Button_Cancel, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));
      Button_Callback.Connect (Resource_Parameter.Button_Add, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Button_Callback.Connect (Resource_Parameter.Button_Affect, "clicked",
         Button_Callback.To_Marshaller (On_Affect_Pressed'access));

      Add(Resource_Parameter.Hbox_Add_Cancel,Resource_Parameter.
         Button_Add);
      Add(Resource_Parameter.Hbox_Add_Cancel,Resource_Parameter.
         Button_Cancel);
      Add(Resource_Parameter.Vbox_Main,Resource_Parameter.Hbox_Add_Cancel);


   end Initialize;

end Resources_Parameters_Pkg ;
