------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--  This package contains all callbacks of the graphical editor. There is 3
--  type of callbacks : pixmap and menu callbacks which do not call the 
--  the framework and callbacks which call the framework
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Config;
use Config;

with Translate;
use Translate;

with Project;
use Project;

with References;
with Manual;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Resources;
use Resources;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Time_Units;
use Time_Units;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Densities_Table_Package;
use Task_Set.Density_Table_Package;
use Task_Set.Generic_Task_Set;

with Buffers;
use Buffers;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Dependencies;
use Dependencies;
with Task_Dependencies;
use Task_Dependencies;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Unbounded_Strings;
use Unbounded_Strings;

with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;

with Display_Resources_Set_Pkg;
use Display_Resources_Set_Pkg;
with Display_Messages_Set_Pkg;
use Display_Messages_Set_Pkg;

with Scheduler;
use Scheduler;
with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Call_Framework;
use Call_Framework;
use Call_Framework.Framework_Request_Package;
use Call_Framework.Framework_Response_Package;



with Call_Scheduling_Framework;
use Call_Scheduling_Framework;
with Call_Buffer_Framework;
use Call_Buffer_Framework;
use Call_Buffer_Framework.Buffer_Result_Package;
with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;


with Glib;
use Glib;
with Gtk.Widget;
use Gtk.Widget;


with Mmi_Tools;
use Mmi_Tools;

with Processors_Parameters_Pkg;
use Processors_Parameters_Pkg;
with Display_Tasks_Set_Pkg;
use Display_Tasks_Set_Pkg;
with Display_Buffers_Set_Pkg;
use Display_Buffers_Set_Pkg;
with Update_Tasks_Set_Pkg;
use Update_Tasks_Set_Pkg;
with Display_Processors_Set_Pkg;
use Display_Processors_Set_Pkg;
with Update_Processors_Set_Pkg;
use Update_Processors_Set_Pkg;
with Update_Resources_Set_Pkg;
use Update_Resources_Set_Pkg;
with Update_Resources_Set_Pkg.Callbacks;
use Update_Resources_Set_Pkg.Callbacks;
with Update_Buffers_Set_Pkg;
use Update_Buffers_Set_Pkg;
with Update_Messages_Set_Pkg;
use Update_Messages_Set_Pkg;
with Draw_Scheduling_Pkg;
use Draw_Scheduling_Pkg;
with Scrolling_Draw_Buffer_Pkg;
use Scrolling_Draw_Buffer_Pkg;
with Scrolling_Editor_Pkg;
use Scrolling_Editor_Pkg;
with Scrolling_Text_Pkg;
use Scrolling_Text_Pkg;
with Question_Pkg;
use Question_Pkg;
with Unbounded_Strings;
use Unbounded_Strings;
with Resources_Parameters_Pkg;
use Resources_Parameters_Pkg;
with Messages_Parameters_Pkg;
use Messages_Parameters_Pkg;
with Buffers_Parameters_Pkg;
use Buffers_Parameters_Pkg;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with Options_Pkg;
use Options_Pkg;
with Duplicate_Pkg;
use Duplicate_Pkg;



with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;

with Text_Io;
use Text_Io;




package body Root_Window_Pkg.Callbacks is

   use Gtk.Arguments;


   procedure Write_Title_And_Text (
         Root_Window   : in     Root_Window_Access;      
         Response_List : in     Framework_Response_Table ) is 

   begin
      for J in 0..Response_List.Nb_Entries-1 loop
         Write_Title(Root_Window, Response_List.Entries(J).Title);
         Write_Text(Root_Window, Response_List.Entries(J).Text);
      end loop;

   end Write_Title_And_Text;


   ----------------------------------------------------------------------
   --   Callbacks which call the framework
   ----------------------------------------------------------------------



   -- Scheduling tools : scheduling simulation and feasibility tests
   --


   procedure Check_Scheduling is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;

      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_Basics;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window, Response_List);

   end Check_Scheduling;



   function Draw_Scheduling_Callback (
         Value : Unbounded_String ) 
     return Boolean is 


      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  
      A_Param       : Parameter_Ptr;  

      Last_Period : Natural := Config.Max_Scheduling_Period - 1;  

      Max_Period : Natural := 0;  
      Ok         : Boolean;  

   begin


      To_Natural(Value,Max_Period,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return False;
      end if;

      if(Max_Period>=Last_Period)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) & Lb_Must_Be(
               Current_Language) & To_Unbounded_String(" < ")
            & Last_Period'Img);
         return False;
      end if;

      if(Max_Period=0)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be(Current_Language) & To_Unbounded_String(" > 0"));
         return False;
      end if;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Simulation_Time_Line;
      A_Param := new Parameter(Integer_Parameter);
      A_Param.Name:=To_Unbounded_String("period");
      A_Param.Integer_Value:=Max_Period;
      Add(A_Request.Param, A_Param);
      A_Param := new Parameter(Boolean_Parameter);
      A_Param.Name:=To_Unbounded_String("schedule_with_offsets");
      A_Param.Boolean_Value:=Schedule_With_Offsets;
      Add(A_Request.Param, A_Param);
      A_Param := new Parameter(Boolean_Parameter);
      A_Param.Name:=To_Unbounded_String("schedule_with_precedencies");
      A_Param.Boolean_Value:=Schedule_With_Precedencies;
      Add(A_Request.Param, A_Param);
      A_Param := new Parameter(Boolean_Parameter);
      A_Param.Name:=To_Unbounded_String("schedule_with_resources");
      A_Param.Boolean_Value:=Schedule_With_Resources;
      Add(A_Request.Param, A_Param);
      A_Param := new Parameter(Integer_Parameter);
      A_Param.Name:=To_Unbounded_String("seed_value");
      A_Param.Integer_Value:=Common_Seed_Value;
      Add(A_Request.Param, A_Param);

      Add(Request_List, A_Request);

      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Simulation_Basics;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);


      -- Put to the screen the resulting scheduling (for debug)
      -- Put(Call_Scheduling_Framework.Sched.All);
      -- Put(Sys.Processors);

      Write_Title_And_Text(Root_Window, Response_List);

      Draw_Task(Draw_Scheduling,Sys.Tasks,Max_Period,
         Call_Scheduling_Framework.Sched);



      return True;

   end Draw_Scheduling_Callback;



   procedure Schedule_And_Check_Temporal_Constraints is 


      A_Task      : Task_Ptr;  
      Iterator1   : Tasks_Iterator;  
      A_Processor : Processor_Ptr;  
      Iterator2   : Processors_Iterator;  
      Max_Period  : Natural             := 0;  
      Period      : Natural             := 0;  
      Last_Period : Natural             := Config.Max_Scheduling_Period -
        1;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;


      -- First of all : get the largest period of all
      -- the defined tasks
      --
      Reset_Iterator(Iterator1);
      loop

         Current_Element(Sys.Tasks, A_Task, Iterator1);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Max_Period:=Natural'Max(Max_Period,
               Periodic_Task_Ptr(A_Task).Period);
         else
            Max_Period:=Natural'Max(Max_Period,
               A_Task.Deadline);
         end if;

         exit when Is_Last_Element(Sys.Tasks, Iterator1) = True;

         Next_Element(Sys.Tasks, Iterator1);

      end loop;




      -- Then, compute LCM
      --
      Reset_Iterator(Iterator2);

      loop


         Current_Element(Sys.Processors, A_Processor, Iterator2);

         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            begin
               Period:=Scheduling_Period(Sys.
                  Tasks, A_Processor.Name);
               if Period> Max_Period
                     then
                  Max_Period:=Period;
               end if;

            exception
               when Task_Set.Task_Must_Be_Periodic =>
                  null;
               when Task_Set.Task_Model_Error =>
                  null;

            end;
         end if;

         exit when Is_Last_Element(Sys.Processors, Iterator2) = True;

         Next_Element(Sys.Processors, Iterator2);


      end loop;


      if(Max_Period>=Last_Period)
            then
         Max_Period:=Last_Period;
      end if;

      Gtk_New(Question, Lb_Draw(Current_Language), Lb_Schedule_From_To(
            Current_Language),
         To_Unbounded_String(Max_Period'Img),
         Draw_Scheduling_Callback'access);
      Show_All(Question);

   end Schedule_And_Check_Temporal_Constraints;





   -- Buffer tools : bound and bufer utilization
   --

   procedure Check_Buffer is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin

      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
         return;
      end if;


      Initialize(A_Request);
      A_Request.Statement:=Buffer_Bound;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window, Response_List);



   end Check_Buffer;



   procedure On_Draw_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  
      Index         : Scheduling_Range;  


   begin

      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      else

         -- Check if scheduling is computed or not
         --
         if Call_Scheduling_Framework.Sched.Nb_Entries=0
               then
            Show_Message_Box(Lb_Schedule_Before(Current_Language));
            return;
         end if;

         -- For each buffer, compute and draw its size from
         -- 0 to the base period
         --
         Initialize(A_Request);
         A_Request.Statement:=Buffer_Utilization;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);


         Write_Title_And_Text(Root_Window, Response_List);


         -- Now, draw the result on buffer utilization
         --
         for I in 0..Buff.Nb_Entries-1 loop


            -- Firstly, find the processor which hosts the buffer
            --
            for J in 0..Call_Scheduling_Framework.Sched.Nb_Entries-1 loop
               if Call_Scheduling_Framework.Sched.Entries(J).Data.
                     Processor.Name
                     =Buff.Entries(I).Item.Location then
                  Index:=J;
                  exit;
               end if;
            end loop;


            Gtk_New(Scrolling_Draw_Buffer,To_String(Buff.Entries(I).Item.
                  Name));
            Show_All(Scrolling_Draw_Buffer);
            Clear(Scrolling_Draw_Buffer);
            Draw(Scrolling_Draw_Buffer, Buff.Entries(I).Item, Buff.
               Entries(I).Data,
               Call_Scheduling_Framework.Scheduling_Base_Period,
               Time_Unit_Range (Buff.Entries(I).Data.Nb_Entries));
         end loop;


      end if;
   end On_Draw_Buffer_Activate;




   -- Resource tools : blocking time evaluation
   --

   procedure On_Blocking_Time_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Initialize(A_Request);
         A_Request.Statement:=Resource_Compute_Blocking_Time;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);

      end if;
   end On_Blocking_Time_Display_Callback;


   procedure On_Blocking_Time_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Initialize(A_Request);
         A_Request.Statement:=Resource_Set_Blocking_Time;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      end if;
   end On_Blocking_Time_Update_Callback;








   -- Precedency tools : deadline, jitter and priority modifications
   --
   procedure On_Chetto_Priority_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Chetto_Priority_Callback;


   procedure On_Chetto_Priority_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_Chetto_Blazewicz_Priority;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);

      end if;
   end  On_Chetto_Priority_Display_Callback;


   procedure On_Chetto_Priority_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=Dependency_Set_Chetto_Blazewicz_Priority;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;

      end if;
   end  On_Chetto_Priority_Update_Callback;


   procedure On_Chetto_Deadline_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Chetto_Deadline_Callback;


   procedure On_Chetto_Deadline_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_Chetto_Blazewicz_Deadline;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


      end if;
   end  On_Chetto_Deadline_Display_Callback;


   procedure On_Chetto_Deadline_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=Dependency_Set_Chetto_Blazewicz_Deadline;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      end if;
   end  On_Chetto_Deadline_Update_Callback;


   procedure On_Jitter_From_Response_Time_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end  On_Jitter_From_Response_Time_Callback;


   procedure On_Jitter_From_Response_Time_Display_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_End_To_End_Response_Time_All_Steps;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


      end if;
   end  On_Jitter_From_Response_Time_Display_Callback;


   procedure On_Jitter_From_Response_Time_Update_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Set_End_To_End_Response_Time_All_Steps;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      end if;
   end  On_Jitter_From_Response_Time_Update_Callback;


   procedure On_Jitter_From_Response_Time_Display_One_Step_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_End_To_End_Response_Time_One_Step;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);

      end if;
   end  On_Jitter_From_Response_Time_Display_One_Step_Callback;


   procedure On_Jitter_From_Response_Time_Update_One_Step_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 

      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      else
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Set_End_To_End_Response_Time_One_Step;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      end if;
   end  On_Jitter_From_Response_Time_Update_One_Step_Callback;





   -- Set priorities according to a fixed priority scheduler
   --

   function Rm_Priorities_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator   : Processors_Iterator;  
      A_Processor   : Processor_Ptr;  
      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin

      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value
               then

            Initialize(A_Request);
            A_Request.Target:=Value;
            A_Request.Statement:=
               Scheduling_Set_Priorities_According_To_Rate_Monotonic;
            Add(Request_List, A_Request);


            Sequential_Framework_Request (
               Sys, Request_List, Response_List);

            Write_Title_And_Text(Root_Window, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New:=False;
            Is_Saved:=False;


            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);

      end loop;

      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;


   end Rm_Priorities_Callback;



   function Dm_Priorities_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator   : Processors_Iterator;  
      A_Processor   : Processor_Ptr;  
      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  


   begin

      loop

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value
               then

            Initialize(A_Request);
            A_Request.Target:=Value;
            A_Request.Statement:=
               Scheduling_Set_Priorities_According_To_Deadline_Monotonic;
            Add(Request_List, A_Request);


            Sequential_Framework_Request (
               Sys, Request_List, Response_List);

            Write_Title_And_Text(Root_Window, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New:=False;
            Is_Saved:=False;

            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;

   end Dm_Priorities_Callback;


   procedure On_Dm_Priorities_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Set_Priorities_Dm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Dm_Priorities_Callback'access);
         Show_All(Question);
      end if;

   end On_Dm_Priorities_Activate;




   procedure On_Rm_Priorities_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Set_Priorities_Rm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Rm_Priorities_Callback'access);
         Show_All(Question);
      end if;

   end On_Rm_Priorities_Activate;


   -- Random services
   --


   procedure On_Compute_Response_Time_Density_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 


      Response_List : Framework_Response_Table;  
      Request_List  : Framework_Request_Table;  
      A_Request     : Framework_Request;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;


      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries=0
            then
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         return;
      end if;


      Initialize(A_Request);
      A_Request.Statement:=Random_Response_Time_Density;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window, Response_List);


   end On_Compute_Response_Time_Density_Activate;





   ----------------------------------------------------------------------
   --   Menu callbacks without calls to the framework 
   ----------------------------------------------------------------------



   -- Callback to be called when a scheduler is modified
   --
   Current_Processor : Processor_Ptr;  


   procedure Do_Modify_Scheduler (
         Str : in     Unbounded_String ) is 
   begin

      -- Mark the project to be saved
      --
      Is_New:=False;
      Is_Saved:=False;

      Set_String_Behavior ( Parametric_Scheduler(Current_Processor.
            Scheduler.all), Str);
   end Do_Modify_Scheduler ;


   function Edit_Scheduler_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
   begin

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);


         if A_Processor.Name = Value then


            if Get_Name(A_Processor.Scheduler.all) /= Parametric
                  then
               Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
               return True;
            end if;

            Current_Processor:=A_Processor;

            Gtk_New (Scrolling_Editor,
               Lb_Edit_Scheduler(Current_Language),
               Get_String_Behavior(Parametric_Scheduler(A_Processor.
                     Scheduler.all)),
               Do_Modify_Scheduler'access);
            Set_Usize(Scrolling_Editor,800,450);
            Show_All (Scrolling_Editor);
            return True;

         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;

      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return True;
   end  Edit_Scheduler_Callback;


   procedure Clear_Work_Space is 
      Size        : Guint;  
      I_Dont_Care : Boolean;  
   begin
      Size:=Get_Length(Root_Window.Text);
      Set_Point(Root_Window.Text,0);
      I_Dont_Care:=Forward_Delete(Root_Window.Text,Size);
      Clear_Task(Draw_Scheduling);
   end Clear_Work_Space;



   --  Delete functions : called by delete callbacks
   --

   function Delete_Resource_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      My_Iterator : Resources_Iterator;  
      A_Resource  : Resource_Ptr;  
   begin
      loop

         Current_Element(Sys.Resources, A_Resource, My_Iterator);

         if A_Resource.Name = Value
               then

            Delete(Sys.Resources,A_Resource);

            Is_New:=False;
            Is_Saved:=False;

            return True;
         end if;

         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Resource_Not_Found(Current_Language));
      return False;
   end Delete_Resource_Callback;


   function Delete_Buffer_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      My_Iterator : Buffers_Iterator;  
      A_Buffer    : Buffer_Ptr;  
   begin

      loop

         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);

         if A_Buffer.Name = Value
               then

            Delete_All_Dependencies(Sys.Dependencies, Buffer_Dependency,
               A_Buffer.Name);

            Delete(Sys.Buffers,A_Buffer);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      return False;
   end Delete_Buffer_Callback;


   function Delete_Message_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      My_Iterator : Messages_Iterator;  
      A_Message   : Message_Ptr;  
   begin
      loop

         Current_Element(Sys.Messages, A_Message, My_Iterator);

         if A_Message.Name = Value
               then

            Delete_All_Dependencies(Sys.Dependencies,
               Communication_Dependency,
               A_Message.Name);

            Delete(Sys.Messages,A_Message);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      return False;
   end Delete_Message_Callback;


   function Delete_Task_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      My_Iterator : Tasks_Iterator;  
      A_Task      : Task_Ptr;  
   begin
      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);

         if A_Task.Name = Value
               then

            Delete_Task(Sys.Resources, A_Task.Name);
            Delete_Task(Sys.Buffers, A_Task.Name);
            Delete_All_Dependencies(Sys.Dependencies,
               Precedence_Dependency, A_Task.Name);
            Delete(Sys.Tasks,A_Task);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Task_Not_Found(Current_Language));
      return False;
   end Delete_Task_Callback;


   function Delete_Processor_Callback (
         Value : Unbounded_String ) 
     return Boolean is 
      Iterator1   : Processors_Iterator;  
      A_Processor : Processor_Ptr;  
   begin
      Reset_Iterator(Iterator1);

      loop

         Current_Element(Sys.Processors, A_Processor, Iterator1);

         if A_Processor.Name = Value
               then

            Delete_Processors_Dependencies(Sys.Dependencies, A_Processor.
               Name);
            Delete_Processor(Sys.Buffers, A_Processor.Name);
            Delete_Processor(Sys.Resources, A_Processor.Name);
            Delete_Processor(Sys.Tasks, A_Processor.Name);
            Delete(Sys.Processors,A_Processor);

            Is_New:=False;
            Is_Saved:=False;

            Rebuild_Canvas(Graph_Window);
            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, Iterator1) = True;
         Next_Element(Sys.Processors, Iterator1);

      end loop;

      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;
   end Delete_Processor_Callback;


   function On_Root_Window_Delete_Event (
         Object : access Gtk_Widget_Record'Class; 
         Params :        Gtk.Arguments.Gtk_Args   ) 
     return Boolean is 
   begin
      Do_Exit;
      return True;
   end On_Root_Window_Delete_Event;


   procedure On_File_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_File_Activate;


   procedure On_New_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   end On_New_Activate;


   procedure On_Open_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Open;
   end On_Open_Activate;


   procedure On_Save_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save;
   end On_Save_Activate;


   procedure On_Save_As_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Save_As_Activate;


   procedure On_Exit_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Exit;
   end On_Exit_Activate;


   procedure On_New_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Update_Tasks_Set);
         Show_Advanced (Update_Tasks_Set,False);
      end if;
   end On_New_Task_Activate;


   procedure On_Update_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Update_Tasks_Set);
         Show_Advanced (Update_Tasks_Set,True);
      end if;
   end On_Update_Task_Activate;


   procedure On_New_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New (Processors_Parameters);
      Show_All (Processors_Parameters);
   end On_New_Processor_Activate;


   procedure On_Update_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Update_Processors_Set);
         Show_All (Update_Processors_Set);
      end if;
   end On_Update_Processor_Activate;


   procedure On_Tasks_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New (Display_Tasks_Set);
         Show_All (Display_Tasks_Set);
      end if;
   end On_Tasks_List_Activate;


   procedure On_Processors_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Display_Processors_Set);
         Show_All (Display_Processors_Set);
      end if;
   end On_Processors_List_Activate;


   procedure On_Resource_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0) then
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New (Display_Resources_Set);
         Show_All (Display_Resources_Set);
      end if;
   end On_Resource_List_Activate;


   procedure On_Message_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Messages)=0) then
         Show_Message_Box(Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New (Display_Messages_Set);
         Show_All (Display_Messages_Set);
      end if;
   end On_Message_List_Activate;


   procedure On_Buffer_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Buffers)=0) then
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
      else
         Gtk_New (Display_Buffers_Set);
         Show_All (Display_Buffers_Set);
      end if;
   end On_Buffer_List_Activate;


   procedure On_Tools_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Tools_Activate;


   procedure On_Clear_Work_Space_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Clear_Work_Space_Activate;


   procedure On_Draw_Scheduling_Graph_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Draw_Scheduling_Graph_Activate;


   procedure On_Options_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Options);
      Show_All(Options);
   end On_Options_Activate;


   procedure On_Check_Scheduling_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Check_Scheduling_Activate;


   procedure On_Help_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Help_Activate;


   procedure On_Update_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0) then
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New(Update_Resources_Set);
         Show_All(Update_Resources_Set);
         On_Cpu_Change_Show;
      end if;
   end On_Update_Resource_Activate;


   procedure On_New_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         if (Get_Number_Of_Elements(Sys.Tasks)=0)
               then
            Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         else
            Gtk_New(Resource_Parameter);
            Show_All(Resource_Parameter);
         end if;
      end if;
   end On_New_Resource_Activate;


   procedure On_About_Cheddar_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Show_Message_Box(Lb_About_Text(Current_Language));
   end On_About_Cheddar_Activate;


   procedure On_Manual_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Scrolling_Text,Lb_Manual(Current_Language));
      Set_Usize(Scrolling_Text,690,425);


      Write_Title_Small(Scrolling_Text,Manual.Ustr_File_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_File_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Edit_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Edit_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_View_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_View_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Tools_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Tools_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Help_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Help_Menu_Content);

      Show_All(Scrolling_Text);
   end On_Manual_Activate;


   procedure On_Bibliography_References_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Scrolling_Text,Lb_Ref(Current_Language));
      Set_Usize(Scrolling_Text,535, 440);
      Write_Text(Scrolling_Text,References.Text);
      Show_All(Scrolling_Text);
   end On_Bibliography_References_Activate;


   procedure On_Delete_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Task(Current_Language),
            Lb_Task_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Task_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Task_Activate;


   procedure On_Delete_Resource_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Resources)=0)
            then
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      else
         Gtk_New(Question,Lb_Delete_Resource(Current_Language),
            Lb_Resource_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Resource_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Resource_Activate;


   procedure On_Delete_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Processor(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Processor_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Processor_Activate;


   procedure On_New_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Messages_Parameters, To_Unbounded_String("cheddar_window"));
   end On_New_Message_Activate;


   procedure On_Delete_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Messages)=0)
            then
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Message(Current_Language),
            Lb_Message_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Message_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Message_Activate;


   procedure On_Update_Message_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Messages)=0)
            then
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      else
         Gtk_New(Update_Messages_Set);
         Show_All(Update_Messages_Set);
      end if;
   end On_Update_Message_Activate;


   procedure On_New_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Buffers_Parameters, To_Unbounded_String("cheddar_window"));

   end On_New_Buffer_Activate;


   procedure On_Delete_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Buffers)=0)
            then
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Buffer(Current_Language),
            Lb_Buffer_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Buffer_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Buffer_Activate;


   procedure On_Update_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Update_Buffers_Set);
   end On_Update_Buffer_Activate;


   procedure On_Check_Buffer_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Check_Buffer;
   end On_Check_Buffer_Activate;


   procedure On_Blocking_Time_Callback (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Blocking_Time_Callback;


   procedure On_Duplicate_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Gtk_New(Duplicate_Pkg.Duplicate);
      Show_All(Duplicate_Pkg.Duplicate);
   end On_Duplicate_Activate;


   procedure On_Edit_Scheduler_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(
            Lb_Define_Processors_Before(Current_Language));

      else

         if Scrolling_Editor/=null then
            if Scrolling_Editor.Is_Displayed=True
                  then
               Show_Message_Box(
                  Lb_Can_Not_Edit_To_Parametric_Object(Current_Language));
               return;
            end if;
         end if;

         Gtk_New(Question, Lb_Edit_Scheduler(Current_Language),
            Lb_Processor_Name(
               Current_Language) & " : ",
            To_Unbounded_String(""),
            Edit_Scheduler_Callback'access);
         Show_All(Question);

      end if;
   end On_Edit_Scheduler_Activate;





   ----------------------------------------------------------------------
   --   Pixmap callbacks without calls to the framework 
   ----------------------------------------------------------------------


   procedure On_Pixmap_Draw_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Pixmap_Draw_Scheduling_Clicked;


   procedure On_Pixmap_New_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   end On_Pixmap_New_Clicked;


   procedure On_Pixmap_Open_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Open;
   end On_Pixmap_Open_Clicked;


   procedure On_Pixmap_Save_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save;
   end On_Pixmap_Save_Clicked;


   procedure On_Pixmap_Save_As_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Pixmap_Save_As_Clicked;


   procedure On_Pixmap_Exit_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Exit;
   end On_Pixmap_Exit_Clicked;


   procedure On_Pixmap_Clear_Work_Space_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Pixmap_Clear_Work_Space_Clicked;


   procedure On_Pixmap_Dependencies_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Gtk_New(Graph_Window);
      Show(Graph_Window);
   end On_Pixmap_Dependencies_Clicked;


   procedure On_Pixmap_Check_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Pixmap_Check_Scheduling_Clicked;






end Root_Window_Pkg.Callbacks;





