------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Box;
use Gtk.Box;
with Gtk.Menu_Bar;
use Gtk.Menu_Bar;
with Gtk.Menu_Item;
use Gtk.Menu_Item;
with Gtk.Menu;
use Gtk.Menu;
with Gtk.Toolbar;
use Gtk.Toolbar;
with Gtk.Scrolled_Window;
use Gtk.Scrolled_Window;
with Gtk.Text;
use Gtk.Text;
with Gtk.Window;
use Gtk.Window;
with Gtk.Pixmap;
use Gtk.Pixmap;
with Gdk.Font;
use Gdk.Font;
with Gtk.Paned;
use Gtk.Paned;
with Glib;
use Glib;
with Gdk.Color;
use Gdk.Color;


package Root_Window_Pkg is

   type Root_Window_Record is new Gtk_Window_Record with private; 
   type Root_Window_Access is access all Root_Window_Record'Class; 

   procedure Gtk_New (
         Root_Window :    out Root_Window_Access ); 
   procedure Initialize (
         Root_Window : access Root_Window_Record'Class ); 
   procedure Write_Text (
         Root_Window : in     Root_Window_Access; 
         Msg         :        Unbounded_String    ); 
   procedure Write_Title (
         Root_Window : in     Root_Window_Access; 
         Msg         :        Unbounded_String    ); 

   pragma Inline (Write_Text);
   pragma Inline (Write_Title);

   Root_Window : Root_Window_Access;  


private

   type Root_Window_Record is new Gtk_Window_Record with 
      record 
         Vbox            : Gtk_Vbox;  
         Vpaned          : Gtk_Vpaned;  
         Scrolledwindow1 : Gtk_Scrolled_Window;  
         Scrolledwindow2 : Gtk_Scrolled_Window;  
         Text            : Gtk_Text;  
         Font_R          : Gdk_Font;  
         Font_B          : Gdk_Font;  
         Red             : Gdk_Color;  
         Black           : Gdk_Color;  

         Separator1 : Gtk_Menu_Item;  
         Separator2 : Gtk_Menu_Item;  
         Separator3 : Gtk_Menu_Item;  

         Menubar : Gtk_Menu_Bar;  

         File         : Gtk_Menu_Item;  
         File_Menu    : Gtk_Menu;  
         New_File     : Gtk_Menu_Item;  
         Open_File    : Gtk_Menu_Item;  
         Save_File    : Gtk_Menu_Item;  
         Save_As_File : Gtk_Menu_Item;  
         Exit_Program : Gtk_Menu_Item;  

         Edit      : Gtk_Menu_Item;  
         Edit_Menu : Gtk_Menu;  

         Add           : Gtk_Menu_Item;  
         Add_Menu      : Gtk_Menu;  
         New_Task      : Gtk_Menu_Item;  
         New_Processor : Gtk_Menu_Item;  
         New_Resource  : Gtk_Menu_Item;  
         New_Message   : Gtk_Menu_Item;  
         New_Buffer    : Gtk_Menu_Item;  

         Update           : Gtk_Menu_Item;  
         Update_Menu      : Gtk_Menu;  
         Update_Task      : Gtk_Menu_Item;  
         Update_Processor : Gtk_Menu_Item;  
         Update_Resource  : Gtk_Menu_Item;  
         Update_Message   : Gtk_Menu_Item;  
         Update_Buffer    : Gtk_Menu_Item;  

         Delete           : Gtk_Menu_Item;  
         Delete_Menu      : Gtk_Menu;  
         Delete_Task      : Gtk_Menu_Item;  
         Delete_Processor : Gtk_Menu_Item;  
         Delete_Resource  : Gtk_Menu_Item;  
         Delete_Message   : Gtk_Menu_Item;  
         Delete_Buffer    : Gtk_Menu_Item;  

         Duplicate_Object : Gtk_Menu_Item;  
         Edit_Scheduler   : Gtk_Menu_Item;  

         Tools      : Gtk_Menu_Item;  
         Tools_Menu : Gtk_Menu;  

         Scheduling            : Gtk_Menu_Item;  
         Scheduling_Menu       : Gtk_Menu;  
         Draw_Scheduling_Graph : Gtk_Menu_Item;  
         Options               : Gtk_Menu_Item;  
         Check_Scheduling      : Gtk_Menu_Item;  
         Rm_Priorities         : Gtk_Menu_Item;  
         Dm_Priorities         : Gtk_Menu_Item;  

         Clear_Work_Space : Gtk_Menu_Item;  

         Tools_Resource        : Gtk_Menu_Item;  
         Tools_Resource_Menu   : Gtk_Menu;  
         Blocking_Time         : Gtk_Menu_Item;  
         Blocking_Time_Menu    : Gtk_Menu;  
         Blocking_Time_Update  : Gtk_Menu_Item;  
         Blocking_Time_Display : Gtk_Menu_Item;  

         Tools_Dependencies                         : Gtk_Menu_Item;  
         Tools_Dependencies_Menu                    : Gtk_Menu;  
         Chetto_Priority                            : Gtk_Menu_Item;  
         Chetto_Priority_Menu                       : Gtk_Menu;  
         Chetto_Priority_Update                     : Gtk_Menu_Item;  
         Chetto_Priority_Display                    : Gtk_Menu_Item;  
         Chetto_Deadline                            : Gtk_Menu_Item;  
         Chetto_Deadline_Menu                       : Gtk_Menu;  
         Chetto_Deadline_Update                     : Gtk_Menu_Item;  
         Chetto_Deadline_Display                    : Gtk_Menu_Item;  
         Jitter_From_Response_Time                  : Gtk_Menu_Item;  
         Jitter_From_Response_Time_Menu             : Gtk_Menu;  
         Jitter_From_Response_Time_Update           : Gtk_Menu_Item;  
         Jitter_From_Response_Time_Display          : Gtk_Menu_Item;  
         Jitter_From_Response_Time_Update_One_Step  : Gtk_Menu_Item;  
         Jitter_From_Response_Time_Display_One_Step : Gtk_Menu_Item;  
         Tools_Buffer                               : Gtk_Menu_Item;  
         Tools_Buffer_Menu                          : Gtk_Menu;  
         Compute_Buffer                             : Gtk_Menu_Item;  
         Draw_Buffer                                : Gtk_Menu_Item;  

         Tools_Random                  : Gtk_Menu_Item;  
         Tools_Random_Menu             : Gtk_Menu;  
         Compute_Response_Time_Density : Gtk_Menu_Item;  

         Show      : Gtk_Menu_Item;  
         Show_Menu : Gtk_Menu;  

         List            : Gtk_Menu_Item;  
         List_Menu       : Gtk_Menu;  
         Tasks_List      : Gtk_Menu_Item;  
         Processors_List : Gtk_Menu_Item;  
         Resources_List  : Gtk_Menu_Item;  
         Messages_List   : Gtk_Menu_Item;  
         Buffers_List    : Gtk_Menu_Item;  

         Help                    : Gtk_Menu_Item;  
         Help_Menu               : Gtk_Menu;  
         About_Cheddar           : Gtk_Menu_Item;  
         Manual                  : Gtk_Menu_Item;  
         Bibliography_References : Gtk_Menu_Item;  

         Toolbar                 : Gtk_Toolbar;  
         Pixmap_Empty            : Gtk_Pixmap;  
         Pixmap_New              : Gtk_Pixmap;  
         Pixmap_Open             : Gtk_Pixmap;  
         Pixmap_Save             : Gtk_Pixmap;  
         Pixmap_Save_As          : Gtk_Pixmap;  
         Pixmap_Exit             : Gtk_Pixmap;  
         Pixmap_Clear_Work_Space : Gtk_Pixmap;  
         Pixmap_Dependencies     : Gtk_Pixmap;  
         Pixmap_Check_Scheduling : Gtk_Pixmap;  
         Pixmap_Draw_Scheduling  : Gtk_Pixmap;  

         Canvas_Is_Created : Boolean;  
      end record; 

end Root_Window_Pkg;
