------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Project;
use Project;
with Gtk.Enums;
use Gtk.Enums;
with Mmi_Tools;
use Mmi_Tools;

with Gtk.Option_Menu;
use Gtk.Option_Menu;

with Buffers;
use Buffers;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
use Buffers.Buffer_Roles;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
with Processors;
use Processors;
with Processor_Set;
use Processor_Set;

with Task_Dependencies;
use Task_Dependencies;

with Translate;
use Translate;
with Config;
use Config;


with Graph_Window_Pkg;
use Graph_Window_Pkg;


package body Update_Buffers_Set_Pkg.Callbacks is



   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin
      Sys.Buffers:=Update_Buffers_Set.Undo_Buffer;
      Destroy(Update_Buffers_Set);
   end  On_Cancel_Pressed;



   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      A_Buff      : Buffer_Ptr;  
      My_Iterator : Buffers_Iterator;  

   begin
      A_Buff:=Buffer_Modify;


      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Buffers, A_Buff, My_Iterator);

         -- For all buffers, update dependencies :
         -- Delete and then add all dependencies
         --
         Build_Dependencies(Sys.Dependencies, Sys.Tasks, A_Buff);



         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

      Rebuild_Canvas(Graph_Window);


      Is_New:=False;
      Is_Saved:=False;

      Hide(Update_Buffers_Set);

   exception
      when others =>
         Hide(Update_Buffers_Set);

   end On_Ok_Pressed;



   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 

      Task_Name       : Unbounded_String;  
      Role_Size       : Unbounded_String;  
      Role_Type       : Unbounded_String;  
      Row,  
      Current         : Gint;  
      A_Task          : Task_Ptr;  
      Location_Items  : Strings_Table;  
      Role_Type_Items : Strings_Table;  
      Index           : Strings_Range    := Strings_Range'First;  

      My_Iterator : Task_Set.Generic_Task_Set.Iterator;  



   begin
      Row:=Get_Focus_Row(Update_Buffers_Set.Affected_Role_List);
      Task_Name:= To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,0));
      Role_Size:= To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,1));
      Role_Type:= To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,2));

      Update_Buffers_Set.One_Row_Is_Selected:=True;

      -- Create the task_Name Option Menu
      --
      Initialize(Location_Items);
      Index:=0;
      Task_Set.Generic_Task_Set.Reset_Iterator(My_Iterator);
      Destroy(Update_Buffers_Set.Role_Task_Name);
      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         Add(Location_Items,A_Task.Name);
         if A_Task.Name = Task_Name then
            Current:=Gint(Index);
         end if;
         Index:=Index+1;
         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);
      end loop;

      Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Task_Name);
      Update_Buffers_Set.Role_Name_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Update_Buffers_Set.Role_Task_Name,
         Update_Buffers_Set.Role_Name_Menu,
         Location_Items, Current, null);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Task_Name, 1, 2, 0, 1);

      ----- Rebuild a Role_Type Option Menu
      --
      Destroy(Update_Buffers_Set.Role_Type);
      if Role_Type= Lb_Role_Producer(Current_Language) then
         Current:=Gint(0);
      else
         Current:=Gint(1);
      end if;
      Add(Role_Type_Items,Lb_Role_Producer(Current_Language));
      Add(Role_Type_Items,Lb_Role_Consumer(Current_Language));
      Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Type);
      Update_Buffers_Set.Role_Type_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Update_Buffers_Set.Role_Type,Update_Buffers_Set.
         Role_Type_Menu, Role_Type_Items, Current, null);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Type, 1, 2, 2, 3);

      -- End OptionMenu
      --

      Gtk.Gentry.Set_Text(Update_Buffers_Set.Role_Size, To_String(
            Role_Size));
      Show_All(Update_Buffers_Set);
   end On_Click_List;



   procedure Show_Roles (
         A_Buffer_Name : Unbounded_String ) is 

      A_Buffer    : Buffer_Ptr;  
      My_Iterator : Iterator;  
      Index       : Integer    := 0;  

      A_Task       : Task_Ptr;  
      My_Iterator2 : Task_Set.Generic_Task_Set.Iterator;  
      Items        : Strings_Table;  
      Line         : Gint;  
      Role_Share   : Chars_Ptr_Array (1 .. 3);  
      Role_Type    : Unbounded_String;  
      Found        : Boolean                            := True;  


   begin
      Reset_Iterator(My_Iterator);
      Clear(Update_Buffers_Set.Affected_Role_List);

      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         if A_Buffer_Name = A_Buffer.Name then
            if (Natural(A_Buffer.Roles.Nb_Entries) >= 1) then
               for I in 0..Buffer_Roles_Range(Natural(A_Buffer.Roles.
                        Nb_Entries) -1) loop

                  if (A_Buffer.Roles.Entries(I).Data.The_Role = Producer) then
                     Role_Type:=Lb_Role_Producer(Current_Language);
                  else
                     Role_Type:=Lb_Role_Consumer(Current_Language);
                  end if;
                  Role_Share:= To_String(A_Buffer.Roles.Entries(I).Item)
                     + To_String(To_Unbounded_String(Natural'Image(
                           A_Buffer.Roles.Entries(I).Data.Size)))
                     + To_String(Role_Type);
                  Update_Buffers_Set.Current_Role_Index:=
                     Indexed_Table_Range(
                     Index);
                  Index:=Index+1;
                  Line:= Append(Update_Buffers_Set.Affected_Role_List,
                     Role_Share);
               end loop;
            else
               Found:=False;
            end if;
         end if;
         exit when Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

      if not Found then
         Initialize(Items);

         Task_Set.Generic_Task_Set.Reset_Iterator(My_Iterator2);

         loop
            Current_Element(Sys.Tasks, A_Task, My_Iterator2);
            Add(Items,A_Task.Name);
            exit when Is_Last_Element(Sys.Tasks, My_Iterator2) = True;
            Next_Element(Sys.Tasks, My_Iterator2);
         end loop;

         -- Rebuild task option menu & role_Type option menu with something in
         --
         Destroy(Update_Buffers_Set.Role_Task_Name);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Task_Name);
         Update_Buffers_Set.Role_Name_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Buffers_Set.Role_Task_Name,
            Update_Buffers_Set.Role_Name_Menu, Items, -1, null);
         Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
            Role_Task_Name, 1, 2, 0, 1);
         ----- End Rebuild
         --

         Initialize(Items);
         Add(Items, Lb_Role_Producer(Current_Language));
         Add(Items, Lb_Role_Consumer(Current_Language));

         Destroy(Update_Buffers_Set.Role_Type);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Type);
         Update_Buffers_Set.Role_Type_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Buffers_Set.Role_Type,
            Update_Buffers_Set.Role_Type_Menu, Items, -1, null);
         Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
            Role_Type, 1, 2, 2, 3);
         -- End OptionMenu
         --

      end if;
   end Show_Roles;

   function Buffer_Modify return Buffer_Ptr is 

      A_Buffer        : Buffer_Ptr;  
      A_Processor     : Processor_Ptr;  
      My_Iterator     : Iterator;  
      Size_String,  
      Location_String,  
      Name_String     : Unbounded_String;  
      Size            : Integer           := 0;  
      Ok              : Boolean;  
      Cpu_Index       : Gint;  
      Items           : String_List.Glist;  

   begin
      if Update_Buffers_Set.Last_Buffer_Selected = To_Unbounded_String("") then
         A_Buffer:=Search_Buffer(Sys.Buffers,To_Unbounded_String(Get_Text(
                  Get_Entry(Update_Buffers_Set.Buffer))));
         return A_Buffer;
      end if;

      Name_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.Name));
      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         if (Name_String = A_Buffer.Name) then
            if(A_Buffer.Name/=Update_Buffers_Set.Last_Buffer_Selected) then
               Show_Message_Box(Lb_Buffer(Current_Language) &
                  Lb_Already_Defined(Current_Language));
               Sys.Buffers:=Update_Buffers_Set.Undo_Buffer;
               return Search_Buffer(Sys.Buffers,Update_Buffers_Set.
                  Last_Buffer_Selected);
            end if;
         end if;
         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Buffers_Set.Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;
      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Location_String :=A_Processor.Name;

      Size_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Buffer_Size));
      To_Integer(Size_String, Size, Ok);

      if not Ok then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         return A_Buffer;
      end if;

      if Size<=0 then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & To_Unbounded_String(">0") );
         return A_Buffer;
      end if;

      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         if A_Buffer.Name = Update_Buffers_Set.Last_Buffer_Selected then
            exit;
         end if;
         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

      Update_Buffers_Set.Last_Buffer_Selected:=A_Buffer.Name;

      if Name_String/=A_Buffer.Name or
            A_Buffer.Location/=Location_String or
            A_Buffer.Size/= Size then
         Update_Buffer(Sys.Buffers,Update_Buffers_Set.
            Last_Buffer_Selected, Name_String,Size,
            Location_String,A_Buffer.Roles);

         -- Rebuild the combo
         --
         Reset_Iterator(My_Iterator);
         Update_Buffers_Set.Last_Buffer_Selected:= Name_String;
         String_List.Append(Items,"");
         loop
            Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
            String_List.Append(Items,To_String(A_Buffer.Name));
            exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
            Next_Element(Sys.Buffers, My_Iterator);
         end loop;
         Set_Popdown_Strings(Update_Buffers_Set.Buffer,Items);
         -- End rebuild
         --
      end if;
      return Search_Buffer(Sys.Buffers,Update_Buffers_Set.
         Last_Buffer_Selected);
   end Buffer_Modify;


   procedure On_Buffer_Change (
         Object : access Gtk_Editable_Record'Class ) is 

      A_Buff      : Buffer_Ptr;  
      Current     : Gint;  
      Buff_Str    : Unbounded_String;  
      A_Processor : Processor_Ptr;  
      Items       : Strings_Table;  
      Index       : Strings_Range    := Strings_Range'First;  

      My_Iterator2 : Processor_Set.Generic_Processor_Set.Iterator;  

   begin
      Buff_Str:=To_Unbounded_String(Get_Text(Get_Entry(Update_Buffers_Set.
               Buffer)));
      if (Buff_Str /= To_Unbounded_String("")) then
         A_Buff:=Search_Buffer(Sys.Buffers,Buff_Str);
         Gtk.Gentry.Set_Text(Update_Buffers_Set.Name, To_String(A_Buff.
               Name));
         Gtk.Gentry.Set_Text(Update_Buffers_Set.Buffer_Size,
            To_String(To_Unbounded_String(Natural'Image(A_Buff.Size))));

         ----- Rebuild The CPU Option Menu & select the right item
         Initialize(Items);
         Index:=0;
         Processor_Set.Generic_Processor_Set.Reset_Iterator(My_Iterator2);

         loop
            Current_Element(Sys.Processors, A_Processor, My_Iterator2);
            Add(Items, A_Processor.Name);
            if A_Processor.Name = A_Buff.Location then
               Current:=Gint(Index);
            end if;
            Index:=Index+1;
            exit when Is_Last_Element(Sys.Processors, My_Iterator2) = True;
            Next_Element(Sys.Processors, My_Iterator2);
         end loop;
         Destroy(Update_Buffers_Set.Buffer_Location);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Buffer_Location);
         Update_Buffers_Set.Processor_Menu:= Widget_Slist.Null_List;

         Build_Option_Menu (Update_Buffers_Set.Buffer_Location,
            Update_Buffers_Set.Processor_Menu,
            Items, Current, null);
         Attach_Defaults(Update_Buffers_Set.Table_Buffer,
            Update_Buffers_Set.Buffer_Location,1,2,1,2);
         ----- End Rebuild
         --

         Update_Buffers_Set.Last_Buffer_Selected := A_Buff.Name;
         Show_Roles(A_Buff.Name);
      else

         Gtk.Gentry.Set_Text(Update_Buffers_Set.Name, "");
         Gtk.Gentry.Set_Text(Update_Buffers_Set.Buffer_Size,"");
         ----- Rebuild an empty CPU Option Menu
         --
         Initialize(Items);
         Destroy(Update_Buffers_Set.Buffer_Location);
         Processor_Set.Generic_Processor_Set.Reset_Iterator(My_Iterator2);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Buffer_Location);
         Update_Buffers_Set.Processor_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Buffers_Set.Buffer_Location,
            Update_Buffers_Set.Processor_Menu, Items, -1, null);
         Attach_Defaults(Update_Buffers_Set.Table_Buffer,
            Update_Buffers_Set.Buffer_Location,1,2,1,2);
         ----- End Rebuild
         --

         ----- Rebuild an empty Task Option Menu
         --
         Destroy(Update_Buffers_Set.Role_Task_Name);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Task_Name);
         Update_Buffers_Set.Role_Name_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Buffers_Set.Role_Task_Name,
            Update_Buffers_Set.Role_Name_Menu, Items, -1, null);
         Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
            Role_Task_Name, 1, 2, 0, 1);

         ----- End Rebuild
         --
         Gtk.Gentry.Set_Text(Update_Buffers_Set.Role_Size, "");
         ----- Rebuild an empty RoleType Option Menu
         Destroy(Update_Buffers_Set.Role_Type);
         Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Type);
         Update_Buffers_Set.Role_Type_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Buffers_Set.Role_Type,
            Update_Buffers_Set.Role_Type_Menu, Items, -1, null);
         Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
            Role_Type, 1, 2, 2, 3);
         -- End OptionMenu
         --

         Clear(Update_Buffers_Set.Affected_Role_List);
      end if;
      Show_All(Update_Buffers_Set);
   end  On_Buffer_Change;



   procedure On_Buffer_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      A_Buffer : Buffer_Ptr;  

   begin
      A_Buffer:=Buffer_Modify;
   end On_Buffer_Modify_Pressed;



   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Ok            : Boolean;  
      Size,  
      Buff_Size     : Natural;  
      Buffer_Name,  
      Size_String,  
      Task_String,  
      Buff_Location : Unbounded_String;  
      A_Buffer      : Buffer_Ptr;  
      Task_Index,  
      Type_Index    : Gint;  
      A_Task        : Task_Ptr;  
      Role_Type     : Role;  

   begin
      if Get_Rows(Update_Buffers_Set.Affected_Role_List) = 0 then
         Show_Message_Box(Lb_Select_Row_Before(Current_Language));
         return;
      end if;

      if Update_Buffers_Set.One_Row_Is_Selected /= True then
         Show_Message_Box(Lb_Select_Row_Before(Current_Language));
         return;
      end if;

      Buffer_Name:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Buffer)));
      A_Buffer:=Search_Buffer(Sys.Buffers,Buffer_Name);

      Size_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Role_Size));
      To_Natural(Size_String, Size, Ok);

      if Ok=False then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         return;
      end if;

      Task_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Tasks)) -
         Gint(Selected_Button (Update_Buffers_Set.Role_Name_Menu)));
      Task_Index:=Task_Index-1;
      Get_Element_Number(Sys.Tasks,A_Task,Tasks_Range(Task_Index));
      Task_String :=A_Task.Name;

      Type_Index:= 2 - Gint(Selected_Button (Update_Buffers_Set.
            Role_Type_Menu));
      Type_Index:=Type_Index - 1;
      if(Type_Index=0) then
         Role_Type:=Role(Producer);
      else
         Role_Type:=Role(Consumer);
      end if;

      Buff_Location:=A_Buffer.Location;
      Buff_Size:=A_Buffer.Size;

      A_Buffer.Roles.Entries(Update_Buffers_Set.Current_Role_Index).Data.
         The_Role:=Role_Type;
      A_Buffer.Roles.Entries(Update_Buffers_Set.Current_Role_Index).Data.
         Size :=Size;
      A_Buffer.Roles.Entries(Update_Buffers_Set.Current_Role_Index).Item:=
         Task_String;

      Update_Buffer(Sys.Buffers,Update_Buffers_Set.Last_Buffer_Selected,
         Buffer_Name, Buff_Size,Buff_Location , A_Buffer.Roles);

      Show_Roles(A_Buffer.Name);
      Update_Buffers_Set.One_Row_Is_Selected:=False;
   end On_Modify_Pressed;



   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Ok          : Boolean;  
      Size        : Integer;  
      Task_Index,  
      Type_Index  : Gint;  
      Buffer_Name,  
      Size_String,  
      Task_String : Unbounded_String;  
      A_Buffer    : Buffer_Ptr;  
      Role_Type   : Role;  
      A_Task      : Task_Ptr;  

   begin
      Size_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Role_Size));
      To_Integer(Size_String, Size, Ok);

      if not Ok then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         return;
      end if;

      if (Size<=0) then
         Show_Message_Box( Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & To_Unbounded_String(" >0 ") );
         return;
      end if;

      Task_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Tasks)) -
         Gint(Selected_Button (Update_Buffers_Set.Role_Name_Menu)));
      Task_Index:=Task_Index-1;
      Get_Element_Number(Sys.Tasks,A_Task,Tasks_Range(Task_Index));
      Task_String :=A_Task.Name;

      Type_Index:= 2- Gint(Selected_Button (Update_Buffers_Set.
            Role_Type_Menu));
      Type_Index:=Type_Index -1;
      if(Type_Index=0) then
         Role_Type:=Role(Producer);
      else
         Role_Type:=Role(Consumer);
      end if;

      Buffer_Name:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Buffer)));
      A_Buffer:=Search_Buffer(Sys.Buffers,Buffer_Name);

      A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Item:=Task_String;
      A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Data.Size:=Size;
      A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Data.The_Role:=
         Role_Type;
      A_Buffer.Roles.Nb_Entries:=A_Buffer.
         Roles.Nb_Entries+1;
      Update_Buffer(Sys.Buffers, Update_Buffers_Set.Last_Buffer_Selected,
         A_Buffer.Name, A_Buffer.Size,A_Buffer.Location, A_Buffer.Roles);

      Show_Roles(A_Buffer.Name);
      Gtk.Gentry.Set_Text(Update_Buffers_Set.Role_Size,"");
   end On_Add_Pressed;


   procedure On_Delete_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Ok                : Boolean;  
      Buff_Size,  
      Role_Size_Clist   : Natural;  
      Index_Del,  
      Row               : Gint;  
      Buffer_Name,  
      Buff_Location     : Unbounded_String;  
      Name_Clist,  
      Role_Begin_String,  
      Role_End_String   : Unbounded_String;  
      A_Buffer          : Buffer_Ptr;  
      Role_List_Share   : Buffer_Roles_Table;  
      Role_Type_Clist   : Role;  

   begin
      if Update_Buffers_Set.One_Row_Is_Selected /=True then
         Show_Message_Box(Lb_Select_Row_Before(Current_Language));
         return;
      end if;
      Update_Buffers_Set.One_Row_Is_Selected :=False;

      Buffer_Name:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Buffer)));
      A_Buffer:=Search_Buffer(Sys.Buffers,Buffer_Name);

      if Get_Rows(Update_Buffers_Set.Affected_Role_List) = 0 then
         Show_Message_Box(Lb_Select_Row_Before(Current_Language));
         return;
      end if;
      Row:=Get_Focus_Row(Update_Buffers_Set.Affected_Role_List);
      Name_Clist        := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Affected_Role_List,Row,0));
      Role_Begin_String := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Affected_Role_List,Row,2));
      Role_End_String   := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Affected_Role_List,Row,1));
      To_Natural(Role_End_String, Role_Size_Clist, Ok);

      if Role_Begin_String= To_String(Lb_Role_Producer(Current_Language)) then
         Role_Type_Clist:=Role(Producer);
      else
         Role_Type_Clist:=Role(Consumer);
      end if;

      Buff_Location:=A_Buffer.Location;
      Buff_Size:=A_Buffer.Size;
      Index_Del:=0;
      for Index in 0..Buffer_Roles_Range(Natural(A_Buffer.Roles.
               Nb_Entries) -1) loop
         if (A_Buffer.Roles.Entries(Index).Item = Name_Clist)
               and A_Buffer.Roles.Entries(Index).Data.The_Role =
               Role_Type_Clist
               and A_Buffer.Roles.Entries(Index).Data.Size =
               Role_Size_Clist
               then
            null;
         else
            Role_List_Share.Entries(Buffer_Roles_Range(Index_Del)):=
               A_Buffer.Roles.Entries(Buffer_Roles_Range(Index));
            Index_Del:=Index_Del+1;
         end if;
      end loop;
      Role_List_Share.Nb_Entries:=Buffer_Roles_Range(Natural(A_Buffer.
            Roles.Nb_Entries) -1);

      Update_Buffer(Sys.Buffers,Update_Buffers_Set.Last_Buffer_Selected,
         Buffer_Name, Buff_Size,Buff_Location , Role_List_Share);
      Show_Roles(Buffer_Name);

      Gtk.Gentry.Set_Text(Update_Buffers_Set.Role_Size,"");
   end On_Delete_Pressed;



end Update_Buffers_Set_Pkg.Callbacks;




