------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk.Window;
use Gtk.Window;
with Gtk.Clist;
use Gtk.Clist;
with Translate;
use Translate;
with Config;
use Config;
with Mmi_Tools;
use Mmi_Tools;
with Gtkada.Types;
use Gtkada.Types;

with Gtk.Table;
use Gtk.Table;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Gtk.Enums;
use Gtk.Enums;
with Glib;
use Glib;
with Gtk;
use Gtk;
with Gtk.Window;
use Gtk.Window;
with Gtk.Table;
use Gtk.Table;
with Gtk.Label;
use Gtk.Label;
with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Button;
use Gtk.Button;
with Gtk.Option_Menu;
use Gtk.Option_Menu;
with Gtk.Box;
use Gtk.Box;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Frame;
use Gtk.Frame;

with Project;
use Project;

with Gtk.Combo;
use Gtk.Combo;

with  Unbounded_Strings;
use  Unbounded_Strings;
use  Unbounded_Strings.Strings_Table_Package;

with Buffers;
use Buffers;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
use Buffers.Buffer_Roles;
with Processor_Set;
use Processor_Set;
with Update_Buffers_Set_Pkg.Callbacks;
use Update_Buffers_Set_Pkg.Callbacks;

package body Update_Buffers_Set_Pkg is

   procedure Gtk_New (
         Update_Buffers_Set :    out Update_Buffers_Set_Access ) is 

   begin
      Update_Buffers_Set := new Update_Buffers_Set_Record;
      Update_Buffers_Set_Pkg.Initialize(Update_Buffers_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Buffers_Set : access Update_Buffers_Set_Record'Class ) is 
      A_Buffer        : Buffer_Ptr;  
      Location_Items,  
      Role_Type_Items : Strings_Table;  
      My_Iterator     : Iterator;  
      My_Iterator2    : Processor_Set.Generic_Processor_Set.Iterator;  
      Tmp_Entry       : Gtk.Gentry.Gtk_Entry;  
   begin
      -- Build the Buffer Combo
      --
      Reset_Iterator(My_Iterator);
      String_List.Append(Update_Buffers_Set.Buffer_Items,"");
      loop
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         String_List.Append(Update_Buffers_Set.Buffer_Items,To_String(
               A_Buffer.Name));
         exit when Is_Last_Element(Sys.Buffers, My_Iterator) = True;
         Next_Element(Sys.Buffers, My_Iterator);
      end loop;

      Gtk.Window.Initialize(Update_Buffers_Set , Window_Toplevel);
      Set_Policy (Update_Buffers_Set, True, True, True);
      Set_Title (Update_Buffers_Set, To_String(Lb_Update_Buffer(
               Current_Language)));
      Set_Position (Update_Buffers_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Buffers_Set, True);
      Grab_Focus(Update_Buffers_Set);
      Gtk.Box.Gtk_New_Vbox(Update_Buffers_Set.Vbox_Main, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Buffers_Set.Hbox_Menu, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Buffers_Set.Hbox_List, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Buffers_Set.Hbox_Confirm, False, 10);

      Add(Update_Buffers_Set, Update_Buffers_Set.Vbox_Main);
      Add(Update_Buffers_Set.Vbox_Main, Update_Buffers_Set.Hbox_Menu);
      Add(Update_Buffers_Set.Vbox_Main, Update_Buffers_Set.Hbox_List);
      Add(Update_Buffers_Set.Vbox_Main, Update_Buffers_Set.Hbox_Confirm);

      Update_Buffers_Set.Undo_Buffer:= Sys.Buffers;

      -- Finalize the combo
      Gtk_New(Update_Buffers_Set.Buffer);
      Set_Popdown_Strings(Update_Buffers_Set.Buffer,Update_Buffers_Set.
         Buffer_Items);

      Gtk.Frame.Gtk_New(Update_Buffers_Set.Frame_Buffer,"");

      Gtk.Box.Gtk_New_Vbox(Update_Buffers_Set.Vbox_Frame_Buffer, False, 10);
      Gtk.Box.Gtk_New_Vbox(Update_Buffers_Set.Vbox_Menu, False, 10);


      --Create the CPU Option Menu
      Initialize(Location_Items);

      Processor_Set.Generic_Processor_Set.Reset_Iterator(My_Iterator2);
      Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Buffer_Location);
      Update_Buffers_Set.Processor_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Update_Buffers_Set.Buffer_Location,
         Update_Buffers_Set.Processor_Menu, Location_Items, -1, null);
      --End Option Menu

      Gtk.Gentry.Gtk_New(Update_Buffers_Set.Name,50);
      Gtk.Gentry.Gtk_New(Update_Buffers_Set.Buffer_Size,50);

      Gtk.Table.Gtk_New (Update_Buffers_Set.Table_Buffer,2, 4, False);

      Gtk.Label.Gtk_New(Update_Buffers_Set.Buffer_Combo_Label,To_String(
            Lb_Buffer_Name(Current_Language) & Lb_Colon));

      Gtk.Label.Gtk_New(Update_Buffers_Set.Buffer_Location_Label,
         To_String(Lb_Processor_Name(Current_Language)));
      Gtk.Label.Gtk_New(Update_Buffers_Set.Buffer_Size_Label,To_String(
            Lb_Size(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Buffers_Set.Buffer_Label,To_String(
            Lb_Buffer_Name(Current_Language) & Lb_Colon));
      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Buffer_Modify,
         To_String(Lb_Modify(Current_Language)));

      Add(Update_Buffers_Set.Hbox_Menu, Update_Buffers_Set.Vbox_Menu);
      Add(Update_Buffers_Set.Vbox_Menu,Update_Buffers_Set.
         Buffer_Combo_Label);
      Add(Update_Buffers_Set.Vbox_Menu, Update_Buffers_Set.Buffer);



      Add(Update_Buffers_Set.Hbox_Menu, Update_Buffers_Set.Frame_Buffer);
      Add(Update_Buffers_Set.Frame_Buffer, Update_Buffers_Set.
         Table_Buffer);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Buffer_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Name,1,2,0,1);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Buffer_Location_Label,0,1,1,2);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Buffer_Location,1,2,1,2);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Buffer_Size_Label,0,1,2,3);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Buffer_Size,1,2,2,3);
      Attach_Defaults(Update_Buffers_Set.Table_Buffer, Update_Buffers_Set.
         Button_Buffer_Modify,0,2,3,4);

      Gtk.Scrolled_Window.Gtk_New(Update_Buffers_Set.
         Scroll_Affected_Role_List, null, null);
      Add(Update_Buffers_Set.Hbox_List,Update_Buffers_Set.
         Scroll_Affected_Role_List);
      Update_Buffers_Set.Affect_List_Titles:= To_String(Lb_Task(
            Current_Language))
         +To_String(Lb_Size(Current_Language))
         +To_String(Lb_Role_Type(Current_Language));
      Gtk.Clist.Gtk_New(Update_Buffers_Set.Affected_Role_List,3,
         Update_Buffers_Set.Affect_List_Titles);
      Set_Column_Width(Update_Buffers_Set.Affected_Role_List, 0, 90);
      Set_Usize(Update_Buffers_Set.Affected_Role_List,200,20);
      Add(Update_Buffers_Set.Scroll_Affected_Role_List,Update_Buffers_Set.
         Affected_Role_List);

      Gtk.Frame.Gtk_New(Update_Buffers_Set.Frame_Role,"");
      Add(Update_Buffers_Set.Hbox_List, Update_Buffers_Set.Frame_Role);

      Gtk.Box.Gtk_New_Vbox(Update_Buffers_Set.Vbox_Frame_Role, False, 10);
      Add(Update_Buffers_Set.Frame_Role, Update_Buffers_Set.
         Vbox_Frame_Role);

      Gtk.Label.Gtk_New(Update_Buffers_Set.Role_Task_Name_Label,To_String(
            Lb_Task(Current_Language)));
      Gtk.Label.Gtk_New(Update_Buffers_Set.Role_Size_Label,To_String(
            Lb_Size(Current_Language)));
      Gtk.Label.Gtk_New(Update_Buffers_Set.Role_Type_Label, To_String(
            Lb_Role_Type(Current_Language)));


      --Create the RoleType option Menu
      Initialize(Role_Type_Items);
      Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Type);
      Update_Buffers_Set.Role_Type_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Update_Buffers_Set.Role_Type,Update_Buffers_Set.
         Role_Type_Menu, Role_Type_Items, -1, null);
      --End OptionMenu

      --Create the taskName Option Menu
      Initialize(Location_Items);
      Gtk.Option_Menu.Gtk_New(Update_Buffers_Set.Role_Task_Name);
      Update_Buffers_Set.Role_Name_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Update_Buffers_Set.Role_Task_Name,
         Update_Buffers_Set.Role_Name_Menu, Location_Items, -1, null);
      --End Option Menu

      Gtk.Gentry.Gtk_New(Update_Buffers_Set.Role_Size,50);

      Gtk.Table.Gtk_New (Update_Buffers_Set.Table_Role,4, 3, False);
      Add(Update_Buffers_Set.Vbox_Frame_Role, Update_Buffers_Set.
         Table_Role);

      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Add_Role, To_String(
            Lb_Add(Current_Language)));
      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Remove_Role, To_String(
            Lb_Delete(Current_Language)));
      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Modify, To_String(
            Lb_Modify(Current_Language)));

      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Task_Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Task_Name, 1, 2, 0, 1);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Size_Label, 0, 1, 1, 2);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Size, 1, 2, 1, 2);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Type_Label, 0, 1, 2, 3);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Role_Type, 1, 2, 2, 3);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Button_Add_Role, 0, 1, 3, 4);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Button_Remove_Role , 1, 2, 3, 4);
      Attach_Defaults(Update_Buffers_Set.Table_Role, Update_Buffers_Set.
         Button_Modify, 2, 3, 3, 4);

      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Add, To_String(
            Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Buffers_Set.Button_Cancel, To_String(
            Lb_Cancel(Current_Language)));

      Add(Update_Buffers_Set.Hbox_Confirm, Update_Buffers_Set.Button_Add);
      Add(Update_Buffers_Set.Hbox_Confirm, Update_Buffers_Set.
         Button_Cancel);

      Button_Callback.Connect (Update_Buffers_Set.Button_Cancel, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));
      Set_Editable(Get_Entry(Update_Buffers_Set.Buffer),False);
      Tmp_Entry:=Get_Entry(Update_Buffers_Set.Buffer);
      Combo_Callback.Connect (Tmp_Entry, "changed",
         Combo_Callback.To_Marshaller (On_Buffer_Change'access));

      List_Callback.Connect (Update_Buffers_Set.Affected_Role_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'access));

      Button_Callback.Connect (Update_Buffers_Set.Button_Buffer_Modify,
         "clicked",
         Button_Callback.To_Marshaller (On_Buffer_Modify_Pressed'access));

      Button_Callback.Connect (Update_Buffers_Set.Button_Modify, "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'access));
      Button_Callback.Connect (Update_Buffers_Set.Button_Remove_Role,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'access));
      Button_Callback.Connect (Update_Buffers_Set.Button_Add_Role,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'access));
      Button_Callback.Connect (Update_Buffers_Set.Button_Add, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Initialize(Update_Buffers_Set.My_Roles);
      Show_All(Update_Buffers_Set);
   exception
      when Buffer_Set.Generic_Buffer_Set.Empty_Set=>
         begin
            Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
         end;


   end Initialize;

end Update_Buffers_Set_Pkg;
