------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Project;
use Project;
with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;
with Translate;
use Translate;
with Config;
use Config;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
with Update_Messages_Set_Pkg;
use Update_Messages_Set_Pkg;
with Unbounded_Strings;
use Unbounded_Strings;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with Mmi_Tools;
use Mmi_Tools;



package body Update_Messages_Set_Pkg.Callbacks  is

   procedure Show_Messages (
         A_Message_Name : Unbounded_String ) is 
      A_Message          : Message_Ptr;  
      My_Iterator        : Iterator;  
      Line               : Gint;  
      Message_Share      : Chars_Ptr_Array (1 .. 5);  
      The_Name           : Unbounded_String;  
      The_Jitter         : Unbounded_String;  
      The_Deadline       : Unbounded_String;  
      The_Period         : Unbounded_String;  
      The_Size           : Unbounded_String;  
      A_Periodic_Message : Periodic_Message_Ptr;  

   begin
      Clear(Update_Messages_Set.Messages_List);
      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Messages, A_Message, My_Iterator);
         The_Name  := A_Message.Name;
         The_Deadline := To_Unbounded_String(Natural'Image(A_Message.
               Deadline));
         The_Size     := To_Unbounded_String(Natural'Image(A_Message.Size));
         if(A_Message.Message_Type=Periodic_Type) then
            A_Periodic_Message := Periodic_Message_Ptr(A_Message);
            The_Jitter   := To_Unbounded_String(Natural'Image(
                  A_Periodic_Message.Jitter));
            The_Period  := To_Unbounded_String(Natural'Image(
                  A_Periodic_Message.Period));
         end if;
         Message_Share:= To_String(The_Name)+To_String(The_Jitter)+
            To_String(The_Deadline)+To_String(The_Period)+To_String(
            The_Size);
         Line:= Append(Update_Messages_Set.Messages_List, Message_Share);

         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);
      end loop;


   end Show_Messages;

   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Sys.Messages:=Update_Messages_Set.Undo_Message;
      Destroy(Update_Messages_Set);
   end  On_Cancel_Pressed;


   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Is_New:=False;
      Is_Saved:=False;

      Rebuild_Canvas(Graph_Window);
      Hide(Update_Messages_Set);
   end On_Ok_Pressed;


   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 
      Name             : Unbounded_String;  
      Message_Jitter   : Unbounded_String;  
      Message_Deadline : Unbounded_String;  
      Message_Period   : Unbounded_String;  
      Message_Size     : Unbounded_String;  
      Row              : Gint;  
   begin

      Row:=Get_Focus_Row(Update_Messages_Set.Messages_List);
      Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,0));
      Update_Messages_Set.Current_Message := Name;
      Message_Jitter := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,1));
      Message_Deadline := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Messages_List,Row,2));
      Message_Period := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,3));
      Message_Size := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,4));

      Gtk.Gentry.Set_Text(Update_Messages_Set.Message_Name, To_String(
            Name));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Message_Jitter, To_String(
            Message_Jitter));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Message_Deadline, To_String(
            Message_Deadline));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Message_Period, To_String(
            Message_Period));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Message_Size, To_String(
            Message_Size));

   end On_Click_List;

   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok               : Boolean;  
      Message_Name     : Unbounded_String;  
      Message_Jitter   : Natural;  
      Message_Deadline : Natural;  
      Message_Period   : Natural;  
      Message_Size     : Natural;  

      Message_Jitter_String   : Unbounded_String;  
      Message_Deadline_String : Unbounded_String;  
      Message_Period_String   : Unbounded_String;  
      Message_Size_String     : Unbounded_String;  
      A_Message               : Message_Ptr;  
      My_Iterator             : Iterator;  
   begin
      Message_Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Message_Name));
      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Messages, A_Message, My_Iterator);
         if (Message_Name = A_Message.Name) then
            if(A_Message.Name/=Update_Messages_Set.Current_Message) then
               Show_Message_Box(Lb_Message(Current_Language) &
                  Lb_Already_Defined(Current_Language));
               Sys.Messages:=Update_Messages_Set.Undo_Message;
               return;
            end if;
         end if;
         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);
      end loop;
      Message_Jitter_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Message_Jitter));
      Message_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Messages_Set.Message_Deadline));
      Message_Period_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Message_Period));
      Message_Size_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Message_Size));

      Ok:=True;
      To_Natural(Message_Jitter_String,Message_Jitter,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      To_Natural(Message_Deadline_String,Message_Deadline,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Deadline(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      To_Natural(Message_Period_String,Message_Period,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      To_Natural(Message_Size_String,Message_Size,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be_Numeric(
               Current_Language));
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      if (Message_Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      if (Message_Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      if (Message_Period/=0 and Message_Deadline<Message_Jitter)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      if (Message_Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      if (Message_Size<=0)
            then
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         Sys.Messages:=Update_Messages_Set.Undo_Message;
         return;
      end if;

      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Messages, A_Message, My_Iterator);
         if A_Message.Name = Update_Messages_Set.Current_Message then
            Update_Message (Sys.Messages, A_Message.Name, Message_Name,
               Message_Size, Message_Period, Message_Deadline,
               Message_Jitter);
            Update_Messages_Set.Current_Message := Message_Name;
            exit;
         end if;

         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);

      end loop;

      Show_Messages(A_Message.Name);

   end On_Modify_Pressed;


   procedure On_Delete_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Message_Name : Unbounded_String;  
      A_Message    : Message_Ptr;  
      My_Iterator  : Iterator;  
   begin
      Message_Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Message_Name));

      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Messages, A_Message, My_Iterator);
         if A_Message.Name = Update_Messages_Set.Current_Message then
            if Get_Number_Of_Elements(Sys.Messages)>1 then
               Delete(Sys.Messages,A_Message);
               Show_Messages(A_Message.Name);
               exit;
            else
               Destroy(Update_Messages_Set);
               Delete(Sys.Messages,A_Message);
               exit;
            end if;
         end if;
         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;
         Next_Element(Sys.Messages, My_Iterator);

      end loop;

   end On_Delete_Pressed;

end Update_Messages_Set_Pkg.Callbacks;
