------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk.Clist;
use Gtk.Clist;
with Gtk.Table;
use Gtk.Table;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Glib;
use Glib;
with Gtk.Window;
use Gtk.Window;
with Gtk.Label;
use Gtk.Label;
with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Button;
use Gtk.Button;
with Gtk.Box;
use Gtk.Box;
with Gtk.Frame;
use Gtk.Frame;
with Gtk.Enums;
use Gtk.Enums;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Project;
use Project;
with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;
with Update_Messages_Set_Pkg.Callbacks;
use Update_Messages_Set_Pkg.Callbacks;
with Translate;
use Translate;
with Config;
use Config;

package body Update_Messages_Set_Pkg is

   procedure Gtk_New (
         Update_Messages_Set :    out Update_Messages_Set_Access ) is 

   begin
      Update_Messages_Set := new Update_Messages_Set_Record;
      Update_Messages_Set_Pkg.Initialize(Update_Messages_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Messages_Set : access Update_Messages_Set_Record'Class ) is 
      A_Message   : Message_Ptr;  
      My_Iterator : Iterator;  

      The_Name           : Unbounded_String;  
      The_Jitter         : Unbounded_String;  
      The_Deadline       : Unbounded_String;  
      The_Period         : Unbounded_String;  
      The_Size           : Unbounded_String;  
      A_Line             : Chars_Ptr_Array (1 .. 5);  
      Line               : Gint;  
      A_Periodic_Message : Periodic_Message_Ptr;  

   begin
      Gtk.Window.Initialize(Update_Messages_Set, Window_Toplevel);
      Set_Policy (Update_Messages_Set, True, True, True);
      Set_Title (Update_Messages_Set, To_String(Lb_Update_Message(
               Current_Language)));
      Set_Position (Update_Messages_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Messages_Set, True);
      Grab_Focus(Update_Messages_Set);

      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Messages_Set.Vbox_Main, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Messages_Set.Hbox_Message, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Messages_Set.Hbox_Confirm, False, 10);
      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_Choice, False, 10);

      Add(Update_Messages_Set, Update_Messages_Set.Vbox_Main);
      Add(Update_Messages_Set.Vbox_Main, Update_Messages_Set.Hbox_Message);
      Add(Update_Messages_Set.Vbox_Main, Update_Messages_Set.Hbox_Confirm);

      Update_Messages_Set.Undo_Message:= Sys.Messages;

      Reset_Iterator(My_Iterator);

      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_Choice, False, 10);

      Gtk.Frame.Gtk_New(Update_Messages_Set.Frame_Message,"");
      Gtk.Table.Gtk_New (Update_Messages_Set.Message_Table,3, 6, False);
      Set_Border_Width (Update_Messages_Set.Message_Table, 5);
      Add(Update_Messages_Set.Hbox_Message, Update_Messages_Set.
         Vbox_Choice);
      Add(Update_Messages_Set.Hbox_Message, Update_Messages_Set.
         Frame_Message);
      Add(Update_Messages_Set.Frame_Message, Update_Messages_Set.
         Message_Table);

      Gtk.Box.Gtk_New_Hbox (Update_Messages_Set.Hbox_Up,False,0);
      Set_Border_Width (Update_Messages_Set.Hbox_Up, 10);

      Gtk.Box.Gtk_New_Hbox (Update_Messages_Set.Hbox_Down,False,10);

      Gtk.Box.Gtk_New_Hbox (Update_Messages_Set.Hbox_Modif,False,5);
      Set_Border_Width (Update_Messages_Set.Hbox_Modif, 10);

      Add(Update_Messages_Set.Vbox_Choice,Update_Messages_Set.Hbox_Up);

      Add(Update_Messages_Set.Vbox_Choice,Update_Messages_Set.Hbox_Down);

      Gtk.Label.Gtk_New(Update_Messages_Set.Message_Name_Label, To_String(
            Lb_Name(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Messages_Set.Message_Jitter_Label,
         To_String(Lb_Jitter(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Messages_Set.Message_Deadline_Label,
         To_String(Lb_Deadline(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Messages_Set.Message_Period_Label,
         To_String(Lb_Period(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Messages_Set.Message_Size_Label, To_String(
            Lb_Size(Current_Language) & Lb_Colon));

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Message_Name,50);
      Gtk.Gentry.Gtk_New(Update_Messages_Set.Message_Jitter,50);
      Gtk.Gentry.Gtk_New(Update_Messages_Set.Message_Deadline,50);
      Gtk.Gentry.Gtk_New(Update_Messages_Set.Message_Period,50);
      Gtk.Gentry.Gtk_New(Update_Messages_Set.Message_Size,50);

      Gtk.Button.Gtk_New(Update_Messages_Set.Button_Modify,    To_String(
            Lb_Modify(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Button_Remove_Message,
         To_String(Lb_Delete(Current_Language)));

      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Jitter_Label, 0, 1, 1, 2);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Deadline_Label, 0, 1, 2, 3);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Period_Label, 0, 1, 3, 4);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Size_Label, 0, 1, 4, 5);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Name, 1, 2, 0, 1);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Jitter, 1, 2, 1, 2);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Deadline, 1, 2, 2, 3);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Period, 1, 2, 3, 4);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Message_Size, 1, 2, 4, 5);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.Hbox_Modif, 0, 3, 5, 6);

      Add(Update_Messages_Set.Hbox_Modif, Update_Messages_Set.
         Button_Modify);
      Add(Update_Messages_Set.Hbox_Modif, Update_Messages_Set.
         Button_Remove_Message);

      Gtk.Scrolled_Window.Gtk_New(Update_Messages_Set.Scroll_List, null, null);
      Add(Update_Messages_Set.Hbox_Down,Update_Messages_Set.Scroll_List);

      Update_Messages_Set.List_Titles:= To_String(Lb_Name(
            Current_Language))
         +To_String(Lb_Jitter(Current_Language))
         +To_String(Lb_Deadline(Current_Language))
         +To_String(Lb_Period(Current_Language))
         +To_String(Lb_Size(Current_Language));

      Gtk.Clist.Gtk_New(Update_Messages_Set.Messages_List,5,
         Update_Messages_Set.List_Titles);
      Set_Usize(Update_Messages_Set.Messages_List,300,200);
      Add(Update_Messages_Set.Scroll_List,Update_Messages_Set.
         Messages_List);
      Reset_Iterator(My_Iterator);
      Reset_Iterator(My_Iterator);

      loop

         Current_Element(Sys.Messages, A_Message, My_Iterator);
         The_Name  := A_Message.Name;
         The_Deadline := To_Unbounded_String(Natural'Image(A_Message.
               Deadline));
         The_Size     := To_Unbounded_String(Natural'Image(A_Message.Size));
         if(A_Message.Message_Type=Periodic_Type) then
            A_Periodic_Message := Periodic_Message_Ptr(A_Message);
            The_Jitter   := To_Unbounded_String(Natural'Image(
                  A_Periodic_Message.Jitter));
            The_Period  := To_Unbounded_String(Natural'Image(
                  A_Periodic_Message.Period));
         end if;

         --Create the line
         A_Line := To_String(The_Name)+To_String(The_Jitter)+To_String(
            The_Deadline)+To_String(The_Period)+To_String(The_Size);
         --Display the Line
         Line := Append(Update_Messages_Set.Messages_List, A_Line);

         exit when Is_Last_Element(Sys.Messages, My_Iterator) = True;

         Next_Element(Sys.Messages, My_Iterator);

      end loop;



      Gtk.Button.Gtk_New(Update_Messages_Set.Button_Add, To_String(
            Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Button_Cancel, To_String(
            Lb_Cancel(Current_Language)));

      Add(Update_Messages_Set.Hbox_Confirm, Update_Messages_Set.
         Button_Add);
      Add(Update_Messages_Set.Hbox_Confirm, Update_Messages_Set.
         Button_Cancel);

      List_Callback.Connect (Update_Messages_Set.Messages_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'access));
      Button_Callback.Connect (Update_Messages_Set.Button_Modify,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'access));
      Button_Callback.Connect (Update_Messages_Set.Button_Remove_Message,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'access));
      Button_Callback.Connect (Update_Messages_Set.Button_Add, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Button_Callback.Connect (Update_Messages_Set.Button_Cancel,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));
   end Initialize;

end Update_Messages_Set_Pkg;
