------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use Text_Io;

with Mmi_Tools;
use Mmi_Tools;


with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Project;
use Project;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;
with Scheduler.Dynamic_Priority.Edf;
use Scheduler.Dynamic_Priority.Edf;
with Scheduler.Fixed_Priority.Rm;
use Scheduler.Fixed_Priority.Rm;
with Scheduler.Dynamic_Priority.Llf;
use Scheduler.Dynamic_Priority.Llf;
with Scheduler.Fixed_Priority.Dm;
use Scheduler.Fixed_Priority.Dm;
with Scheduler.Fixed_Priority.Hpf;
use Scheduler.Fixed_Priority.Hpf;
with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;

with Config;
use Config;

with Translate;
use Translate;

with Scrolling_Editor_Pkg;
use Scrolling_Editor_Pkg;



package body Update_Processors_Set_Pkg.Callbacks is


   procedure On_Edit_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Sched         : Natural;  
      Nb_Processors : Processors_Range := Get_Number_Of_Elements (Sys.Processors);
         



   begin

      -- Looking for the processor to edit
      --
      for I in 0..Nb_Processors-1 loop

         if Update_Processors_Set.Entries(I).Edit_Button
               = Gtk_Button(Object) then

            -- Only parametric scheduling can be edited
            --
            Sched:=Gtk.Radio_Menu_Item.Selected_Button (
               Update_Processors_Set.Entries(I).
               Clist_Scheduler_Menu);
            if (Sched/=0)
                  then
               Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
               return;
            end if;


            Gtk_New (Scrolling_Editor,
               Lb_Edit(Current_Language) &
               Unbounded_Sp &
               Lb_Scheduler_Behavior(Current_Language),
               Update_Processors_Set.Entries(I).Behavior_String);
            Set_Usize(Scrolling_Editor,800,450);
            Show_All (Scrolling_Editor);

         end if;
      end loop;

   end On_Edit_Pressed;


   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Nb_Processors : Processors_Range := Get_Number_Of_Elements (Sys.Processors);
         





      type A_Processor is 
         record 
            Name        : Unbounded_String;  
            A_Scheduler : Scheduler_Ptr;  
            Quantum     : Unbounded_String;  
         end record; 
      type Processors_List is array (Processors_Range) of A_Processor; 

      List    : Processors_List;  
      K,  
      L       : Processors_Range;  
      Sched   : Natural;  
      Quantum : Integer;  
      Ok      : Boolean;  

   begin


      for I in 0..Nb_Processors-1 loop

         L:=I+1;

         List(I).Name:=To_Unbounded_String(Get_Text(Update_Processors_Set.
               Entries(I).Processor_Name));

         if List(I).Name=""
               then
            Show_Message_Box(Lb_Processor_Name(Current_Language) &
               Lb_Mandatory(Current_Language) &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         Sched:=Gtk.Radio_Menu_Item.Selected_Button (
            Update_Processors_Set.Entries(I).
            Clist_Scheduler_Menu);

         if (Sched=5)
               then
            List(I).A_Scheduler := new Edf_Scheduler;
         end if;
         if (Sched=4)
               then
            List(I).A_Scheduler := new Llf_Scheduler;
         end if;
         if (Sched=3)
               then
            List(I).A_Scheduler := new Rm_Scheduler;
         end if;
         if (Sched=2)
               then
            List(I).A_Scheduler := new Dm_Scheduler;
         end if;
         if (Sched=1)
               then
            List(I).A_Scheduler := new Hpf_Scheduler;
         end if;
         if (Sched=0)
               then
            List(I).A_Scheduler := new Parametric_Scheduler;

            -- Save behavior
            --
            Set_String_Behavior(Parametric_Scheduler_Ptr(List(I).
                  A_Scheduler).all,
               Update_Processors_Set.Entries(I).Behavior_String.all);

         end if;


         if Get_Active(Update_Processors_Set.Entries(I).Scheduler_Option)
               then
            Set_Preemptive(List(I).A_Scheduler.all, Preemptive);
         else
            Set_Preemptive(List(I).A_Scheduler.all, Not_Preemptive);
         end if;


         List(I).Quantum:=To_Unbounded_String(Get_Text(
               Update_Processors_Set.Entries(I).Quantum));
         if(List(I).Quantum="")
               then
            Quantum:=0;
         else
            To_Integer(List(I).Quantum,Quantum,Ok);
            if not Ok
                  then
               Show_Message_Box("Quantum" &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         if (Quantum<0)
               then
            Show_Message_Box("Quantum" &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;


         if (Quantum/=0) and
               (Sched/=1)
               then
            Show_Message_Box(Lb_Quantum_Control(Current_Language)
               & To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;


         Set_Quantum(List(I).A_Scheduler.all, Quantum);


      end loop;


      for I in 0..Nb_Processors-1 loop
         for J in 0..Nb_Processors-1 loop
            if ( (I /= J)
                  and (List(I).Name=List(J).Name) )
                  then
               L:=I+1;
               K:=J+1;


               Show_Message_Box(Lb_Processor_Name(Current_Language) &
                  Lb_Already_Defined(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ", " & K'Img &
                     ")") );
               return;
            end if;
         end loop;
      end loop;

      Reset(Sys.Processors);

      for I in 0..Nb_Processors-1 loop
         Add_Processor(Sys.Processors,List(I).Name,List(I).A_Scheduler);
      end loop;

      Is_New:=False;
      Is_Saved:=False;

      Hide(Update_Processors_Set);

   end On_Ok_Pressed;



   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin
      Hide(Update_Processors_Set);
   end On_Cancel_Pressed;

end Update_Processors_Set_Pkg.Callbacks;
