------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gdk.Types;
use Gdk.Types;
with Gtk.Enums;
use Gtk.Enums;
with Callbacks_Cheddar;
use Callbacks_Cheddar;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Scheduler;
use Scheduler;
with Processors;
use Processors;
with Project;
use Project;
with Mmi_Tools;
use Mmi_Tools;

with Update_Processors_Set_Pkg.Callbacks;
use Update_Processors_Set_Pkg.Callbacks;

with Scheduler.Parametrics;
use Scheduler.Parametrics;

with Config;
use Config;
with Translate;
use Translate;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;



package body Update_Processors_Set_Pkg is

   procedure Gtk_New (
         Update_Processors_Set :    out Update_Processors_Set_Access ) is 

   begin
      Update_Processors_Set := new Update_Processors_Set_Record;
      Update_Processors_Set_Pkg.Initialize (Update_Processors_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Processors_Set : access Update_Processors_Set_Record'Class ) is 


      A_Processor  : Processor_Ptr;  
      My_Scheduler : Scheduler_Ptr;  
      My_Iterator  : Iterator;  
      Ligne        : Guint         := 1;  
      Sched_Id     : Gint;  
      Items        : Strings_Table;  


   begin

      Initialize(Items);

      Add(Items,To_Unbounded_String("Earliest Deadline First"));
      Add(Items,To_Unbounded_String("Least Laxity First"));
      Add(Items,To_Unbounded_String("Rate Monotonic"));
      Add(Items,To_Unbounded_String("Deadline Monotonic"));
      Add(Items,To_Unbounded_String("Highiest Priority First/POSIX 4"));
      Add(Items,To_Unbounded_String("Parametric scheduler"));


      Gtk.Window.Initialize (Update_Processors_Set, Window_Toplevel);
      Set_Title (Update_Processors_Set, To_String(Lb_Update_Processor(
               Current_Language)));
      Set_Policy (Update_Processors_Set, True, True, True);
      Set_Position (Update_Processors_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Processors_Set, True);
      Set_Extension_Events (Update_Processors_Set, Extension_Events_All);
      Set_Flags (Update_Processors_Set, Can_Default);
      Grab_Focus (Update_Processors_Set);

      Gtk_New (Update_Processors_Set.Table, Guint(3+
            Get_Number_Of_Elements(Sys.Processors)), 3, True);
      Set_Row_Spacings (Update_Processors_Set.Table, 8);
      Set_Col_Spacings (Update_Processors_Set.Table, 7);
      Add (Update_Processors_Set, Update_Processors_Set.Table);

      Gtk_New (Update_Processors_Set.Processor_Name, To_String(
            Lb_Processor_Name(Current_Language)));
      Set_Alignment (Update_Processors_Set.Processor_Name, 0.5, 0.5);
      Set_Padding (Update_Processors_Set.Processor_Name, 0, 0);
      Set_Justify (Update_Processors_Set.Processor_Name, Justify_Center);
      Set_Line_Wrap (Update_Processors_Set.Processor_Name, False);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Processor_Name, 0, 1, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Update_Processors_Set.Scheduler_Name, To_String(
            Lb_Scheduler(Current_Language)));
      Set_Alignment (Update_Processors_Set.Scheduler_Name, 0.5, 0.5);
      Set_Padding (Update_Processors_Set.Scheduler_Name, 0, 0);
      Set_Justify (Update_Processors_Set.Scheduler_Name, Justify_Center);
      Set_Line_Wrap (Update_Processors_Set.Scheduler_Name, False);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Scheduler_Name, 1, 2, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Update_Processors_Set.Quantum, "Quantum");
      Set_Alignment (Update_Processors_Set.Quantum, 0.5, 0.5);
      Set_Padding (Update_Processors_Set.Quantum, 0, 0);
      Set_Justify (Update_Processors_Set.Quantum, Justify_Center);
      Set_Line_Wrap (Update_Processors_Set.Quantum, False);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Quantum, 2, 3, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Update_Processors_Set.Scheduler_Option, "Options");
      Set_Alignment (Update_Processors_Set.Scheduler_Option, 0.5, 0.5);
      Set_Padding (Update_Processors_Set.Scheduler_Option, 0, 0);
      Set_Justify (Update_Processors_Set.Scheduler_Option, Justify_Center);
      Set_Line_Wrap (Update_Processors_Set.Scheduler_Option, False);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Scheduler_Option, 3, 4, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Update_Processors_Set.Edit_Label,
         To_String(Lb_Scheduler_Behavior(Current_Language)));
      Set_Alignment (Update_Processors_Set.Edit_Label, 0.5, 0.5);
      Set_Padding (Update_Processors_Set.Edit_Label, 0, 0);
      Set_Justify (Update_Processors_Set.Edit_Label, Justify_Center);
      Set_Line_Wrap (Update_Processors_Set.Edit_Label, False);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Edit_Label, 4, 5, 0, 1,
         0, 0,
         0, 0);



      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         Gtk_New (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Processor_Name);
         Set_Editable (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Processor_Name, False);
         Set_Max_Length (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Processor_Name, 0);
         Set_Text (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Processor_Name,
            To_String((A_Processor.all.Name)));
         Set_Visibility (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Processor_Name, True);
         Attach (Update_Processors_Set.Table, Update_Processors_Set.
            Entries(Processors_Range(Ligne-1)).Processor_Name, 0, 1, Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         My_Scheduler:=A_Processor.all.Scheduler;


         if (Get_Name(My_Scheduler.all) = Earliest_Deadline_First)
               then
            Sched_Id:=0;
         end if;
         if (Get_Name(My_Scheduler.all) = Least_Laxity_First)
               then
            Sched_Id:=1;
         end if;
         if (Get_Name(My_Scheduler.all) = Rate_Monotonic)
               then
            Sched_Id:=2;
         end if;
         if (Get_Name(My_Scheduler.all) = Deadline_Monotonic)
               then
            Sched_Id:=3;
         end if;
         if (Get_Name(My_Scheduler.all) = Highiest_Priority_First)
               then
            Sched_Id:=4;
         end if;
         if (Get_Name(My_Scheduler.all) = Parametric)
               then
            Sched_Id:=5;
         end if;

         Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Clist_Scheduler_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Processors_Set.Entries(
               Processors_Range(Ligne-1)).Scheduler_Menu,
            Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Clist_Scheduler_Menu, Items, Sched_Id, null);
         Attach (Update_Processors_Set.Table, Update_Processors_Set.
            Entries(Processors_Range(Ligne-1)).Scheduler_Menu, 1, 2, Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Quantum);
         Set_Editable (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Quantum, True);
         Set_Max_Length (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Quantum, 0);
         Set_Text (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Quantum, To_String(Get_Quantum(My_Scheduler.all)) );
         Set_Visibility (Update_Processors_Set.Entries(Processors_Range(
                  Ligne-1)).Quantum, True);
         Attach (Update_Processors_Set.Table, Update_Processors_Set.
            Entries(Processors_Range(Ligne-1)).Quantum, 2, 3, Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Scheduler_Option, To_String(Lb_Preemptive(Current_Language)));
         if (Get_Preemptive(My_Scheduler.all) = Preemptive)
               then
            Set_Active(Update_Processors_Set.Entries(Processors_Range(
                     Ligne-1)).Scheduler_Option, True);
         else
            Set_Active(Update_Processors_Set.Entries(Processors_Range(
                     Ligne-1)).Scheduler_Option, False);
         end if;
         Attach (Update_Processors_Set.Table, Update_Processors_Set.
            Entries(Processors_Range(Ligne-1)).Scheduler_Option,  3, 4,
            Ligne, Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Edit_Button, To_String(Lb_Edit(Current_Language)));

         Attach (Update_Processors_Set.Table, Update_Processors_Set.
            Entries(Processors_Range(Ligne-1)).Edit_Button,  4, 5,
            Ligne, Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Button_Callback.Connect(Update_Processors_Set.Entries(
               Processors_Range(Ligne-1)).Edit_Button, "clicked",
            Button_Callback.To_Marshaller (On_Edit_Pressed'access));


         Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
            Behavior_String:=
            new Unbounded_String;

         if (Get_Name(My_Scheduler.all) = Parametric)
               then
            Update_Processors_Set.Entries(Processors_Range(Ligne-1)).
               Behavior_String.all:=Get_String_Behavior(
               Parametric_Scheduler(My_Scheduler.all));
         end if;

         Ligne:=Ligne+1;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);


      end loop;




      Gtk_New_Hseparator (Update_Processors_Set.Hseparator);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.
         Hseparator, 0, 5, Ligne, Ligne+1,
         Fill, Expand or Fill,
         0, 0);


      Gtk_New_Hbox (Update_Processors_Set.Hbox, True, 0);
      Set_Border_Width (Update_Processors_Set.Hbox, 6);
      Attach (Update_Processors_Set.Table, Update_Processors_Set.Hbox, 0,
         5, Ligne+1, Ligne+2,
         Fill, Expand or Fill,
         0, 0);


      Gtk_New (Update_Processors_Set.Ok_Button, To_String(Lb_Modify(
               Current_Language)));
      Pack_Start (Update_Processors_Set.Hbox, Update_Processors_Set.
         Ok_Button, True,
         True, 0);

      Gtk_New_Vseparator (Update_Processors_Set.Vseparator);
      Set_Sensitive (Update_Processors_Set.Vseparator, False);
      Pack_Start (Update_Processors_Set.Hbox, Update_Processors_Set.
         Vseparator, True, True, 0);

      Gtk_New (Update_Processors_Set.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Update_Processors_Set.Hbox, Update_Processors_Set.
         Cancel_Button, True, True, 0);


      Button_Callback.Connect
         (Update_Processors_Set.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Button_Callback.Connect
         (Update_Processors_Set.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));



   end Initialize;

end Update_Processors_Set_Pkg;
