------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Project;
use Project;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Resources;
use Resources;
use Resources.Task_Lists;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Update_Resources_Set_Pkg;
use Update_Resources_Set_Pkg;

with Config;
use Config;

with Gtk.Check_Button;
use Gtk.Check_Button;

with Mmi_Tools;
use Mmi_Tools;

with Unbounded_Strings;
use Unbounded_Strings;

with Translate;
use Translate;


with Text_Io;
use Text_Io;


package body Update_Resources_Set_Pkg.Callbacks is

   procedure Verify_Task_Exist is 
   begin
      null;
   end Verify_Task_Exist;

   procedure On_Check_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      My_Iterator : Processors_Iterator;  
      A_Processor : Processor_Ptr;  
      Menu_Tmp    : String_List.Glist;  
      Zero        : Guint               := 0;  
   begin
      Update_Resources_Set.Resource_Menu:=Menu_Tmp;
      if Get_Active(Update_Resources_Set.Check_Change_Cpu) = True
            then
         loop
            Current_Element(Sys.Processors, A_Processor, My_Iterator);
            String_List.Append(Update_Resources_Set.Change_Cpu_Menu,
               To_String(A_Processor.Name));
            exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
            Next_Element(Sys.Processors, My_Iterator);
         end loop;
         if String_List.Length(Update_Resources_Set.Resource_Menu) = Zero
               then
            String_List.Append (Update_Resources_Set.Resource_Menu, " ");
         end if;
         Set_Popdown_Strings (Update_Resources_Set.Change_Cpu_List,
            Update_Resources_Set.Change_Cpu_Menu);
         Show(Update_Resources_Set.Change_Cpu_List);
      else
         Hide(Update_Resources_Set.Change_Cpu_List);
      end if;

   end On_Check_Pressed;

   procedure Show_Tasks (
         A_Resource_Name : Unbounded_String ) is 
      A_Resource  : Resource_Ptr;  
      My_Iterator : Resources_Iterator;  
      Line        : Gint;  
      Task_Share  : Chars_Ptr_Array (1 .. 3);  
   begin

      Reset_Iterator(My_Iterator);
      Clear(Update_Resources_Set.Affected_Task_List);

      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if To_String(A_Resource.Name) = To_String(A_Resource_Name)
               then
            for Index in 0..A_Resource.Task_Tab.Nb_Entries -1 loop

               Task_Share:= To_String(A_Resource.Task_Tab.Entries(Index).
                  Item)
                  +To_String(To_Unbounded_String(Natural'Image(A_Resource.
                        Task_Tab.Entries(Index).Data.Task_Begin)))
                  +To_String(To_Unbounded_String(Natural'Image(A_Resource.
                        Task_Tab.Entries(Index).Data.Task_End)));

               Line:= Append(Update_Resources_Set.Affected_Task_List,
                  Task_Share);

            end loop;
            exit;
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

   end Show_Tasks;

   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Sys.Resources:=Update_Resources_Set.Undo_Resource;
      Destroy(Update_Resources_Set);
   end  On_Cancel_Pressed;

   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Is_New:=False;
      Is_Saved:=False;

      Resource_Modify;
      Hide(Update_Resources_Set);
   end On_Ok_Pressed;

   procedure On_Cpu_Change (
         Object : access Gtk_Editable_Record'Class ) is 
   begin
      Update_Resources_Set.Undo_Resource:= Sys.Resources;
      On_Cpu_Change_Show;
   end On_Cpu_Change;

   procedure On_Cpu_Change_Show is 
      A_Resource      : Resource_Ptr;  
      My_Iterator     : Resources_Iterator;  
      Processor       : Unbounded_String;  
      Combo_Task_Name : Unbounded_String;  
      Protocol        : Unbounded_String;  
      Menu_Tmp        : String_List.Glist;  
      Zero            : Guint              := 0;  
   begin
      Resource_Modify;
      if Get_Active(Update_Resources_Set.Check_Change_Cpu) = True
            then
         Set_Active(Update_Resources_Set.Check_Change_Cpu, False);
         Hide(Update_Resources_Set.Change_Cpu_List);
      end if;

      Reset_Iterator(My_Iterator);
      Processor:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Resources_Set.Cpu)));
      Update_Resources_Set.Last_Cpu_Selected := Processor;

      Update_Resources_Set.Resource_Menu:=Menu_Tmp;
      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Location = Processor
               then
            String_List.Append (Update_Resources_Set.Resource_Menu,
               To_String(A_Resource.Name));
            Protocol:=To_Unbounded_String(A_Resource.Protocol'Img);
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;
      if String_List.Length(Update_Resources_Set.Resource_Menu) = Zero
            then
         String_List.Append (Update_Resources_Set.Resource_Menu, " ");
      end if;

      String_List.List_Reverse(Update_Resources_Set.Resource_Menu);
      Set_Popdown_Strings (Update_Resources_Set.Resource,
         Update_Resources_Set.Resource_Menu);

      Combo_Task_Name := To_Unbounded_String(Gtk.Gentry.Get_Text(
            Get_Entry(Update_Resources_Set.Resource)));
      Gtk.Gentry.Set_Text(Update_Resources_Set.State, To_String(
            To_Unbounded_String(Integer'Image(A_Resource.State))));

      if Protocol = "NO_PROTOCOL"
            then
         Set_Active(Update_Resources_Set.Button_No_Protocol, True);
      else
         if Protocol = "PRIORITY_CEILING_PROTOCOL"
               then
            Set_Active(Update_Resources_Set.Button_Pcp, True);
         else
            if Protocol = "PRIORITY_INHERITANCE_PROTOCOL"
                  then
               Set_Active(Update_Resources_Set.Button_Pip, True);
            end if;
         end if;
      end if;

   end On_Cpu_Change_Show;

   procedure Resource_Modify is 
      Protocol     : Resources_Type;  
      A_Resource   : Resource_Ptr;  
      My_Iterator  : Resources_Iterator;  
      State_String : Unbounded_String;  
      Cpu          : Unbounded_String;  
      State        : Integer            := 0;  
      Ok           : Boolean;  
   begin

      if Update_Resources_Set.Last_Resource_Selected = To_Unbounded_String(
            "")
            then
         Set_Active(Update_Resources_Set.Check_Change_Cpu, False);
         Hide(Update_Resources_Set.Change_Cpu_List);
         return;
      end if;

      Reset_Iterator(My_Iterator);

      if Get_Active(Update_Resources_Set.Button_No_Protocol) = True
            then
         Protocol := No_Protocol;
      else
         if Get_Active(Update_Resources_Set.Button_Pcp) = True
               then
            Protocol := Priority_Ceiling_Protocol;
         else
            if Get_Active(Update_Resources_Set.Button_Pip) = True
                  then
               Protocol := Priority_Inheritance_Protocol;
            end if;
         end if;
      end if;

      State_String:=To_Unbounded_String(Get_Text(Update_Resources_Set.
            State));
      if State_String=""
            then
         State:=0;
      else
         To_Integer(State_String, State, Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Resource_State(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;


      loop

         Current_Element(Sys.Resources, A_Resource, My_Iterator);


         if A_Resource.Name = Update_Resources_Set.Last_Resource_Selected
               then
            exit;
         end if;
         if Is_Last_Element(Sys.Resources, My_Iterator) = True
               then
            raise Resource_Not_Found;
         end if;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

      if Get_Active(Update_Resources_Set.Check_Change_Cpu) = True
            then
         Cpu:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
                  Change_Cpu_List)));
         Set_Active(Update_Resources_Set.Check_Change_Cpu, False);
         Hide(Update_Resources_Set.Change_Cpu_List);
      else
         Cpu:=A_Resource.Location;
      end if;


      Update_Resource (Sys.Resources,
         A_Resource.Name, State, Cpu,  To_Unbounded_String(Protocol'Img),
         A_Resource.Task_Tab);

   end Resource_Modify;

   procedure On_Change_Resource (
         Widget : access Gtk_Editable_Record'Class ) is 
      Protocol      : Unbounded_String;  
      Resource_Name : Unbounded_String;  
      A_Resource    : Resource_Ptr;  
      My_Iterator   : Resources_Iterator;  
   begin
      Resource_Modify;
      if To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
                  Resource))) = To_Unbounded_String("")
            then
         Update_Resources_Set.Last_Resource_Selected:= To_Unbounded_String(
            "");
         return;
      end if;

      Reset_Iterator(My_Iterator);
      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Name = To_Unbounded_String(Get_Text(Get_Entry(
                     Update_Resources_Set.Resource)))
               then
            Protocol:=To_Unbounded_String(A_Resource.Protocol'Img);
            exit;
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

      Gtk.Gentry.Set_Text(Update_Resources_Set.State, To_String(
            To_Unbounded_String(Integer'Image(A_Resource.State))));

      if Protocol = "No Protocol"
            then
         Set_Active(Update_Resources_Set.Button_No_Protocol, True);
      else
         if Protocol = "PCP"
               then
            Set_Active(Update_Resources_Set.Button_Pcp, True);
         else
            if Protocol = "PIP"
                  then
               Set_Active(Update_Resources_Set.Button_Pip, True);
            end if;
         end if;
      end if;
      Resource_Name:=To_Unbounded_String(To_String(A_Resource.Name));
      Update_Resources_Set.Last_Resource_Selected := Resource_Name;
      Show_Tasks(Resource_Name);

   end On_Change_Resource;

   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 
      Name       : Unbounded_String;  
      Task_Begin : Unbounded_String;  
      Task_End   : Unbounded_String;  
      Row        : Gint;  
   begin

      Row:=Get_Focus_Row(Update_Resources_Set.Affected_Task_List);
      Name       := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,0));
      Task_Begin := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,1));
      Task_End   := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,2));

      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Begin, To_String(
            Task_Begin));
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_End, To_String(
            Task_End));

   end On_Click_List;

   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok                : Boolean;  
      Name              : Unbounded_String;  
      State             : Integer;  
      Cpu               : Unbounded_String;  
      Protocol          : Unbounded_String;  
      State_String      : Unbounded_String;  
      Task_Name         : Unbounded_String;  
      Task_Begin        : Natural;  
      Task_End          : Natural;  
      Task_Begin_String : Unbounded_String;  
      Task_End_String   : Unbounded_String;  
      Task_Begin_Clist  : Natural          := 0;  
      Task_End_Clist    : Natural          := 0;  
      Row               : Gint;  
      Name_Clist        : Unbounded_String;  
      A_Resource        : Resource_Ptr;  

      Task_List_Share : Resources.Task_List;  
      My_Iterator     : Resources_Iterator;  

      Ite2   : Tasks_Iterator;  
      A_Task : Task_Ptr;  


   begin
      Reset_Iterator(My_Iterator);

      Cpu:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Cpu)));


      State_String:=To_Unbounded_String(Get_Text(Update_Resources_Set.
            State));
      To_Natural(State_String, State, Ok);

      if State_String=""
            then
         State:=0;
      else
         To_Integer(State_String, State, Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Resource_State(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;


      Name:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Resource)));



      if Get_Active(Update_Resources_Set.Button_No_Protocol) = True
            then
         Protocol:=To_Unbounded_String(Resources_Type'Image(
               No_Protocol));
      else
         if Get_Active(Update_Resources_Set.Button_Pcp) = True
               then
            Protocol:=To_Unbounded_String(Resources_Type'Image(
                  Priority_Ceiling_Protocol));
         else
            if Get_Active(Update_Resources_Set.Button_Pip) = True
                  then
               Protocol:=To_Unbounded_String(Resources_Type'Image(
                     Priority_Inheritance_Protocol));
            end if;
         end if;
      end if;

      Task_Name := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Task_Name));
      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_Begin));
      Task_End_String :=  To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_End));


      To_Natural(Task_Begin_String, Task_Begin, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;

      To_Natural(Task_End_String, Task_End, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_End(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;
      if (Task_Begin<=0)
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if (Task_End<=0)
            then
         Show_Message_Box(   Lb_Task_End(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if Task_Begin > Task_End
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) & Lb_Must_Be(
               Current_Language)
            & To_Unbounded_String(" <= ")
            & Lb_Task_End(Current_Language) );
         return;
      end if;

      Reset_Iterator(Ite2);
      loop
         Current_Element(Sys.Tasks, A_Task, Ite2);

         if A_Task.Name=Task_Name
               then
            if (Task_End>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_End(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );
               return;
            end if;
            if (Task_Begin>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_Begin(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );
               return;
            end if;
            exit;
         end if;
         exit when Is_Last_Element(Sys.Tasks, Ite2) = True;

         Next_Element(Sys.Tasks, Ite2);
      end loop;




      Row:=Get_Focus_Row(Update_Resources_Set.Affected_Task_List);
      Name_Clist        := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,0));
      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,1));
      Task_End_String   := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,2));

      To_Natural(Task_Begin_String, Task_Begin_Clist, Ok);
      To_Natural(Task_End_String, Task_End_Clist, Ok);

      if Name_Clist /= Task_Name
            then
         Show_Message_Box(Lb_Update_Resource_Control(Current_Language));
         return;
      end if;

      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Name = To_Unbounded_String(Get_Text(Get_Entry(
                     Update_Resources_Set.Resource)))
               then
            for Index in 0..A_Resource.Task_Tab.Nb_Entries-1 loop

               if A_Resource.Task_Tab.Entries(Index).Item = Task_Name
                     and A_Resource.Task_Tab.Entries(Index).Data.
                     Task_Begin = Task_Begin_Clist
                     and A_Resource.Task_Tab.Entries(Index).Data.Task_End =
                     Task_End_Clist
                     then
                  A_Resource.Task_Tab.Entries(Index).Data.Task_Begin:=
                     Task_Begin;
                  A_Resource.Task_Tab.Entries(Index).Data.Task_End:=
                     Task_End;
                  Task_List_Share:=A_Resource.Task_Tab;

                  exit;
               end if;
            end loop;
            exit;
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

      Update_Resource(Sys.Resources, Name, State, Cpu, Protocol,
         Task_List_Share);
      Show_Tasks(A_Resource.Name);


   end On_Modify_Pressed;

   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok                : Boolean;  
      Name              : Unbounded_String;  
      State             : Natural;  
      Cpu               : Unbounded_String;  
      Protocol          : Unbounded_String;  
      State_String      : Unbounded_String;  
      Task_Begin        : Natural;  
      Task_End          : Natural;  
      Task_Begin_String : Unbounded_String;  
      Task_End_String   : Unbounded_String;  
      A_Resource        : Resource_Ptr;  

      Task_List_Share   : Resources.Task_List;  
      Affect_Task_Share : Affected_Task_List;  
      Task_Name         : Unbounded_String;  
      My_Iterator       : Resources_Iterator;  

      A_Task : Task_Ptr;  
      Ite2   : Tasks_Iterator;  

      Too_Many_Affected_Tasks : exception;  

   begin
      Reset_Iterator(My_Iterator);

      Cpu:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Cpu)));

      State_String:=To_Unbounded_String(Get_Text(Update_Resources_Set.
            State));
      To_Natural(State_String, State, Ok);

      Name:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Resource)));


      if Get_Active(Update_Resources_Set.Button_No_Protocol) = True
            then
         Protocol:=To_Unbounded_String(Resources_Type'Image(
               No_Protocol));
      else
         if Get_Active(Update_Resources_Set.Button_Pcp) = True
               then
            Protocol:=To_Unbounded_String(Resources_Type'Image(
                  Priority_Ceiling_Protocol));
         else
            if Get_Active(Update_Resources_Set.Button_Pip) = True
                  then
               Protocol:=To_Unbounded_String(Resources_Type'Image(
                     Priority_Inheritance_Protocol));
            end if;
         end if;
      end if;

      Task_Name := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Task_Name));

      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_Begin));
      Task_End_String :=  To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_End));


      To_Natural(Task_Begin_String, Task_Begin, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;

      To_Natural(Task_End_String, Task_End, Ok);
      if not Ok
            then
         Show_Message_Box(   Lb_Task_End(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language) );
         return;
      end if;


      if (Task_Begin<=0)
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if (Task_End<=0)
            then
         Show_Message_Box(   Lb_Task_End(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if Task_Begin > Task_End
            then
         Show_Message_Box(   Lb_Task_Begin(Current_Language) & Lb_Must_Be(
               Current_Language)
            & To_Unbounded_String(" <= ")
            & Lb_Task_End(Current_Language) );
         return;
      end if;


      Reset_Iterator(Ite2);
      loop
         Current_Element(Sys.Tasks, A_Task, Ite2);

         if A_Task.Name=Task_Name
               then
            if (Task_End>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_End(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );
               return;
            end if;
            if (Task_Begin>A_Task.Capacity)
                  then
               Show_Message_Box(   Lb_Task_Begin(Current_Language) &
                  Lb_Must_Be(Current_Language)
                  & To_Unbounded_String(" <= ")
                  & Lb_Capacity(Current_Language) );
               return;
            end if;
            exit;
         end if;

         exit when Is_Last_Element(Sys.Tasks, Ite2) = True;

         Next_Element(Sys.Tasks, Ite2);
      end loop;



      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Name = To_Unbounded_String(Get_Text(Get_Entry(
                     Update_Resources_Set.Resource))) then

            if (Max_Tasks = Natural(A_Resource.Task_Tab.Nb_Entries)) then
               raise Too_Many_Affected_Tasks;

            else

               Affect_Task_Share.Task_Begin := Task_Begin;
               Affect_Task_Share.Task_End := Task_End;
               A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
                  Data:=Affect_Task_Share;
               A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
                  Item:=Task_Name;
               A_Resource.Task_Tab.Nb_Entries:=A_Resource.Task_Tab.
                  Nb_Entries+1;

               Task_List_Share:=A_Resource.Task_Tab;

               exit;


            end if;
            exit;
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

      if Get_Active(Update_Resources_Set.Check_Change_Cpu) = True
            then
         Cpu:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
                  Change_Cpu_List)));
         Update_Resource(Sys.Resources, Name, State, Cpu, Protocol,
            Task_List_Share);
      else
         Update_Resource(Sys.Resources, Name, State, Cpu, Protocol,
            Task_List_Share);
      end if;
      Show_Tasks(A_Resource.Name);

      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Name,"");
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Begin,"");
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_End,"");

   end On_Add_Pressed;


   procedure On_Delete_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok                : Boolean;  
      Name              : Unbounded_String;  
      State             : Natural;  
      Cpu               : Unbounded_String;  
      Protocol          : Unbounded_String;  
      Row               : Gint;  
      State_String      : Unbounded_String;  
      Name_Clist        : Unbounded_String;  
      Task_Begin        : Natural;  
      Task_End          : Natural;  
      Task_Begin_String : Unbounded_String;  
      Task_End_String   : Unbounded_String;  
      Task_End_Clist    : Natural          := 0;  
      Task_Begin_Clist  : Natural          := 0;  
      A_Resource        : Resource_Ptr;  

      Task_List_Share : Resources.Task_List;  
      Task_Name       : Unbounded_String;  
      My_Iterator     : Resources_Iterator;  

   begin
      Reset_Iterator(My_Iterator);

      Cpu:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Cpu)));

      State_String:=To_Unbounded_String(Get_Text(Update_Resources_Set.
            State));
      To_Natural(State_String, State, Ok);

      Name:=To_Unbounded_String(Get_Text(Get_Entry(Update_Resources_Set.
               Resource)));

      if Get_Active(Update_Resources_Set.Button_No_Protocol) = True
            then
         Protocol:=To_Unbounded_String(Resources_Type'Image(
               No_Protocol));
      else
         if Get_Active(Update_Resources_Set.Button_Pcp) = True
               then
            Protocol:=To_Unbounded_String(Resources_Type'Image(
                  Priority_Ceiling_Protocol));
         else
            if Get_Active(Update_Resources_Set.Button_Pip) = True
                  then
               Protocol:=To_Unbounded_String(Resources_Type'Image(
                     Priority_Inheritance_Protocol));
            end if;
         end if;
      end if;



      Task_Name := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Task_Name));
      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_Begin));
      Task_End_String :=  To_Unbounded_String(Get_Text(
            Update_Resources_Set.Task_End));
      To_Natural(Task_Begin_String, Task_Begin, Ok);
      To_Natural(Task_End_String, Task_End, Ok);

      Row:=Get_Focus_Row(Update_Resources_Set.Affected_Task_List);
      Name_Clist        := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,0));
      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,1));
      Task_End_String   := To_Unbounded_String(Get_Text(
            Update_Resources_Set.Affected_Task_List,Row,2));

      To_Natural(Task_Begin_String, Task_Begin_Clist, Ok);
      To_Natural(Task_End_String, Task_End_Clist, Ok);

      if Name_Clist /= Task_Name
            then
         Show_Message_Box(Lb_Update_Resource_Control(Current_Language));
         return;
      end if;

      loop
         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Name = To_Unbounded_String(Get_Text(Get_Entry(
                     Update_Resources_Set.Resource))) then
            for Index in 0..A_Resource.Task_Tab.Nb_Entries-1 loop

               if (A_Resource.Task_Tab.Entries(Index).Item = Task_Name)
                     and A_Resource.Task_Tab.Entries(Index).Data.
                     Task_Begin = Task_Begin_Clist
                     and A_Resource.Task_Tab.Entries(Index).Data.Task_End =
                     Task_End_Clist
                     then
                  A_Resource.Task_Tab.Entries(Index) := A_Resource.
                     Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries-1);
                  A_Resource.Task_Tab.Nb_Entries:=A_Resource.Task_Tab.
                     Nb_Entries-1;
               end if;

            end loop;

            Task_List_Share:=A_Resource.Task_Tab;

            exit;
         end if;
         exit when Is_Last_Element(Sys.Resources, My_Iterator) = True;
         Next_Element(Sys.Resources, My_Iterator);
      end loop;

      Update_Resource(Sys.Resources, Name, State, Cpu, Protocol,
         Task_List_Share);


      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Name, "");
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_Begin, "");
      Gtk.Gentry.Set_Text(Update_Resources_Set.Task_End, "");


      Show_Tasks(A_Resource.Name);

   end On_Delete_Pressed;

end Update_Resources_Set_Pkg.Callbacks;
