------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk.Window;
use Gtk.Window;
with Gtk.Clist;
use Gtk.Clist;
with Translate;
use Translate;
with Config;
use Config;
with Gtk.Combo;
use Gtk.Combo;

with  Unbounded_Strings;
use  Unbounded_Strings;
use  Unbounded_Strings.Strings_Table_Package;

with Gtk.Table;
use Gtk.Table;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Gtk;
use Gtk;
with Gtk.Window;
use Gtk.Window;
with Gtk.Table;
use Gtk.Table;
with Gtk.Label;
use Gtk.Label;
with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Button;
use Gtk.Button;
with Gtk.Option_Menu;
use Gtk.Option_Menu;
with Gtk.Box;
use Gtk.Box;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Frame;
use Gtk.Frame;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Project;
use Project;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Update_Resources_Set_Pkg.Callbacks;
use Update_Resources_Set_Pkg.Callbacks;

package body Update_Resources_Set_Pkg is

   procedure Gtk_New (
         Update_Resources_Set :    out Update_Resources_Set_Access ) is 

   begin
      Update_Resources_Set := new Update_Resources_Set_Record;
      Update_Resources_Set_Pkg.Initialize(Update_Resources_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Resources_Set : access Update_Resources_Set_Record'Class ) is 
      A_Processor : Processor_Ptr;  
      Items       : Strings_Table;  
      Index       : Strings_Range := Strings_Range'First;  

      My_Iterator   : Iterator;  
      Gwendal_Entry : Gtk.Gentry.Gtk_Entry;  
      Reg_Entry     : Gtk.Gentry.Gtk_Entry;  
   begin
      Gtk.Window.Initialize(Update_Resources_Set , Window_Toplevel);
      Set_Title (Update_Resources_Set, To_String(Lb_Update_Resource(
               Current_Language)));
      Set_Policy (Update_Resources_Set, True, True, True);
      Set_Position (Update_Resources_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Resources_Set, True);
      Grab_Focus(Update_Resources_Set);
      Gtk.Box.Gtk_New_Vbox(Update_Resources_Set.Vbox_Main, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Resources_Set.Hbox_Menu, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Resources_Set.Hbox_List, False, 10);
      Gtk.Box.Gtk_New_Hbox(Update_Resources_Set.Hbox_Confirm, False, 10);

      Add(Update_Resources_Set, Update_Resources_Set.Vbox_Main);
      Add(Update_Resources_Set.Vbox_Main, Update_Resources_Set.Hbox_Menu);
      Add(Update_Resources_Set.Vbox_Main, Update_Resources_Set.Hbox_List);
      Add(Update_Resources_Set.Vbox_Main, Update_Resources_Set.
         Hbox_Confirm);

      Update_Resources_Set.Undo_Resource:= Sys.Resources;

      Initialize(Items);
      Index:=0;
      Reset_Iterator(My_Iterator);

      Gtk.Combo.Gtk_New(Update_Resources_Set.Cpu);
      Set_Editable(Get_Entry(Update_Resources_Set.Cpu),False);
      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         String_List.Append(Update_Resources_Set.Processor_Menu, To_String(
               A_Processor.Name));
         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
         Next_Element(Sys.Processors, My_Iterator);
      end loop;
      Set_Popdown_Strings (Update_Resources_Set.Cpu, Update_Resources_Set.
         Processor_Menu);

      Gtk.Frame.Gtk_New(Update_Resources_Set.Frame_Resource,"");

      Gtk.Box.Gtk_New_Vbox(Update_Resources_Set.Vbox_Frame_Resource, False,
         10);
      Gtk.Box.Gtk_New_Vbox(Update_Resources_Set.Vbox_Menu, False, 10);

      Gtk.Combo.Gtk_New(Update_Resources_Set.Resource);
      Set_Editable(Get_Entry(Update_Resources_Set.Resource),False);

      Gtk.Gentry.Gtk_New(Update_Resources_Set.State,50);

      Gtk.Table.Gtk_New(Update_Resources_Set.Table_Protocol,1, 3, False);

      Gtk.Label.Gtk_New(Update_Resources_Set.Resource_Combo_Label,
         To_String(Lb_Use_Protocol(Current_Language)));
      Gtk.Label.Gtk_New(Update_Resources_Set.Resource_State_Label,
         To_String(Lb_Resource_State(Current_Language)));
      Gtk.Label.Gtk_New(Update_Resources_Set.Resource_Label,To_String(
            Lb_Resource_Filter(Current_Language)));
      Gtk.Label.Gtk_New(Update_Resources_Set.Processor_Label,To_String(
            Lb_Cpu_Filter(Current_Language)));

      Gtk.Frame.Gtk_New(Update_Resources_Set.Frame_Cpu,"");
      Gtk.Check_Button.Gtk_New(Update_Resources_Set.Check_Change_Cpu,
         To_String(Lb_Modify(Current_Language)));
      Set_Active(Update_Resources_Set.Check_Change_Cpu, True);
      Gtk.Gentry.Gtk_New(Update_Resources_Set.Change_Cpu, 50);
      Gtk.Box.Gtk_New_Hbox(Update_Resources_Set.Hbox_Cpu, False, 10);
      Gtk.Box.Gtk_New_Vbox(Update_Resources_Set.Vbox_Frame_Cpu, False, 5);
      Gtk.Combo.Gtk_New(Update_Resources_Set.Change_Cpu_List);
      Set_Editable(Get_Entry(Update_Resources_Set.Change_Cpu_List),False);

      Add(Update_Resources_Set.Hbox_Menu, Update_Resources_Set.Vbox_Menu);
      Add(Update_Resources_Set.Vbox_Menu, Update_Resources_Set.Frame_Cpu);
      Add(Update_Resources_Set.Frame_Cpu, Update_Resources_Set.
         Vbox_Frame_Cpu);
      Add(Update_Resources_Set.Vbox_Frame_Cpu, Update_Resources_Set.
         Processor_Label);
      Add(Update_Resources_Set.Vbox_Frame_Cpu, Update_Resources_Set.Cpu);
      Add(Update_Resources_Set.Vbox_Frame_Cpu, Update_Resources_Set.
         Hbox_Cpu);
      Add(Update_Resources_Set.Hbox_Cpu, Update_Resources_Set.
         Check_Change_Cpu);
      Add(Update_Resources_Set.Hbox_Cpu, Update_Resources_Set.
         Change_Cpu_List);
      Add(Update_Resources_Set.Vbox_Menu, Update_Resources_Set.
         Resource_Label);
      Add(Update_Resources_Set.Vbox_Menu, Update_Resources_Set.Resource);

      Add(Update_Resources_Set.Hbox_Menu, Update_Resources_Set.
         Frame_Resource);
      Add(Update_Resources_Set.Frame_Resource, Update_Resources_Set.
         Vbox_Frame_Resource);
      Add(Update_Resources_Set.Vbox_Frame_Resource, Update_Resources_Set.
         Resource_State_Label);
      Add(Update_Resources_Set.Vbox_Frame_Resource, Update_Resources_Set.
         State);
      Add(Update_Resources_Set.Vbox_Frame_Resource, Update_Resources_Set.
         Resource_Combo_Label);
      Add(Update_Resources_Set.Vbox_Frame_Resource, Update_Resources_Set.
         Table_Protocol);

      Gtk_New(Update_Resources_Set.Button_No_Protocol, Label =>
         "No Protocol");
      Gtk_New(Update_Resources_Set.Button_Pip, Group =>
         Update_Resources_Set.Button_No_Protocol, Label => "PIP");
      Gtk_New(Update_Resources_Set.Button_Pcp, Group =>
         Update_Resources_Set.Button_No_Protocol, Label => "PCP");

      Attach_Defaults(Update_Resources_Set.Table_Protocol,
         Update_Resources_Set.Button_No_Protocol, 0, 1, 0, 1);
      Attach_Defaults(Update_Resources_Set.Table_Protocol,
         Update_Resources_Set.Button_Pip, 1, 2, 0, 1);
      Attach_Defaults(Update_Resources_Set.Table_Protocol,
         Update_Resources_Set.Button_Pcp, 2, 3, 0, 1);

      Gtk.Scrolled_Window.Gtk_New(Update_Resources_Set.
         Scroll_Affected_Task_List, null, null);
      Add(Update_Resources_Set.Hbox_List,Update_Resources_Set.
         Scroll_Affected_Task_List);
      Update_Resources_Set.Affect_List_Titles:= To_String(Lb_Task(
            Current_Language))
         +To_String(Lb_Task_Begin(Current_Language))
         +To_String(Lb_Task_End(Current_Language));
      Gtk.Clist.Gtk_New(Update_Resources_Set.Affected_Task_List,3,
         Update_Resources_Set.Affect_List_Titles); -- 3 Columns
      Set_Column_Width(Update_Resources_Set.Affected_Task_List, 0, 90);
      Set_Usize(Update_Resources_Set.Affected_Task_List,200,20);
      Add(Update_Resources_Set.Scroll_Affected_Task_List,
         Update_Resources_Set.Affected_Task_List);

      Gtk.Frame.Gtk_New(Update_Resources_Set.Frame_Task,"");
      Add(Update_Resources_Set.Hbox_List, Update_Resources_Set.Frame_Task);

      Gtk.Box.Gtk_New_Vbox(Update_Resources_Set.Vbox_Frame_Task, False, 10);
      Add(Update_Resources_Set.Frame_Task, Update_Resources_Set.
         Vbox_Frame_Task);

      Gtk.Label.Gtk_New(Update_Resources_Set.Task_Name_Label,To_String(
            Lb_Task_Name(Current_Language)));
      Gtk.Label.Gtk_New(Update_Resources_Set.Task_Begin_Label,To_String(
            Lb_Task_Begin(Current_Language)));
      Gtk.Label.Gtk_New(Update_Resources_Set.Task_End_Label, To_String(
            Lb_Task_End(Current_Language)));

      Gtk.Gentry.Gtk_New(Update_Resources_Set.Task_Name,50);
      Gtk.Gentry.Gtk_New(Update_Resources_Set.Task_Begin,50);
      Gtk.Gentry.Gtk_New(Update_Resources_Set.Task_End,50);

      Gtk.Table.Gtk_New (Update_Resources_Set.Table_Task,4, 3, False);
      Add(Update_Resources_Set.Vbox_Frame_Task, Update_Resources_Set.
         Table_Task);

      Gtk.Button.Gtk_New(Update_Resources_Set.Button_Add_Task, To_String(
            Lb_Add(Current_Language)));
      Gtk.Button.Gtk_New(Update_Resources_Set.Button_Remove_Task,
         To_String(Lb_Delete(Current_Language)));
      Gtk.Button.Gtk_New(Update_Resources_Set.Button_Modify, To_String(
            Lb_Modify(Current_Language)));

      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_Name, 1, 2, 0, 1);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_Begin_Label, 0, 1, 1, 2);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_Begin, 1, 2, 1, 2);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_End_Label, 0, 1, 2, 3);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Task_End, 1, 2, 2, 3);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Button_Add_Task, 0, 1, 3, 4);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Button_Remove_Task , 1, 2, 3, 4);
      Attach_Defaults(Update_Resources_Set.Table_Task,
         Update_Resources_Set.Button_Modify, 2, 3, 3, 4);

      Gtk.Button.Gtk_New(Update_Resources_Set.Button_Add, To_String(
            Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Resources_Set.Button_Cancel, To_String(
            Lb_Cancel(Current_Language)));

      Add(Update_Resources_Set.Hbox_Confirm, Update_Resources_Set.
         Button_Add);
      Add(Update_Resources_Set.Hbox_Confirm, Update_Resources_Set.
         Button_Cancel);

      Button_Callback.Connect (Update_Resources_Set.Button_Cancel,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));

      List_Callback.Connect (Update_Resources_Set.Affected_Task_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'access));

      Gwendal_Entry:=Get_Entry(Update_Resources_Set.Resource);
      Combo_Callback.Connect (Gwendal_Entry, "changed",
         Combo_Callback.To_Marshaller (On_Change_Resource'access));

      Reg_Entry:=Get_Entry(Update_Resources_Set.Cpu);
      Combo_Callback.Connect(Reg_Entry, "changed",
         Combo_Callback.To_Marshaller (On_Cpu_Change'access));

      Button_Callback.Connect (Update_Resources_Set.Button_Modify,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'access));
      Button_Callback.Connect (Update_Resources_Set.Button_Remove_Task,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'access));
      Button_Callback.Connect (Update_Resources_Set.Button_Add_Task,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'access));
      Button_Callback.Connect (Update_Resources_Set.Button_Add, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Button_Callback.Connect (Update_Resources_Set.Check_Change_Cpu,
         "clicked",
         Button_Callback.To_Marshaller (On_Check_Pressed'access));

   end Initialize;

end Update_Resources_Set_Pkg;
