------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Mmi_Tools;
use Mmi_Tools;

with Text_Io;
use Text_Io;

with Translate;
use Translate;

with Config;
use Config;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Parameters;
use Parameters;
use Parameters.Parameters_Table_Package;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Project;
use Project;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;
with Gtkada.File_Selection;
use Gtkada.File_Selection;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with Graph_Item_Pkg;
use Graph_Item_Pkg;
with Graph_Add_Pkg;
use Graph_Add_Pkg;
with My_Canvas;
use My_Canvas;


with Double_Util;
use Double_Util;
with Ada.Numerics.Float_Random;
use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;
use Ada.Numerics.Aux;



package body Update_Tasks_Set_Pkg.Callbacks is


   procedure Page_Switch (
         Notebook : access Gtk_Notebook_Record'Class; 
         Page     : in     Gtk.Gtk_Notebook_Page;     
         Page_Num : in     Gtk_Notebook               ) is 
      pragma Warnings (Off, Page_Num);
      Old_Page : Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);  
      Pixmap   : Gtk_Pixmap;  
   begin
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);

      if Old_Page /= null then
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Closed,
            Update_Tasks_Set.Book_Closed_Mask);
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Closed,
            Update_Tasks_Set.Book_Closed_Mask);
      end if;
   end Page_Switch;


   procedure Show_Tasks (
         A_Task_Name : Unbounded_String ) is 
      A_Task          : Task_Ptr;  
      My_Iterator     : Tasks_Iterator;  
      Line            : Gint;  
      Task_Share      : Chars_Ptr_Array (1 .. 12);  
      Capacity        : Unbounded_String;  
      Start_Time      : Unbounded_String;  
      Jitter          : Unbounded_String;  
      Deadline        : Unbounded_String;  
      Period          : Unbounded_String;  
      Priority        : Unbounded_String;  
      Blocking_Time   : Unbounded_String;  
      Parametric_File : Unbounded_String;  
      Seed_Value      : Unbounded_String;  

      A_Periodic_Task   : Periodic_Task_Ptr;  
      A_Poisson_Task    : Poisson_Task_Ptr;  
      A_Parametric_Task : Parametric_Task_Ptr;  

      Policy    : Unbounded_String;  
      Task_Type : Unbounded_String;  

   begin
      Clear(Update_Tasks_Set.Tasks_List);

      Reset_Iterator(My_Iterator);
      if not Is_Empty(Sys.Tasks) then
         loop

            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            Capacity      := To_Unbounded_String(
               Natural'Image(A_Task.Capacity));
            Deadline      := To_Unbounded_String(
               Natural'Image(A_Task.Deadline));
            Start_Time    := To_Unbounded_String(
               Natural'Image(A_Task.Start_Time));
            Priority      := To_Unbounded_String(
               Natural'Image(Natural(A_Task.Priority)));
            Blocking_Time := To_Unbounded_String(
               Natural'Image(A_Task.Blocking_Time));

            case A_Task.Task_Type is
               when Periodic_Type =>
                  A_Periodic_Task := Periodic_Task_Ptr(A_Task);
                  Jitter    := To_Unbounded_String(
                     Natural'Image(A_Periodic_Task.Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(A_Periodic_Task.Period));
                  Parametric_File   := To_Unbounded_String("");
                  Seed_Value :=  To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Periodic");
               when Aperiodic_Type =>
                  Jitter    := To_Unbounded_String(" 0");
                  Period    := To_Unbounded_String(" 0");
                  Parametric_File   := To_Unbounded_String("");
                  Seed_Value :=  To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Aperiodic");
               when Poisson_Type =>
                  A_Poisson_Task := Poisson_Task_Ptr(A_Task);
                  Jitter    := To_Unbounded_String(
                     Natural'Image(A_Poisson_Task.Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(A_Poisson_Task.Period));
                  Seed_Value :=  To_Unbounded_String(
                     Natural'Image(A_Poisson_Task.Seed));
                  Parametric_File   := To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Poisson");
               when Parametric_Type =>
                  A_Parametric_Task := Parametric_Task_Ptr(A_Task);
                  Parametric_File   := A_Parametric_Task.File_Name;
                  Seed_Value :=  To_Unbounded_String("");
                  Jitter    := To_Unbounded_String(" 0");
                  Period    := To_Unbounded_String(" 0");
                  Task_Type := To_Unbounded_String("Parametric");
            end case;


            case A_Task.Policy is
               when Sched_Fifo  =>
                  Policy := To_Unbounded_String("Fifo");
               when Sched_Rr =>
                  Policy := To_Unbounded_String("RR");
               when Sched_Others =>
                  Policy := To_Unbounded_String("Other");
               when others =>
                  null;
            end case;


            -- Create the line
            --
            Task_Share := To_String(A_Task.Name)+To_String(Task_Type)+
               To_String(A_Task.Location)+To_String(Policy)+
               To_String(Priority)+To_String(Capacity)+To_String(Jitter)+
               To_String(Deadline)+To_String(Period)+To_String(Start_Time)+
               To_String(Blocking_Time)+To_String(Parametric_File);
            -- Display the Line
            --
            Line := Append(Update_Tasks_Set.Tasks_List, Task_Share);

            exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;

            Next_Element(Sys.Tasks, My_Iterator);

         end loop;
      end if;
   end Show_Tasks;


   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 

      Name            : Unbounded_String;  
      Capacity        : Unbounded_String;  
      Jitter          : Unbounded_String;  
      Deadline        : Unbounded_String;  
      Period          : Unbounded_String;  
      Start_Time      : Unbounded_String;  
      Priority        : Unbounded_String;  
      Blocking_Time   : Unbounded_String;  
      Parametric_File : Unbounded_String;  
      Row             : Gint;  
      My_Iterator     : Processor_Set.Generic_Processor_Set.Iterator;  
      A_Processor     : Processor_Ptr;  
      Index           : Strings_Range                                := 
        Strings_Range'First;  

      A_Task            : Task_Ptr;  
      Current           : Gint;  
      Task_Str          : Unbounded_String;  
      Location_Items    : Strings_Table;  
      Parameter_Items   : array (Parameters_Range) of Strings_Table;  
      Current_Task_Type : Gint;  
      Current_Policy    : Gint;  

   begin
      Row:=Get_Focus_Row(Update_Tasks_Set.Tasks_List);
      Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Tasks_List,
            Row,0));
      Update_Tasks_Set.Current_Task := Name;
      Priority      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,4));
      Capacity      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,5));
      Jitter        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,6));
      Deadline      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,7));
      Period        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,8));
      Start_Time    := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,9));
      Blocking_Time := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,10));
      Parametric_File := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,11));

      Gtk.Gentry.Set_Text(Update_Tasks_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Priority, To_String(Priority));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Capacity, To_String(Capacity));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Jitter, To_String(Jitter));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Deadline, To_String(Deadline));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Period, To_String(Period));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Start_Time, To_String(
            Start_Time));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Blocking_Time, To_String(
            Blocking_Time));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.File_Name, To_String(
            Parametric_File));

      Task_Str:=Update_Tasks_Set.Current_Task;
      if (Task_Str /= To_Unbounded_String("")) then
         A_Task:=Search_Task(Sys.Tasks,Task_Str);
         Update_Tasks_Set.Current_Task_Ptr := A_Task;



         ----- Rebuild The CPU Option Menu & select the right item
         --
         Initialize(Location_Items);
         Index:=0;
         Processor_Set.Generic_Processor_Set.Reset_Iterator(My_Iterator);

         loop
            Current_Element(Sys.Processors, A_Processor, My_Iterator);
            Add(Location_Items,A_Processor.Name);
            if A_Processor.Name = A_Task.Location then
               Current:=Gint(Index);
            end if;
            Index:=Index+1;
            exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;
            Next_Element(Sys.Processors, My_Iterator);
         end loop;

         Update_Tasks_Set.Clist_Processor_Menu:= Widget_Slist.Null_List;

         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Processor_Menu);

         Build_Option_Menu (Update_Tasks_Set.Processor_Menu,
            Update_Tasks_Set.Clist_Processor_Menu,
            Location_Items, Current, null);


         Show_All(Update_Tasks_Set.Processor_Menu);
         Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
            Processor_Menu,1,2,2,3);

         Show_All(Update_Tasks_Set);


         -- rebuild type menu
         --
         Initialize(Location_Items);
         Add(Location_Items,To_Unbounded_String("Periodic"));
         Add(Location_Items,To_Unbounded_String("Aperiodic"));
         Add(Location_Items,To_Unbounded_String("Poisson"));
         Add(Location_Items,To_Unbounded_String("Parametric"));

         Current_Task_Type := Gint'(Tasks_Type'Pos(A_Task.Task_Type));

         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Task_Type_Menu);
         Update_Tasks_Set.Clist_Task_Type_Menu:= Widget_Slist.Null_List;

         Build_Option_Menu (Update_Tasks_Set.Task_Type_Menu,
            Update_Tasks_Set.
            Clist_Task_Type_Menu, Location_Items, Current_Task_Type, null);
         Show_All(Update_Tasks_Set.Task_Type_Menu);
         Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
            Task_Type_Menu,1,2,1,2);
         Show_All(Update_Tasks_Set);


         -- rebuild policy menu
         --
         Initialize(Location_Items);
         Add(Location_Items,To_Unbounded_String("SCHED_FIFO"));
         Add(Location_Items,To_Unbounded_String("SCHED_RR"));
         Add(Location_Items,To_Unbounded_String("SCHED_OTHERS"));

         case A_Task.Policy is
            when Sched_Fifo   =>
               Current_Policy:=0;
            when Sched_Rr     =>
               Current_Policy:=1;
            when Sched_Others =>
               Current_Policy:=2;
            when others =>
               null;
         end case;

         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Policy_Menu);
         Update_Tasks_Set.Clist_Policy_Menu:= Widget_Slist.Null_List;

         Build_Option_Menu (Update_Tasks_Set.Policy_Menu, Update_Tasks_Set.
            Clist_Policy_Menu, Location_Items, Current_Policy, null);
         Show_All(Update_Tasks_Set.Policy_Menu);
         Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
            Policy_Menu,1,2,3,4);
         Show_All(Update_Tasks_Set);

         if A_Task.Task_Type = Poisson_Type then
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,
               Poisson_Task_Ptr(A_Task).Seed'Img);

         else
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,"");

         end if;

         Update_Tasks_Set.Current_Task := A_Task.Name;

         -- Offset information
         --
         for I in 0..Offsets_Range'Last-1 loop
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Act, "");
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Val, "");
         end loop;
         for I in 0 .. A_Task.Offset.Nb_Entries-1 loop
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Act,
               A_Task.Offset.Entries(I).Activation'Img);
            Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Val,
               A_Task.Offset.Entries(I).Value'Img);
         end loop;


         -- User defined parameters
         --
         for I in 0..Parameters_Range'Last-1 loop
            Set_Text(Update_Tasks_Set.Param(I).Entry_Name, "");
            Set_Text(Update_Tasks_Set.Param(I).Entry_Value, "");
         end loop;

         for I in A_Task.Param.Nb_Entries .. Parameters_Range'Last-1 loop
            Initialize(Parameter_Items(I));
            Add(Parameter_Items(I),To_Unbounded_String("Integer"));
            Add(Parameter_Items(I),To_Unbounded_String("Double"));
            Add(Parameter_Items(I),To_Unbounded_String("String"));
            Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

            Destroy(Update_Tasks_Set.Param(I).Parameter_Menu);
            Update_Tasks_Set.Param(I).Clist_Parameter_Menu :=
               Widget_Slist.Null_List;
            Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Param(I).
               Parameter_Menu);

            Build_Option_Menu (Update_Tasks_Set.Param(I).Parameter_Menu,
               Update_Tasks_Set.Param(I).Clist_Parameter_Menu,
               Parameter_Items(I),
               -1, null);
            Attach (Update_Tasks_Set.Parameter_Table, Update_Tasks_Set.
               Param(I).Parameter_Menu, 1, 2,
               1+Guint(I), 2+Guint(I),
               Expand or Fill, 0,
               0, 0);
         end loop;



         for I in 0..A_Task.Param.Nb_Entries-1 loop
            Set_Text(Update_Tasks_Set.Param(I).Entry_Name, To_String(
                  A_Task.Param.Entries(I).Name));


            if A_Task.Param.Entries(I).Ptype=Boolean_Parameter then
               Set_Text(Update_Tasks_Set.Param(I).Entry_Value, A_Task.
                  Param.Entries(I).Boolean_Value'Img);
               Current:=3;
            else
               if A_Task.Param.Entries(I).Ptype=Integer_Parameter then
                  Set_Text(Update_Tasks_Set.Param(I).Entry_Value, A_Task.
                     Param.Entries(I).Integer_Value'Img);
                  Current:=0;
               else
                  if A_Task.Param.Entries(I).Ptype=Double_Parameter then
                     Set_Text(Update_Tasks_Set.Param(I).Entry_Value,
                        To_String(Format(A_Task.Param.Entries(I).
                              Double_Value)));
                     Current:=1;
                  else
                     Set_Text(Update_Tasks_Set.Param(I).Entry_Value,
                        To_String(A_Task.Param.Entries(I).String_Value));
                     Current:=2;
                  end if;
               end if;
            end if;

            Initialize(Parameter_Items(I));
            Add(Parameter_Items(I),To_Unbounded_String("Integer"));
            Add(Parameter_Items(I),To_Unbounded_String("Double"));
            Add(Parameter_Items(I),To_Unbounded_String("String"));
            Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

            Destroy(Update_Tasks_Set.Param(I).Parameter_Menu);
            Update_Tasks_Set.Param(I).Clist_Parameter_Menu := Widget_Slist.
               Null_List;
            Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Param(I).
               Parameter_Menu);

            Build_Option_Menu (Update_Tasks_Set.Param(I).Parameter_Menu,
               Update_Tasks_Set.Param(I).Clist_Parameter_Menu,
               Parameter_Items(I),
               Current, null);
            Attach (Update_Tasks_Set.Parameter_Table, Update_Tasks_Set.
               Param(I).Parameter_Menu, 1, 2,
               1+Guint(I), 2+Guint(I),
               Expand or Fill, 0,
               0, 0);


         end loop;
         Show_All(Update_Tasks_Set);

      end if;


   end On_Click_List;


   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Is_New:=False;
      Is_Saved:=False;

      Rebuild_Canvas(Graph_Window);
      Hide(Update_Tasks_Set);
   end On_Ok_Pressed;

   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Ok                        : Boolean;  
      Task_Name                 : Unbounded_String;  
      Task_Processor            : Unbounded_String;  
      Task_Type                 : Tasks_Type                         := 
        Periodic_Type;  
      Task_Policy               : Policies                           := 
        Sched_Fifo;  
      Task_Type_Index           : Gint                               := 
        0;  
      Policy_Index              : Gint                               := 
        0;  
      Task_Priority             : Priority_Range;  
      Task_Priority2            : Integer;  
      Task_Capacity             : Integer;  
      Task_Jitter               : Integer;  
      Task_Deadline             : Integer;  
      Task_Period               : Integer;  
      Task_Start_Time           : Integer;  
      Task_Blocking_Time        : Integer;  
      Task_Seed_Value           : Integer;  
      Task_Offset               : Offsets_Table;  
      Task_Param                : Parameters_Table;  
      Task_Priority_String      : Unbounded_String;  
      Task_Capacity_String      : Unbounded_String;  
      Task_Jitter_String        : Unbounded_String;  
      Task_Deadline_String      : Unbounded_String;  
      Task_Period_String        : Unbounded_String;  
      Task_Start_Time_String    : Unbounded_String;  
      Task_Blocking_Time_String : Unbounded_String;  
      Task_File_Name_String     : Unbounded_String;  
      Task_Seed_Value_String    : Unbounded_String;  
      A_Task                    : Task_Ptr;  
      Cpu_Index                 : Gint                               := 
        0;  
      A_Processor               : Processor_Ptr;  
      My_Iterator               : Task_Set.Generic_Task_Set.Iterator;  
      Current                   : Unbounded_String;  
      A_Offset                  : Offset_Type;  
      A_Param                   : Parameter_Ptr;  
      A_Param_Type              : Gint;  

   begin

      --
      -- First task page : general information
      --

      Task_Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Name));


      if Task_Name=""
            then
         Show_Message_Box(
            Lb_Task_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         return;
      end if;


      if not Is_A_Valid_Identifier(Task_Name)
            then
         Show_Message_Box(Lb_Task_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;



      Reset_Iterator(My_Iterator);
      if not Is_Empty(Sys.Tasks) then
         loop
            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            if (Task_Name = A_Task.Name) then
               if(A_Task.Name/=Update_Tasks_Set.Current_Task) then
                  Show_Message_Box(Lb_Task_Name(Current_Language) &
                     Lb_Already_Defined(Current_Language));
                  return;
               end if;
            end if;
            exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
            Next_Element(Sys.Tasks, My_Iterator);
         end loop;
      end if;

      Task_Priority_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Priority));
      Task_Capacity_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Capacity));
      Task_Jitter_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Jitter));
      Task_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Deadline));
      Task_Period_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Period));
      Task_Start_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Start_Time));
      Task_Blocking_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Blocking_Time));
      Task_File_Name_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.File_Name));
      Task_Seed_Value_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Seed_Value));
      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Tasks_Set.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Task_Processor :=A_Processor.Name;

      -- Convert value from menu to Tasks_Type
      Task_Type_Index:= Gint(3 - Selected_Button (Update_Tasks_Set.
            Clist_Task_Type_Menu));

      Task_Type := Tasks_Type'Val(Task_Type_Index);

      -- Convert value from menu to Policies
      Policy_Index:= Gint(2 - Selected_Button (Update_Tasks_Set.
            Clist_Policy_Menu));

      Task_Policy := Policies'Val(Policy_Index);

      To_Integer(Task_Priority_String,Task_Priority2, Ok);
      if not Ok
            then
         if Task_Priority_String /= "" then
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Capacity_String,Task_Capacity, Ok);
      if not Ok
            then
         if Task_Capacity_String /= "" then
            Show_Message_Box(Lb_Capacity(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Jitter_String,Task_Jitter, Ok);
      if not Ok
            then
         if Task_Jitter_String /= "" then
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Deadline_String,Task_Deadline, Ok);
      if not Ok
            then
         if Task_Deadline_String /= "" then
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Period_String,Task_Period, Ok);
      if not Ok
            then
         if Task_Period_String /= "" then
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Seed_Value_String,Task_Seed_Value, Ok);
      if not Ok
            then
         if Task_Seed_Value_String /= "" then
            Show_Message_Box(Lb_Seed(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;


      case Task_Type is
         when Periodic_Type =>
            if Task_Period<=0 then
               Show_Message_Box(Lb_Periodic(Current_Language) &
                  Lb_Require_Period(Current_Language));
               return;
            end if;

         when Aperiodic_Type =>
            if Task_Period/=0 then
               Show_Message_Box(Lb_Aperiodic(Current_Language) &
                  Lb_No_Period(Current_Language));
               return;
            end if;

         when Poisson_Type =>
            if Task_Period<=0 then
               Show_Message_Box(Lb_Poisson_Process(Current_Language) &
                  Lb_Require_Period(Current_Language));
               return;
            end if;

         when Parametric_Type =>
            if Task_Period/=0 then
               Show_Message_Box(Lb_Parametric_Task(Current_Language) &
                  Lb_No_Period(Current_Language));
               return;
            end if;
      end case;

      To_Integer(Task_Start_Time_String,Task_Start_Time, Ok);
      if not Ok
            then
         if Task_Start_Time_String /= "" then
            Show_Message_Box(Lb_Start_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Blocking_Time_String,Task_Blocking_Time, Ok);
      if not Ok
            then
         if Task_Blocking_Time_String /= "" then
            Show_Message_Box(Lb_Blocking_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;



      if (Task_Capacity<=0)
            then
         Show_Message_Box(Lb_Capacity(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Task_Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Task_Deadline<Task_Jitter)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         return;
      end if;

      if (Task_Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Start_Time<0)
            then
         Show_Message_Box(Lb_Start_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Blocking_Time<0)
            then
         Show_Message_Box(Lb_Blocking_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if ((Natural(Task_Priority2)<Natural(Priority_Range'First))
            or (Natural(Task_Priority2)>Natural(Priority_Range'Last)) )
            then
         Show_Message_Box(Lb_Invalid_Priority(Current_Language));
         return;
      end if;

      Task_Priority := Priority_Range(Task_Priority2);


      if (Task_Priority2 /= 0) and
            (Task_Policy=Sched_Others)
            then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;

      if (Task_Priority2 = 0) and
            (Task_Policy/=Sched_Others)
            then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;

      if (To_String(Task_File_Name_String) /= ""
            and Task_Type /= Parametric_Type) then
         Show_Message_Box(Lb_File_For_Parametric_Only(Current_Language));
         return;
      end if;

      --
      --- Second page of the widget : user defined parameters
      --
      Initialize(Task_Param);

      for I in 0..Parameters_Range'Last-1 loop

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
               Param(I).Entry_Name));


         if Current /= Empty_String then


            A_Param_Type:=  Gint(Selected_Button (Update_Tasks_Set.
                  Param(I).Clist_Parameter_Menu));


            if A_Param_Type = 3
                  then
               A_Param:= new Parameter(Integer_Parameter);
            else
               if A_Param_Type = 2
                     then
                  A_Param:= new Parameter(Double_Parameter);
               else
                  if A_Param_Type = 1
                        then
                     A_Param:= new Parameter(String_Parameter);
                  else
                     A_Param:= new Parameter(Boolean_Parameter);
                  end if;
               end if;
            end if;


            A_Param.Name:=Current;

            Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
                  Param(I).Entry_Value));

            -- Integer case
            --
            if A_Param_Type=3 then
               if(Current="")
                     then
                  A_Param.Integer_Value:=0;
               else
                  To_Integer(Current,A_Param.Integer_Value,Ok);
                  if not Ok
                        then
                     Show_Message_Box(To_Unbounded_String("Parameter " &
                           Integer'Image(Integer(I+1)) & " ") &
                        Lb_Must_Be(Current_Language) & To_Unbounded_String(
                           " integer"));
                     return;
                  end if;
               end if;

               -- Double case
               --
            else
               if A_Param_Type=2 then
                  if(Current="")
                        then
                     A_Param.Double_Value:=0.0;
                  else
                     To_Double(Current,A_Param.Double_Value,Ok);
                     if not Ok
                           then
                        Show_Message_Box(To_Unbounded_String("Parameter " &
                              Integer'Image(Integer(I+1)) & " ") &
                           Lb_Must_Be(Current_Language) &
                           To_Unbounded_String(" double"));
                        return;
                     end if;
                  end if;

                  -- String case
                  --
               else
                  if A_Param_Type=1 then
                     A_Param.String_Value:=Current;

                     -- Boolean case
                     --
                  else
                     if(Current="")
                           then
                        A_Param.Boolean_Value:=True;
                     else
                        To_Boolean(Current,A_Param.Boolean_Value,Ok);
                        if not Ok
                              then
                           Show_Message_Box(To_Unbounded_String(
                                 "Parameter " &
                                 Integer'Image(Integer(I+1)) & " ") &
                              Lb_Must_Be(Current_Language) &
                              To_Unbounded_String(" boolean"));
                           return;
                        end if;
                     end if;

                  end if;
               end if;
            end if;


            Add(Task_Param, A_Param);
         end if;

      end loop;


      --
      -- Offset information
      --
      Initialize(Task_Offset);

      for I in 0..Offsets_Range'Last-1 loop


         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Val));
         if(Current="")
               then
            A_Offset.Value:=0;
         else
            To_Integer(Current, A_Offset.Value,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ")") & Lb_Value(
                     Current_Language));
               return;
            end if;
         end if;

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Act));
         if(Current="")
               then
            A_Offset.Activation:=0;
         else
            To_Integer(Current, A_Offset.Activation,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ") ") &
                  Lb_Activation(
                     Current_Language));
               return;
            end if;
         end if;

         if A_Offset.Activation/=0
               then
            Add(Task_Offset,A_Offset);
         end if;

      end loop;



      for I in 0..Task_Offset.Nb_Entries-1 loop

         if Task_Offset.Entries(I).Value<0
               then
            Show_Message_Box(To_Unbounded_String("Offset/val.") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0"));
            return;
         end if;

         if Task_Offset.Entries(I).Activation<0
               then
            Show_Message_Box(To_Unbounded_String("Offset/activation") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") );
            return;
         end if;

      end loop;


      --
      -- Update task set
      --

      Add_Task (Sys.Tasks, Task_Name, Task_Processor, Task_Type,
         Natural(Task_Start_Time), Natural(Task_Capacity),
         Natural(Task_Period), Natural(Task_Deadline),
         Natural(Task_Jitter), Natural(Task_Blocking_Time),
         Task_Priority,Task_Policy, Task_Offset, Task_Param,
         Task_File_Name_String, Task_Seed_Value);

      Update_Tasks_Set.Current_Task := Task_Name;

      Reset_Iterator(My_Iterator);
      Current_Element(Sys.Tasks, A_Task, My_Iterator);
      Show_Tasks(A_Task.Name);

      if Add_New_Item /= null then
         Set_Name(Items_List(Last_Item-1),Task_Name);
         Refresh_Canvas(Graph_Window.Buffer);
      end if;


   exception
      when Task_Already_Defined =>
         begin
            Show_Message_Box(Lb_Task_Name(Current_Language) &
               Lb_Already_Defined(Current_Language) );
            return;
         end;

   end On_Add_Pressed;

   procedure On_Seed_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Seed : Generator;  
      Nmb  : Integer;  
   begin
      -- Give random value for the tasks seed
      --
      Reset(Seed);
      -- range from integer'first .. integer'last
      Nmb := Integer(Double(Random(Seed))*Double(Integer'Last-Integer'
            First)+
         Double(Integer'First));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,Nmb'Img);

   end On_Seed_Pressed;


   procedure On_Delete_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      My_Iterator : Task_Set.Generic_Task_Set.Iterator;  
      A_Task      : Task_Ptr;  

   begin

      Reset_Iterator(My_Iterator);
      if not Is_Empty(Sys.Tasks) then
         loop

            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            if A_Task.Name = Update_Tasks_Set.Current_Task then

               Delete(Sys.Tasks, A_Task);

               -- Clear main entrys
               Update_Tasks_Set.Current_Task := To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Priority, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Capacity, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Jitter, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Deadline, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Period, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Start_Time, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Blocking_Time, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.File_Name, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value, "");

               -- Clear User defined parameters
               for I in 0..Parameters_Range'Last-1 loop
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Param(I).
                     Entry_Name, "");
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Param(I).
                     Entry_Value, "");
               end loop;

               -- Clear Offset parameters
               for I in 0..Offsets_Range'Last-1 loop
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).
                     Entry_Val, "");
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).
                     Entry_Act, "");
               end loop;

               exit;
            end if;

            exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
            Next_Element(Sys.Tasks, My_Iterator);

         end loop;
         Show_Tasks(A_Task.Name);
      end if;

   end On_Delete_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Sys.Tasks:=Update_Tasks_Set.Undo_Task;
      Destroy(Update_Tasks_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      if Visible_Is_Set(Update_Tasks_Set.Vbox_List) then
         Show_Advanced(Update_Tasks_Set,False);
      else
         Show_Advanced(Update_Tasks_Set,True);
      end if;


   end On_Advanced_Pressed;

   procedure On_File_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin
      Gtk.Gentry.Set_Text(Update_Tasks_Set.File_Name,
         (File_Selection_Dialog(
               To_String(Lb_Open_File(Current_Language)),
               "",   -- default directory
               False, -- dir only
               False))); -- must exist


   end On_File_Pressed;

   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
      Ok                        : Boolean;  
      Task_Name                 : Unbounded_String;  
      Task_Processor            : Unbounded_String;  
      Task_Type                 : Tasks_Type;  
      Task_Policy               : Policies;  
      Task_Type_Index           : Gint;  
      Policy_Index              : Gint;  
      Task_Priority             : Priority_Range;  
      Task_Priority2            : Integer;  
      Task_Capacity             : Integer;  
      Task_Jitter               : Integer;  
      Task_Deadline             : Integer;  
      Task_Period               : Integer;  
      Task_Start_Time           : Integer;  
      Task_Blocking_Time        : Integer;  
      Task_Seed_Value           : Integer;  
      Task_Offset               : Offsets_Table;  
      Task_Param                : Parameters_Table;  
      Task_Priority_String      : Unbounded_String;  
      Task_Capacity_String      : Unbounded_String;  
      Task_Jitter_String        : Unbounded_String;  
      Task_Deadline_String      : Unbounded_String;  
      Task_Period_String        : Unbounded_String;  
      Task_Start_Time_String    : Unbounded_String;  
      Task_Blocking_Time_String : Unbounded_String;  
      Task_File_Name_String     : Unbounded_String;  
      Task_Seed_Value_String    : Unbounded_String;  
      --      Task_Predictable          : Boolean;
      A_Task       : Task_Ptr;  
      Cpu_Index    : Gint;  
      A_Processor  : Processor_Ptr;  
      My_Iterator  : Task_Set.Generic_Task_Set.Iterator;  
      Current      : Unbounded_String;  
      A_Offset     : Offset_Type;  
      A_Param      : Parameter_Ptr;  
      A_Param_Type : Gint;  

   begin

      --
      -- First task page : general information
      --

      Task_Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Name));
      if Update_Tasks_Set.Current_Task = "" then
         Show_Message_Box(Lb_Select_Task_First(Current_Language));
         return;
      end if;

      Reset_Iterator(My_Iterator);
      if not Is_Empty(Sys.Tasks) then
         loop
            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            if (Task_Name = A_Task.Name) then
               if(A_Task.Name/=Update_Tasks_Set.Current_Task) then
                  Show_Message_Box(Lb_Task_Name(Current_Language) &
                     Lb_Already_Defined(Current_Language));
                  return;
               end if;
            end if;
            exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
            Next_Element(Sys.Tasks, My_Iterator);
         end loop;
      else
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         return;
      end if;

      Task_Priority_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Priority));
      Task_Capacity_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Capacity));
      Task_Jitter_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Jitter));
      Task_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Deadline));
      Task_Period_String := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Period));
      Task_Start_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Start_Time));
      Task_Blocking_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Blocking_Time));
      Task_File_Name_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.File_Name));
      Task_Seed_Value_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Seed_Value));
      --      Task_Predictable := Get_Active(Update_Tasks_Set.Predictable_Button);

      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Tasks_Set.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Task_Processor :=A_Processor.Name;

      -- Convert value from menu to Tasks_Type
      Task_Type_Index:= Gint(3 - Selected_Button (Update_Tasks_Set.
            Clist_Task_Type_Menu));

      Task_Type := Tasks_Type'Val(Task_Type_Index);

      -- Convert value from menu to Policies
      Policy_Index:= Gint(2 - Selected_Button (Update_Tasks_Set.
            Clist_Policy_Menu));

      Task_Policy := Policies'Val(Policy_Index);

      To_Integer(Task_Priority_String,Task_Priority2, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Priority(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return;
      end if;

      To_Integer(Task_Capacity_String,Task_Capacity, Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Capacity(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return;
      end if;

      To_Integer(Task_Jitter_String,Task_Jitter, Ok);
      if not Ok
            then
         if Task_Jitter_String /= "" then
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Deadline_String,Task_Deadline, Ok);
      if not Ok
            then
         if Task_Deadline_String /= "" then
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Period_String,Task_Period, Ok);
      if not Ok
            then
         if Task_Period_String /= "" then
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Seed_Value_String,Task_Seed_Value, Ok);
      if not Ok
            then
         if Task_Seed_Value_String /= "" then
            Show_Message_Box(Lb_Seed(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Start_Time_String,Task_Start_Time, Ok);
      if not Ok
            then
         if Task_Start_Time_String /= "" then
            Show_Message_Box(Lb_Start_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      To_Integer(Task_Blocking_Time_String,Task_Blocking_Time, Ok);
      if not Ok
            then
         if Task_Blocking_Time_String /= "" then
            Show_Message_Box(Lb_Blocking_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            return;
         end if;
      end if;

      if ((Natural(Task_Priority2)<Natural(Priority_Range'First))
            or (Natural(Task_Priority2)>Natural(Priority_Range'Last)) )
            then
         Show_Message_Box(Lb_Invalid_Priority(Current_Language));
         return;
      end if;

      Task_Priority := Priority_Range(Task_Priority2);


      if (Task_Priority2 /= 0) and (Task_Policy=Sched_Others) then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;

      if (Task_Priority2 = 0) and
            (Task_Policy/=Sched_Others)
            then
         Show_Message_Box(Lb_Policy_Control(Current_Language) );
         return;
      end if;

      if (Task_Capacity<=0)
            then
         Show_Message_Box(Lb_Capacity(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      if (Task_Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         return;
      end if;

      case Task_Type is
         when Periodic_Type =>
            if Task_Period<=0 then
               Show_Message_Box(Lb_Periodic(Current_Language) &
                  Lb_Require_Period(Current_Language));
               return;
            end if;

         when Aperiodic_Type =>
            if Task_Period>0 then
               Show_Message_Box(Lb_Aperiodic(Current_Language) &
                  Lb_No_Period(Current_Language));
               return;
            end if;

         when Poisson_Type =>
            if Task_Period<=0 then
               Show_Message_Box(Lb_Poisson_Process(Current_Language) &
                  Lb_Require_Period(Current_Language));
               return;
            end if;

         when Parametric_Type =>
            if Task_Period>0 then
               Show_Message_Box(Lb_Parametric_Task(Current_Language) &
                  Lb_No_Period(Current_Language));
               return;
            end if;
      end case;

      if (Task_Deadline<Task_Jitter)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         return;
      end if;

      if (Task_Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Start_Time<0)
            then
         Show_Message_Box(Lb_Start_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (Task_Blocking_Time<0)
            then
         Show_Message_Box(Lb_Blocking_Time(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         return;
      end if;

      if (To_String(Task_File_Name_String) /= ""
            and Task_Type /= Parametric_Type) then
         Show_Message_Box(Lb_File_For_Parametric_Only(Current_Language));
         return;
      end if;

      --
      --- Second page of the widget : user defined parameters
      --
      Initialize(Task_Param);

      for I in 0..Parameters_Range'Last-1 loop

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
               Param(I).Entry_Name));


         if Current /= Empty_String then


            A_Param_Type:=  Gint(Selected_Button (Update_Tasks_Set.
                  Param(I).Clist_Parameter_Menu));


            if A_Param_Type = 3
                  then
               A_Param:= new Parameter(Integer_Parameter);
            else
               if A_Param_Type = 2
                     then
                  A_Param:= new Parameter(Double_Parameter);
               else
                  if A_Param_Type = 1
                        then
                     A_Param:= new Parameter(String_Parameter);
                  else
                     A_Param:= new Parameter(Boolean_Parameter);
                  end if;
               end if;
            end if;


            A_Param.Name:=Current;

            Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
                  Param(I).Entry_Value));

            -- Integer case
            --
            if A_Param_Type=3 then
               if(Current="")
                     then
                  A_Param.Integer_Value:=0;
               else
                  To_Integer(Current,A_Param.Integer_Value,Ok);
                  if not Ok
                        then
                     Show_Message_Box(To_Unbounded_String("Parameter " &
                           Integer'Image(Integer(I+1)) & " ") &
                        Lb_Must_Be(Current_Language) & To_Unbounded_String(
                           " integer"));
                     return;
                  end if;
               end if;

               -- Double case
               --
            else
               if A_Param_Type=2 then
                  if(Current="")
                        then
                     A_Param.Double_Value:=0.0;
                  else
                     To_Double(Current,A_Param.Double_Value,Ok);
                     if not Ok
                           then
                        Show_Message_Box(To_Unbounded_String("Parameter " &
                              Integer'Image(Integer(I+1)) & " ") &
                           Lb_Must_Be(Current_Language) &
                           To_Unbounded_String(" double"));
                        return;
                     end if;
                  end if;

                  -- String case
                  --
               else
                  if A_Param_Type=1 then
                     A_Param.String_Value:=Current;

                     -- Boolean case
                     --
                  else
                     if(Current="")
                           then
                        A_Param.Boolean_Value:=True;
                     else
                        To_Boolean(Current,A_Param.Boolean_Value,Ok);
                        if not Ok
                              then
                           Show_Message_Box(To_Unbounded_String(
                                 "Parameter " &
                                 Integer'Image(Integer(I+1)) & " ") &
                              Lb_Must_Be(Current_Language) &
                              To_Unbounded_String(" boolean"));
                           return;
                        end if;
                     end if;

                  end if;
               end if;
            end if;


            Add(Task_Param, A_Param);
         end if;

      end loop;


      --
      -- Offset information
      --
      Initialize(Task_Offset);

      for I in 0..Offsets_Range'Last-1 loop


         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Val));
         if(Current="")
               then
            A_Offset.Value:=0;
         else
            To_Integer(Current, A_Offset.Value,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ")") & Lb_Value(
                     Current_Language));
               Sys.Tasks:=Update_Tasks_Set.Undo_Task;
               return;
            end if;
         end if;

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Act));
         if(Current="")
               then
            A_Offset.Activation:=0;
         else
            To_Integer(Current, A_Offset.Activation,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ") ") &
                  Lb_Activation(
                     Current_Language));
               return;
            end if;
         end if;

         if A_Offset.Activation/=0
               then
            Add(Task_Offset,A_Offset);
         end if;

      end loop;



      for I in 0..Task_Offset.Nb_Entries-1 loop

         if Task_Offset.Entries(I).Value<0
               then
            Show_Message_Box(To_Unbounded_String("Offset/val.") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0"));
            return;
         end if;

         if Task_Offset.Entries(I).Activation<0
               then
            Show_Message_Box(To_Unbounded_String("Offset/activation") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") );
            return;
         end if;

      end loop;


      --
      -- Update task set
      --

      Reset_Iterator(My_Iterator);
      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         if A_Task.Name = Update_Tasks_Set.Current_Task then

            Delete(Sys.Tasks, A_Task);

            Add_Task (Sys.Tasks, Task_Name, Task_Processor, Task_Type,
               Natural(Task_Start_Time), Natural(Task_Capacity),
               Natural(Task_Period), Natural(Task_Deadline),
               Natural(Task_Jitter), Natural(Task_Blocking_Time),
               Task_Priority,Task_Policy, Task_Offset, Task_Param,
               Task_File_Name_String, Task_Seed_Value);


            Update_Tasks_Set.Current_Task := Task_Name;
            exit;
         end if;


         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);

      end loop;

      Show_Tasks(A_Task.Name);

   end On_Modify_Pressed;




end Update_Tasks_Set_Pkg.Callbacks;
