------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gdk;
with Gdk.Bitmap;
use Gdk.Bitmap;
with Gdk.Pixmap;
use Gdk.Pixmap;
with Gtk.Enums;
use Gtk.Enums;
with Gtk.Frame;
use Gtk.Frame;
with Gtk.Label;
use Gtk.Label;
with Gtk.Notebook;
use Gtk.Notebook;
with Gtk.Pixmap;
use Gtk.Pixmap;
with Gtk.Separator;
use Gtk.Separator;
with Gtk;
use Gtk;

with Gtk.Window;
use Gtk.Window;
with Gtk.Table;
use Gtk.Table;
with Gtk.Label;
use Gtk.Label;
with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Separator;
use Gtk.Separator;
with Gtk.Button;
use Gtk.Button;
with Gtk.Box;
use Gtk.Box;
with Gtk.Hbutton_Box;
use Gtk.Hbutton_Box;
with Gtk.Scrolled_Window;
use Gtk.Scrolled_Window;
with Gtk.Option_Menu;
use Gtk.Option_Menu;
with Gtk.Widget;
use Gtk.Widget;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Gtk.Clist;
use Gtk.Clist;
with Gtkada.Types;
use Gtkada.Types;
with Gtk.Frame;
use Gtk.Frame;
with Gtk.Scrolled_Window;
use Gtk.Scrolled_Window;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Gtk.Enums;
use Gtk.Enums;

with Tasks_Widget_Pkg;
use Tasks_Widget_Pkg;

with Systems;
use Systems;

package Update_Tasks_Set_Pkg is

   type Update_Tasks_Set_Record is new Gtk_Window_Record with private; 
   type Update_Tasks_Set_Access is access all Update_Tasks_Set_Record'Class; 

   procedure Gtk_New (
         Update_Tasks_Set :    out Update_Tasks_Set_Access ); 
   procedure Initialize (
         Update_Tasks_Set : access Update_Tasks_Set_Record'Class ); 
   procedure Show_Advanced (
         Update_Tasks_Set : in     Update_Tasks_Set_Access; 
         Advanced         : in     Boolean                  ); 

   Update_Tasks_Set : Update_Tasks_Set_Access;  
   Parent           : Unbounded_String;  


private

   type Update_Tasks_Set_Record is new Gtk_Window_Record with 
      record 
         Scroll : Gtk_Scrolled_Window;  

         Tasks_List           : Gtk.Clist.Gtk_Clist;  
         Task_Type_Menu       : Gtk_Option_Menu;  
         Clist_Task_Type_Menu : Widget_Slist.Gslist;  
         Processor_Menu       : Gtk_Option_Menu;  
         Clist_Processor_Menu : Widget_Slist.Gslist;  
         Policy_Menu          : Gtk_Option_Menu;  
         Clist_Policy_Menu    : Widget_Slist.Gslist;  
         Task_Items           : String_List.Glist;  
         Current_Task         : Unbounded_String;  
         Current_Task_Ptr     : Task_Ptr;  
         Name                 : Gtk_Entry;  
         Start_Time           : Gtk_Entry;  
         Capacity             : Gtk_Entry;  
         Period               : Gtk_Entry;  
         Deadline             : Gtk_Entry;  
         Jitter               : Gtk_Entry;  
         Priority             : Gtk_Entry;  
         Blocking_Time        : Gtk_Entry;  
         Seed_Value           : Gtk_Entry;  
         File_Name            : Gtk_Entry;  

         Name_Label          : Gtk_Label;  
         Task_Type_Label     : Gtk_Label;  
         Processor_Label     : Gtk_Label;  
         Start_Time_Label    : Gtk_Label;  
         Capacity_Label      : Gtk_Label;  
         Period_Label        : Gtk_Label;  
         Deadline_Label      : Gtk_Label;  
         Jitter_Label        : Gtk_Label;  
         Priority_Label      : Gtk_Label;  
         Policy_Label        : Gtk_Label;  
         Blocking_Time_Label : Gtk_Label;  
         File_Name_Label     : Gtk_Label;  
         Seed_Label          : Gtk_Label;  
         Offset_Label        : Gtk_Label;  
         Offset_Act_Label    : Gtk_Label;  
         Offset_Val_Label    : Gtk_Label;  

         Frame_Task      : Gtk.Frame.Gtk_Frame;  
         Task_Table      : Gtk.Table.Gtk_Table;  
         Offset_Table    : Gtk.Table.Gtk_Table;  
         Parameter_Table : Gtk.Table.Gtk_Table;  

         Vbox_Main    : Gtk.Box.Gtk_Box;  
         Hbox_Task    : Gtk.Box.Gtk_Box;  
         Hbox_Confirm : Gtk.Box.Gtk_Box;  
         Vbox_List    : Gtk.Box.Gtk_Box;  
         Hbox_Modify  : Gtk.Box.Gtk_Box;  
         Vbox_Update  : Gtk.Box.Gtk_Box;  
         Hbox_File    : Gtk.Box.Gtk_Box;  
         Hbox_Seed    : Gtk.Box.Gtk_Box;  

         Hbutton_Box_Modify : Gtk_Hbutton_Box;  

         Modify_Button   : Gtk_Button;  
         Advanced_Button : Gtk_Button;  
         Ok_Button       : Gtk_Button;  
         Cancel_Button   : Gtk_Button;  
         Add_Button      : Gtk_Button;  
         Delete_Button   : Gtk_Button;  
         Seed_Button     : Gtk_Button; 
         -- to generate new seed for task 
         File_Button     : Gtk_Button;  

         Hseparator1 : Gtk_Hseparator;  
         Vseparator  : Gtk_Vseparator;  
         Vseparator2 : Gtk_Vseparator;  

         Child_Page : Gtk_Frame;  
         Label_Box  : Gtk_Box;  
         Menu_Box   : Gtk_Box;  
         Pixmap     : Gtk_Pixmap;  
         Vbox       : Gtk_Box;  
         Button     : Gtk_Button;  
         Lab        : Gtk_Label;  

         --         Predictable_Button : Gtk_Radio_Button;
         --         Unpredictable_Button : Gtk_Radio_Button;
         --         Seed_Group : Widget_Slist.Gslist;
         Offset : Widget_Offset_Table;  
         Param  : Widget_Parameter_Table;  

         Book_Opened      : Gdk_Pixmap;  
         Book_Opened_Mask : Gdk_Bitmap;  
         Book_Closed      : Gdk_Pixmap;  
         Book_Closed_Mask : Gdk_Bitmap;  
         Notebook         : Gtk_Notebook;  

         Undo_Task : Tasks_Set;  

         Scroll_List : Gtk_Scrolled_Window;  
         List_Titles : Chars_Ptr_Array (1 .. 12);  

         Sys      : System;  
         Advanced : Boolean := False;  
      end record; 



end Update_Tasks_Set_Pkg;
