------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;



package body Discrete_Util is

   package Integer_Io is new Text_Io.Integer_Io(Discrete_Element);


   function Format (
         F : in     Discrete_Element ) 
     return Unbounded_String is 
      S      : String (1 .. 500);  
      Result : Unbounded_String;  
   begin
      Integer_Io.Put(S,F,10);

      -- suppress blank
      for I in 1..S'Length loop
         if S(I) /= ' '
               then
            Append(Result,S(I));
         end if;
      end loop;

      return(Result);
   end Format;



   function Gcd (
         A : in     Discrete_Element; 
         B : in     Discrete_Element  ) 
     return Discrete_Element is 

      Z      : Discrete_Element;  
      Temp_A : Discrete_Element := A;  
      Temp_B : Discrete_Element := B;  

   begin
      loop
         if Temp_B=0
               then
            return Temp_A;
         end if;
         Z:=Temp_A mod Temp_B;
         Temp_A:=Temp_B;
         Temp_B:=Z;
      end loop;
   end Gcd;

   function Lcm (
         A : in     Discrete_Element; 
         B : in     Discrete_Element  ) 
     return Discrete_Element is 

      Tmp : Double := Double (A) * Double (B) / Double (Gcd (A, B));  
   begin
      return Discrete_Element(Double'Floor(Tmp));
   end Lcm;


end Discrete_Util;


