------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Text_Io;
use Text_Io;





with Unchecked_Deallocation;




package body Lists is


   procedure Free is 
   new Unchecked_Deallocation
      (Cell, Cell_Ptr);


   procedure Free is 
   new Unchecked_Deallocation
      (Element, Element_Ptr);


   procedure Add (
         L : in out List;       
         E : in     Element_Ptr ) is 

      New_Cell : Cell_Ptr;  

   begin
      New_Cell:=new Cell;
      New_Cell.Info:=E;

      New_Cell.Next:=L.Head;
      L.Head:=New_Cell;

      if (L.Tail = null)
            then
         L.Tail := New_Cell;
      else
         New_Cell.Next.Previous:=New_Cell;
      end if;


   end Add;



   procedure Put (
         L : in     List ) is 

      Current : Cell_Ptr := L.Head;  

   begin

      while Current/=null loop
         Put(Current.Info.all);
         Current:=Current.Next;
      end loop;
   end Put;



   procedure Delete (
         L : in out List;       
         E : in     Element_Ptr ) is 

      Current  : Cell_Ptr := L.Head;  
      Is_Found : Boolean  := False;  

   begin
      while Current/=null loop
         if Current.Info=E
               then
            Is_Found:=True;

            if (Current.Previous=null) -- Delete the last item
                  and (Current.Next=null)
                  then
               L.Head:=null;
               L.Tail:=null;
            else

               if (Current.Previous=null) -- Delete the Head item
                     then
                  L.Head:=Current.Next;
                  Current.Next.Previous:=null;
               else

                  if (Current.Next = null) -- delete the Tail item
                        then
                     L.Tail := Current.Previous;
                     Current.Previous.Next:=null;
                  else
                     -- Middle item
                     Current.Previous.Next:=Current.Next;
                     Current.Next.Previous:=Current.Previous;
                  end if;
               end if;
            end if;

            Free(Current.Info);
            Free(Current);
            exit;
         end if;

         Current:=Current.Next;
      end loop;

      if not Is_Found then
         raise Element_Not_Found;
      end if;

   end Delete;


   procedure Duplicate (
         Src  : in     List; 
         Dest : in out List  ) is 

      Current : Cell_Ptr := Src.Tail;  

   begin

      while Current/=null loop


         Add(Dest,Current.Info);

         Current:=Current.Previous;
      end loop;

   end Duplicate;


   function Get_Head (
         L : in     List ) 
     return Element_Ptr is 

   begin
      return(L.Head.Info);
   end Get_Head;


   function Get_Tail (
         L : in     List ) 
     return Element_Ptr is 

   begin
      return(L.Tail.Info);
   end Get_Tail;


   function Is_Empty (
         L : in     List ) 
     return Boolean is 
   begin
      return (L.Head=null) and (L.Tail=null);
   end Is_Empty;




end Lists;
