------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




generic
   type Element is private; 
   with procedure Put (
         E : in     Element ); 


package Lists is



   type Cell is private; 
   type List is private; 

   type Element_Ptr is access Element; 

   Element_Not_Found : exception;


   procedure Add (
         L : in out List;       
         E : in     Element_Ptr ); 


   procedure Put (
         L : in     List ); 


   procedure Delete (
         L : in out List;       
         E : in     Element_Ptr ); 


   procedure Duplicate (
         Src  : in     List; 
         Dest : in out List  ); 


   function Is_Empty (
         L : in     List ) 
     return Boolean; 


   function Get_Head (
         L : in     List ) 
     return Element_Ptr; 


   function Get_Tail (
         L : in     List ) 
     return Element_Ptr; 




private

   type Cell_Ptr is access Cell; 

   type Cell is 
      record 
         Next     : Cell_Ptr;  
         Previous : Cell_Ptr;  
         Info     : Element_Ptr;  
      end record; 


   type List is 
      record 
         Head : Cell_Ptr;  
         Tail : Cell_Ptr;  
      end record; 



end Lists;
