------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--  This package provides complement definitions to easily manipulate
--  unbounded_string
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;

with Convert_Unbounded_Strings;

with Tables;




package Unbounded_Strings is


   -- Some standard strings constants 
   --
   Unbounded_Sp : constant Unbounded_String := To_Unbounded_String (" ");
      

   Unbounded_Lf : constant Unbounded_String := To_Unbounded_String ("" &
     Ascii.Lf);  
   Unbounded_Ht : constant Unbounded_String := To_Unbounded_String ("" &
     Ascii.Ht);  
   Empty_String : constant Unbounded_String := To_Unbounded_String ("");
      





   type Unbounded_String_Ptr is access Unbounded_String; 


   ----------------------------------------------
   -- String tables
   ----------------------------------------------

   -- Maximum number of strings composing a table of strings
   --
   Max_Strings : constant Natural := 100;  


   procedure Initialize (
         St :    out Unbounded_String ); 

   package Strings_Table_Package is new Tables(Unbounded_String,
      Max_Strings,Put);
   use Strings_Table_Package;

   subtype Strings_Range is Strings_Table_Package.Table_Range;
   subtype Strings_Table is Strings_Table_Package.Table;



   ------------------------------------------------------
   -- Convert unbounded_strings to standard numeric types 
   ------------------------------------------------------

   procedure To_Natural is 
   new Convert_Unbounded_Strings(Natural,0);
   procedure To_Integer is 
   new Convert_Unbounded_Strings(Integer,0);
   procedure To_Boolean is 
   new Convert_Unbounded_Strings(Boolean,True);


   pragma Inline (To_Natural);
   pragma Inline (To_Integer);
   pragma Inline (To_Boolean);

   procedure To_Double (
         From : in     Unbounded_String; 
         To   :    out Double;           
         Ok   :    out Boolean           ); 


end Unbounded_Strings;
