------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2005   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the EA 2215 Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
-- This package contains every text labels used in the different parts
-- of the Cheddar graphical editor. The text is given for each language
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Config;
use Config;

package Translate  is

   pragma Elaborate_Body;

   type Labels is array (Languages'range) of Unbounded_String; 



   ----------------------------------------------------
   -- Labels common on every languages
   ----------------------------------------------------
   Lb_Point,  
   Lb_Colon,  
   Lb_Minus,  
   Lb_Equal,  
   Lb_Comma : Unbounded_String;  


   ----------------------------------------------------
   --   Menu and sub menu titles
   ----------------------------------------------------
   Lb_Root_Title,  
   Lb_File,  
   Lb_New,  
   Lb_Open,  
   Lb_Save,  
   Lb_Edit_Scheduler,  
   Lb_Save_As,  
   Lb_Exit,  

   Lb_Edit,  
   Lb_View,  

   Lb_Tools,  
   Lb_Compute_Buffer,  
   Lb_Draw_Buffer,  
   Lb_Clear,  
   Lb_Draw,  
   Lb_Check,  
   Lb_Compute_And_Display,  
   Lb_Compute_And_Update,  
   Lb_Compute_And_Display_One_Step,  
   Lb_Compute_And_Update_One_Step,  

   Lb_Help,  
   Lb_Manual,  
   Lb_Ref,  
   Lb_About,  
   Lb_About_Text : Labels;  


   ----------------------------------------------------
   --   Add, delete, display, list and update
   ----------------------------------------------------
   Lb_Update,  
   Lb_Update_Task,  
   Lb_Update_Message,  
   Lb_Update_Buffer,  
   Lb_Update_Processor,  
   Lb_Update_Resource  : Labels;  

   Lb_Add,  
   Lb_Add_Task,  
   Lb_Add_Processor,  
   Lb_Add_Buffer,  
   Lb_Add_Message,  
   Lb_Add_Resource  : Labels;  

   Lb_List,  
   Lb_Tasks_List,  
   Lb_Processors_List,  
   Lb_Resources_List,  
   Lb_Messages_List,  
   Lb_Buffers_List    : Labels;  

   Lb_Delete,  
   Lb_Delete_Task,  
   Lb_Delete_Processor,  
   Lb_Delete_Resource,  
   Lb_Delete_Message,  
   Lb_Delete_Buffer    : Labels;  


   ------------------------------------------------------------
   --   Errors on messages/tasks/processors/resources/buffers
   ------------------------------------------------------------
   Lb_Resource_Not_Found,  
   Lb_Message_Not_Found,  
   Lb_Task_Not_Found,  
   Lb_Processor_Not_Found,  
   Lb_Buffer_Not_Found    : Labels;  

   Lb_Define_Tasks_Before,  
   Lb_Define_Processors_Before,  
   Lb_Define_Messages_Before,  
   Lb_Define_Resources_Before,  
   Lb_Define_Buffers_Before    : Labels;  

   ----------------------------------------------------
   --    I/O Errors and messages
   ----------------------------------------------------
   Lb_Open_File,  
   Lb_Save_File,  
   Lb_Not_Saved_File,  
   Lb_Can_Not_Open_File,  
   Lb_Can_Not_Save_File,  
   Lb_File_Name         : Labels;  

   Lb_Open_Project,  
   Lb_Save_Project,  
   Lb_Not_Saved_Project,  
   Lb_Can_Not_Open_Project,  
   Lb_Can_Not_Save_Project : Labels;  



   ----------------------------------------------------
   --   Widget text fields
   ----------------------------------------------------
   --
   --  Processor widgets
   --
   Lb_Only_On_Parametric,  
   Lb_Scheduler_Behavior,  
   Lb_Read_From_File,  
   Lb_Quantum_Control,  
   Lb_Processor,  
   Lb_Processor_Name,  
   Lb_Scheduler,  
   Lb_Option,  
   Lb_Preemptive,  
   Lb_Not_Preemptive,  
   Lb_Invalid_Identifier : Labels;  

   --
   --  Tasks widgets
   --
   Lb_Task_Name,  
   Lb_Task,  
   Lb_Task_Type,  
   Lb_Capacity,  
   Lb_Deadline,  
   Lb_Period,  
   Lb_Jitter,  
   Lb_Priority,  
   Lb_Policy,  
   Lb_Policy_Control,  
   Lb_Start_Time,  
   Lb_Seed,  
   Lb_Predictable,  
   Lb_Unpredictable,  
   Lb_Offset_Val,  
   Lb_Offset_Act,  
   Lb_Offsets,  
   Lb_Activations,  
   Lb_Activation,  
   Lb_Values,  
   Lb_Value,  
   Lb_User_Defined_Parameter,  
   Lb_File_For_Parametric_Only,  
   Lb_Main,  
   Lb_No_Period,  
   Lb_Require_Period,  
   Lb_Select_Task_First,  
   Lb_Can_Not_Modify,  
   Lb_Task_Not_Permitted       : Labels;  

   --
   --  Message widgets
   --
   Lb_Message,  
   Lb_Message_Name : Labels;  

   --
   --  Buffer widgets
   --
   Lb_Buffer,  
   Lb_Buffer_Name,  
   Lb_Size,  
   Lb_Time,  
   Lb_Maximum_Buffer_Size,  
   Lb_Utilization_Level,  
   Lb_Bound_Buffer,  
   Lb_Role_Type,  
   Lb_Role,  
   Lb_Role_Consumer,  
   Lb_Role_Producer,  
   Lb_Role_List,  
   Lb_Buffer_List,  
   Lb_Buffer_Information  : Labels;  

   --
   --  Resource widgets
   --
   Lb_Resource_Name,  
   Lb_Resource,  
   Lb_Update_Resource_Control,  
   Lb_Use_Protocol,  
   Lb_Resource_Filter,  
   Lb_Cpu_Filter,  
   Lb_View_Resource_State,  
   Lb_View_Resource_Protocol,  
   Lb_Modify,  
   Lb_Resource_State,  
   Lb_Task_Begin,  
   Lb_Task_End,  
   Lb_Resources,  
   Lb_Resource_Affect,  
   Lb_Resource_Information    : Labels;  

   --
   --  Dependencies graph widgets
   --
   Lb_New_Buffer_Role,  
   Lb_Disp1,  
   Lb_Disp2,  
   Lb_Disp3,  
   Lb_Create,  
   Lb_Select,  
   Lb_Name,  
   Lb_Select_Row_Before,  
   Lb_Precedencies,  
   Lb_Arrow,  
   Lb_New_Item,  
   Lb_Graph_View,  
   Lb_Choose_Item       : Labels;  

   --
   -- Scheduling options
   --
   Lb_Scheduling_Options,  
   Lb_Schedule_According_To : Labels;  

   --
   -- Seed options
   --
   Lb_Seed_Options,  
   Lb_Task_Specific,  
   Lb_All_Predictable,  
   Lb_All_Unpredictable : Labels;  

   --
   -- Duplicate widget
   --
   Lb_Duplicate,  
   Lb_Old,  
   Lb_Object    : Labels;  



   --
   -- Others ...
   --
   Lb_Advanced,  
   Lb_Cancel,  
   Lb_Warning,  
   Lb_See,  
   Lb_Equation,  
   Lb_Theorem,  
   Lb_Inferior,  
   Lb_Superior,  
   Lb_Lemma,  
   Lb_Close,  
   Lb_And,  
   Lb_Continue : Labels;  



   ----------------------------------------------------
   --   Data controls
   ----------------------------------------------------
   Lb_Invalid_Priority,  
   Lb_Mandatory,  
   Lb_Already_Defined,  
   Lb_Must_Be,  
   Lb_Must_Be_Numeric,  
   Lb_Parameter        : Labels;  


   ----------------------------------------------------
   --   Parametric scheduler messages
   ----------------------------------------------------
   --
   -- Parser and parametric simulation errors
   --
   Lb_Type_Error,  
   Lb_Statement_Error,  
   Lb_Variable_Error,  
   Lb_Syntax_Error,  
   Lb_Identifier_Already_Exists,  
   Lb_Undeclared_Identifier,  
   Lb_Identifier,  
   Lb_Integer_Conversion_Error,  
   Lb_Double_Conversion_Error,  
   Lb_Index_Error,  
   Lb_Uncompatible_Type_Error,  
   Lb_Can_Not_Edit_To_Parametric_Object,  
   Lb_Operation_Not_Permitted           : Labels;  


   ----------------------------------------------------
   --   Logic messages/errors (scheduling, buffer
   --           and blocking time)
   ----------------------------------------------------
   --
   -- Blocking logic
   --
   Lb_Compute_Blocking_Error1,  
   Lb_Compute_Blocking_Error2,  
   Lb_Blocking_Time,  
   Lb_Inject_Success          : Labels;  


   --
   -- Scheduling logic
   --
   Lb_Simulation_Error,  
   Lb_Priorities_Error,  
   Lb_Set_Priorities_Dm,  
   Lb_Set_Priorities_Rm,  
   Lb_Set_Priorities,  
   Lb_Schedule_From_To,  
   Lb_Periodic_Situation,  
   Lb_Scheduling,  
   Lb_Free_Unit_Time,  
   Lb_Utilization_With_Period,  
   Lb_Utilization_With_Deadline,  
   Lb_Utilization_Bound,  
   Lb_Task_Response_Time,  
   Lb_Task_Response_Time_From_Simulation,  
   Lb_Scheduling_Period,  
   Lb_No_Missed_Deadline,  
   Lb_Number_Of_Preemption,  
   Lb_Priorities,  

   Lb_Compute_Error_1,  
   Lb_Compute_Error_2,  
   Lb_Compute_Error_3,  
   Lb_Compute_Error_4,  
   Lb_Compute_Error_5,  
   Lb_Compute_Error_6,  
   Lb_Compute_Error_7,  
   Lb_Compute_Error_8,  
   Lb_Compute_Error_9,  
   Lb_Compute_Error_10,  
   Lb_Compute_Error_11,  
   Lb_Compute_Error_12,  
   Lb_Compute_Error_13,  
   Lb_Compute_Error_14,  
   Lb_Compute_Error_15,  
   Lb_Compute_Error_18,  
   Lb_Compute_Error_19,  

   Lb_Check_Deadline,  
   Lb_Periodic,  
   Lb_Aperiodic,  
   Lb_Poisson_Process,  
   Lb_Parametric_Task,  
   Lb_Completion_Time : Labels;  




   --
   -- Buffer logic
   --
   Lb_Harmonic,  
   Lb_Compute_Error_16,  
   Lb_Compute_Error_17,  
   Lb_Schedule_Before,  
   Lb_Buffer_Utilization,  
   Lb_Compute_Error_20   : Labels;  



   --
   -- Dependencies logic
   --
   Lb_Chetto,  
   Lb_Jitter_From_Response_Time_All_Steps,  
   Lb_Deadlines_Are_Updated,  
   Lb_Priorities_Are_Updated,  
   Lb_Jitters_Are_Updated,  
   Lb_Chetto_Error1,  
   Lb_Chetto_Error2,  
   Lb_Chetto_Error3,  
   Lb_Chetto_Deadline,  
   Lb_Chetto_Priority,  
   Lb_Jitter_From_Response_Time_One_Step,  
   Lb_Jitter_From_Response_Time,  
   Lb_Precedencies_Period_Error           : Labels;  


   --
   -- Random tools
   --
   Lb_Random,  
   Lb_Compute_Response_Time_Density,  
   Lb_Response_Time_Density,  
   Lb_Response_Time                 : Labels;  




end Translate;










